/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __DPTX_REG_H__
#define __DPTX_REG_H__

/* Constants */
#define GB02MAC2043		0
#define GB02MAC2044		1
#define GB02MAC2045		2

/* Controller Version Registers */
#define GB02MAC2046			0x00
#define GB02MAC2047			0x31303061

#define GB02MAC2048			0x04

#define GB02MAC2049				0x08
#define GB02MAC2050               0x9001
#define GB02MAC2051		0x16c3

/* DPTX Configuration Register */
#define GB02MAC2052			0x100

/* CoreControl Register */
#define GB02MAC2053			0x200
#define GB02MAC2054			0x204

/* MST */
#define GB02MAC2055(n)	(0x210 + (n) * 4)

#define GB02MAC2056 14

/* Video Registers. N=0-3 */
#define GB02MAC2057(n)		(0x300  + 0x10000 * (n))
#define GB02MAC2058(n)	(0x304  + 0x10000 * (n))
#define GB02MAC2059(n)	(0x308  + 0x10000 * (n))
#define GB02MAC2060(n)	(0x30C  + 0x10000 * (n))
#define GB02MAC2062(n)		(0x310  + 0x10000 * (n))
#define GB02MAC2063(n)		(0x314  + 0x10000 * (n))
#define GB02MAC2064(n)		(0x318  + 0x10000 * (n))
#define GB02MAC2065(n)		(0x31c  + 0x10000 * (n))
#define GB02MAC2066(n)		(0x320  + 0x10000 * (n))
#define GB02MAC2067(n)		(0x324  + 0x10000 * (n))
#define GB02MAC2068(n)		(0x328  + 0x10000 * (n))
#define GB02MAC2069(n)		(0x32C  + 0x10000 * (n))
#define GB02MAC2070(n)		(0x3804 + 0x10000 * (n))
#define GB02MAC2071(n)		(0x3808 + 0x10000 * (n))
#define GB02MAC2072(n)		(0x380C + 0x10000 * (n))
#define GB02MAC2073(n)		(0x3810 + 0x10000 * (n))
#define GB02MAC2074(n)		(0x3814 + 0x10000 * (n))
#define GB02MAC2075(n)		(0x3800 + 0x10000 * (n))
#define GB02MAC2076	0x330
#define GB02MAC2077 16
#define GB02MAC2078 1
/* Audio Registers */
#define GB02MAC2079		0x400
#define GB02MAC2080			0x3904
#define GB02MAC2081			0x3908
#define GB02MAC2082			0x390C
#define GB02MAC2083			0x3910
#define GB02MAC2084			0x3914
#define GB02MAC2085			0x3918

/* SecondaryDataPacket Registers */
#define GB02MAC2086		0x500
#define GB02MAC2087        0x504
#define GB02MAC2088			0x508
#define GB02MAC2089			0x600

/* PHY Layer Control Registers */
#define GB02MAC2090			0xA00
#define GB02MAC2091			0xA04
#define GB02MAC2092			0xA08
#define GB02MAC2093			0xA0C
#define GB02MAC2094			0xA10
#define GB02MAC2095		0xC00
#define GB02MAC2096		0xC04
#define GB02MAC2097		0xC08

/* AUX Channel Interface Registers */
#define GB02MAC2098			0xB00
#define GB02MAC2099			0xB04
#define GB02MAC2100			0xB08
#define GB02MAC2101			0xB0C
#define GB02MAC2102			0xB10
#define GB02MAC2103			0xB14
#define GB02MAC2104		0xb40
#define GB02MAC2105		0xb44
#define GB02MAC2106	0xb48

/* Interrupt Registers */
#define GB02MAC2107			0xD00
#define GB02MAC2108			0xD04
#define GB02MAC2109			0xD08
#define GB02MAC2110			0xD0C

/* HDCP Registers */
#define GB02MAC2111		0xE00
#define GB02MAC2112		0xE10

/* I2C APB Registers */
#define GB02MAC2113			0xc006c
#define GB02MAC2114			0xc0000
#define GB02MAC2115			0xc0004
#define GB02MAC2116		0xc0010
#define GB02MAC2117	BIT(4)

/* RAM Registers */

#define GB02MAC2118(n)		(0x381C + 0x10000 * (n))
#define GB02MAC2119(n)	(0x3820 + 0x10000 * (n))
#define GB02MAC2120(n)	(0x3824 + 0x10000 * (n))

#define GB02MAC2121	12
#define DPTX_VG_RAM_ADDR_START_MASK	GENMASK(12, 0)
#define GB02MAC2122	0
#define GB02MAC2123	BIT(0)
#define GB02MAC2124	0
#define DPTX_VG_WRT_RAM_DATA_MASK	GENMASK(0, 7)

/* Register Bitfields */
#define GB02MAC2125		16
#define DPTX_ID_DEVICE_ID_MASK		GENMASK(31, 16)
#define GB02MAC2126		0
#define DPTX_ID_VENDOR_ID_MASK		GENMASK(15, 0)

#define GB02MAC2127	16
#define DPTX_CONFIG1_NUM_STREAMS_MASK	GENMASK(18, 16)
#define GB02MAC2128	19
#define DPTX_CONFIG1_MP_MODE_MASK	GENMASK(21, 19)
#define GB02MAC2129	1
#define GB02MAC2130	2
#define GB02MAC2131	4
#define GB02MAC2132		BIT(22)
#define GB02MAC2133	25
#define DPTX_CONFIG1_NUM_DSC_ENC_MASK	GENMASK(28, 25)
#define GB02MAC2134		BIT(29)

#define GB02MAC2135		BIT(1)
#define GB02MAC2136		BIT(2)
#define GB02MAC2137		BIT(3)
#define GB02MAC2138	BIT(25)
#define GB02MAC2139		BIT(26)
#define GB02MAC2140	BIT(28)
#define GB02MAC2141	BIT(29)
#define GB02MAC2142	BIT(16)

#define GB02MAC2143	BIT(0)
#define GB02MAC2144		BIT(1)
#define GB02MAC2145		BIT(2)
#define GB02MAC2146	BIT(3)
#define GB02MAC2147		BIT(4)
#define GB02MAC2148(n)	BIT(5 + n)
#define GB02MAC2149 (GB02MAC2143 |		\
			    GB02MAC2145 |		\
			    GB02MAC2146 |	\
			    GB02MAC2147)

#define GB02MAC2150	0
#define DPTX_PHYIF_CTRL_TPS_SEL_MASK	GENMASK(3, 0)
#define GB02MAC2151	0
#define GB02MAC2152		1
#define GB02MAC2153		2
#define GB02MAC2154		3
#define GB02MAC2155		4
#define GB02MAC2156	5
#define GB02MAC2157	6
#define GB02MAC2158	7
#define GB02MAC2159	8
#define GB02MAC2160	9
#define GB02MAC2161	4
#define DPTX_PHYIF_CTRL_RATE_MASK	GENMASK(5, 4)
#define GB02MAC2162	6
#define DPTX_PHYIF_CTRL_LANES_MASK	GENMASK(7, 6)
#define GB02MAC2163	0x0	//1.62G
#define GB02MAC2164	0x1	//2.7G
#define GB02MAC2165	0x2 //5.4G
#define GB02MAC2166	0x3	//8.1G
#define GB02MAC2167	0x4
#define GB02MAC2168	0x5
#define GB02MAC2169	0x6
#define GB02MAC2170	0x7
#define GB02MAC2172	(GB02MAC2170 - GB02MAC2166 - 1)
#define GB02MAC2175	8
#define DPTX_PHYIF_CTRL_XMIT_EN_MASK	GENMASK(11, 8)
#define GB02MAC2177(lane)	BIT(8 + lane)
#define GB02MAC2180(lane)	BIT(12 + lane)
#define GB02MAC2183		BIT(16)
#define GB02MAC2185 17
#define DPTX_PHYIF_CTRL_LANE_PWRDOWN_MASK GENMASK(20, 17)
#define GB02MAC2188		BIT(25)

#define GB02MAC2190(lane)	(6 * lane)
#define DPTX_PHY_TX_EQ_PREEMP_MASK(lane)	GENMASK(6 * lane + 1, 6 * lane)
#define GB02MAC2192(lane)	(6 * lane + 2)
#define DPTX_PHY_TX_EQ_VSWING_MASK(lane)	GENMASK(6 * lane + 3, \
							6 * lane + 2)

#define GB02MAC2195	0
#define DPTX_AUX_CMD_REQ_LEN_MASK	GENMASK(3, 0)
#define GB02MAC2197	BIT(4)
#define GB02MAC2199		8
#define DPTX_AUX_CMD_ADDR_MASK		GENMASK(27, 8)
#define GB02MAC2200		28
#define DPTX_AUX_CMD_TYPE_MASK		GENMASK(31, 28)
#define GB02MAC2201		0x0
#define GB02MAC2202		0x1
#define GB02MAC2203		0x2
#define GB02MAC2204		0x4
#define GB02MAC2205	0x8

#define GB02MAC2206	0
#define DPTX_AUX_STS_STATUS_MASK	GENMASK(3, 0)
#define GB02MAC2207		0x0
#define GB02MAC2208	0x1
#define GB02MAC2210	0x2
#define GB02MAC2211	0x4
#define GB02MAC2212	0x8
#define GB02MAC2213		8
#define DPTX_AUX_STS_AUXM_MASK		GENMASK(15, 8)
#define GB02MAC2214	BIT(16)
#define GB02MAC2215		BIT(17)
#define GB02MAC2216		BIT(18)
#define GB02MAC2217	19
#define DPTX_AUX_STS_BYTES_READ_MASK	GENMASK(23, 19)
#define GB02MAC2218		BIT(24)

#define GB02MAC2219			BIT(0)
#define GB02MAC2220		BIT(1)
#define GB02MAC2221			BIT(2)
#define GB02MAC2222	BIT(3)
#define GB02MAC2223			BIT(4)
#define GB02MAC2224	BIT(5)
#define GB02MAC2225	BIT(6)
#define GB02MAC2226		BIT(7)
#define GB02MAC2227	(GB02MAC2219 |			\
				 GB02MAC2220 |			\
				 GB02MAC2221 |			\
				 GB02MAC2222 |		\
				 GB02MAC2223 |			\
				 GB02MAC2224 |	\
				 GB02MAC2225 |	\
				 GB02MAC2226)
#define GB02MAC2229			BIT(0)
#define GB02MAC2231		BIT(1)
#define GB02MAC2233			BIT(2)
#define GB02MAC2235	BIT(3)
#define GB02MAC2236			BIT(4)
#define GB02MAC2237	BIT(5)
#define GB02MAC2238	BIT(6)
#define GB02MAC2239			BIT(7)
#define GB02MAC2240	(GB02MAC2229 |			\
				 GB02MAC2231 |		\
				 GB02MAC2233 |		\
				 GB02MAC2235 |	\
				 GB02MAC2236 |			\
				 /* GB02MAC2237 |*/ \
				 /* GB02MAC2238 |*/ \
				 GB02MAC2239)

#define GB02MAC2241			BIT(0)
#define GB02MAC2242		BIT(1)
#define GB02MAC2243		BIT(2)
#define GB02MAC2244	BIT(3)
#define GB02MAC2245		BIT(8)
#define GB02MAC2246		4
#define DPTX_HPDSTS_STATE_MASK		GENMASK(6, 4)
#define GB02MAC2247		16
#define DPTX_HPDSTS_TIMER_MASK		GENMASK(31, 16)

#define GB02MAC2248		GB02MAC2241
#define GB02MAC2249	GB02MAC2242
#define GB02MAC2250	GB02MAC2243
#define GB02MAC2251	GB02MAC2244

#define GB02MAC2252	BIT(5)

#define GB02MAC2253		6
#define DPTX_AG_CONFIG1_WORD_WIDTH_MASK			GENMASK(9, 6)
#define GB02MAC2254			14
#define GB02MAC2255			BIT(14)
#define GB02MAC2256			24
#define DPTX_AG_CONFIG3_CH_NUMCL0_MASK			GENMASK(27, 24)
#define GB02MAC2257			28
#define DPTX_AG_CONFIG3_CH_NUMCR0_MASK			GENMASK(31, 28)
#define GB02MAC2258			0
#define DPTX_AG_CONFIG4_SAMP_FREQ_MASK			GENMASK(3, 0)
#define GB02MAC2259		8
#define DPTX_AG_CONFIG4_WORD_LENGTH_MASK		GENMASK(11, 8)
#define GB02MAC2260		12
#define DPTX_AG_CONFIG4_ORIG_SAMP_FREQ_MASK		GENMASK(15, 12)
#define GB02MAC2261			0
#define GB02MAC2262			BIT(0)
#define GB02MAC2263			12
#define DPTX_AUD_CONFIG1_NCH_MASK			GENMASK(14, 12)
#define GB02MAC2264		5
#define DPTX_AUD_CONFIG1_DATA_WIDTH_MASK		GENMASK(9, 5)
#define GB02MAC2265		1
#define DPTX_AUD_CONFIG1_DATA_EN_IN_MASK		GENMASK(4, 1)
#define GB02MAC2266			24
#define DPTX_AUD_CONFIG1_ATS_VER_MASK			GENMASK(29, 24)

#define GB02MAC2267                16
#define DPTX_VSAMPLE_CTRL_VMAP_BPC_MASK                 GENMASK(20, 16)
#define GB02MAC2268		21
#define DPTX_VSAMPLE_CTRL_MULTI_PIXEL_MASK		GENMASK(22, 21)
#define GB02MAC2269			BIT(23)
#define GB02MAC2270			GB02MAC2045
#define GB02MAC2271			GB02MAC2044
#define GB02MAC2272			GB02MAC2043
#define GB02MAC2273                      29
#define DPTX_VIDEO_VMSA2_BPC_MASK                       GENMASK(31, 29)
#define GB02MAC2276                      25
#define DPTX_VIDEO_VMSA2_COL_MASK                       GENMASK(28, 25)
#define GB02MAC2278                  31
#define GB02MAC2280           30 // ignore MSA
#define DPTX_VIDEO_VMSA3_PIX_ENC_MASK                   GENMASK(31, 30)
#define GB02MAC2282			BIT(0)
#define GB02MAC2284			BIT(1)
#define GB02MAC2286			BIT(2)
#define GB02MAC2288			BIT(0)
#define GB02MAC2290			BIT(1)
#define GB02MAC2292			2
#define GB02MAC2294			16
#define GB02MAC2296			16
#define GB02MAC2298			0
#define GB02MAC2300			0
#define GB02MAC2302				16
#define GB02MAC2304			0
#define GB02MAC2306				16
#define GB02MAC2308			0
#define GB02MAC2310			16
#define GB02MAC2311			0
#define DPTX_VIDEO_CONFIG5_TU_MASK			GENMASK(6, 0)
#define GB02MAC2314            14
#define DPTX_VIDEO_CONFIG5_TU_FRAC_MASK_MST             GENMASK(19, 14)
#define GB02MAC2316            16
#define DPTX_VIDEO_CONFIG5_TU_FRAC_MASK_SST             GENMASK(19, 16)
#define GB02MAC2318		7
#define DPTX_VIDEO_CONFIG5_INIT_THRESHOLD_MASK		GENMASK(13, 7)

#define GB02MAC2321			12
#define DPTX_VG_CONFIG1_BPC_MASK			GENMASK(14, 12)
#define GB02MAC2323			17
#define DPTX_VG_CONFIG1_PATTERN_MASK			GENMASK(18, 17)
#define GB02MAC2325		19
#define DPTX_VG_CONFIG1_MULTI_PIXEL_MASK		GENMASK(20, 19)
#define GB02MAC2328			GB02MAC2045
#define GB02MAC2330			GB02MAC2044
#define GB02MAC2332			GB02MAC2043
#define GB02MAC2333			BIT(0)
#define GB02MAC2336			BIT(1)
#define GB02MAC2337			BIT(2)
#define GB02MAC2339				BIT(3)
#define GB02MAC2341			BIT(5)
#define GB02MAC2343			BIT(6)
#define GB02MAC2345		BIT(7)
#define GB02MAC2347			BIT(15)
#define GB02MAC2349			12
#define GB02MAC2351			0
#define GB02MAC2353			16

#define GB02MAC2355			BIT(0)
#define GB02MAC2357			BIT(1)
#define GB02MAC2359			BIT(2)
#define GB02MAC2361				1
#define GB02MAC2363				1
#define GB02MAC2365                              3
#define GB02MAC2367				9
#define GB02MAC2369				7
#define GB02MAC2371				7
#define GB02MAC2373				0xF
#define GB02MAC2375				0xF
#define GB02MAC2377					BIT(15)

#define GB02MAC2379				0x3614
#define GB02MAC2381                                   0xE00
#define GB02MAC2383                            0x3618
#define GB02MAC2385				0x3630
#define GB02MAC2387				0x3620
#define GB02MAC2389				0x3624
#define GB02MAC2391				0x361C
#define GB02MAC2393				0xE0C
#define GB02MAC2395				0xE08
#define GB02MAC2397					0xE04
#define GB02MAC2399                              0x3628
#define GB02MAC2401                       0x362c

#define GB02MAC2403			7
#define GB02MAC2405			BIT(7)
#define GB02MAC2407			BIT(0)
#define GB02MAC2409			BIT(3)
#define GB02MAC2411				BIT(5)
#define GB02MAC2412				BIT(6)
#define GB02MAC2413				BIT(7)
#define GB02MAC2414				BIT(8)
#define DPTX_REVOC_LIST_MASK				GENMASK(31, 24)
#define GB02MAC2415				23
#define DPTX_REVOC_SIZE_MASK				GENMASK(23, 8)
#define GB02MAC2416			0
#define GB02MAC2417			BIT(0)
#define GB02MAC2418			0
#define DPTX_IDPK_DATA_INDEX_MASK			GENMASK(5, 0)
#define GB02MAC2419			6
#define GB02MAC2420			BIT(6)
#define GB02MAC2423				BIT(2)
#define GB02MAC2425					BIT(6)
#define GB02MAC2427				BIT(1)

#define GB02MAC2429					0x0230
#define DPTX_DSC_NUM_ENC_MSK			GENMASK(3,0)
#define GB02MAC2431					0x0234
#define GB02MAC2433		BIT(18)
#define GB02MAC2435		18

#define DPTX_DSC_ENC_STREAM_SEL(i)				GENMASK((2 * i + 1),(2 * i))
#define GB02MAC2438(i)		((i)*2)
#define GB02MAC2439		22

// Stream should be [1 .. 4],  SST = 1
#define DPTX_DSC_PPS(stream, i)			((0x3a00 + (stream -1) * 0x10000) + (i * 0x4))
#define GB02MAC2442					1

// PPS SDPs
#define GB02MAC2444				0
#define GB02MAC2446				4
#define GB02MAC2448			24
#define GB02MAC2450			5
#define DPTX_DSC_BPP_HIGH_MASK				GENMASK(9, 8)
#define DPTX_DSC_BPP_LOW_MASK				GENMASK(7, 0)
#define DPTX_DSC_BPP_LOW_MASK1				GENMASK(15, 8)
#define GB02MAC2452					28

#define GB02MAC2455					0x104
#define GB02MAC2457				16
#define DSC_MAX_NUM_LINES_MASK				GENMASK(31, 16)

#define GB02MAC2459					0x334
#define GB02MAC2461			0
#define GB02MAC2463			5
#define GB02MAC2465	16
#define DPTX_DSC_LSTEER_XMIT_DELAY_MASK		GENMASK(31, 16)
#define DPTX_DSC_LSTEER_FRAC_SHIFT_MASK		GENMASK(9, 5)
#define DPTX_DSC_LSTEER_INT_SHIFT_MASK		GENMASK(4, 0)

#define GB02MAC2468 8 // For Interop testing only RGB 8bpp 8bpc
#define GB02MAC2470 8 // For Interop testing only RGB 8bpp 8bpc

#endif
