/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright (ST) 2012 Rajeev Kumar (rajeevkumar.linux@gmail.com)
 */

#ifndef __SOUND_DESIGNWARE_I2S_H
#define __SOUND_DESIGNWARE_I2S_H

#include <linux/dmaengine.h>
#include <linux/types.h>

/*
 * struct i2s_clk_config_data - represent i2s clk configuration data
 * @chan_nr: number of channel
 * @data_width: number of bits per sample (8/16/24/32 bit)
 * @sample_rate: sampling frequency (8Khz, 16Khz, 32Khz, 44Khz, 48Khz)
 */
struct i2s_clk_config_data {
	int chan_nr;
	u32 data_width;
	u32 sample_rate;
};

/* I2S DMA registers */
#define I2S_RXDMA		0x01C0
#define I2S_TXDMA		0x01C8

#define TWO_CHANNEL_SUPPORT	2	/* up to 2.0 */
#define FOUR_CHANNEL_SUPPORT	4	/* up to 3.1 */
#define SIX_CHANNEL_SUPPORT	6	/* up to 5.1 */
#define EIGHT_CHANNEL_SUPPORT	8	/* up to 7.1 */

#define	ASAMPLE_RATE_32		0
#define	ASAMPLE_RATE_44_1	1
#define	ASAMPLE_RATE_48		2
#define	ASAMPLE_RATE_88_2	3
#define	ASAMPLE_RATE_96		4
#define	ASAMPLE_RATE_176_4	5
#define	ASAMPLE_RATE_192	6

#define PCM_WIDTH_BIT_16	16
#define PCM_WIDTH_BIT_20	20
#define PCM_WIDTH_BIT_24	24
#define PCM_WIDTH_BIT_32	32

#endif /*  __SOUND_DESIGNWARE_I2S_H */
