/* Copyright (C) Xi'an Sietium Electronics Co.Ltd */
/*
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/debugfs.h>
#include <linux/uaccess.h>
#include <linux/io.h>
#include <asm/irq.h>
#include <linux/uaccess.h>
#include <linux/delay.h>
#include <linux/pci.h>
#include <linux/miscdevice.h>
#include <linux/interrupt.h>
#include <linux/sysfs.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/string.h>
#include <linux/gfp.h>
#include <linux/mm_types.h>
#include <linux/mm.h>
#include <linux/highmem.h>
#include <linux/slab.h>
#include "local.h"

#include "common/xt.h"
#include "common/gb_common.h"
#include "audio/gb02_fpga_v2vdma.h"

void axi_dma_disable(struct audio_dma_ctrl_info *chip)
{
	u32 val;

	val = dma_reg_read32(chip, DMAC_CFG);
	val &= ~DMAC_EN_MASK;
	dma_reg_write32(chip, DMAC_CFG, val);
}

void axi_dma_enable(struct audio_dma_ctrl_info *chip)
{
	u32 val;

	val = dma_reg_read32(chip, DMAC_CFG);
	val |= DMAC_EN_MASK;
	dma_reg_write32(chip, DMAC_CFG, val);
}

void axi_dma_irq_disable(struct audio_dma_ctrl_info *chip)
{
	u32 val;

	val = dma_reg_read32(chip, DMAC_CFG);
	val &= ~INT_EN_MASK;
	dma_reg_write32(chip, DMAC_CFG, val);
}

void axi_dma_irq_enable(struct audio_dma_ctrl_info *chip)
{
	u32 val;

	val = dma_reg_read32(chip, DMAC_CFG);
	val |= INT_EN_MASK;
	dma_reg_write32(chip, DMAC_CFG, val);
}

void axi_chan_irq_disable(struct axi_dma_chan *chan, u32 irq_mask)
{
	u32 val;

	if (likely(irq_mask == DWAXIDMAC_IRQ_ALL))
		chan_reg_write32(chan, CH_INTSTATUS_ENA, DWAXIDMAC_IRQ_NONE);
	else {
		val = chan_reg_read32(chan, CH_INTSTATUS_ENA);
		val &= ~irq_mask;
		chan_reg_write32(chan, CH_INTSTATUS_ENA, val);
	}
}

void axi_chan_irq_set(struct axi_dma_chan *chan, u32 irq_mask)
{
	chan_reg_write32(chan, CH_INTSTATUS_ENA, irq_mask);
}

void axi_chan_irq_sig_set(struct axi_dma_chan *chan, u32 irq_mask)
{
	chan_reg_write32(chan, CH_INTSIGNAL_ENA, irq_mask);
}

void axi_chan_irq_clear(struct axi_dma_chan *chan, u32 irq_mask)
{
	chan_reg_write32(chan, CH_INTCLEAR, irq_mask);
}

u32 axi_chan_irq_read(struct axi_dma_chan *chan)
{
	return chan_reg_read32(chan, CH_INTSTATUS);
}

void axi_chan_disable(struct axi_dma_chan *chan)
{
	u32 val;

	val = dma_reg_read32(chan->chip, DMAC_CHEN);
	val &= ~(BIT(chan->id) << DMAC_CHAN_EN_SHIFT);
	val |=   BIT(chan->id) << DMAC_CHAN_EN_WE_SHIFT;
	dma_reg_write32(chan->chip, DMAC_CHEN, val);
}

static inline void axi_chan_enable(struct axi_dma_chan *chan)
{
	u32 val;

	val = dma_reg_read32(chan->chip, DMAC_CHEN);
	val |= BIT(chan->id) << DMAC_CHAN_EN_SHIFT |
		BIT(chan->id) << DMAC_CHAN_EN_WE_SHIFT;
	dma_reg_write32(chan->chip, DMAC_CHEN, val);
}

void axi_dma_hw_init(struct audio_dma_ctrl_info *chip)
{
	u32 i;

	for (i = 0; i < DW_AXI_DMA_CHANNELS; i++) {
		axi_chan_irq_disable(&chip->chan[i], DWAXIDMAC_IRQ_ALL);
		axi_chan_disable(&chip->chan[i]);
	}
}

static inline bool axi_chan_is_hw_enable(struct axi_dma_chan *chan)
{
	u32 val;

	val = dma_reg_read32(chan->chip, DMAC_CHEN);

	return !!(val & (BIT(chan->id) << DMAC_CHAN_EN_SHIFT));
}

int restart_dma_axi_ch(struct audio_dma_ctrl_info *dma_ctrl,
	u64 addr, int chid)
{
	struct axi_dma_chan *chan = &(dma_ctrl->chan[chid]);

#ifdef	AUDIO_USE_NOC
	/* CHX_SAR */
//	gb_printf(KERN_INFO, "--------dma src addr =%llx--\n", addr);
	chan_reg_write64(chan, CH_SAR, cpu_to_le64(addr));
#else
	gb_printf(KERN_INFO, "---restart dma---dma src addr =%llx--\n", addr);
	chan_reg_write64(chan, CH_SAR, cpu_to_le64(addr));

#endif
	axi_chan_enable(chan);
	return 0;
}

void set_dma_data_len(struct audio_dma_ctrl_info *dma_ctrl,
	int abytes, int chid)
{
	struct axi_dma_chan *chan = &(dma_ctrl->chan[chid]);
	u32 val;

	/* CHX_BLOCK_TS */
	val = (abytes >> DWAXIDMAC_TRANS_WIDTH_32) - 1;
	chan_reg_write32(chan, CH_BLOCK_TS, cpu_to_le32(val));
}

void audio_dma_enable_handle(struct audio_dma_ctrl_info *dma_ctrl)
{
	/* DMAC_CFGREG */
	axi_dma_enable(dma_ctrl);
	axi_dma_irq_enable(dma_ctrl);

	/* DMAC_COMMON_INTSTATUS_ENA */
	dma_reg_write32(dma_ctrl, DMAC_COMMON_INTSTATUS_ENA, GENMASK(20, 0));
	/* DMAC_COMMON_INTSIGNAL_ENA */
	dma_reg_write32(dma_ctrl, DMAC_COMMON_INTSIGNAL_ENA, GENMASK(20, 0));
}

static void gb02_dma_single_blk_mode_start(struct audio_dma_ctrl_info *dma_ctrl,
	int abytes, int chid)
{
	struct axi_dma_chan *chan = &(dma_ctrl->chan[chid]);
	u32 val = 0, irq_mask = 0;

#ifdef	AUDIO_USE_NOC
	/* CHX_SAR */
	gb_printf(KERN_INFO, "dma src addr = %llx\n",
		dma_ctrl->dma_src_paddr[chid]);
	chan_reg_write64(chan, CH_SAR,
		cpu_to_le64(dma_ctrl->dma_src_paddr[chid]));
#else
	gb_printf(KERN_INFO, "dma src addr = %llx\n",
		dma_ctrl->dma_src_paddr[chid]);
	chan_reg_write64(chan, CH_SAR,
		cpu_to_le64(dma_ctrl->dma_src_paddr[chid]));

#endif
	/* CHX_DAR */
#ifdef	AUDIO_USE_NOC
	gb_printf(KERN_INFO, "--dma dts addr =%llx--\n",
		dma_ctrl->dma_dst_paddr + chid * AUDIO_STEP);
	val = dma_ctrl->dma_dst_paddr;
	chan_reg_write64(chan, CH_DAR,
		cpu_to_le64(dma_ctrl->dma_dst_paddr + chid * AUDIO_STEP));
#else
	gb_printf(KERN_INFO, "--------dma dts addr =%llx--\n",
		dma_ctrl->dma_dst_paddr + chid * AUDIO_STEP);
	val = dma_ctrl->dma_dst_paddr;
	chan_reg_write64(chan, CH_DAR,
		cpu_to_le64(dma_ctrl->dma_dst_paddr + chid * AUDIO_STEP));
#endif
	/* CHX_BLOCK_TS */
	val = (abytes >> DWAXIDMAC_TRANS_WIDTH_32) - 1;
	gb_printf(KERN_INFO, "%s-%d: i2s start  abytes = 0x%x, val = 0x%x\n",
		__func__, __LINE__,  abytes, val);
	chan_reg_write32(chan, CH_BLOCK_TS, cpu_to_le32(val));
	/* CHX_CTL */
	val = (DWAXIDMAC_BURST_TRANS_LEN_8 << CH_CTL_L_DST_MSIZE_POS |
		DWAXIDMAC_BURST_TRANS_LEN_8 << CH_CTL_L_SRC_MSIZE_POS |
		DW_AXI_DMA_DATA_WIDTH << CH_CTL_L_DST_WIDTH_POS |
		DW_AXI_DMA_DATA_WIDTH << CH_CTL_L_SRC_WIDTH_POS |
		DWAXIDMAC_CH_CTL_L_NOINC << CH_CTL_L_DST_INC_POS |
		DWAXIDMAC_CH_CTL_L_INC << CH_CTL_L_SRC_INC_POS |
		CH_CTL_L_DST_MAST);
	val &= ~(CH_CTL_L_SRC_MAST);
	chan_reg_write32(chan, CH_CTL_L, val);
	chan_reg_write32(chan, CH_CTL_H, 0x48240);

	/* CHX_CFG */
	val = (DWAXIDMAC_MBLK_TYPE_CONTIGUOUS
		<< CH_CFG_L_DST_MULTBLK_TYPE_POS |
		DWAXIDMAC_MBLK_TYPE_CONTIGUOUS
		<< CH_CFG_L_SRC_MULTBLK_TYPE_POS);
	chan_reg_write32(chan, CH_CFG_L, val);
	val = (DWAXIDMAC_TT_FC_MEM_TO_PER_DST << CH_CFG_H_TT_FC_POS |
		DW_AXI_DMA_TEST_CHAN_PRIORITY << CH_CFG_H_PRIORITY_POS |
		DWAXIDMAC_HS_SEL_HW << CH_CFG_H_HS_SEL_DST_POS |
		DWAXIDMAC_HS_SEL_HW << CH_CFG_H_HS_SEL_SRC_POS);
	switch (chid) {
	case 0:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a0009);
		break;
	case 1:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a1089);
		break;
	case 2:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a2109);
		break;
	case 3:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a3189);
		break;
	case 4:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a4209);
		break;
	case 5:
		chan_reg_write32(chan, CH_CFG_H, 0x7f8a5289);
		break;
	default:
		break;
	}
	chan_reg_write32(chan, CH_SWHSSRC, 0x00000003);

	/* CH_INTSTATUS_ENA */
	irq_mask = DWAXIDMAC_IRQ_DMA_TRF | DWAXIDMAC_IRQ_ALL_ERR;
	axi_chan_irq_sig_set(chan, irq_mask);

	/* Generate 'suspend' status but don't generate interrupt */
	irq_mask |= DWAXIDMAC_IRQ_SUSPENDED;
	axi_chan_irq_set(chan, irq_mask);

	/* DMAC_CHENREG */
	axi_chan_enable(chan);
}

int audio_dma_single_blk_mode(struct audio_dma_ctrl_info *dma_ctrl,
	int data_bytes, int chid)
{
	int ret = 0;

	/* second, prepare for DMA transferring */
	gb02_dma_single_blk_mode_start(dma_ctrl, data_bytes, chid);
	return ret;
}

void axi_chan_handle_err(struct axi_dma_chan *chan, u32 status)
{
	 axi_chan_disable(chan);
}

void axi_chan_block_xfer_complete(struct axi_dma_chan *chan)
{
	if (unlikely(axi_chan_is_hw_enable(chan))) {
		gb_printf(KERN_ERR, "BUG: channel not idle!\n");
		axi_chan_disable(chan);
	}
}
