/* Copyright (C) Xi'an Sietium Electronics Co.Ltd */
/*
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef __GB02_FPGA_V2VDMA_H
#define __GB02_FPGA_V2VDMA_H

#define	AUDIO_USE_NOC
#define	GB02_PLD
#ifndef	AUDIO_USE_NOC
#define	DDR_STEP	(2 * 1024)
#define	AUDIO_STEP	0x50000//0x500000
#define	APB_I2S_OFFSET	0x10000//0x100000
#define	APB_I2S_BASE	0x100000//0x100000
#define		FPGA_DMA_OFFSET  0xc0000000
#define		FPGA_AUDIO_OFFSET	0x1000000

#else

#define	AUDIO_STEP	0x800//0x500000
#define	AINS_OFFSET	CS_S1_AUDIO_BASE//0x80400000
#define	ADINS_OFFSET	CS_S4_DMA_BASE//0x81000000
#define	DDRINS_OFFSET	0x800000000
#define	AINS_BASE	0x80000000
#endif

#define DW_AXI_DMA_CHANNELS	(6)
#define DW_AXI_DMA_NR_MASTERS (1)
#define DW_AXI_DMA_DATA_WIDTH (2) /* DWAXIDMAC_TRANS_WIDTH_256 */
#define DW_AXI_DMA_MAX_BURST_LENGTH (255) /* DWAXIDMAC_ARWLEN_256 */
#define DW_AXI_DMA_SINGLE_BLCOK_TEST_CHAN_NO (0) /* channel 0 */
#define DW_AXI_DMA_TEST_CHAN_PRIORITY (1)

#define KERNEL_PAGE_SIZE				(0x1000) /* 4KB */
#define DMA_SINGLE_BLK_SIZE_TEST			(0x400000) /* 4MB */
#define DMA_SINGLE_BLK_TS_SIZE				(0x4000) /* 16KB */
#define DMA_SINGLE_BLK_SRC_OFFSET			(0x100000)
#define DMA_SINGLE_BLK_DST_OFFSET			(0x500000)

#define DMA_MULTI_BLK_LINKLIST_ELEMENT_SIZE (0x40)

#define V2VDMA_BASE_ADDR (0xc0000000) /* view at V2VDMA side */

/* LLI == Linked List Item */
struct __packed axi_dma_lli {
	__le64	sar;
	__le64	dar;
	__le32	block_ts_lo;
	__le32	block_ts_hi;
	__le64	llp;
	__le32	ctl_lo;
	__le32	ctl_hi;
	__le32	sstat;
	__le32	dstat;
	__le32	status_lo;
	__le32	ststus_hi;
	__le32	reserved_lo;
	__le32	reserved_hi;
};

struct axi_dma_desc {
	struct axi_dma_lli	lli;
	struct axi_dma_chan	*chan;
	struct list_head	xfer_list;
	u64	phys;
};

struct axi_dma_chan {
	void __iomem *chan_regs;
	u8 id;
	struct audio_dma_ctrl_info *chip;
	atomic_t descs_allocated;
	struct axi_dma_desc *first_desc;
};

struct audio_dma_ctrl_info {
	void __iomem *regs;
	struct axi_dma_chan chan[DW_AXI_DMA_CHANNELS];
	int chan_count;
	unsigned __iomem *dma_src_addr;
	unsigned __iomem *dma_dst_addr;

	u64	dma_src_paddr[DW_AXI_DMA_CHANNELS];
	u64	dma_dst_paddr;
	int	tsfd_num;
	dma_addr_t dma_src_handle;
	dma_addr_t dma_dst_handle;

	u64 pos[DW_AXI_DMA_CHANNELS];
};

static inline u32 dma_reg_read32(struct audio_dma_ctrl_info *chip, u32 offset)
{
	u32 val = 0;

	val = ioread32(chip->regs + offset);
	return val;
}

static inline void dma_reg_write32(struct audio_dma_ctrl_info *chip,
	u32 offset, u32 val)
{
	iowrite32(val, chip->regs + offset);
}

static inline u32 chan_reg_read32(struct axi_dma_chan *chan, u32 offset)
{
	u32 val = 0;

	val = ioread32(chan->chan_regs + offset);
	return val;
}

static inline void chan_reg_write32(struct axi_dma_chan *chan,
	u32 offset, u32 val)
{
	iowrite32(val, chan->chan_regs + offset);
}

static inline void chan_reg_write64(struct axi_dma_chan *chan,
	u32 offset, u64 val)
{
	iowrite32(lower_32_bits(val), chan->chan_regs + offset);
	iowrite32(upper_32_bits(val), chan->chan_regs + offset + 4);
}

void axi_dma_disable(struct audio_dma_ctrl_info *chip);
void axi_dma_irq_disable(struct audio_dma_ctrl_info *chip);
u32 axi_chan_irq_read(struct axi_dma_chan *chan);
void axi_chan_irq_clear(struct axi_dma_chan *chan, u32 irq_mask);
void axi_chan_handle_err(struct axi_dma_chan *chan, u32 status);
void axi_chan_block_xfer_complete(struct axi_dma_chan *chan);
void axi_dma_irq_enable(struct audio_dma_ctrl_info *chip);

#define COMMON_REG_LEN		0x100
#define CHAN_REG_LEN		0x100

/* Common registers offset */
#define DMAC_ID			0x000 /* R DMAC ID */
#define DMAC_COMPVER		0x008 /* R DMAC Component Version */
#define DMAC_CFG		0x010 /* R/W DMAC Configuration */
#define DMAC_CHEN		0x018 /* R/W DMAC Channel Enable */
#define DMAC_CHEN_L		0x018 /* R/W DMAC Channel Enable 00-31 */
#define DMAC_CHEN_H		0x01C /* R/W DMAC Channel Enable 32-63 */
#define DMAC_INTSTATUS		0x030 /* R DMAC Interrupt Status */
#define DMAC_COMMON_INTCLEAR	0x038 /* W DMAC Interrupt Clear */
#define DMAC_COMMON_INTSTATUS_ENA 0x040 /* R DMAC Interrupt Status Enable */
#define DMAC_COMMON_INTSIGNAL_ENA 0x048 /* R/W DMAC Interrupt Signal Enable */
#define DMAC_COMMON_INTSTATUS	0x050 /* R DMAC Interrupt Status */
#define DMAC_RESET		0x058 /* R DMAC Reset Register1 */

/* DMA channel registers offset */
#define CH_SAR			0x000 /* R/W Chan Source Address */
#define CH_DAR			0x008 /* R/W Chan Destination Address */
#define CH_BLOCK_TS		0x010 /* R/W Chan Block Transfer Size */
#define CH_CTL			0x018 /* R/W Chan Control */
#define CH_CTL_L		0x018 /* R/W Chan Control 00-31 */
#define CH_CTL_H		0x01C /* R/W Chan Control 32-63 */
#define CH_CFG			0x020 /* R/W Chan Configuration */
#define CH_CFG_L		0x020 /* R/W Chan Configuration 00-31 */
#define CH_CFG_H		0x024 /* R/W Chan Configuration 32-63 */
#define CH_LLP			0x028 /* R/W Chan Linked List Pointer */
#define CH_STATUS		0x030 /* R Chan Status */
#define CH_SWHSSRC		0x038 /* R/W Chan SW Handshake Source */
#define CH_SWHSDST		0x040 /* R/W Chan SW Handshake Destination */
#define CH_BLK_TFR_RESUMEREQ	0x048 /* W Chan Block Transfer Resume Req */
#define CH_AXI_ID		0x050 /* R/W Chan AXI ID */
#define CH_AXI_QOS		0x058 /* R/W Chan AXI QOS */
#define CH_SSTAT		0x060 /* R Chan Source Status */
#define CH_DSTAT		0x068 /* R Chan Destination Status */
#define CH_SSTATAR		0x070 /* R/W Chan Source Status Fetch Addr */
#define CH_DSTATAR	0x078 /* R/W Chan Destination Status Fetch Addr */
#define CH_INTSTATUS_ENA	0x080 /* R/W Chan Interrupt Status Enable */
#define CH_INTSTATUS		0x088 /* R/W Chan Interrupt Status */
#define CH_INTSIGNAL_ENA	0x090 /* R/W Chan Interrupt Signal Enable */
#define CH_INTCLEAR		0x098 /* W Chan Interrupt Clear */


/* DMAC_CFG */
#define DMAC_EN_POS			0
#define DMAC_EN_MASK			BIT(DMAC_EN_POS)

#define INT_EN_POS			1
#define INT_EN_MASK			BIT(INT_EN_POS)

#define DMAC_CHAN_EN_SHIFT		0
#define DMAC_CHAN_EN_WE_SHIFT		8

#define DMAC_CHAN_SUSP_SHIFT		16
#define DMAC_CHAN_SUSP_WE_SHIFT		24

/* CH_CTL_H */
#define CH_CTL_H_ARLEN_EN		BIT(6)
#define CH_CTL_H_ARLEN_POS		7
#define CH_CTL_H_AWLEN_EN		BIT(15)
#define CH_CTL_H_AWLEN_POS		16

enum {
	DWAXIDMAC_ARWLEN_1		= 0,
	DWAXIDMAC_ARWLEN_2		= 1,
	DWAXIDMAC_ARWLEN_4		= 3,
	DWAXIDMAC_ARWLEN_8		= 7,
	DWAXIDMAC_ARWLEN_16		= 15,
	DWAXIDMAC_ARWLEN_32		= 31,
	DWAXIDMAC_ARWLEN_64		= 63,
	DWAXIDMAC_ARWLEN_128		= 127,
	DWAXIDMAC_ARWLEN_256		= 255,
	DWAXIDMAC_ARWLEN_MIN		= DWAXIDMAC_ARWLEN_1,
	DWAXIDMAC_ARWLEN_MAX		= DWAXIDMAC_ARWLEN_256
};

#define CH_CTL_H_LLI_LAST		BIT(30)
#define CH_CTL_H_LLI_VALID		BIT(31)

/* CH_CTL_L */
#define CH_CTL_L_LAST_WRITE_EN		BIT(30)

#define CH_CTL_L_DST_MSIZE_POS		18
#define CH_CTL_L_SRC_MSIZE_POS		14
#define	CH_CTL_SHADOW_OR_LL_POS		63

enum {
	DWAXIDMAC_BURST_TRANS_LEN_1	= 0,
	DWAXIDMAC_BURST_TRANS_LEN_4,
	DWAXIDMAC_BURST_TRANS_LEN_8,
	DWAXIDMAC_BURST_TRANS_LEN_16,
	DWAXIDMAC_BURST_TRANS_LEN_32,
	DWAXIDMAC_BURST_TRANS_LEN_64,
	DWAXIDMAC_BURST_TRANS_LEN_128,
	DWAXIDMAC_BURST_TRANS_LEN_256,
	DWAXIDMAC_BURST_TRANS_LEN_512,
	DWAXIDMAC_BURST_TRANS_LEN_1024
};

enum {
	DWAXIDMAC_CH_CTL_SHADOW_LL_INVALID	= 0,
	DWAXIDMAC_CH_CTL_SHADOW_LL_VALID
};

#define CH_CTL_L_DST_WIDTH_POS		11
#define CH_CTL_L_SRC_WIDTH_POS		8

#define CH_CTL_L_DST_INC_POS		6
#define CH_CTL_L_SRC_INC_POS		4
enum {
	DWAXIDMAC_CH_CTL_L_INC		= 0,
	DWAXIDMAC_CH_CTL_L_NOINC
};

#define CH_CTL_L_DST_MAST		BIT(2)
#define CH_CTL_L_SRC_MAST		BIT(0)

/* CH_CFG_H */
#define CH_CFG_H_PRIORITY_POS		17
#define CH_CFG_H_HS_SEL_DST_POS		4
#define CH_CFG_H_HS_SEL_SRC_POS		3
enum {
	DWAXIDMAC_HS_SEL_HW		= 0,
	DWAXIDMAC_HS_SEL_SW
};

#define CH_CFG_H_TT_FC_POS		0
enum {
	DWAXIDMAC_TT_FC_MEM_TO_MEM_DMAC	= 0,
	DWAXIDMAC_TT_FC_MEM_TO_PER_DMAC,
	DWAXIDMAC_TT_FC_PER_TO_MEM_DMAC,
	DWAXIDMAC_TT_FC_PER_TO_PER_DMAC,
	DWAXIDMAC_TT_FC_PER_TO_MEM_SRC,
	DWAXIDMAC_TT_FC_PER_TO_PER_SRC,
	DWAXIDMAC_TT_FC_MEM_TO_PER_DST,
	DWAXIDMAC_TT_FC_PER_TO_PER_DST
};

/* CH_CFG_L */
#define CH_CFG_L_DST_MULTBLK_TYPE_POS	2
#define CH_CFG_L_SRC_MULTBLK_TYPE_POS	0
enum {
	DWAXIDMAC_MBLK_TYPE_CONTIGUOUS	= 0,
	DWAXIDMAC_MBLK_TYPE_RELOAD,
	DWAXIDMAC_MBLK_TYPE_SHADOW_REG,
	DWAXIDMAC_MBLK_TYPE_LL
};

/**
 * DW AXI DMA channel interrupts
 *
 * @DWAXIDMAC_IRQ_NONE: Bitmask of no one interrupt
 * @DWAXIDMAC_IRQ_BLOCK_TRF: Block transfer complete
 * @DWAXIDMAC_IRQ_DMA_TRF: Dma transfer complete
 * @DWAXIDMAC_IRQ_SRC_TRAN: Source transaction complete
 * @DWAXIDMAC_IRQ_DST_TRAN: Destination transaction complete
 * @DWAXIDMAC_IRQ_SRC_DEC_ERR: Source decode error
 * @DWAXIDMAC_IRQ_DST_DEC_ERR: Destination decode error
 * @DWAXIDMAC_IRQ_SRC_SLV_ERR: Source slave error
 * @DWAXIDMAC_IRQ_DST_SLV_ERR: Destination slave error
 * @DWAXIDMAC_IRQ_LLI_RD_DEC_ERR: LLI read decode error
 * @DWAXIDMAC_IRQ_LLI_WR_DEC_ERR: LLI write decode error
 * @DWAXIDMAC_IRQ_LLI_RD_SLV_ERR: LLI read slave error
 * @DWAXIDMAC_IRQ_LLI_WR_SLV_ERR: LLI write slave error
 * @DWAXIDMAC_IRQ_INVALID_ERR: LLI invalid error or Shadow register error
 * @DWAXIDMAC_IRQ_MULTIBLKTYPE_ERR: Slave Interface Multiblock type error
 * @DWAXIDMAC_IRQ_DEC_ERR: Slave Interface decode error
 * @DWAXIDMAC_IRQ_WR2RO_ERR: Slave Interface write to read only error
 * @DWAXIDMAC_IRQ_RD2RWO_ERR: Slave Interface read to write only error
 * @DWAXIDMAC_IRQ_WRONCHEN_ERR: Slave Interface write to channel error
 * @DWAXIDMAC_IRQ_SHADOWREG_ERR: Slave Interface shadow reg error
 * @DWAXIDMAC_IRQ_WRONHOLD_ERR: Slave Interface hold error
 * @DWAXIDMAC_IRQ_LOCK_CLEARED: Lock Cleared Status
 * @DWAXIDMAC_IRQ_SRC_SUSPENDED: Source Suspended Status
 * @DWAXIDMAC_IRQ_SUSPENDED: Channel Suspended Status
 * @DWAXIDMAC_IRQ_DISABLED: Channel Disabled Status
 * @DWAXIDMAC_IRQ_ABORTED: Channel Aborted Status
 * @DWAXIDMAC_IRQ_ALL_ERR: Bitmask of all error interrupts
 * @DWAXIDMAC_IRQ_ALL: Bitmask of all interrupts
 */
enum {
	DWAXIDMAC_IRQ_NONE		= 0,
	DWAXIDMAC_IRQ_BLOCK_TRF		= BIT(0),
	DWAXIDMAC_IRQ_DMA_TRF		= BIT(1),
	DWAXIDMAC_IRQ_SRC_TRAN		= BIT(3),
	DWAXIDMAC_IRQ_DST_TRAN		= BIT(4),
	DWAXIDMAC_IRQ_SRC_DEC_ERR	= BIT(5),
	DWAXIDMAC_IRQ_DST_DEC_ERR	= BIT(6),
	DWAXIDMAC_IRQ_SRC_SLV_ERR	= BIT(7),
	DWAXIDMAC_IRQ_DST_SLV_ERR	= BIT(8),
	DWAXIDMAC_IRQ_LLI_RD_DEC_ERR	= BIT(9),
	DWAXIDMAC_IRQ_LLI_WR_DEC_ERR	= BIT(10),
	DWAXIDMAC_IRQ_LLI_RD_SLV_ERR	= BIT(11),
	DWAXIDMAC_IRQ_LLI_WR_SLV_ERR	= BIT(12),
	DWAXIDMAC_IRQ_INVALID_ERR	= BIT(13),
	DWAXIDMAC_IRQ_MULTIBLKTYPE_ERR	= BIT(14),
	DWAXIDMAC_IRQ_DEC_ERR		= BIT(16),
	DWAXIDMAC_IRQ_WR2RO_ERR		= BIT(17),
	DWAXIDMAC_IRQ_RD2RWO_ERR	= BIT(18),
	DWAXIDMAC_IRQ_WRONCHEN_ERR	= BIT(19),
	DWAXIDMAC_IRQ_SHADOWREG_ERR	= BIT(20),
	DWAXIDMAC_IRQ_WRONHOLD_ERR	= BIT(21),
	DWAXIDMAC_IRQ_LOCK_CLEARED	= BIT(27),
	DWAXIDMAC_IRQ_SRC_SUSPENDED	= BIT(28),
	DWAXIDMAC_IRQ_SUSPENDED		= BIT(29),
	DWAXIDMAC_IRQ_DISABLED		= BIT(30),
	DWAXIDMAC_IRQ_ABORTED		= BIT(31),
	DWAXIDMAC_IRQ_ALL_ERR		= (GENMASK(21, 16) | GENMASK(14, 5)),
	DWAXIDMAC_IRQ_ALL		= GENMASK(31, 0)
};

enum {
	DWAXIDMAC_TRANS_WIDTH_8		= 0,
	DWAXIDMAC_TRANS_WIDTH_16,
	DWAXIDMAC_TRANS_WIDTH_32,
	DWAXIDMAC_TRANS_WIDTH_64,
	DWAXIDMAC_TRANS_WIDTH_128,
	DWAXIDMAC_TRANS_WIDTH_256,
	DWAXIDMAC_TRANS_WIDTH_512,
	DWAXIDMAC_TRANS_WIDTH_MAX	= DWAXIDMAC_TRANS_WIDTH_512
};

struct axi_dma_desc *dma_chan_prep_ll(struct audio_dma_ctrl_info *dma_ctrl,
	size_t len);
void
gb02_dma_multi_blk_linklist_mode_start(struct audio_dma_ctrl_info *dma_ctrl);
void axi_dma_hw_init(struct audio_dma_ctrl_info *chip);
int audio_dma_single_blk_mode(struct audio_dma_ctrl_info *dma_ctrl,
	int data_bytes, int ch_id);
int restart_dma_axi_ch(struct audio_dma_ctrl_info *dma_ctrl,
	u64 addr, int chid);
void set_dma_data_len(struct audio_dma_ctrl_info *dma_ctrl,
	int abytes, int chid);
void axi_chan_irq_disable(struct axi_dma_chan *chan, u32 irq_mask);
void axi_chan_disable(struct axi_dma_chan *chan);
void audio_dma_enable_handle(struct audio_dma_ctrl_info *dma_ctrl);
#endif
