/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/version.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/io.h>

#include "gb_audio_procfs.h"
#include "gb02_fpga_v2vdma.h"
#include "i2s_platform.h"
#include "common/gb_common.h"
#include "common/gb_pcie_info.h"

struct proc_dir_entry *audio_proc_parent;

static int gb02_board_show(struct seq_file *m, void *v)
{
	struct gb_pcie_info *pcie_info = get_pcie_gl_info();
	enum gb_board_type gb_type;

	gb_type = gb_get_board_type(pcie_info);
	seq_printf(m, "%d\n", gb_type);
	return 0;
}

static int gb_board_open(struct inode *inode, struct file *file)
{
	struct gbdc_pcie_info *gbpcie_info;
#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	gbpcie_info = PDE_DATA(inode);
#else
	gbpcie_info = pde_data(inode);
#endif	
	gb_printf(KERN_INFO, "chen gb open proc--\n");

	return single_open(file, gb02_board_show, gbpcie_info);

}
static int gb_board_release(struct inode *inode, struct file *file)
{
	int res = single_release(inode, file);

	return res;

}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
static const struct file_operations audio_proc_ops = {
	.open = gb_board_open,
	.read = seq_read,
	.release = gb_board_release,
	.owner = THIS_MODULE,
};
#else
static const struct proc_ops audio_proc_ops = {
	.proc_open = gb_board_open,
	.proc_read = seq_read,
	.proc_release = gb_board_release,
};
#endif

static int gb_audio_edid_show(struct seq_file *m, void *v)
{
	int i, support_audio = 0;
	struct i2s_ctrl_info *i2s_infos = get_i2s_infos();

	for (i = 0; i < DW_AXI_DMA_CHANNELS; i++) {
		support_audio = get_is_support_audio(i2s_infos, i);
		seq_printf(m, "HDMI: %d, edid_audio_support: %d\n", i, support_audio);
	}
	return 0;
}

static int gb_audio_edid_open(struct inode *inode, struct file *file)
{
	gb_printf(KERN_INFO, "gb_audio_edid_open proc--\n");

	return single_open(file, gb_audio_edid_show, NULL);

}
static int gb_audio_edid_release(struct inode *inode, struct file *file)
{
	int res = single_release(inode, file);

	return res;

}

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
static const struct file_operations audio_edid_ops = {
	.open = gb_audio_edid_open,
	.read = seq_read,
	.release = gb_audio_edid_release,
	.owner = THIS_MODULE,
};
#else
static const struct proc_ops audio_edid_ops = {
	.proc_open = gb_audio_edid_open,
	.proc_read = seq_read,
	.proc_release = gb_audio_edid_release,
};
#endif

int gb_audio_create_proc_entry(void)
{
	struct proc_dir_entry *return_parent;

	gb_printf(KERN_INFO, "gb audio enter proc entry\n");

	remove_proc_subtree("gb02_audio", NULL);
	audio_proc_parent = proc_mkdir("gb02_audio", NULL);
	if (!audio_proc_parent) {
		gb_printf(KERN_ERR, "error creating proc entry\n");
		return -ENOMEM;
	}

	return_parent = proc_create_data("gb02_board_type", 0,
		audio_proc_parent, &audio_proc_ops, NULL);
	if (!return_parent) {
		remove_proc_subtree("gb02_audio", NULL);
		return -ENOMEM;
	}

	return_parent = proc_create_data("EDID_audio", 0,
		audio_proc_parent, &audio_edid_ops, NULL);
	if (!return_parent) {
		remove_proc_subtree("gb02_audio", NULL);
		return -ENOMEM;
	}

	gb_printf(KERN_INFO, "gb audio create proc entry\n");
	return 0;
}

void gb_audio_remove_proc(void)
{
	remove_proc_subtree("gb02_audio", NULL);
}
