/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef __GB_SND_CODEC_H__
#define __GB_SND_CODEC_H__
#include <linux/platform_device.h>
#include "i2s_platform.h"

#define MIN_NUM_PERIODS	4
#define MAX_NUM_PERIODS	256
#define MIN_RATE_LEN	16000
#define MAX_RATE_LEN	48000
#define MAX_PERIOD_BYTES	(4096)
#define MIN_PERIOD_BYTES	(256)
#define MAX_BUFFER_BYTES	(4096)
#define PCM_PAGES_SIZE	(32 * 1024)

#define DP_BLOCK_SIZE	(192 * 4)
#define DMA_FIFO_LEN	32768
#define AUDIO_CHANNEL_GROUP	4

/* Sample index defination depends on IEC60958-3@1999 */
#define BIT0_USAGE_LEFT_CHANNEL                    1
#define BIT0_USAGE_RIGHT_CHANNEL                   2
#define BIT2_COPYRIGHT_LEFT_CHANNEL                5
#define BIT2_COPYRIGHT_RIGHT_CHANNEL               6
#define BIT20_FOR_LEFT_CHANNEL                    41
#define BIT21_FOR_RIGHT_CHANNEL                   44
#define BIT24_SAMPLING_FREQUENCY_1_LEFT_CHANNEL   49
#define BIT24_SAMPLING_FREQUENCY_1_RIGHT_CHANNEL  50
#define BIT25_SAMPLING_FREQUENCY_2_LEFT_CHANNEL   51
#define BIT25_SAMPLING_FREQUENCY_2_RIGHT_CHANNEL  52
#define BIT26_SAMPLING_FREQUENCY_3_LEFT_CHANNEL   53
#define BIT26_SAMPLING_FREQUENCY_3_RIGHT_CHANNEL  54
#define BIT27_SAMPLING_FREQUENCY_4_LEFT_CHANNEL   55
#define BIT27_SAMPLING_FREQUENCY_4_RIGHT_CHANNEL  56
#define BIT32_MAXIMUM_WORLD_LENGTH_LEFT_CHANNEL   65
#define BIT32_MAXIMUM_WORLD_LENGTH_RIGHT_CHANNEL  66
#define BIT33_WORLD_LENGTH_1_LEFT_CHANNEL         67
#define BIT33_WORLD_LENGTH_1_RIGHT_CHANNEL        68
#define BIT34_WORLD_LENGTH_2_LEFT_CHANNEL         69
#define BIT34_WORLD_LENGTH_2_RIGHT_CHANNEL        70
#define BIT35_WORLD_LENGTH_3_LEFT_CHANNEL         71
#define BIT35_WORLD_LENGTH_3_RIGHT_CHANNEL        72

#define CHANNEL_STATUS_BIT_SHIFT                  26
#define PREAMBLE_CODE_BIT_SHIFT                   28
#define SAMPLE_CNT_PER_BLOCK                     384

int gb_snd_device_init(struct device *dev);
void i2s_restart_handle(int num, struct i2s_ctrl_info *i2s_cinfo);
void set_audio_param(int dp_num, struct i2s_ctrl_info *i2s_cinfo);
u32 cal_pos_offset(struct snd_pcm_runtime *runtime, u64 pos);
#ifdef AUDIO_SOUND_SWITCH_DIFF_A
int gb_pcm_trigger(struct snd_pcm_substream *substream, int cmd);
#endif
void gb_alsa_audio_init(struct pcie_info_str *pcie_info);
#endif
