#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/debugfs.h>
#include <linux/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/delay.h>
#include <linux/pci.h>
#include <linux/miscdevice.h>
#include <linux/interrupt.h>
#include <linux/sysfs.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/string.h>
#include <linux/gfp.h>
#include <linux/mm_types.h>
#include <linux/mm.h>
#include <linux/highmem.h>
#include <linux/slab.h>
#include "common/xt.h"
#include "gb02_fpga_v2vdma.h"
#include "local.h"
#include "i2s_platform.h"
#include "common/gb_common.h"
#include "gb_snd_codec.h"
#include "designware_i2s.h"
#include "ip/gb_dp.h"
#include "common/gb_pcie_info.h"

static struct i2s_ctrl_info i2s_infos;
struct i2s_ctrl_info *get_i2s_infos(void)
{
	return &i2s_infos;
}

void init_audio_infos(void)
{
	struct i2s_ctrl_info *i2s_ctrl;

	i2s_ctrl = get_i2s_infos();
	memset((void *)i2s_ctrl, 0, sizeof(struct i2s_ctrl_info));
}

static struct dptx *gb_dp_get_dptx_info(int dp_index)
{
	struct gb_pcie_info *pcie_info = get_pcie_gl_info();
	struct dptx *dptx_info = pcie_info->dptx[dp_index];

	return dptx_info;
}

static void set_dptx_audio_desc(struct dptx *dptx, int sample_rate,
	int ch_count, int sample_dep)
{
	struct audio_short_desc *audio_desc;

	audio_desc = &dptx->audio_desc;
	audio_desc->max_sampling_freq = sample_rate;
	audio_desc->max_num_of_channels = ch_count;
	audio_desc->max_bit_per_sample = sample_dep;
}

void gb01_audio_clk_cfg(int chid, int sample_rate, int ch_count,
	int sample_dep, u64 pll_va, u64 base_switch, u64 dp_va)
{
	uint32_t val = 0, pll_val = 0;
	struct dptx *dptx_info;

	gb_printf(KERN_INFO, "%s-%d: id = %d, sapmle = %d, ch_count = %d\n",
		__func__, __LINE__, chid, sample_rate, ch_count);
	dptx_info = gb_dp_get_dptx_info(chid);
	set_dptx_audio_desc(dptx_info, sample_rate, ch_count, sample_dep);
	dptx_config_audio_based_on_edid(dptx_info, DPTX_AUDIO_START_PLAY);

	iowrite32(0x60000000, (void *)(base_switch + 0xd0c));

	pll_val = ioread32((void *)(pll_va + 0x154));

	if (chid < 2) {
		val = ioread32((void *)(pll_va + 0x148));
		val &= ~(0xff << (12 + chid * 8));
	} else {
		val = ioread32((void *)(pll_va + 0x14c));
		val &= ~(0xff << (chid - 2) * 8);
	}
	switch (sample_rate) {
	case ASAMPLE_RATE_32:
		pll_val &= ~(1 << (6 + chid)); // pll0
		if (chid < 2)
			val |= DIVI_PLL_32K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_32K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_44_1:
		pll_val |= 1 << (6 + chid); // pll1
		if (chid < 2)
			val |= DIVI_PLL_44K_48K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_44K_48K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_48:
		pll_val &= ~(1 << (6 + chid)); // pll0
		if (chid < 2)
			val |= DIVI_PLL_44K_48K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_44K_48K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_88_2:
		pll_val |= 1 << (6 + chid); // pll1
		if (chid < 2)
			val |= DIVI_PLL_88K_96K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_88K_96K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_96:
		pll_val &= ~(1 << (6 + chid)); // pll0
		if (chid < 2)
			val |= DIVI_PLL_88K_96K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_88K_96K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_176_4:
		pll_val |= 1 << (6 + chid); // pll1
		if (chid < 2)
			val |= DIVI_PLL_176K_192K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_176K_192K << (chid - 2) * 8;
		break;
	case ASAMPLE_RATE_192:
		pll_val &= ~(1 << (6 + chid)); // pll0
		if (chid < 2)
			val |= DIVI_PLL_176K_192K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_176K_192K << (chid - 2) * 8;
		break;
	default:
		pll_val &= ~(1 << (6 + chid)); // 48k pll0
		if (chid < 2)
			val |= DIVI_PLL_44K_48K << (PLL_I2S0_bit + chid * 8);
		else
			val |= DIVI_PLL_44K_48K << (chid - 2) * 8;
		break;
	}

	iowrite32(pll_val, (void *)(pll_va + 0x154));

	if (chid < 2)
		iowrite32(val, (void *)(pll_va + 0x148));
	else
		iowrite32(val, (void *)(pll_va + 0x14c));

	if (ch_count == TWO_CHANNEL_SUPPORT) {
		if (sample_dep == PCM_WIDTH_BIT_16)
			iowrite32(0x13001202,
			(void *)(dp_va + chid * 0x400000 + 0x400));
		else
			iowrite32(0x13001302,
			(void *)(dp_va + chid * 0x400000 + 0x400));
	} else {
		if (sample_dep == PCM_WIDTH_BIT_16)
			iowrite32(0x1300221e,
			(void *)(dp_va + chid * 0x400000 + 0x400));
		else
			iowrite32(0x1300231e,
			(void *)(dp_va + chid * 0x400000 + 0x400));
	}

	val = ioread32((void *)(dp_va + chid * 0x400000 + 0x0500));
	val &= 0xfffffffc;
	val |= 0x1 << 0;
	val |= 0x1 << 1;
	/* SDP_VERTICAL_CTRL */
	iowrite32(val, (void *)(dp_va + chid * 0x400000 + 0x0500));

	val = ioread32((void *)(dp_va + chid * 0x400000 + 0x0504));
	val &= 0xfffffffc;
	val |= 0x1 << 0;
	val |= 0x1 << 1;
	/* SDP_HORIZONTAL_CTRL */
	iowrite32(val, (void *)(dp_va + chid * 0x400000 + 0x0504));

	iowrite32(0xe08, (void *)(dp_va + chid * 0x400000 + 0x204));
	iowrite32(0x0, (void *)(dp_va + chid * 0x400000 + 0x204));
}

int get_is_support_audio(struct i2s_ctrl_info *i2s_cinfo, int idx)
{
	int ret = DP_DISSUPPORT_AUDIO;

	if ((i2s_cinfo->dp_connect_status[idx] == DP_CONNECT)
	&& (i2s_cinfo->dp_edid_support_audio[idx] == DP_SUPPORT_AUDIO))
		ret = DP_SUPPORT_AUDIO;
	return ret;
}

#ifdef AUDIO_SOUND_SWITCH_DIFF_A
static void handle_stop_sound_pcm(struct i2s_ctrl_info *i2s_ctrl, int dp_num)
{
	struct snd_pcm_substream *substream;
	struct snd_pcm_runtime *runtime;

	if (i2s_ctrl->playback_substream[dp_num] == NULL)
		return;
	substream = i2s_ctrl->playback_substream[dp_num];
	runtime = substream->runtime;
	if (runtime == NULL)
		return;
	if (runtime->status->state == SNDRV_PCM_STATE_RUNNING) {
		gb_pcm_trigger(substream, SNDRV_PCM_TRIGGER_STOP);
//		gb_dma_hw_free(substream);
//		gb_dma_close(substream);
		i2s_ctrl->jack_stop_pcm[dp_num] = 1;
	}
}
#endif

static int get_connect_num(struct i2s_ctrl_info *i2s_infos)
{
	int i, ret_num = 0;

	for (i = 0; i < DW_AXI_DMA_CHANNELS; ++i) {
		if (i2s_infos->dp_connect_status[i] == DP_CONNECT)
			ret_num++;
	}
	return ret_num;
}

static void board_unhot_jack_switch(struct i2s_ctrl_info *i2s_infos,
	int dp_num)
{
	struct gb_pcie_info *pcie_info = get_pcie_gl_info();
	enum gb_board_type gb_type = gb_get_board_type(pcie_info);
	int ret;

	if ((gb_type == PCIE_LPDDR4) || (gb_type == PCIE_C0_200)) {
		if (i2s_infos->jack_init_flag[dp_num] == AUDIO_JACK_INIT) {
			// switch the audio cards, when GVA keeping CONNECT.
			ret = get_connect_num(i2s_infos);
			if (ret > 0)
				snd_jack_report(i2s_infos->jack[dp_num], 0);
		}
	}
}

// determine if only the VGA port is connected
static int judge_only_vga_connect(enum gb_board_type gb_type,
	struct i2s_ctrl_info *i2s_infos)
{
	int ret = -1;

	if ((gb_type != PCIE_LPDDR4) && (gb_type != PCIE_C0_200))
		return ret;
	if ((i2s_infos->dp_connect_status[C0_P21_HDMI_NUM] == DP_DISCONNECT)
	&& (i2s_infos->dp_connect_status[C0_P21_VGA_NUM] == DP_CONNECT))
		ret = 0;
	return ret;
}

static void board_hot_jack_switch(struct i2s_ctrl_info *i2s_infos,
	int dp_num)
{
	struct gb_pcie_info *pcie_info = get_pcie_gl_info();
	enum gb_board_type gb_type = gb_get_board_type(pcie_info);
	int ret;

	if ((gb_type == PCIE_LPDDR4) || (gb_type == PCIE_C0_200)) {
		if (dp_num != C0_P21_VGA_NUM)
			return;
		ret = judge_only_vga_connect(gb_type, i2s_infos);
		if (ret == 0) {
			if (i2s_infos->jack_init_flag[C0_P21_HDMI_NUM] ==
				AUDIO_JACK_INIT)
				snd_jack_report(i2s_infos->jack[C0_P21_HDMI_NUM], 0);
			gb_printf(KERN_INFO, "%s-%d: vga = %d, delete jack\n",
				__func__, __LINE__, dp_num);
		}
	}
}

void set_i2s_ctrl_info(int dp_num, int pcm_channel,
	int audio_sample, int pcm_width, int hotplug_status)
{
	struct i2s_ctrl_info *i2s_ctrl;

	i2s_ctrl = get_i2s_infos();
	i2s_ctrl->dp_connect_status[dp_num] = hotplug_status;
	if (i2s_ctrl->dp_connect_status[dp_num] == DP_DISCONNECT) {
		set_support_audio(dp_num, 0);
		board_unhot_jack_switch(i2s_ctrl, dp_num);
		i2s_ctrl->i2s_pcm_parm[dp_num].pcm_channel =
			TWO_CHANNEL_SUPPORT;
		i2s_ctrl->i2s_pcm_parm[dp_num].audio_sample = ASAMPLE_RATE_48;
		i2s_ctrl->i2s_pcm_parm[dp_num].pcm_width = PCM_WIDTH_BIT_16;
	} else {
		if (i2s_ctrl->dp_edid_support_audio[dp_num] == DP_SUPPORT_AUDIO) {
			if (pcm_channel > EIGHT_CHANNEL_SUPPORT)
				i2s_ctrl->i2s_pcm_parm[dp_num].pcm_channel =
					EIGHT_CHANNEL_SUPPORT;
			else if (pcm_channel <= TWO_CHANNEL_SUPPORT)
				i2s_ctrl->i2s_pcm_parm[dp_num].pcm_channel =
					TWO_CHANNEL_SUPPORT;
			else
				i2s_ctrl->i2s_pcm_parm[dp_num].pcm_channel =
					pcm_channel / 2 * 2;

			if (audio_sample < ASAMPLE_RATE_32
				|| audio_sample > ASAMPLE_RATE_192)
				i2s_ctrl->i2s_pcm_parm[dp_num].audio_sample =
					ASAMPLE_RATE_48;
			else
				i2s_ctrl->i2s_pcm_parm[dp_num].audio_sample =
					audio_sample;
			i2s_ctrl->i2s_pcm_parm[dp_num].pcm_width = pcm_width;
			gb_printf(KERN_INFO, "dp_num=%d, jack on\n", dp_num);
			if (i2s_ctrl->jack_init_flag[dp_num] ==
				AUDIO_JACK_INIT) {
#ifdef AUDIO_SOUND_SWITCH_DIFF_A
				handle_stop_sound_pcm(i2s_ctrl, dp_num);
				if (i2s_ctrl->jack_stop_pcm[dp_num] == 0)
#endif
				snd_jack_report(i2s_ctrl->jack[dp_num],
					SND_JACK_AVOUT);
			}
		} else
			board_hot_jack_switch(i2s_ctrl, dp_num);
	}
	gb_printf(KERN_INFO, "%s-%d: i = %d, channel=%d, sample=%d,\
		connect_status = %d, pcm_width = %d\n",
		__func__, __LINE__, dp_num,
		i2s_ctrl->i2s_pcm_parm[dp_num].pcm_channel,
		i2s_ctrl->i2s_pcm_parm[dp_num].audio_sample,
		i2s_ctrl->dp_connect_status[dp_num], pcm_width);
}

void set_support_audio(int dp_num, int audio_support_status)
{
	struct i2s_ctrl_info *i2s_ctrl;

	i2s_ctrl = get_i2s_infos();
	i2s_ctrl->dp_edid_support_audio[dp_num] = audio_support_status;
	gb_printf(KERN_INFO, "%s-%d: dp_num = %d, audio_support_status = %d\n",
		__func__, __LINE__, dp_num, audio_support_status);
}

void set_audio_reparam(int dp_num, int restart_val)
{
	struct i2s_ctrl_info *i2s_cinfo = get_i2s_infos();
	struct snd_pcm_substream *substream =
		i2s_cinfo->playback_substream[dp_num];
	struct snd_pcm_runtime *runtime;

	if ((i2s_cinfo == NULL) || (substream == NULL)) {
		gb_printf(KERN_INFO, "%s-%d: dp_num = %d, NULL, so return\n",
			__func__, __LINE__, dp_num);
		return;
	}
	if ((i2s_cinfo->dp_connect_status[dp_num] == DP_DISCONNECT)
	|| (i2s_cinfo->dp_edid_support_audio[dp_num] == DP_DISSUPPORT_AUDIO)) {
		gb_printf(KERN_INFO, "%s-%d: dp_num = %d, return\n",
			__func__, __LINE__, dp_num);
		return;
	}
	runtime = substream->runtime;
	if (runtime == NULL) {
		gb_printf(KERN_INFO, "%s-%d: runtime = NULL, so return\n",
			__func__, __LINE__);
		return;
	}

	if (runtime->status->state == SNDRV_PCM_STATE_RUNNING) // running
		i2s_cinfo->audio_reset_parameters[dp_num] = restart_val;
	gb_printf(KERN_INFO, "%s-%d: audio audio_reset_parameters[%d] = %d\n",
		__func__, __LINE__, dp_num,
		i2s_cinfo->audio_reset_parameters[dp_num]);
}

int get_audio_reparam(int dp_num)
{
	struct i2s_ctrl_info *i2s_cinfo;

	i2s_cinfo = get_i2s_infos();
	return i2s_cinfo->audio_reset_parameters[dp_num];
}

irqreturn_t gb_audio_dma_irq_handler(int irq, void *arg)
{
	struct snd_pcm_substream *substream;
	struct snd_pcm_runtime *runtime;
	struct i2s_ctrl_info *i2s_cinfo = get_i2s_infos();
	struct audio_dma_ctrl_info *chip = &i2s_cinfo->dma_info;
	struct axi_dma_chan *chan = NULL;
	u32 status = 0, val = 0;
	u64 bus_addr = 0, bus_vaddr = 0;
	int i = 0;

	if (chip->regs == NULL) {
		gb_printf(KERN_ERR, "%s-%d: chip is NULL\n",
			__func__, __LINE__);
		return IRQ_NONE;
	}
	axi_dma_irq_disable(chip);
	status = dma_reg_read32(chip, DMAC_COMMON_INTSTATUS);
	if (status) {
		dma_reg_write32(chip, DMAC_COMMON_INTCLEAR, status);
		val = dma_reg_read32(chip, DMAC_COMMON_INTSTATUS_ENA);
		val &= ~status;
		dma_reg_write32(chip, DMAC_COMMON_INTSTATUS_ENA, val);
	}

	for (i = 0; i < DW_AXI_DMA_CHANNELS; i++) {
		chan = &chip->chan[i];
		if (chan->chan_regs == NULL) {
			gb_printf(KERN_ERR, "%s-%d: chan is NULL\n",
				__func__, __LINE__);
			return IRQ_NONE;
		}
		status = axi_chan_irq_read(chan);
		axi_chan_irq_clear(chan, status);
		if (status & DWAXIDMAC_IRQ_ALL_ERR) {
			gb_printf(KERN_ERR, "%s-%d: err id =%d, status=0x%x\n",
				__func__, __LINE__, i, status);
			return IRQ_NONE;
		} else if (status & DWAXIDMAC_IRQ_DMA_TRF) {
			substream = i2s_cinfo->playback_substream[i];
			if (substream == NULL) {
				gb_printf(KERN_ERR, "%s-%d: substream is NULL\n",
					__func__, __LINE__);
				axi_dma_irq_enable(chip);
				continue;
			}

			runtime = substream->runtime;
			if (runtime == NULL) {
				gb_printf(KERN_ERR, "%s-%d: runtime is NULL\n",
					__func__, __LINE__);
				axi_dma_irq_enable(chip);
				continue;
			}

			axi_chan_block_xfer_complete(chan);
			chip->pos[i] = (chip->pos[i]
				+ runtime->period_size) % runtime->buffer_size;
			val = cal_pos_offset(runtime,
				frames_to_bytes(runtime, chip->pos[i]));
			bus_addr = i2s_cinfo->aoddr_pbase + BUF_MAX_SIZE * i
				+ val
				- i2s_cinfo->vphy_base + DDRINS_OFFSET;
//			gb_printf(KERN_INFO,"bus_addr = 0x%llx, point = %llu, hw_ptr = %lu, hw_base = %lu\n",
//				bus_addr, chip->pos[i],
//				runtime->status->hw_ptr, runtime->hw_ptr_base);
			/* Solve problem VPU-1438:Begin */
			if (runtime->control->appl_ptr < runtime->status->hw_ptr) {
				bus_vaddr = (u64)i2s_cinfo->ddrwr_va + i2s_cinfo->aoddr_pbase - i2s_cinfo->vphy_base +
							BUF_MAX_SIZE * i + frames_to_bytes(runtime, val);
				memset((void *)bus_vaddr, 0, i2s_cinfo->pcm_bytes[i]);
			}
			/* Solve problem VPU-1438:End */
			axi_dma_irq_enable(chip);
			restart_dma_axi_ch(chip, bus_addr, i);

			if (i2s_cinfo->playback_substream[i])
				snd_pcm_period_elapsed(i2s_cinfo->playback_substream[i]);
		}
	}
	return IRQ_HANDLED;
}

