#ifndef	__I2S_PLATFORM_H__
#define	__I2S_PLATFORM_H__
#include <linux/miscdevice.h>
#include <linux/interrupt.h>
#include "gb02_fpga_v2vdma.h"
#include "local.h"

#ifndef	AUDIO_USE_NOC
#define	CONTROL_REG_BAR 0
#define DMA_DDR_BAR  2
#else
#define DMA_DDR_BAR  0
#define	CONTROL_REG_BAR	2
#endif

#define C0_P21_HDMI_NUM 2
#define C0_P21_VGA_NUM 3

#define DIVI_PLL_32K 149
#define DIVI_PLL_44K_48K 99
#define DIVI_PLL_88K_96K 49
#define DIVI_PLL_176K_192K 24
#define PLL_I2S0_bit	12

//#define AUDIO_SOUND_SWITCH_DIFF_A
//#define DEBUG_SAVE_FILE

/* 18M/6 = 3M */
#define BUF_MAX_SIZE 0x300000
#define AUDIO_DMA_MAX_LEN 0x10000
typedef struct {
	unsigned long bus_address;
	unsigned int size;
	unsigned int audio_id;
	unsigned long translation_offset;
} memalloc_params;

struct pcie_info_str {
	u64	hdwr_pa;
	void	*hdwr_va;
	u64	hdwr_len;

	u64	ddrwr_pa;
	void	*ddrwr_va;
	u64	ddrwr_len;
	u64	base_switch;
	u64	pll_va;
};

struct i2s_pcm_parm {
	int	pcm_channel;
	int	audio_sample;
	int	pcm_width;
};

struct pcm_data_info {
	unsigned char *in_buf;
	unsigned char *out_buf;
};

enum DP_CONNECT_STATUS {
	DP_DISCONNECT = 0,
	DP_CONNECT,
	DP_STATUS_MAX,
};

enum DP_SUPPORT_AUDIO_STATUS {
	DP_DISSUPPORT_AUDIO = 0,
	DP_SUPPORT_AUDIO,
	DP_STATUS_AUDIO_MAX,
};

enum AUDIO_RESTART_STATUS {
	AUDIO_DISNEED_RESTART = 0,
	AUDIO_NEED_RESTART,
	AUDIO_RESTART_MAX,
};

enum AUDIO_SUSPEND_RESTORE_STATUS {
	AUDIO_SUSPEND_DISRESTORE = 0,
	AUDIO_SUSPEND_RESTORE,
	AUDIO_SUSPEND_RESTORE_MAX,
};

enum AUDIO_NEED_MUTE {
        AUDIO_SET_UNMUTE = 0,
        AUDIO_SET_MUTE,
        AUDIO_SET_MUTE_MAX,
};

enum AUDIO_JACK_INIT {
        AUDIO_JACK_UNINIT = 0,
        AUDIO_JACK_INIT,
        AUDIO_JACK_MAX,
};

struct i2s_ctrl_info {
	struct pci_dev *pdev;
	struct snd_card *card;
	struct snd_pcm *pcm;
	struct snd_jack *jack[DW_AXI_DMA_CHANNELS];
	u64	aoddr_pbase;	//audio phy base
	u64	vphy_base;	//vram phy base
	int	abuffer_size;	
	struct audio_dma_ctrl_info dma_info;
	struct dw_i2s_dev i2s_info;
	u64	base_switch;
	u64	pll_va;
	u64	dp_va;
	void	*ddrwr_va;
	int     pcm_bytes[DW_AXI_DMA_CHANNELS];
	int     suspend_play_flag[DW_AXI_DMA_CHANNELS];
	int	dp_connect_status[DW_AXI_DMA_CHANNELS];	// 1 connect; 0 disconnect
	int	dp_edid_support_audio[DW_AXI_DMA_CHANNELS];	// 1 support; 0 dissupport
	int	audio_reset_parameters[DW_AXI_DMA_CHANNELS]; // 1 need; 0 unneed
	unsigned int g_sample_cnt[DW_AXI_DMA_CHANNELS];
	int	jack_init_flag[DW_AXI_DMA_CHANNELS];
#ifdef AUDIO_SOUND_SWITCH_DIFF_A
	int	jack_stop_pcm[DW_AXI_DMA_CHANNELS];
#endif
	struct i2s_pcm_parm	i2s_pcm_parm[DW_AXI_DMA_CHANNELS];
	struct pcm_data_info audio_data_info[DW_AXI_DMA_CHANNELS];
	struct snd_pcm_substream *playback_substream[DW_AXI_DMA_CHANNELS];
#ifdef DEBUG_SAVE_FILE
	struct file *fp;
#endif
};

struct i2s_ctrl_info *get_i2s_infos(void);
void init_audio_infos(void);
irqreturn_t gb_audio_dma_irq_handler(int irq, void *arg);
void gb01_audio_clk_cfg(int chid, int sample_rate, int ch_count,
	int sample_dep, u64 pll_va, u64 base_switch, u64 dp_va);
void set_i2s_ctrl_info(int dp_num, int pcm_channel,
	int audio_sample, int pcm_width, int hotplug_status);
void set_support_audio(int dp_num, int audio_support_status);
void set_audio_reparam(int dp_num, int restart_val);
int get_audio_reparam(int dp_num);
int get_is_support_audio(struct i2s_ctrl_info *i2s_cinfo, int idx);
#endif
