/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/delay.h>
#include <linux/mutex.h>
#include <linux/version.h>
#include <linux/irq.h>
#include <linux/kernel.h>
#ifndef		__GB_COMMON_H__
#define		__GB_COMMON_H__
#include "common/gb_uk.h"

#define	KB	(1UL<<10)
#define	MB	(1UL<<20)
#define	GB	(1UL<<30)
struct pci_dev;
struct mutex;
#define GB_DC_DP
#define PALADING_MODE
//#define FPGA_MODE
#define	GB_DEC_MODULE
#define	GB_ENC_MODULE
struct gb_pcie_info;
//#define	GB_PRINT_TMP_DEBUG
//#define	GB02_PLD

extern u64 get_genbu_ecc_len(void);
extern atomic_t log_en;
#define gb_printf(level, format, arg...)				\
	do {								\
		if (atomic_read(&log_en) >= simple_strtol(&level[1], NULL, 10)) \
			printk(level format, ##arg);			\
	} while(0)							\

#define	GB_HPD_THREAD
#define GB_DRM_MODE_FMT    "\"%s\": %px %d %d %d %d %d %d %d %d %d 0x%x 0x%x %d"

#define GB_DRM_MODE_ARG(m) \
	(m)->name, m, (m)->clock, \
	(m)->hdisplay, (m)->hsync_start, (m)->hsync_end, (m)->htotal, \
	(m)->vdisplay, (m)->vsync_start, (m)->vsync_end, (m)->vtotal, \
	(m)->type, (m)->flags,(m)->status

#define dump_func_begin \
	pr_info("----%s begin----\n", __func__)
#define dump_func_param(fmt,...) \
	pr_info("%s: "fmt,__func__,##__VA_ARGS__)
#define dump_func_end \
	pr_info("----%s end----\n", __func__)
#define dump_current  \
	pr_info("%s, ********thread name %s,pid %d\n",\
		__func__,current->comm,current->pid)
#define dump_caller \
	pr_info("*****dump caller:%pS->%pS->%s \n",\
		__builtin_return_address(1),__builtin_return_address(0),__func__)
#define dump_mode(mode, con) \
	if(con) \
		pr_info("mode %px:%d x %d@%d Hz\n", mode, mode->hdisplay, mode->vdisplay, drm_mode_vrefresh(mode))
#define dump_mode_more(mode, con) \
	if (con) \
		pr_info(GB_DRM_MODE_FMT, GB_DRM_MODE_ARG(mode))
#define dump_vmode_pad(vpad,vidx) \
	pr_info("vidx %d,%d x %d:wpad(%d) hpad(%d)\n",vidx,vpad[vidx].vwidth, vpad[vidx].vheight, \
		vpad[vidx].max_wpad, vpad[vidx].max_hpad)
#define dump_mode_pad(pad,idx) \
	pr_info("%d:%d x %d:top(%d),right(%d),bottom(%d),left(%d)\n",idx,pad[idx].width, pad[idx].height, \
		pad[idx].top, pad[idx].right,pad[idx].bottom,pad[idx].left)
/* def vram max size 32GB*/
#define GB_VRAM_MAX_VRAM_SIZE (32*GB)
/* def vpu len large 8GB to 32GB vram*/
#define GB_VPU_LARGE_LEN (8*GB)

/* Vram address partition domain */
/*
	VRAM_DEF:|DC & VPU|GPU|MCU|AUDIO|HDMAC|PERF|MMU |LL| ->
	VRAM_TYPE:|  TTM  |TTM|PHY|PHY  |PHY  |PHY |BIT |PHY| ->
	VRAM_LEN:|   8G   |RES|2M |18M  |0MB  |1MB |64MB|24MB|
*/
/* Reserved for VPU */
#define VPU_LEN             (0*MB)
/* Reserved for MCU */
#define MCU_LEN             (2*MB)
/* Reserved for AUDIO */
#define AUDIO_LEN           (18*MB)
/* Reserved for HDMAC */
#define HDMAC_VRAM_LEN      (0*MB)
/* Reserved for V2V */
#define PERF_CNT_LEN        (1*MB)
/* Reserved for MMU */
#define MMU_LEN             (64*MB)
/* Reserved for DMA LL TABLE */
#define LL_TABLE_LEN        (24*MB)
/* Reserved for ECC */
#define ECC_LEN        (get_genbu_ecc_len())
/* Reserved for ALL SIZE */
#define RESERVE_ALL         (VPU_LEN + MCU_LEN + AUDIO_LEN + HDMAC_VRAM_LEN + PERF_CNT_LEN + MMU_LEN + LL_TABLE_LEN + ECC_LEN)
/* Reserved for GPU */
#define GPU_LEN             (get_genbu_vram_size() - RESERVE_ALL)

/* start addr define */
#define DC_FB_VRM_START_ADDR	(0 * GB)
#define VPU_VRM_START_ADDR	(DC_FB_VRM_START_ADDR)
#define XPU_VRM_START_ADDR	(DC_FB_VRM_START_ADDR + VPU_LEN)
#define MCU_VRAM_START_ADDR	(XPU_VRM_START_ADDR + GPU_LEN)
#define AUDIO_VRAM_START_ADDR	(MCU_VRAM_START_ADDR + MCU_LEN)
#define HDMAC_VRAM_START_ADDR	(AUDIO_VRAM_START_ADDR + AUDIO_LEN)
#define PERF_CNT_START_ADDR	(HDMAC_VRAM_START_ADDR + HDMAC_VRAM_LEN)
#define MMU_START_ADDR		(PERF_CNT_START_ADDR + PERF_CNT_LEN)
#define LL_TABLE_START_ADDR	(MMU_START_ADDR + MMU_LEN)

#define PCI_BAR_MAX_CNT 6
#define CONNECTOR_MAX_CNT 6
#define IRQ_NUM 16

enum gb_board_type {
	PCIE_FULL_FUNC_DDR4 = 0,
	PCIE_LPDDR4,
	PCIE_FULL_LPDDR4,
	/*M6FL8G 4HDMI+2DP*/
	PCIE_M6FL8G_LPDDR4,
	/*HIE1LP4 4HDMI*/
	PCIE_HIE1LP4_LPDDR4,
	PCIE_C0_200,
	/*M4HL8G 4HDMI*/
	PCIE_M4HL8G_LPDDR4,
	CQ2040_MXM_M60,
	BOARD_TYPE_MAX,
};

enum genbu_asic_type {
	GENBU_FPGA = 0,
	GENBU_01,
	GENBU_02,
	GENBU_TYPE_MAX,
};

/* this enumerates display type. */
enum gb_drm_output_type {
	GB_DISPLAY_TYPE_NONE,
	/* RGB or CPU Interface. */
	GB_DISPLAY_TYPE_LCD,
	/* HDMI Interface. */
	GB_DISPLAY_TYPE_HDMI,
	/* DP Interface. */
	GB_DISPLAY_TYPE_DP,
	/* Virtual Display Interface. */
	GB_DISPLAY_TYPE_LVDS,
};

struct gb_dma_ll_bitmap {
	unsigned long *dma_ll_mem_bitmap;
	unsigned long dma_ll_mem_bitmap_maxno;
	struct mutex lock;
};

struct gb_edma_para {
	unsigned int wr_channel_usage;
	unsigned int rd_channel_usage;
	struct gb_dma_ll_bitmap ll_bitmap;
	struct mutex channel_rd_lock;
	struct mutex channel_wr_lock;
	struct mutex dma_lock;
	struct hdma_ctrl_info *dma_ctrl;
};

extern int gb_hdmac_init(struct gb_pcie_info *pcie_info);
extern void gb_hdmac_release(void);
extern int gb_v2vdma_init(struct gb_pcie_info *pcie_info);
extern void gb_v2vdma_release(struct gb_pcie_info *pcie_info);
extern void gb02_ddr_init(const void __iomem *mcu_peri_bar_p, const void __iomem *sys_peri_bar_p,
		const void __iomem *ddr_bar_p, const void __iomem *peri_base_bar_p);
extern int pcie_hdma_init(struct gb_pcie_info *pcie_info);
extern int gb_dp_init(int dp_index,
	struct gb_pcie_info *pcie_info, bool is_edp);

extern int gb_dp_pm_resume(int dp_index,
	struct gb_pcie_info *pcie_info, bool is_edp);
extern int gb_dp_pm_suspend(int dp_index,
	struct gb_pcie_info *pcie_info, bool is_edp);
extern enum gb_board_type gb_get_board_type(struct gb_pcie_info *gpi);
extern bool gb_connector_is_vga(int connector_id);
extern int gb_get_gpu_utilization(u32 *gpu_ut);
extern void gb_get_gpu_utilization_2d(u32 *gpu_ut_2d, unsigned long *gpu_mem_used);
void HDMI_power_reset(struct gb_pcie_info *gpi, int dp_index);
#endif
