#ifndef __COMM_IRQ_H__
#define __COMM_IRQ_H__

#include <linux/interrupt.h>
#include "xt.h"

#define GPU_MIRQ_ENABLED
#define IRQ_INTEN_L		(CS_IRQ_CTRL_BASE + 0x0)
#define IRQ_INTEN_H		(CS_IRQ_CTRL_BASE + 0x4)
#define IRQ_INTMASK_L	(CS_IRQ_CTRL_BASE + 0x8)
#define IRQ_INTMASK_H	(CS_IRQ_CTRL_BASE + 0xC)
#define IRQ_INTFORCE_L	(CS_IRQ_CTRL_BASE + 0x10)
#define IRQ_INTFORCE_H	(CS_IRQ_CTRL_BASE + 0x14)
#define IRQ_INTRAW_L	(CS_IRQ_CTRL_BASE + 0x18)
#define IRQ_INTRAW_H	(CS_IRQ_CTRL_BASE + 0x1C)
#define IRQ_INTSTA_L	(CS_IRQ_CTRL_BASE + 0x20)
#define IRQ_INTSTA_H	(CS_IRQ_CTRL_BASE + 0x24)
#define IRQ_MASK_STA_L	(CS_IRQ_CTRL_BASE + 0x28)
#define IRQ_MASK_STA_H	(CS_IRQ_CTRL_BASE + 0x2C)
#define IRQ_FSTA_L		(CS_IRQ_CTRL_BASE + 0x30)
#define IRQ_FSTA_H		(CS_IRQ_CTRL_BASE + 0x34)

struct gb_pcie_info;

enum gb_irq_ip_type {
	irq_type_comp,
	irq_type_v2vdma,
	irq_type_audio,
	irq_type_dec0,
	irq_type_dec1,
	irq_type_enc,
	irq_type_dc,
	irq_type_gpu,
	irq_type_dp,
	irq_type_hdmac,
	irq_type_total,
};

enum audio_irq {
	AUDIO_IRQ,
	AUDIO_DMA_IRQ,
	AUDIO_IRQ_TOTAL,
};

enum gpu_irq {
	GPU_MMU_IRQ,
	GPU_JOB_IRQ,
	GPU_GPU_IRQ,
	GPU_EVENT_IRQ,
	GPU_IRQ_TOTAL,
};

enum dc_irq {
	DC_DE_IRQ,
	DC_SE_IRQ,
	DC_IRQ_TOTAL,
};

enum gb_irq_bit {
	dc0_irqde		= 0,
	dc0_irqse		= 1,
	dc1_irqde		= 2,
	dc1_irqse		= 3,
	dc2_irqde		= 4,
	dc2_irqse		= 5,
	dc3_irqde		= 6,
	dc3_irqse		= 7,
	dc4_irqde		= 8,
	dc4_irqse		= 9,
	dc5_irqde		= 10,
	dc5_irqse		= 11,
	dp0_intr		= 12,
	dp1_intr		= 13,
	dp2_intr		= 14,
	dp3_intr		= 15,
	dp4_intr		= 16,
	dp5_intr		= 17,
	ddr0_int_o		= 18,
	ddr1_int_o		= 19,
	ddr2_int_o		= 20,
	ddr3_int_o		= 21,
	v2vdma_intr		= 22,
	hdmac_interupt	= 23,
	audio_dma_intr	= 24,
	i2s0_intr		= 25,
	i2s1_intr		= 26,
	i2s2_intr		= 27,
	i2s3_intr		= 28,
	i2s4_intr		= 29,
	i2s5_intr		= 30,
	dec0_INTR		= 31,
	dec1_INTR		= 32,
	enc0_INTR		= 33,
	enc1_INTR		= 34,
	enc2_INTR		= 35,
	gpu_IRQMMU		= 36,
	gpu_IRQJOB		= 37,
	gpu_IRQGPU		= 38,
	gpu_IRQEVENT	= 39,
	sen_volt_int_nomal	= 40,
	sen_volt_int_fatal	= 41,
	sen_temp_int_nomal	= 42,
	sen_temp_int_fatal	= 43,
	gb_irq_bit_total,
};
/*
#define DISABLE_IRQ (GENMASK_ULL(ddr3_int_o, ddr0_int_o) |\
	GENMASK_ULL(dc5_irqse, dc0_irqde) |\
	GENMASK_ULL(sen_temp_int_fatal, enc1_INTR) |\
	GENMASK_ULL(i2s5_intr, i2s0_intr) |\
	BIT(hdmac_interupt))
*/
#define DISABLE_IRQ (GENMASK_ULL(63, gb_irq_bit_total) |\
	GENMASK_ULL(ddr3_int_o, ddr0_int_o) |\
        GENMASK_ULL(sen_temp_int_fatal, enc1_INTR) |\
        GENMASK_ULL(i2s5_intr, i2s0_intr) |\
        GENMASK_ULL(v2vdma_intr, v2vdma_intr) |\
        BIT(hdmac_interupt))
extern int gb_pcie_enable_msi(struct gb_pcie_info *pcie_info);
extern int gb_pcie_init_irqs(struct gb_pcie_info *pcie_info);
extern int gb_pcie_free_irqs(struct gb_pcie_info *pcie_info);
extern int gb_pcie_disable_msi(struct gb_pcie_info *pcie_info);
extern int gb_init_comp_irq(struct gb_pcie_info *pcie_info);
extern void gb_release_comp_irq(struct gb_pcie_info *pcie_info);
extern int get_ip_irq_vector_index(struct gb_pcie_info *pcie_info,
	enum gb_irq_ip_type ip_type,
	int ip_irq_index);
extern irqreturn_t gb_gpu_common_irq_handler(int irq, void *data);
extern irqreturn_t gb_v2vdma_irq_handler(int irq, void *arg);
extern irqreturn_t gb_edma_irq_handler(int irq, void *arg);
extern irqreturn_t gb_dptx_irq_handler(int irq, void *dev);
extern irqreturn_t gb_dc_vsync_irq_handler(int irq, void *dev, int crtc_id);

extern void gb_set_irq_mask_en(void);
#endif
