/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "gb_pcie_info.h"

u64 get_genbu_vram_size_from_board(void)
{
	u64 vram_size = 0;
	struct gb_pcie_info *gbdev = get_pcie_gl_info();
	int mcu_peri_bar_id = get_genbu_mcu_peri_bar_id(gbdev->gb_dev_info);

	vram_size = base_reg_read(gbdev->pci_bars[mcu_peri_bar_id].mmio, BOARD_VRAM_SIZE_OFFSET);

	if(vram_size > 0 && vram_size <= BOARD_VRAM_SIZE_MAX){
		gb_printf(KERN_INFO, "get vram size from board: 0x%llx\n", vram_size);
		return vram_size*GB;
	}else{
		gb_printf(KERN_ERR, "%s get vram size faild, mcu_peri_bar_id:%d\n",
			__func__, mcu_peri_bar_id);
		return get_genbu_vram_size();
	}
}

enum genbu_asic_type get_genbu_gb_type(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->gb_type;
}

u64 get_vpu_dynamic_vram_size(void)
{
	u64 vram_size = get_genbu_vram_size();
	if (vram_size == GB_VRAM_MAX_VRAM_SIZE)
		return GB_VPU_LARGE_LEN;
	else
		return vram_size / 2;
}

u64 get_genbu_vram_size(void)
{
	struct gb_pcie_info *gbdev = get_pcie_gl_info();
	int ddr_var_cnt = get_genbu_pci_bar_cnt(gbdev->gb_dev_info);
	int ddr_bar_id = get_genbu_ddr_bar_id(gbdev->gb_dev_info);

	if (ddr_bar_id <= ddr_var_cnt)
		return gbdev->pci_bars[ddr_bar_id].len;

	gb_printf(KERN_ERR, "%s get vram size faild, ddr_bar_id:%d ddr_var_cnt:%d\n",
			__func__, ddr_bar_id, ddr_var_cnt);
	return 0;
}

int get_genbu_gpu_reg_bar_id(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->gpu_reg_bar_id;
}

int get_genbu_ddr_bar_id(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->ddr_bar_id;
}

int get_genbu_pci_bar_cnt(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->pci_bar_cnt;
}

u64 get_genbu_gb_reserve_size(struct gb_asic_dev_info *gb_dev_info)
{
	return RESERVE_ALL;
}

int get_genbu_irq_num(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->irq_num;
}

int get_genbu_mcu_peri_bar_id(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->mcu_peri_bar_id;
}

int get_genbu_peri_base_bar_id(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->peri_base_bar_id;
}

int get_genbu_dc_reg_bar_id(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->dc_reg_bar_id;
}
int get_genbu_crtc_num(struct gb_asic_dev_info *gb_dev_info)
{
	return gb_dev_info->crtc_num;
}

