#ifndef GB_PCIE_INFO_H
#define GB_PCIE_INFO_H

#include "common/gb_common.h"
#include "common/gb-peripherals-common.h"

#define		GB02_CRTC_NUM		6
#define		GB02_CONNECTER_NUM	6
#define		GB02_PLANE_NUM		4
#define		GB_CRTC_NUM		2
#define		GB_CONNECTER_NUM	2
#define		GB_PLANE_NUM		4
#define		FPGA_CRTC_NUM		1
#define		FPGA_CONNECTER_NUM	1
#define		FPGA_PLANE_NUM		1

#define		BOARD_VRAM_SIZE_OFFSET	(0x1000000 + 0x3fc00 + 4 * 8)
#define		BOARD_VRAM_SIZE_MAX	32

struct gb_asic_dev_info {
	enum genbu_asic_type gb_type;
	//unsigned long vram_size;
	unsigned long dc_base_offset;
	unsigned int crtc_step;
	int gpu_reg_bar_id;
	int dc_reg_bar_id;
	int ddr_bar_id;
	int mcu_peri_bar_id;
	int peri_base_bar_id;
	int ipc_bar_id;
	int pci_bar_cnt;
	int crtc_num;
	int connector_num;
	int irq_num;
};

struct firmware_info {
     unsigned int hdmi_firmware_maj;
     unsigned int hdmi_firmware_min;
     unsigned int vga_firmware_maj;
     unsigned int vga_firmware_min;
     unsigned int version_info;
};

struct gb_pci_bar {
	phys_addr_t base;
	void __iomem *mmio;
	resource_size_t len;
};

struct gb_pcie_info {
	struct pci_dev *pdev;
	struct gb_pci_bar pci_bars[PCI_BAR_MAX_CNT];
	int irq_vector_num;
	int msi_enabled;
	int irqs[IRQ_NUM];
	struct gb_asic_dev_info *gb_dev_info;
	struct gb_edma_para edma_para;
	void *vdec0_info;
	void *vdec1_info;
	void *venc_info;
	void *ao_info;
	struct gb_device *gbdev;
	struct dptx* dptx[CONNECTOR_MAX_CNT];
	int board_id;
	struct firmware_info f_info;
};

#define GENBU_FPGA_DEV_INFO \
	.gb_type = GENBU_FPGA, \
	.crtc_step = 0x100000, \
	.ddr_bar_id = 0, \
	.ipc_bar_id = -1, \
	.pci_bar_cnt = 5, \
	.gpu_reg_bar_id = 2, \
	.dc_reg_bar_id = 2, \
	.dc_base_offset = 0x100000, \
	.connector_num = FPGA_CONNECTER_NUM, \
	.crtc_num = FPGA_CRTC_NUM, \
	.irq_num = 16, \

#define GENBU_01_DEV_INFO \
	.gb_type = GENBU_01, \
	.crtc_step = 0x20000, \
	.ddr_bar_id = 0, \
	.ipc_bar_id = 3, \
	.pci_bar_cnt = 4, \
	.gpu_reg_bar_id = 2, \
	.dc_reg_bar_id = 2, \
	.dc_base_offset = 0x100000, \
	.connector_num = GB_CONNECTER_NUM, \
	.crtc_num = GB_CRTC_NUM, \
	.irq_num = 1, \

#define GENBU_02_DEV_INFO \
	.gb_type = GENBU_02, \
	.crtc_step = 0x20000, \
	.ddr_bar_id = 2, \
	.ipc_bar_id = -1, \
	.pci_bar_cnt = 5, \
	.mcu_peri_bar_id = 0, \
	.gpu_reg_bar_id = 1, \
	.dc_reg_bar_id = 1, \
	.peri_base_bar_id = 4, \
	.dc_base_offset = 0x0, \
	.connector_num = GB02_CONNECTER_NUM, \
	.crtc_num = GB02_CRTC_NUM, \
	.irq_num = 8, \

enum genbu_asic_type get_genbu_gb_type(struct gb_asic_dev_info *gb_dev_info);
u64 get_vpu_dynamic_vram_size(void);
u64 get_genbu_vram_size(void);
int get_genbu_gpu_reg_bar_id(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_ddr_bar_id(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_pci_bar_cnt(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_irq_num(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_mcu_peri_bar_id(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_peri_base_bar_id(struct gb_asic_dev_info *gb_dev_info);
u64 get_genbu_gb_reserve_size(struct gb_asic_dev_info *gb_dev_info);
int get_genbu_dc_reg_bar_id(struct gb_asic_dev_info *gb_dev_info);
struct gb_pcie_info *get_pcie_gl_info(void);
int get_genbu_crtc_num(struct gb_asic_dev_info *gb_dev_info);
u64 get_genbu_vram_size_from_board(void);
#endif
