/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef		__GB_DEVICE_H__
#define		__GB_DEVICE_H__
#include <linux/atomic.h>
#include <linux/version.h>
#if KERNEL_VERSION(5, 0, 0) < LINUX_VERSION_CODE
#include <linux/io-pgtable.h>
#endif
#include <linux/spinlock.h>
#include <drm/drm_device.h>
#include <drm/drm_mm.h>
#include <drm/gpu_scheduler.h>
#include "gb_mmu.h"
#include "gb_gem.h"  //include gb_bo.h
#include "gb_pcie_info.h"
//#include "gb_ttm.h"
#include "vpu/vpu_comm/gb_vpu.h"
#include "vpu/vpu_mm/gb_vpu_ttm.h"
#include "gb_ip_offset.h"
#include "gb_ttm.h"
#include "kms/device/gb_ip.h"
#include <generated/uapi/linux/version.h>
#include "mcu_peripherals/gpu_freq/gb_devfreq.h"


struct gb_device;
struct gb_mmu;
struct gb_stage_slot;
struct gb_stage;
//#define GB_PRINT_VERBOSE
//#define DEBUG_FOR_JC_DATA
#define MMU_ENABLED
#define GPU_MIRQ_ENABLED
#define NUM_JOB_SLOTS		3
#define NUM_THREAD_AS		8
#define NUM_JOB_SUBMIT_LIMIT	1
#define GB_HAS_KMS_DEV
#define	GB_AUDIO
//#define	GB_VDEC
//#define	GB_VDEC1
//#define	GB_VENC
//#define	GB_DEC_MODULE
//#define	GB_ENC_MODULE
//#define	GB_DP_MODULE
#define	GB_GPU_MODULE
#ifdef GB_DC_DP
//#define GB_ALLSCREEN
#endif
#define	GB_PERIPHERALS
#define	GB_GPU
#define KB      (1UL<<10)
#define MB      (1UL<<20)
#define GB      (1UL<<30)
#define GB_PCI_NAME		"genbu_pcie"

enum {
	GB_RST_IDLE = 0,
	GB_RST_PENDING = 1,
	GB_RST_HW_STARTING = 2,
	GB_RST_HW_COMPLETE = 3,
};
enum {
	FULL_DDR4_VAL = 0,
	FULL_LPDDR4_VAL,
	PCIE_LPDDR4_VAL,
	PCIE_M6FL8G_LPDDR4_VAL,
	PCIE_HIE1LP4_LPDDR4_VAL,
	PCIE_CQ2040_P21_VAL,
	PCIE_M4HL8G_LPDDR4_VAL = 7,
	CQ2040_MXM_M60_VAL,
	PCIE_C0_200_VAL = 9,
	PCIE_HIE1LP42_LPDDR4_VAL = 11,
};

struct gb_features {
	u16 id;
	u16 revision;

	u64 shader_present;
	u64 tiler_present;
	u64 l2_present;
	u64 stack_present;
	u32 as_present;
	u32 ss_present;

	u32 l2_features;
	u32 tiler_features;
	u32 mem_features;
	u32 mmu_features;
	u32 thread_features;
	u32 max_threads;
	u32 thread_max_workgroup_sz;
	u32 thread_max_barrier_sz;
	u32 coherency_features;
	u32 texture_features[4];
	u32 ss_features[16];

	u32 nr_core_groups;
	u32 heap_memory_growable;
	u32 vpu_bo_save_flag;
};

struct gb_devfreq_slot {
	ktime_t busy_time;
	ktime_t idle_time;
	ktime_t time_last_update;
	bool busy;
};

#define VRAM_ALL_SIZE(x)                (get_genbu_vram_size())

#define GPU_PAGE_SIZE                   (4UL*KB)
#define GPU_PAGE_SHIFT                  (12UL)
#define GPU_PAGE_MASK                   (~(GPU_PAGE_SIZE-1))
#define PG_SIZE_MULTI                   (PAGE_SIZE / GPU_PAGE_SIZE)

// Normal page alloced base.
#define VRAM_FOR_NORMAL_PG_BASE(x) 	(get_genbu_gb_reserve_size(x))
#define GPU_ALL_SIZE(x) 		(VRAM_ALL_SIZE(x) - VRAM_FOR_NORMAL_PG_BASE(x))
#define GPU_PAGE_CNT_IN_CPU_PG_SIZE(x)  (GPU_ALL_SIZE(x) / PAGE_SIZE)
#define GPU_PAGE_CNT_IN_GPU_PG_SIZE(x)  (GPU_ALL_SIZE(x) / GPU_PAGE_SIZE)
#define UNALIGNED_VRAM_SIZE(x)          (GPU_ALL_SIZE(x) - (GPU_PAGE_CNT_IN_CPU_PG_SIZE(x) * PAGE_SIZE))  // Summary the unaliagned size by PAGE_SIZE
#define VRAM_FOR_MMU_PAGE_TOP_BASE(x) 	(MMU_START_ADDR + MMU_LEN)
/* Inside MIPS arch, PAGE_SIZE is 16KB, others' are 4KB */
#define MMU_PG_USG_BITS_LEN(x)          ((GPU_PAGE_CNT_IN_GPU_PG_SIZE(x) + 63UL) / (64UL))
#define VRAM_PG_USG_BITS_LEN(x)         ((GPU_PAGE_CNT_IN_CPU_PG_SIZE(x) + 63UL) / (64UL))

struct gb_page_ctrl {
        spinlock_t ddr_lock;
        atomic_t used_pages;
        atomic_t used_mmu_pages;
        volatile u64 page_highest_addr;
        volatile u64 mmu_lowest_addr;
        volatile u64 *usage;
        volatile u64 *mmu_usage;
};

enum system_working_status {
	HIBERNATE_SLEEP = 0,
	SUSPEND_SLEEP,
	SYSTEM_WORKING,
	SYSTEM_MAX_STATUS,
};

struct gb_device {
	struct device *dev;
	struct drm_device *ddev;
	struct gbdc_device *gbdc_dev;
	struct gb_pcie_info *gb_pcie;
	struct gpu_info *gpu_device_info;
	struct gb02_mcu_peripherals_info gb_peri[PERIPHERALS_DEVICE_NUM];

	void __iomem *gpu_reg_base;
	struct regulator *regulator;
	struct reset_control *rstc;

	struct gb_features features;

	struct version_str version;

	spinlock_t as_lock;
	spinlock_t mmu_hw_lock;
	struct mutex mmu_as_lock;
	unsigned long as_in_use_mask;
	unsigned long as_alloc_mask;
	struct list_head as_lru_list;

	struct gb_stage_slot *ss;

	struct gb_stage *stages[NUM_JOB_SLOTS];
	struct list_head scheduled_stages;

	struct mutex sched_lock;
	struct job_submit_limit {
		spinlock_t lock;
		struct semaphore job_semaphore;
		unsigned hw_job_limit;
	} jsl;
	struct {
		struct spsc_queue       remain_job_queue;
		void *resume_job[NUM_THREAD_AS][NUM_JOB_SLOTS];
		struct workqueue_struct *reset_workq;
		struct work_struct work;
		atomic_t pending;
	} reset;

	struct gb_file_priv *priv[NUM_THREAD_AS];
	struct mutex gb_file_priv_lock;

	spinlock_t hw_irq_lock;

	struct {
		struct gb_devfreq *devfreq;
		struct thermal_cooling_device *cooling;
		unsigned long cur_freq;
		unsigned long cur_volt;
		struct gb_devfreq_slot slot[NUM_JOB_SLOTS];
	} devfreq;

	struct gb_page_ctrl gb_page_ctrl;
	struct gb_page *global_gb_page;
	struct gb_page *mmu_top_page;
	//struct gb_asic_dev_info *dev_info;
	struct mutex vram_mutex;
	struct gb_mmu_mode *mmu_mode;

	struct gb_mm_info gb_mm;

	struct gbip_offset_base  ipoffset_base;

	//for vpu
	struct rw_semaphore		exclusive_lock;
	struct rw_semaphore		mclk_lock;
	atomic_t used_dec0_cnt;
	atomic_t used_dec1_cnt;
	struct list_head vpu_bo_list_head;
	//struct gb_gem			gem;
	atomic64_t			vram_usage;

		/* memory management */
	void			*vpu_ttm;
	struct dec_vcmd		init_dvcmd;
	struct enc_vcmd		init_evcmd;
	void  *gbdec_data;
	void  *gbenc_data;
	//u64 vram_pin_size;

	struct gb_pcie_info pcie_info;

	struct mutex v2v_mutex;
	struct mutex rb_mutex;
	enum system_working_status sleep_status;
	atomic_t perf_time_per;
	struct timer_list perf_timer;
};

struct gb_list_node {
	struct drm_file *drm_file;
	struct gb_file_priv *gb_priv;
	struct list_head list;
	struct rb_root gpu_root_node;
	struct rb_root cpu_root_node;
};

struct gb_file_priv {
	struct gb_device *gbdev;

	struct drm_sched_entity sched_entity[NUM_JOB_SLOTS];
	struct task_struct *pcb;
	struct drm_file *drm_file;

	struct mutex perfcnt_lock;
	struct gb_list_node list_node;
	int dec_id;
	int cmdbuf_type;
/*nomal vram=1; vcmd buff = 2; vcmd status = 3 */
	__u32  bo_type;
	void *dec_priv;
	void *dec1_priv;
	void *enc_priv;
};
static inline struct gb_device *to_gb_device(struct drm_device *ddev)
{
	return ddev->dev_private;
}
void gbip_offset_init(struct gb_device *gbdev);
int gb_device_init(struct gb_device *gbdev, struct gb_asic_dev_info *gb_dev_info);
void gb_device_release(struct gb_device *gbdev);
void gb_device_reset(struct gb_device *gbdev);

int gb_device_resume(struct device *dev);
int gb_device_suspend(struct device *dev);
struct gb_file_priv* get_gl_perf_priv(void);
void gb_perf_timer_func(void);
void gb_perf_cnt_priv_init(struct gb_device *gbdev);

const char *gb_exception_name(struct gb_device *gbdev, u32 exception_code);

#define reg_write(dev, reg, data) writel(data, dev->gpu_reg_base + reg)
#define reg_read(dev, reg) readl(dev->gpu_reg_base + reg)

#define gpu_write(dev, reg, data) writel(data, dev->gpu_reg_base + reg)
#define gpu_read(dev, reg) readl(dev->gpu_reg_base + reg)

#define stage_write(dev, reg, data) writel(data, dev->gpu_reg_base + (reg))
#define stage_read(dev, reg) readl(dev->gpu_reg_base + (reg))

#define mmu_write(dev, reg, data) writel(data, dev->gpu_reg_base + reg)
#define mmu_read(dev, reg) readl(dev->gpu_reg_base + reg)

#endif
