/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/module.h>
#include <linux/pci.h>
#include <linux/pagemap.h>
#include <linux/pm_runtime.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/version.h>
#include <linux/miscdevice.h>
#include <linux/platform_device.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 12, 0))
#include <linux/timekeeping32.h>
#else
#include <linux/timekeeping.h>
#endif
#include <drm/drm_drv.h>
#include <drm/drm_ioctl.h>
#include <drm/drm_syncobj.h>
#include <drm/drm_utils.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 8, 0)
#include <drm/drm_pci.h>
#endif
#include <drm/drm_auth.h>
#include <drm/drm_crtc_helper.h>
#include <drm/drm_fb_helper.h>
#include <drm/drm_atomic_helper.h>
#include <drm/drm_gem.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0))
#include <linux/dma-resv.h>
#else
#include <linux/reservation.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
#include <drm/drm_aperture.h>
#endif
#include <drm/drm_file.h>
#if (defined CONFIG_CENTOS && !defined SYS_CENTOS7_COMPILE_ENV) \
        || LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)
#include <drm/drm_probe_helper.h>
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
#include <drm/ttm/ttm_bo_driver.h>
#else
#include <drm/ttm/ttm_bo.h>
#endif

#include "vpu/vpu_mm/gb_vpu_gem_comm.h"
#include "vpu/vpu_mm/gb_vpu_gem.h"
#include "gb_device.h"
#include "gb_regs.h"
#include "gb_gem.h"
#include "gb_mmu.h"
#include "gb_stage.h"
#include "gb_gpu.h"
#include "gb_pcie_info.h"
#include "gb_pages_alloc.h"
#include "kms/gbdc_mm.h"
#include "kms/gb_kms.h"
#include "kms/gbdc_crtc.h"
#include "gb_gpu_irq.h"
#include "gb_osl.h"
#ifdef	GB_AUDIO
#include "audio/local.h"
#include "audio/gb02_fpga_v2vdma.h"
#include "audio/i2s_platform.h"
#include "audio/gb_snd_codec.h"
#include "audio/gb_audio_procfs.h"
#endif
#include "ip/gb_hdmac.h"
#include "ip/gb_dp.h"
#include "ip/gb_v2vdma.h"
#include "common/gb_common.h"
#include "common/gb_gpuinfo.h"
#include "common/xt.h"
#include "common/gb_irq.h"
#include "gb_ttm.h"
#include "gpu_test/gb_easy_shell.h"
#include "gpu_test/gb_mem_tool.h"
#include "kms/device/gbdc_ops.h"
#include "kms/device/gb_device_ctrl.h"
#include "vpu/vpu_dec/vpu_vcmd.h"
#include "vpu/vpu_dec1/vpu_vcmd.h"
#include "vpu/vpu_enc/vpu_vc8000E_vcmd.h"
#include "gb_firmware_upgrade.h"
#include "gb_debugfs.h"
#include "mcu_peripherals/gpu_freq/gpu_freq.h"
#include "common/gb02_pcie_resizebar.h"
#include "common/gb_kernel_ver.h"
#include "gbdc_infinity.h"

#define DRIVER_NAME "genbu"
#define DRIVER_DESC "GB DRM Driver"
#define DRIVER_DATE "2021-0901"
#define DRIVER_MAJOR 0
#define DRIVER_MINOR 0
#define DRIVER_PATCHLEVEL 0

extern bool gb_rb_tree;
extern bool gb_time_debug;
extern int job_timeout_ms;
extern bool perf_cnt;
extern bool performance2_set;
extern int performance_ms;

unsigned long gBaseHdwr; /* PCI base register address (Hardware address) */
unsigned long gBaseDDRHw;        /* PCI base register address (memalloc) */
unsigned long gBaseDDRLen;
unsigned long gBaseSwitch;
unsigned long gBasePll;
u32  gBaseLen;                   /* Base register address Length */

u8 *hd_va;
u8 *ddr_va;
struct gb_device *gl_gbdev;
extern int gbdc_load(struct drm_device *drm, unsigned long flags);
#if !(defined CONFIG_CENTOS || defined SYS_CENTOS7_9_2009 || \
	defined SYS_CENTOS7_COMPILE_ENV) && \
	LINUX_VERSION_CODE < KERNEL_VERSION(4, 11, 0)
extern int gbdc_unload(struct drm_device *dev);
#else
extern void gbdc_unload(struct drm_device *dev);
#endif

static int gb_ioctl_get_param(struct drm_device *ddev, void *data, struct drm_file *file)
{
	struct drm_gb_get_param *param = data;
	struct gb_device *gbdev = ddev->dev_private;

	if (param->pad != 0)
		return -EINVAL;

#define GB_FEATURE(name, member)			\
	case DRM_GB_PARAM_ ## name:		\
		param->value = gbdev->features.member;	\
		break
#define GB_FEATURE_ARRAY(name, member, max)			\
	case DRM_GB_PARAM_ ## name ## 0 ...			\
		DRM_GB_PARAM_ ## name ## max:			\
		param->value = gbdev->features.member[param->param -	\
			DRM_GB_PARAM_ ## name ## 0];		\
		break
	switch (param->param) {
		GB_FEATURE(GPU_PROD_ID, id);
		GB_FEATURE(GPU_REVISION, revision);

		GB_FEATURE(SHADER_PRESENT, shader_present);
		GB_FEATURE(TILER_PRESENT, tiler_present);
		GB_FEATURE(L2_PRESENT, l2_present);
		GB_FEATURE(STACK_PRESENT, stack_present);
		GB_FEATURE(AS_PRESENT, as_present);
		GB_FEATURE(SS_PRESENT, ss_present);
		GB_FEATURE(L2_FEATURES, l2_features);
		GB_FEATURE(TILER_FEATURES, tiler_features);
		GB_FEATURE(MEM_FEATURES, mem_features);
		GB_FEATURE(MMU_FEATURES, mmu_features);
		GB_FEATURE(THREAD_FEATURES, thread_features);
		GB_FEATURE(MAX_THREADS, max_threads);
		GB_FEATURE(THREAD_MAX_WORKGROUP_SZ,
				thread_max_workgroup_sz);
		GB_FEATURE(THREAD_MAX_BARRIER_SZ,
				thread_max_barrier_sz);
		GB_FEATURE(COHERENCY_FEATURES, coherency_features);
		GB_FEATURE_ARRAY(TEXTURE_FEATURES, texture_features, 3);
		GB_FEATURE_ARRAY(SS_FEATURES, ss_features, 15);
		GB_FEATURE(NR_CORE_GROUPS, nr_core_groups);
		GB_FEATURE(GROW_HEAP_MEMORY, heap_memory_growable);
		GB_FEATURE(VPU_BO_SAVE_FLAG, vpu_bo_save_flag);
		default:
			return -EINVAL;

	}

	return 0;
}

struct list_head gb_list_head = {0};

void list_node_release(struct gb_file_priv *gb_priv)
{
	struct gb_device *gbdev;
	struct drm_file *drm_file;
	struct gb_list_node *task_list_node_tmp, *tmp;

	if (!gb_priv) {
		gb_printf(KERN_ERR, "%s input gb_priv is null \n", __func__);
		return;
	}

	gbdev = gb_priv->gbdev;
	if (!gbdev) {
		gb_printf(KERN_ERR, "%s input gbdev is null \n", __func__);
		return;
	}

	drm_file = gb_priv->drm_file;
	list_for_each_entry_safe(task_list_node_tmp, tmp, &gb_list_head, list) {
		if (task_list_node_tmp->drm_file == drm_file) {
			list_del_init(&task_list_node_tmp->list);
			break;
		}
	}
}

/*
 * mode: 0 - gpu_va  1 - cpu_va
 */
struct rb_root *gb_get_rb_root(struct gb_file_priv *gb_priv, int mode)
{
	struct drm_file *drm_file;
	struct gb_list_node *gb_list_node_tmp, *tmp;
	struct gb_device *gbdev;

	if (!gb_priv) {
		gb_printf(KERN_INFO, "%s input gb_priv  == NULL\n", __func__);
		return NULL;
	}

	gbdev = gb_priv->gbdev;
	if (!gbdev) {
		gb_printf(KERN_INFO, "%s input gbdev  == NULL\n", __func__);
		return NULL;
	}

	drm_file = gb_priv->drm_file;
	list_for_each_entry_safe(gb_list_node_tmp, tmp, &gb_list_head, list) {
		if (gb_list_node_tmp->drm_file == drm_file)
			break;
	}

	if (mode)
		return &gb_list_node_tmp->cpu_root_node;
	else
		return &gb_list_node_tmp->gpu_root_node;

	gb_printf(KERN_ERR, "%s get mode:%d rb_root faild,drm_file:0x%llx\n",
			__func__, mode, (unsigned long long)drm_file);
	return NULL;
}



/*
*	compare_iovaddr -- compare the double gpu_va addr
*	@gpu_va: compare gpu_va in rb_tree
*	@iovaddr: rb_node gpu_va init data
*	@size: rb_node gpu_va init size data
*	return DOMAIN_LEFT intput gpu_va is in the rb_node left,
*		   DOMAIN_RIGHT intput gpu_va is in the rb_node right,
*		   IN_DOMAIN intput gpu_va is in the rb_node,
*	       0 is success
*/
int compare_iovaddr(__u64 gpu_va, __u64 iovaddr, __u32 size)
{
	if (iovaddr || gpu_va || size) {
		//gb_printf(KERN_INFO, "%s gpu_va:0x%llx iovaddr:0x%llx size:0x%x\n",
		//		__func__, (long long)gpu_va, (long long)iovaddr, size);
		if (gpu_va < iovaddr)
			/*domain left*/
			return DOMAIN_LEFT;
		else if (gpu_va >= (iovaddr + size))
			/*domain right*/
			return DOMAIN_RIGHT;
		else if (gpu_va >= iovaddr && gpu_va < (iovaddr + size))
			/*in domain*/
			return IN_DOMAIN;
	}
	/*error domain*/
	return -EINVAL;
}

/*
*	compare_cpuvaddr -- compare the double cpu_va addr
*	@cpu_va: compare cpu_va in rb_tree
*	@va_start: mmap cpu addr start
*	@va_end: mmap cpu addr end
*	return DOMAIN_LEFT intput cpu_va is in the rb_node left,
*		   DOMAIN_RIGHT intput gpu_va is in the rb_node right,
*		   IN_DOMAIN intput gpu_va is in the rb_node,
*	       0 is success
*/
int compare_cpuvaddr(__u64 cpu_va, __u64 va_start, __u64 va_end)
{
	if (cpu_va || va_start || va_end) {
		//gb_printf(KERN_INFO, "%s cpu_va:0x%llx va_start:0x%llx va_end:0x%llx\n",
		//	__func__, cpu_va, va_start, va_end);
		if (cpu_va < va_start)
			/*domain left*/
			return DOMAIN_LEFT;
		else if (cpu_va >= va_end)
			/*domain right*/
			return DOMAIN_RIGHT;
		else if (cpu_va >= va_start && cpu_va < va_end)
			/*in domain*/
			return IN_DOMAIN;
	}
	/*error domain*/
	return -EINVAL;
}

struct gb_rb_node *gb_bo_rb_init(void)
{
	struct gb_rb_node *base_rb_node;
	base_rb_node = kmalloc(sizeof(struct gb_rb_node), GFP_KERNEL);
	if (!base_rb_node) {
		gb_printf(KERN_ERR, "%s kmalloc base_rb_node error \n", __func__);
		return NULL;
	}
	return base_rb_node;
}

/*
 * Clean cpu_va rb_tree before node insert
 *
 * Dirver would not know user application release vma space, rb
 * node can never be deleted on time. This may cause spatial
 * overlap among nodes in rb_tree.
 *
 * We need to reclaim all node that may overlap with the new node
 *
 * All we have to ensure than the same tree can only hit on one
 * rb_node per address.
 */
int gb_rb_clean(struct rb_root *rb_root, struct gb_rb_node *new)
{
	struct rb_node	*node;
	unsigned long vm_start, vm_end;

	if (!rb_root || !new) {
		gb_printf(KERN_ERR, "%s rb_root(%#llx) is null or node(%#llx) is null\n",
			__func__, (long long)rb_root, (long long)new);
		return -EINVAL;
	}

	node = rb_root->rb_node;
	vm_start = new->gb_gem_bo->vm_info.vm_start;
	vm_end = new->gb_gem_bo->vm_info.vm_end;

	while (node) {
		struct gb_rb_node *tmp = container_of(node,
				struct gb_rb_node, gb_node);
		int result;
		result = compare_cpuvaddr(vm_start,
					tmp->gb_gem_bo->vm_info.vm_start,
					tmp->gb_gem_bo->vm_info.vm_end);

		if (result == DOMAIN_LEFT)
			node = node->rb_left;
		else if (result == DOMAIN_RIGHT)
			node = node->rb_right;
		else if (result == IN_DOMAIN) {
			/*found it, tmp need to release*/
			rb_erase(&tmp->gb_node, rb_root);
			kvfree(tmp);
			node = rb_root->rb_node;
		}
	}
	return 0;
}

int gb_bo_rb_insert(struct rb_root *rb_root, struct gb_rb_node *data, int mode)
{
	struct rb_node **new = &(rb_root->rb_node), *parent = NULL;
	if (!rb_root || !data)
		return -EINVAL;

	while (*new) {
		struct gb_rb_node *this = container_of(*new, struct gb_rb_node, gb_node);
		int result;
		if (mode)
			result = compare_cpuvaddr(data->gb_gem_bo->vm_info.vm_start,
					this->gb_gem_bo->vm_info.vm_start,
					this->gb_gem_bo->vm_info.vm_end);
		else
			result = compare_iovaddr(data->gb_gem_bo->vm_info.iovaddr,
					this->gb_gem_bo->vm_info.iovaddr,
					this->gb_gem_bo->vm_info.size);
		parent = *new;
		if (result == DOMAIN_LEFT) {
			new = &((*new)->rb_left);
		} else if (result == DOMAIN_RIGHT) {
			new = &((*new)->rb_right);
		} else if (result == IN_DOMAIN) {
			gb_printf(KERN_INFO, "%s 0x%lx is exist bypass to insert \n",
				__func__, data->gb_gem_bo->vm_info.iovaddr);
			return 0;
		} else {
			gb_printf(KERN_ERR, "%s 0x%lx is compare error,ret:%d\n",
				__func__, data->gb_gem_bo->vm_info.iovaddr, result);
			return result;
		}
	}
	rb_link_node(&data->gb_node, parent, new);
	rb_insert_color(&data->gb_node, rb_root);
	gb_printf(KERN_INFO, "%s insert 0x%lx\n", __func__,
		data->gb_gem_bo->vm_info.iovaddr);

	return 0;
}

struct gb_rb_node *gb_bo_rb_search(struct rb_root *rb_root, __u64 vaddr, int mode)
{
	struct rb_node	*node;

	if (!rb_root || !vaddr) {
		gb_printf(KERN_INFO, "%s rb_root is null rb_root:0x%llx vaddr:0x%llx mode:0x%d\n",
			__func__, (long long)rb_root, (long long)vaddr, mode);
		return NULL;
	}

	node = rb_root->rb_node;

	while (node) {
		struct gb_rb_node *data = container_of(node, struct gb_rb_node, gb_node);
		int result;
		if (!data->gb_gem_bo)
			return NULL;

		if (mode)
			result = compare_cpuvaddr(vaddr,
				data->gb_gem_bo->vm_info.vm_start,
				data->gb_gem_bo->vm_info.vm_end);
		else
			result = compare_iovaddr(vaddr,
				data->gb_gem_bo->vm_info.iovaddr,
				data->gb_gem_bo->vm_info.size);

		if (result == DOMAIN_LEFT)
			node = node->rb_left;
		else if (result == DOMAIN_RIGHT)
			node = node->rb_right;
		else if (result == IN_DOMAIN)
			return data;
	}

	return NULL;
}

void gb_bo_rb_create(struct gb_gem_object *bo, struct drm_gb_create_bo *args)
{
	int ret = 0;
	struct gb_rb_node *gb_rb_data;

	if (!gb_rb_tree)
		return;
	mutex_lock(&bo->gbdev->rb_mutex);
	/* init vm_info */
	gb_rb_data = gb_bo_rb_init();
	if (gb_rb_data) {
		bo->vm_info.iovaddr = bo->base.iovaddr;
		bo->vm_info.size = args->size;
		bo->vm_info.vm_start = 0;
		bo->vm_info.vm_end = 0;

		/* init gb_rb_data and insert it */
		gb_rb_data->gb_gem_bo = bo;
		ret = gb_bo_rb_insert(gb_get_rb_root(bo->private_data,
			GPU_ADDR_MODE), gb_rb_data, GPU_ADDR_MODE);
		if (ret)
			gb_printf(KERN_ERR, "%s insert erro %d\n", __func__, ret);
	} else {
		gb_printf(KERN_ERR, "%s init erro %pk\n", __func__, gb_rb_data);
	}
	gb_printf(KERN_INFO, "%s bo:0x%llx phy offset: 0x%llx",
		__func__, (unsigned long long)bo, args->offset_pa);
	if (ret)
		pr_err("%s bo rb create faild %d\n", __func__, ret);
	mutex_unlock(&bo->gbdev->rb_mutex);
}

int gb_bo_rb_mmap(struct gb_gem_object *bo, struct vm_area_struct *vma)
{
	int ret = 0;
	struct rb_root *gpu_rb_root;
	struct rb_root *cpu_rb_root;
	struct gb_rb_node *gb_bo_rb_node;
	struct gb_rb_node *gb_rb_cpu_node;
	struct gb_rb_node *data;

	if (bo->vm_info.iovaddr) {
		/* update vm_info */
		gpu_rb_root = gb_get_rb_root(bo->private_data, GPU_ADDR_MODE);
		if (!gpu_rb_root) {
			gb_printf(KERN_ERR, "%s get gpu rb root error\n",
				__func__);
			return -EINVAL;
		}

		gb_bo_rb_node = gb_bo_rb_search(gpu_rb_root,
							bo->vm_info.iovaddr, GPU_ADDR_MODE);
		if (gb_bo_rb_node) {
			/*
			 * Since same bo only has one rb node in cpu addr tree
			 * It is necessary to check if rb node exists
			 * Next, check if rb node needs to be updated
			 */

			cpu_rb_root = gb_get_rb_root(bo->private_data,
							CPU_ADDR_MODE);
			if (!cpu_rb_root) {
				gb_printf(KERN_ERR, "%s get cpu rb root error\n", __func__);
				return -EINVAL;
			}

			/*
			 * Yes I know this is ugly, but what can I do.
			 * User task direct call drmUnmap to unmap bo, gpu driver would
			 * never know.
			 * File_operation doesn't have munmap, I have to consider the
			 * situation rb_node is going to insert has same vm_start with
			 * rb_node from cpu_va rb_tree.
			 *
			 * It is necessary to check if rb_node vm_start conflict before
			 * node actually insert. If does check whether rb node vm_info
			 * needs to be updated. If does delete the node then insert a
			 * new one.
			 */
			data = gb_bo_rb_search(cpu_rb_root,
				gb_bo_rb_node->gb_gem_bo->vm_info.vm_start,
				CPU_ADDR_MODE);

			if (data) {
				if (gb_bo_rb_node->gb_gem_bo->vm_info.vm_start == vma->vm_start
					&& gb_bo_rb_node->gb_gem_bo->vm_info.vm_end == vma->vm_end
					&& gb_bo_rb_node->gb_gem_bo == bo) {
					/* node exist and works fine, DO NOTHING */
					return ret;
				} else {
					/* rb node need to update, delete it */
					gb_printf(KERN_INFO, "%s: vminfo need to update, rb_node:%#llx, bo:%#llx\n",
							__func__, (long long)data->gb_gem_bo, (long long)data->gb_gem_bo);
					rb_erase(&data->gb_node, cpu_rb_root);
					kvfree(data);
				}
			}

			gb_bo_rb_node->gb_gem_bo->vm_info.vm_start = vma->vm_start;
			gb_bo_rb_node->gb_gem_bo->vm_info.vm_end = vma->vm_end;

			gb_rb_cpu_node = gb_bo_rb_init();
			gb_rb_cpu_node->gb_gem_bo =  gb_bo_rb_node->gb_gem_bo;

			/* pre processing before insertion */
			ret = gb_rb_clean(cpu_rb_root, gb_rb_cpu_node);
			if (ret) {
				gb_printf(KERN_ERR, "in %s clean the multi tree faild %d\n", __func__, ret);
				return ret;
			}

			ret = gb_bo_rb_insert(cpu_rb_root, gb_rb_cpu_node, CPU_ADDR_MODE);
			if (ret) {
				gb_printf(KERN_ERR, "in %s something goes really wrong, rb_node insert failed\n", __func__);
				kvfree(gb_rb_cpu_node);
				/* cpu_va do not has this bo rb node */
				return -EINVAL;
			}
		}
	}

	return ret;
}

void gb_bo_rb_delete(struct rb_root *rb_root, __u64 vaddr, int mode)
{
	struct gb_rb_node *data = NULL;
	if (!rb_root || !vaddr)
		return;
	data = gb_bo_rb_search(rb_root, vaddr, mode);
	if (data) {
		gb_printf(KERN_INFO, "%s delete 0x%llx\n", __func__, (long long)vaddr);
		rb_erase(&data->gb_node, rb_root);
		kvfree(data);
	}
}

void gb_rb_delete(struct gb_gem_object *bo, struct gb_file_priv *gb_priv)
{
	struct rb_root *rb_root = NULL;
	if (!gb_rb_tree)
		return;
		/* delete bo_rb_node */
	if (!bo || !gb_priv)
		return;
	mutex_lock(&bo->gbdev->rb_mutex);
	rb_root = gb_get_rb_root(gb_priv, GPU_ADDR_MODE);
	if (rb_root) {
		gb_bo_rb_delete(rb_root, bo->base.iovaddr, GPU_ADDR_MODE);
	}

	rb_root = gb_get_rb_root(gb_priv, CPU_ADDR_MODE);
	if (rb_root) {
		gb_bo_rb_delete(rb_root, bo->vm_info.vm_start, CPU_ADDR_MODE);
	}
	mutex_unlock(&bo->gbdev->rb_mutex);

}

#ifdef GB_BO_RB_PRINTK
void gb_bo_rb_printk(struct rb_root *rb_root)
{
	struct rb_node *node;
	struct gb_rb_node *gb_tmp;
	for (node = rb_first(rb_root); node; node = rb_next(node)) {
		gb_tmp = rb_entry(node, struct gb_rb_node, gb_node);
		gb_printf(KERN_DEBUG, "iovaddr:0x%lx,",
			gb_tmp->gb_gem_bo->vm_info.iovaddr);
		gb_printf(KERN_DEBUG, "vm_start:0x%lx,",
			gb_tmp->gb_gem_bo->vm_info.vm_start);
		gb_printf(KERN_DEBUG, "vm_end:0x%lx\n",
			gb_tmp->gb_gem_bo->vm_info.vm_end);
	}
}
#endif

int gb_ioctl_create_bo(struct drm_device *dev, void *data,
		struct drm_file *file)
{
	struct gb_gem_object *bo;
	int domain = 0;
	struct drm_gb_create_bo *args = data;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	struct timespec t1, t2;
#else
	struct timespec64 t1, t2;
#endif
	long time_use1;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	getnstimeofday(&t1);
#else
	ktime_get_real_ts64(&t1);
#endif

	if (!args->size ||
	    (args->flags & ~GB_VALID_FLAG))
		return -EINVAL;

	if ((args->flags & GB_BO_HEAP) &&
	    !(args->flags & GB_BO_NOEXEC))
		return -EINVAL;

	if (args->domain & GPU_DOMAIN_FLAG)
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 10, 0)
		domain = TTM_PL_FLAG_VRAM;
#else
		domain = TTM_PL_VRAM;
#endif
	else if (args->domain & VPU_DOMAIN_FLAG)
		domain = GB_VPU_DOMAIN;

	if (!!(args->flags & GB_BO_FB))
		domain = GB_VPU_DOMAIN;

	bo = gb_gpu_gem_ttm_create_with_handle(file, dev, args->size, domain,
			args->flags, false, &args->handle);

	if (IS_ERR(bo))
		return PTR_ERR(bo);

	args->offset_va = bo->base.iovaddr;
	args->offset_pa = bo->base.iopaddr;

	if (gb_rb_tree)
		gb_bo_rb_create(bo, args);

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	getnstimeofday(&t2);
#else
	ktime_get_real_ts64(&t2);
#endif
	time_use1 = (t2.tv_sec - t1.tv_sec) * 1000000 +
		(t2.tv_nsec - t1.tv_nsec)/1000;
	if (gb_time_debug) {
		pr_info("[%s,pid:%d] %s:cost time: %ld\n",
			current->comm,current->pid,
			 __func__, time_use1);
	}

	return 0;
}
#ifdef GB_ALLSCREEN
int gb_ioctl_create_fb_bo(struct drm_device *dev, void *data,
		struct drm_file *file)
{
	int ret = 0;
	int crtc_id;
	struct drm_gb_create_fb *args = data;
	struct drm_gb_create_bo *bo = &args->bo;
	struct gb_fb_info *fb_base_info = NULL;
	struct gb_gem_object *gb_bo = NULL;
	struct drm_gem_object *obj = NULL;
	int fb_type = args->fb_type;

	fb_base_info = gb_get_fb_info_point();
	crtc_id = gbdc_check_first_dc_num();
	gb_printf(KERN_INFO, "[%s]crtc_id is %d, size %x", __func__, crtc_id, bo->size);
	ret = gb_ioctl_create_bo(dev, (void *)bo, file);
	if (ret) {
		gb_printf(KERN_ERR, "[%s] create new fb bo failed", __func__);
		return ret;
	}

	obj = idr_find(&file->object_idr, bo->handle);
    if (!obj) {
        ret = -ENOENT;
        return ret;
    }

	gb_bo = gem_ob_to_gb_gem(obj);

	if (gb_bo->is_fb_bo) {
		atomic_set(&fb_base_info->info[fb_type].flag, 1);
		fb_base_info->info[fb_type].fb_addr = gb_bo->base.pages[0];
		fb_base_info->info[fb_type].size = bo->size;
		fb_base_info->info[fb_type].height = fb_base_info->crtc_plane[crtc_id].height;
		fb_base_info->info[fb_type].width = fb_base_info->crtc_plane[crtc_id].width;
	}
	return ret;
}
#endif

static int gb_lookup_bos(struct drm_device *dev, struct drm_file *file_priv,
			 struct drm_gb_submit *args, struct gb_stage *stage)
{

#if (KERNEL_VERSION(5, 2, 0) >= LINUX_VERSION_CODE)
	int ret = 0;
	u32 *handles;
	struct drm_gem_object **objs, *obj;
	int i;
#endif
	if (!args->bo_handle_count)
		return 0;

	stage->bo_count = args->bo_handle_count;

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	stage->implicit_fences = kvmalloc_array(stage->bo_count,
				  sizeof(struct dma_fence *),
				  GFP_KERNEL | __GFP_ZERO);
	if (!stage->implicit_fences)
		return -ENOMEM;
#endif

#if (KERNEL_VERSION(5, 2, 0) >= LINUX_VERSION_CODE)
	objs = kvmalloc_array(stage->bo_count, sizeof(struct drm_gem_object *),
		GFP_KERNEL | __GFP_ZERO);
	if (!objs)
		return -ENOMEM;

	handles = kvmalloc_array(stage->bo_count, sizeof(u32), GFP_KERNEL);
	if (!handles) {
		ret = -ENOMEM;
		kvfree(objs);
		return ret;
	}

	if (copy_from_user(handles, (void __user *)(uintptr_t)args->bo_handles, stage->bo_count * sizeof(u32))) {
		ret = -EFAULT;
		DRM_DEBUG("Failed to copy in GEM handles\n");
		kvfree(objs);
		kvfree(handles);
		return ret;
	}

	spin_lock(&file_priv->table_lock);

	for (i = 0; i < stage->bo_count; i++) {
		/* Check if we currently have a reference on the object */
		obj = idr_find(&file_priv->object_idr, handles[i]);
		if (!obj) {
			gb_printf(KERN_ERR,"[%s] cant find handle %d\n", __func__, handles[i]);
			ret = -ENOENT;
			break;
		}
		drm_gem_object_get(obj);
		objs[i] = obj;
	}
	spin_unlock(&file_priv->table_lock);
	stage->bos = objs;

	if (handles) {
		kvfree(handles);
	}

	return ret;
#else
	return drm_gem_objects_lookup(file_priv,
				      (void __user *)(uintptr_t)args->bo_handles,
				      stage->bo_count, &stage->bos);
#endif
}

static int gb_copy_in_sync(struct drm_device *dev, struct drm_file *file_priv,
		           struct drm_gb_submit *args, struct gb_stage *stage)
{
	u32 *handles = NULL;
	int ret = 0;
	int i;

	stage->in_fence_count = args->in_sync_count;
	if (!stage->in_fence_count)
		return 0;

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	stage->in_fences = kvmalloc_array(stage->in_fence_count,
					sizeof(struct dma_fence *),
					GFP_KERNEL | __GFP_ZERO);
	if (!stage->in_fences) {
		DRM_DEBUG("Failed to allocate stage in fences\n");
		return -ENOMEM;
	}
#endif

	handles = kvmalloc_array(stage->in_fence_count, sizeof(u32), GFP_KERNEL);
	if (!handles) {
		DRM_DEBUG("Failed to allocate incoming syncobj handles\n");
		kvfree(stage->in_fences);
		return -ENOMEM;
	}

	if (copy_from_user(handles,
			   (void __user *)(uintptr_t)args->in_syncs,
			   stage->in_fence_count * sizeof(u32))) {
		ret = -EFAULT;
		DRM_DEBUG("Failed to copy in syncobj handles\n");
		goto fail_res;
	}

	for (i = 0; i < stage->in_fence_count; i++) {

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
#if KERNEL_VERSION (5,0,0) < LINUX_VERSION_CODE
		ret = drm_syncobj_find_fence(file_priv, handles[i],
		 0, 0, &stage->in_fences[i]);
#else
		ret = drm_syncobj_find_fence(file_priv, handles[i],
		 			&stage->in_fences[i]);
#endif
		if (ret) {
			gb_printf(KERN_ERR, "%s %d find usr handles[%d]:%d err,ret:%d\n",
				__func__, __LINE__, i, handles[i], ret);
			goto fail_fence;
		}
#endif

#if KERNEL_VERSION (6, 6, 0) < LINUX_VERSION_CODE
		ret = drm_sched_job_add_syncobj_dependency(&stage->base, file_priv,
							   handles[i], 0);
		if (ret) {
			gb_printf(KERN_ERR, "%s %d find usr handles[%d]:%d err,ret:%d\n",
				__func__, __LINE__, i, handles[i], ret);
			goto fail_fence;
		}
#endif
	}

	kvfree(handles);
	return 0;

fail_fence:
	while (--i >= 0)
		dma_fence_put(stage->in_fences[i]);
fail_res:
	kvfree(stage->in_fences);
	kvfree(handles);
	return ret;
}

static int gb_ioctl_submit(struct drm_device *dev, void *data,
		struct drm_file *file)
{
	struct gb_device *gbdev = dev->dev_private;
	struct drm_gb_submit *args = (struct drm_gb_submit *)data;
	struct drm_syncobj *sync_out = NULL;
	struct gb_stage *stage;
	int ret = 0, slot;
	int i;
	char time_buf[GB_SYS_TIME_LEN];

	gb_printf(KERN_DEBUG, "%s:%d----handle =0x%llx--\n",
		__func__, __LINE__, (long long)args->bo_handles);

	if (!args->sc && !args->bo_handle_count) {
		ret = gbvpu_submit_ioctl(file, args->bo_handles);
		return ret;
	}

	if (!args->sc)
		return -EINVAL;

	if (args->slot_req && args->slot_req != GB_SD_REQ_FS)
		return -EINVAL;

	if (args->out_sync > 0) {
		sync_out = drm_syncobj_find(file, args->out_sync);
		if (!sync_out)
			return -ENODEV;
	}

	stage = kvzalloc(sizeof(*stage), GFP_KERNEL);
	if (!stage) {
		ret = -ENOMEM;
		goto fail_out_sync;
	}

	kref_init(&stage->refcount);

	if (gb_time_debug) {
		sys_printtime(time_buf);
		pr_info("[%s,pid:%d] %s %s --stage:%llx\n",
			current->comm,current->pid,
			__func__, time_buf, (long long)stage);
	}

	stage->gbdev = gbdev;
	stage->sc = args->sc;
	stage->slot_req = args->slot_req;
	stage->reset_time = 0;
	stage->flush_id = gb_gpu_get_latest_flush_id(gbdev);
	stage->file_priv = file->driver_priv;
	stage->user_pid = current->pid;

	slot = gb_stage_get_slot(stage);
	if (gb_time_debug) {
		sys_printtime(time_buf);
		pr_info("[%s,pid:%d] %s %s --stage:%llx slot:%d\n",
			current->comm,current->pid,
			__func__, time_buf, (long long)stage, slot);
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
	ret = drm_sched_job_init(&stage->base, &stage->file_priv->sched_entity[slot], NULL);
	if (ret) {
		gb_printf(KERN_ERR, "%s: drm_sched_job_init ret %d\n",
				__func__, ret);
		goto out_put_job;
	}
#endif

	ret = gb_copy_in_sync(dev, file, args, stage);
	if (ret) {
		gb_printf(KERN_ERR, "%s: gb_copy_in_sync failed %d\n", __func__, ret);
		goto fail_out_sync;
	}

	ret = gb_lookup_bos(dev, file, args, stage);
	if (ret) {
		gb_printf(KERN_ERR, "%s: gb_lookup_bos failed %d\n", __func__, ret);
		goto fail_in_fence;
	}

	ret = gb_stage_push(stage);
	if (ret) {
		gb_printf(KERN_ERR, "%s: gb_stage_push failed %d\n", __func__, ret);
		goto fail_in_fence;
	}
	gb_printf(KERN_DEBUG, "%s: submit stage %pK\n", __func__, stage);

	if (sync_out) {
		drm_syncobj_replace_fence(sync_out, stage->render_done_fence);
		drm_syncobj_put(sync_out);
	}

	if (kref_read(&stage->refcount) >= 1)
		gb_stage_put(stage);

	return ret;

fail_in_fence:
	if (stage->bos) {
		for (i = 0; i < stage->bo_count; i++)
			if (stage->bos[i]) 
		#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0))
				drm_gem_object_put_unlocked(stage->bos[i]);
		#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
				drm_gem_object_put(stage->bos[i]);
		#else
				drm_gem_object_unreference_unlocked(stage->bos[i]);
		#endif
		kvfree(stage->bos);
	}

	if (stage->implicit_fences)
		kvfree(stage->implicit_fences);

	if (stage->in_fences) {
		for (i = 0; i < stage->in_fence_count; i++)
			dma_fence_put(stage->in_fences[i]);
		kvfree(stage->in_fences);
	}

fail_out_sync:
	if (sync_out)
		drm_syncobj_put(sync_out);

out_put_job:
	if (stage) {
		if (kref_read(&stage->refcount) > 1)
			gb_stage_put(stage);
		kfree(stage);
	}

	return ret;
}

static int gb_ioctl_wait_bo(struct drm_device *dev, void *data,
		            struct drm_file *file_priv)
{
	long ret = 0;
	struct drm_gb_wait_bo *args = data;
	struct drm_gem_object *gem_obj;
	struct gb_gem_object *gb_bo;
	unsigned long timeout = nsecs_to_jiffies(args->timeout_ns);
	struct gb_device *gbdev = dev->dev_private;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	struct timespec t1, t2;
#else
	struct timespec64 t1, t2;
#endif
	long time_use1;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	getnstimeofday(&t1);
#else
	ktime_get_real_ts64(&t1);
#endif

	if (!args->handle && args->pad) {
		ret = gbvpu_wait_bo_ioctl(file_priv, args->pad);
		return ret;
	}

	if (args->pad)
		return -EINVAL;
	gem_obj = drm_gem_object_lookup(file_priv, args->handle);
	if (!gem_obj)
		return -ENOENT;
	gb_bo = gem_ob_to_gb_gem(gem_obj);
	gb_printf(KERN_DEBUG, "%s:%d--ip domain =%x--\n",
		__func__, __LINE__, gb_bo->domain);

	if (atomic_read(&gbdev->reset.pending))
		return -EBUSY;

#if KERNEL_VERSION(5, 3, 0) < LINUX_VERSION_CODE
	gb_printf(KERN_DEBUG, "%s: %pK wait bo %pK timeout %ld\n", __func__,
			current, gpu_mmbo_to_gb_bo(to_drm_gem_gbmem_obj(gem_obj)), timeout);
#else
#endif

#if (KERNEL_VERSION(5, 4, 0) > LINUX_VERSION_CODE)
	/* ret = reservation_object_wait_timeout_rcu(gb_bo->resv, true, true, timeout);
	 * // for UOS 1050: 4.19 kernel */
	ret = reservation_object_wait_timeout_rcu(gb_bo->gb_base.ttm_bo.bo.resv, true, true, timeout);
#elif (KERNEL_VERSION(5, 14, 0) > LINUX_VERSION_CODE) && (KERNEL_VERSION(5, 4, 0) <= LINUX_VERSION_CODE)
	ret = dma_resv_wait_timeout_rcu(gem_obj->resv, true, true, timeout);
#else
#if (KERNEL_VERSION(6, 6, 0) <= LINUX_VERSION_CODE)
	ret = dma_resv_wait_timeout(gem_obj->resv, DMA_RESV_USAGE_READ, true, timeout);
#else
	ret = dma_resv_wait_timeout(gem_obj->resv, true, true, timeout);
#endif
#endif
	/* ret == 0 means not signaled,
	 * ret > 0 means signaled
	 * ret < 0 means interrupted before timeout
	 * */
	if (!ret)
		ret = timeout ? -ETIMEDOUT : -EBUSY;

	gb_printf(KERN_DEBUG, "%s: %pK wait bo %pK ok timeout %ld ret %ld\n", __func__, current,
				gpu_mmbo_to_gb_bo(to_drm_gem_gbmem_obj(gem_obj)), timeout, ret);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
		drm_gem_object_put(gem_obj);
#else
		drm_gem_object_put_unlocked(gem_obj);
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	getnstimeofday(&t2);
#else
	ktime_get_real_ts64(&t2);
#endif
	time_use1 = (t2.tv_sec - t1.tv_sec) * 1000000 +
		(t2.tv_nsec - t1.tv_nsec) / 1000;
	if (gb_time_debug) {
		pr_info("[%s,pid:%d] %s:cost time: %ld\n",
			current->comm,current->pid,
			__func__, time_use1);
	}

	return ret;
}

static int gb_ioctl_mmap_bo(struct drm_device *dev, void *data,
		      struct drm_file *file_priv)
{
	struct drm_gb_mmap_bo *args = data;
	struct drm_gem_object *gem_obj;
	struct gb_gem_object *gb_bo;
	int ret = 0;

	if (args->flags != 0) {
		DRM_INFO("unknown mmap_bo flags: %d\n", args->flags);
		gb_printf(KERN_ERR, "%s unknown mmap_bo flags: %d\n",
			__func__, args->flags);
		return -EINVAL;
	}

	gem_obj = drm_gem_object_lookup(file_priv, args->handle);
	if (!gem_obj) {
		DRM_DEBUG("Failed to look up GEM BO %d\n", args->handle);
		gb_printf(KERN_ERR, "%s Failed to look up GEM BO %d\n",
			__func__, args->handle);
		return -ENOENT;
	}

#ifdef MMU_ENABLED
	/* Don't allow mmapping of heap objects, as heap pages aren't alloced yet */
	if (gem_ob_to_gb_gem(gem_obj)->is_heap_growable)
		return -EINVAL;
#endif

	gb_bo = gem_ob_to_gb_gem(gem_obj);

#ifdef GB_GEM_TTM
	args->offset = gb_ttm_bo_mmap_offset(&gb_bo->gb_base.ttm_bo);
#else
	args->offset = drm_vma_node_offset_addr(&gb_bo->gb_base.ttm_bo.bo.base.vma_node);
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0))
	drm_gem_object_put_unlocked(gem_obj);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
	drm_gem_object_put(gem_obj);
#else
	drm_gem_object_unreference_unlocked(gem_obj);
#endif

	return ret;
}

/*Resolve encoding import display data, and return bo vpaddr*/
#define ENC_IMPORT_FLAG 1

static int gb_ioctl_get_bo_offset(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct drm_gb_get_bo_offset *args = data;
	struct drm_gem_object *gem_obj;
	struct gb_gem_object *bo;
	gem_obj = drm_gem_object_lookup(file_priv, args->handle);
	if (!gem_obj) {
		DRM_DEBUG("Failed to look up GEM BO %d\n", args->handle);
		return -ENOENT;
	}
	bo = gem_ob_to_gb_gem(gem_obj);

	if (ENC_IMPORT_FLAG == args->pad) {
		args->offset = bo->base.iopaddr;
	} else {
		args->offset = bo->base.iovaddr;
	}
	gb_printf(KERN_INFO, "args->offset: 0x%llx %s\n", args->offset, __func__);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0))
	drm_gem_object_put_unlocked(gem_obj);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
	drm_gem_object_put(gem_obj);
#else
	drm_gem_object_unreference_unlocked(gem_obj);
#endif

	return 0;
}

static int gb_ioctl_get_bo_vaddr(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	return gb_ioctl_get_bo_offset(dev, data, file_priv);
}

static int gb_ioctl_get_bo_addr_mapping(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	__u64 real_cpu_offset;
	__u64 vaddr;
	int mode;
	struct rb_root *gb_rb_root;
	struct gb_rb_node *gb_rb_data = NULL;
	struct drm_gb_get_bo_cpu_va *args = (struct drm_gb_get_bo_cpu_va *)data;
	struct gb_file_priv *gb_priv = file_priv->driver_priv;

	if (!gb_rb_tree) {
		gb_printf(KERN_ERR, "%s gb rb tree is not enabled\n",
			__func__);
		return -EINVAL;
	}

	if (args->gpu_va && !args->cpu_va) {
		mode = GPU_ADDR_MODE;
		vaddr = args->gpu_va;
	} else if (!args->gpu_va && args->cpu_va) {
		mode = CPU_ADDR_MODE;
		vaddr = args->cpu_va;
	} else {
		gb_printf(KERN_ERR, "%s args error,gpu_va:0x%llx cpu_va:0x%llx\n",
			__func__, (long long)args->gpu_va, (long long)args->cpu_va);
		return -ENOMEM;
	}

	gb_rb_root = gb_get_rb_root(gb_priv, mode);
	if (!gb_rb_root) {
		gb_printf(KERN_ERR, "%s gb_bo_tree is null, mode:%d, drm_file:0x%llx\n",
			__func__, mode, (long long)file_priv);
		return -ENOMEM;
	}

	gb_rb_data = gb_bo_rb_search(gb_rb_root, vaddr, mode);
	if (gb_rb_data == NULL) {
		gb_printf(KERN_ERR, "%s Get cpu_va is null, gpu_va:0x%llx\n",
			__func__, (unsigned long long)args->gpu_va);
		return -EINVAL;
	}

	if (mode) {
		real_cpu_offset = args->cpu_va -
			gb_rb_data->gb_gem_bo->vm_info.vm_start;
		args->gpu_va = gb_rb_data->gb_gem_bo->vm_info.iovaddr +
			real_cpu_offset;
	} else {
		real_cpu_offset = args->gpu_va -
			gb_rb_data->gb_gem_bo->vm_info.iovaddr;
		args->cpu_va = gb_rb_data->gb_gem_bo->vm_info.vm_start +
			real_cpu_offset;
	}
	args->cpu_va_start = gb_rb_data->gb_gem_bo->vm_info.vm_start;
	args->cpu_va_end = gb_rb_data->gb_gem_bo->vm_info.vm_end;

#ifdef GB_KMD_DBG
	args->gpu_pa = gb_rb_data->gb_gem_bo->base.iopaddr + real_cpu_offset;
#endif

	return 0;

}

static int gb_ioctl_set_shed_time(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct genbu_set_sched_time *timems = (struct genbu_set_sched_time *)data;
	if (timems->timems <= 0)
		return -EINVAL;
	gb_change_job_timeout_ms(timems->timems);
	return 0;
}
static int gb_pci_bar_map(struct gb_pci_bar *bar, bool wc)
{
	if (bar->len) {
		request_mem_region(bar->base, bar->len, "gb");

		if (wc)
			bar->mmio = ioremap_wc(bar->base, bar->len);
		else
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
			bar->mmio = ioremap_nocache(bar->base, bar->len);
#else
			bar->mmio = ioremap(bar->base, bar->len);
#endif

		if (!bar->mmio)
			return -EFAULT;
	} else {
		bar->mmio = NULL;
	}
	gb_printf(KERN_INFO, "%s-%d: bar->iomap 0x%llx, wc = %d\n",
	 __func__, __LINE__, (long long)bar->mmio, wc);
	return 0;
}

u32 get_gpu_freq(void)
{
	struct gb_pcie_info *gb_pcie = get_pcie_gl_info();
	u32 gpu_freq = get_cur_gpu_freq();

	if (GENBU_02 == get_genbu_gb_type(gb_pcie->gb_dev_info))
		return gpu_freq ;
	else
		return 0;
}

void gb_enable_perfcnt(struct gb_file_priv *gb_priv)
{
	u32 prfcnt_config;

	if (!gb_priv) {
		gb_printf(KERN_ERR, "%s err gb_priv NULL\n", __func__);
		return;
	}

	/* Configure perf cnt*/
	prfcnt_config = gb_mmu_as_get() << PRFCNT_CONFIG_AS_SHIFT;
	//reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_CONFIG),
	//		prfcnt_config | PRFCNT_CONFIG_MODE_OFF | (1 << 8));
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_CONFIG),
			prfcnt_config | PRFCNT_CONFIG_MODE_OFF);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_BASE_LO),
			PERF_CNT_GPU_VA_START & 0xFFFFFFFF);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_BASE_HI),
			PERF_CNT_GPU_VA_START >> 32);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_SM_EN),
			0xffffffff);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_SHADER_EN),
			0xffffffff);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_MMU_L2_EN),
			0xffffffff);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_TILER_EN),
			0xffffffff);
	if (performance2_set) {
		reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_CONFIG),
				prfcnt_config | PRFCNT_CONFIG_MODE_MANUAL_NO_CLEAR | (1 << 8));
	} else {
		reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_CONFIG),
				prfcnt_config | PRFCNT_CONFIG_MODE_MANUAL_NO_CLEAR);
	}
}

void gb_disable_perfcnt(struct gb_file_priv *gb_priv)
{
	if (!gb_priv) {
		gb_printf(KERN_ERR, "%s err gb_priv NULL\n", __func__);
		return;
	}

	/* Configure perf cnt*/
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_CONFIG),
			PRFCNT_CONFIG_MODE_OFF);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_BASE_LO), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_BASE_HI), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_SM_EN), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_SHADER_EN), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_MMU_L2_EN), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(PRFCNT_TILER_EN), 0);
	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(GPU_COMMAND),
		GPU_COMMAND_PRFCNT_CLEAR);
	mdelay(10);
}

void gb_dump_perfcnt(struct gb_file_priv *gb_priv)
{
	if (!gb_priv) {
		gb_printf(KERN_ERR, "%s err gb_priv NULL\n", __func__);
		return;
	}

	reg_write(gb_priv->gbdev, GPU_CONTROL_REG(GPU_COMMAND),
			GPU_COMMAND_PRFCNT_SAMPLE);
	/* wait gpu write perf data to vram */
	mdelay(100);
}

int gl_perf_dump_cnt;
void gb_dump_l2_perfcnt(struct gb_file_priv *gb_priv)
{
	u32 l2_dbg_cmd[4] = {0xdeb005e0, 0xdeb005a0, 0xdeb00560, 0xdeb00520};
	u32 i, j, offset;
	u64 reg_val;

	if (!gb_priv) {
		pr_info("%s err gb_priv NULL\n", __func__);
		return;
	}

	pr_info("========l2 perf cnt %d dump begin========\n",
			gl_perf_dump_cnt);
	for (i = 0; i < 4; i++) {
		offset = 0x100;
		for (j = 0; j < 64; j++) {
			reg_write(gb_priv->gbdev, GPU_CONTROL_REG(0xfe0),
				offset);
			reg_write(gb_priv->gbdev, GPU_CONTROL_REG(0x54),
				l2_dbg_cmd[i]);
			udelay(20);
			reg_val = reg_read(gb_priv->gbdev, GPU_CONTROL_REG(0xfe0)) |
				((u64)reg_read(gb_priv->gbdev, GPU_CONTROL_REG(0xfe4)) << 32);
			pr_info("l2_%d 0x%x:%llu\n",
				i, offset, reg_val);
			offset += 4;
		}
	}
	pr_info("========l2 perf cnt %d dump end========\n",
			gl_perf_dump_cnt);
}

struct gb_file_priv* gb_get_in_used_priv(void)
{
	/* TODO: get any gb_file_priv in used and lock it*/
	return get_gl_perf_priv();
}

u32 perf_time = 100;
u32 gb_cal_utilization(void __iomem *gb_perf_base)
{
	u32 gpu_ut, gpu_freq;
	u32* gpu_active = (u32*)(gb_perf_base + PRFCNT_GPU_ACT_OFF);

	gpu_freq = get_gpu_freq();
	if (!gpu_freq) {
		gb_printf(KERN_ERR, "%s get gpu ut err, gpu_active:0x%x, gpu_freq:0x%x\n",
			__func__, *gpu_active, gpu_freq);
		return 0xffffffff;
	}

	gpu_ut = ((u64)*gpu_active * 100) / (gpu_freq / (1000 / perf_time));
	return gpu_ut;
}

void __iomem *gb_get_perf_dump_ptr(struct gb_file_priv *gb_priv)
{
	u32 ddr_bar_id;

	ddr_bar_id = get_genbu_ddr_bar_id(
			gb_priv->gbdev->gb_pcie->gb_dev_info);
	return (void __iomem*)gb_priv->gbdev->gb_pcie->pci_bars[ddr_bar_id].mmio +
			PERF_CNT_START_ADDR;
}

int gb_get_gpu_utilization(u32 *gpu_ut)
{
	struct gb_file_priv *gb_priv = NULL;
	void __iomem *gb_perf_base = NULL;
	gb_priv = gb_get_in_used_priv();
	*gpu_ut = 0;
	if (gb_priv == NULL)
		/* if no gb_priv is in used, then no proc use gpu */
		return 0;

	gb_perf_base = gb_get_perf_dump_ptr(gb_priv);
	/* TODO: get gb_priv lock */
	mutex_lock(&gb_priv->perfcnt_lock);
	memset_io(gb_perf_base, 0, GB_PERF_SIZE);
	gb_enable_perfcnt(gb_priv);
	/* sleep for perf increase cnt */
	msleep(perf_time);
	gb_dump_perfcnt(gb_priv);
	/* sleep for gpu write perf data to vram */
	msleep(50);
	*gpu_ut = gb_cal_utilization(gb_perf_base);
	if (*gpu_ut > 100)
		*gpu_ut = 100;
	gb_disable_perfcnt(gb_priv);
	mutex_unlock(&gb_priv->perfcnt_lock);
	return 0;
}

void gb_perf_timer_func(void)
{
	int i, gap;
	u32 *dump_kaddr, *dump_kaddr_base;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	struct timespec starttime, endtime;
#else
	struct timespec64 starttime, endtime;
#endif
	struct gb_file_priv *gb_perf_priv = get_gl_perf_priv();

	pr_info("========perf cnt dump start to sample now========\n");
	dump_kaddr_base = (u32 *)gb_get_perf_dump_ptr(gb_perf_priv);
	memset_io(dump_kaddr_base, 0, GB_PERF_SIZE);
	gb_enable_perfcnt(get_gl_perf_priv());

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0)
	getnstimeofday(&starttime);
	msleep(performance_ms);
	getnstimeofday(&endtime);
#else
	ktime_get_real_ts64(&starttime);
	msleep(performance_ms);
	ktime_get_real_ts64(&endtime);
#endif

	gb_dump_perfcnt(gb_perf_priv);
	gl_perf_dump_cnt++;
	gap = (endtime.tv_sec - starttime.tv_sec) * 1000000 +
		(endtime.tv_nsec - starttime.tv_nsec) / 1000;
	pr_info("========perf cnt %d dump %d time data========\n",
			gl_perf_dump_cnt, gap);
	for (i = 0; i < GB_PERF_SIZE; i += 16) {
		dump_kaddr = dump_kaddr_base + (i / 4);
		pr_info("0x2000%04x: %08x %08x %08x %08x\n", i,
					*dump_kaddr, *(dump_kaddr + 1),
					*(dump_kaddr + 2), *(dump_kaddr + 3));
	}
	pr_info("========perf cnt %d dump end=========\n",
			gl_perf_dump_cnt);

	gb_dump_l2_perfcnt(gb_perf_priv);
	gb_disable_perfcnt(gb_perf_priv);

	/* just run once, set to false */
	perf_cnt = false;

}

int get_pthread_name(void)
{
	if (strncmp(current->comm, "Xorg", 4) &&
		strncmp(current->comm, "systemd", 7) &&
		strncmp(current->comm, "kworker", 7) &&
		strncmp(current->comm, "modprobe", 8) &&
		strncmp(current->comm, "plymouthd", 9) &&
		strncmp(current->comm, "insmod", 6))
		return 1;
	else
		return 0;
}

static int gb_open(struct drm_device *dev, struct drm_file *file)
{
#ifdef GB_GPU
	int ret;
	struct gb_device *gbdev = dev->dev_private;
	struct gb_file_priv *gb_priv;

	gb_priv = kzalloc(sizeof(*gb_priv), GFP_KERNEL);
	if (!gb_priv)
		return -ENOMEM;

	gb_priv->gbdev = gbdev;
	gb_priv->drm_file = file;
	gb_priv->pcb = current;
	gb_priv->list_node.drm_file = file;
	gb_priv->list_node.cpu_root_node = RB_ROOT;
	gb_priv->list_node.gpu_root_node = RB_ROOT;
	gb_priv->list_node.gb_priv = gb_priv;
	file->driver_priv = gb_priv;

	ret = gb_stage_open(gb_priv);
	if (ret) {
		gb_printf(KERN_ERR, "%s gb_stage_open err ! %d\n",
			__func__, ret);
		goto err_stage;
	}
/*
#ifdef GB_DEC_MODULE
	ret = gb_vpu_vcmd_open(dev, file, &gb_priv->dec_priv,
		       &gb_priv->dec1_priv, &gb_priv->enc_priv);
	if (ret) {
		gb_printf(KERN_ERR, "%s gb_vpu_vcmd_open err ! %d\n",
			__func__, ret);
		goto err_stage;
	}
#endif
*/
	/* rb_list node must add at last */
	mutex_lock(&gbdev->rb_mutex);
	list_add(&gb_priv->list_node.list, &gb_list_head);
	mutex_unlock(&gbdev->rb_mutex);

	gb_printf(KERN_INFO, "%s invoke ok! drm_file:0x%llx\n",
		__func__, (long long)file);

	return 0;

err_stage:
	return ret;
#else
	return 0;
#endif
}

static void
gb_postclose(struct drm_device *dev, struct drm_file *file)
{
#ifdef GB_GPU
	struct gb_file_priv *gb_priv = file->driver_priv;
	gb_printf(KERN_INFO, "%s close drm_file:0x%llx\n",
		__func__, (unsigned long long)file);

	gb_vpu_postclose(dev, file);
	gb_stage_close(gb_priv);

	mutex_lock(&gb_priv->gbdev->rb_mutex);
	list_node_release(gb_priv);
	mutex_unlock(&gb_priv->gbdev->rb_mutex);

	kfree(gb_priv);
	gb_priv = NULL;
#endif
}

static int
gb_ioctl_get_gpu_info(struct drm_device *dev, void *data, struct drm_file *file)
{
	struct gpu_info *gb_info = get_gbinfo_handle();
	struct gpu_info *args = data;

	gb_get_mm_size_info(TTM_PL_VRAM);
	//gb_get_vpumm_size_info();

	memcpy(args, gb_info, sizeof(struct gpu_info));

	return 0;
}

static const struct drm_ioctl_desc gb_drm_driver_ioctls[] = {
	DRM_IOCTL_DEF_DRV(GB_SUBMIT, gb_ioctl_submit, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_CREATE_BO, gb_ioctl_create_bo, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_WAIT_BO, gb_ioctl_wait_bo, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_MMAP_BO, gb_ioctl_mmap_bo, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_GET_PARAM, gb_ioctl_get_param, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_GET_BO_OFFSET, gb_ioctl_get_bo_offset, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_GET_BO_ADDR_MAPPING,
			gb_ioctl_get_bo_addr_mapping, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_GET_GPU_INFO,
			gb_ioctl_get_gpu_info, DRM_RENDER_ALLOW),

	DRM_IOCTL_DEF_DRV(GB_VPU_VCMD_OPEN,
			gb_ioctl_vpu_vcmd_open, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_VPU_VCMD_CLOSE,
			gb_ioctl_vpu_vcmd_close, DRM_RENDER_ALLOW),
/*
	DRM_IOCTL_DEF_DRV(GB_VPU_VCMDBUF_FREE,
	gb_ioctl_vpu_vcmdbuf_free, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_VPU_VCMDBUF_RESERVE,
		gb_ioctl_vpu_vcmdbuf_resv, DRM_RENDER_ALLOW),
*/
	DRM_IOCTL_DEF_DRV(GB_VPU_VCMDBUF_RESERVE,
		gb_ioctl_vpu_vcmdbuf_resv, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_VPU_CMD_BUF_FREE,
		gb_ioctl_vpu_cmdbuf_free, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_VPU_REG_OPS,
			gb_ioctl_vpu_reg_ops, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_DMA_TRANS_TO_FB,
			gb_edma_translate_to_fb, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_DMA_TRANS_TO_RAM,
			gb_edma_translate_to_ram, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_V2VDMA_TRANS,
			gb_v2vdma_translate, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_HDMA_OFFSET_TRANS,
			gb_edma_offset_translate, DRM_RENDER_ALLOW),
#ifdef GB_ALLSCREEN
	DRM_IOCTL_DEF_DRV(GB_GET_FB,
			gb_get_fb_base_info, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_CREATE_FB,
			gb_ioctl_create_fb_bo, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_SWITCH_FB,
			gb_switch_fb, DRM_RENDER_ALLOW),
#endif
	DRM_IOCTL_DEF_DRV(GB_SET_RENDER_SIZE,
			gb_set_render_size, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_GET_BO_VADDR,
			gb_ioctl_get_bo_vaddr, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_SET_SCHED_TIME_MS,
			gb_ioctl_set_shed_time, DRM_RENDER_ALLOW),
	DRM_IOCTL_DEF_DRV(GB_INFINITY_ATOMIC,
			gbdc_infinity_atomic_ioctl, 0),
};

int gb_drm_release(struct inode *inode, struct file *filp)
{
	struct drm_file *file_priv = filp->private_data;
	struct gb_file_priv *gb_priv = file_priv->driver_priv;
	gb_wait_sched_complete(gb_priv);
	return drm_release(inode, filp);
}

static int gb_drm_flush(struct file *filp, fl_owner_t id)
{
	int i;
	long timeout = msecs_to_jiffies(job_timeout_ms);
	struct drm_file *file_priv = filp->private_data;
	struct gb_file_priv *gb_priv = file_priv->driver_priv;

	for (i = 0; i < NUM_JOB_SLOTS; i++)
		timeout = drm_sched_entity_flush(&gb_priv->sched_entity[i], timeout);

	return timeout >= 0 ? 0 : timeout;
}

static const struct file_operations gb_drm_driver_fops = {
        .owner = THIS_MODULE,
        .open = drm_open,
        .flush = gb_drm_flush,
        .release = gb_drm_release,
        .unlocked_ioctl = drm_ioctl,
        .mmap = gb_mmap,
        .poll = drm_poll,
        .read = drm_read,
        .compat_ioctl = drm_compat_ioctl,
        .llseek = noop_llseek,
};

vm_fault_t gb_cpu_vm_fault(struct vm_fault *vmf)
{
#if 0
	struct vm_area_struct *vma = vmf->vma;
	struct drm_gem_object *obj = vma->vm_private_data;
	struct gb_gem_object *bo = to_gb_bo(obj);
#endif
	return VM_FAULT_SIGBUS;
}

const struct vm_operations_struct gb_vm_ops = {
        .fault = gb_cpu_vm_fault,
        .open = drm_gem_vm_open,
        .close = drm_gem_vm_close,
};

static struct drm_driver gb_drm_driver = {
	.driver_features	= DRIVER_RENDER | DRIVER_GEM | DRIVER_SYNCOBJ
#ifdef GB_DC_DP
	| DRIVER_MODESET
#endif
//#ifdef CONFIG_GBDC_ATOMIC_FEATURE
	| DRIVER_ATOMIC
//#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0)
	| DRIVER_PRIME
#endif
	,
	.load = gbdc_load,
	.unload = gbdc_unload,
#if !(defined SYS_CENTOS7_COMPILE_ENV || defined SYS_CENTOS7_9_2009) && \
	LINUX_VERSION_CODE < KERNEL_VERSION(4, 14, 0)
	.set_busid = drm_pci_set_busid,
#endif
	.open			= gb_open,
	.postclose		= gb_postclose,
	.lastclose		= drm_fb_helper_lastclose,
	.ioctls			= gb_drm_driver_ioctls,
	.num_ioctls		= ARRAY_SIZE(gb_drm_driver_ioctls),
	.fops			= &gb_drm_driver_fops,

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 10, 0)
	.gem_free_object 	= gb_gem_free_object,
	.gem_vm_ops             = &gb_vm_ops,
	.gem_open_object        = gb_gem_open,
	.gem_close_object       = gb_gem_close,
	.dumb_destroy = drm_gem_dumb_destroy,
#endif
	.dumb_create = gbdc_dumb_create,
	.dumb_map_offset = gbdc_dumb_mmap_offset,
#if LINUX_VERSION_CODE > KERNEL_VERSION(5, 3, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0)
	.prime_handle_to_fd = gb_drm_gem_prime_handle_to_fd,
	.prime_fd_to_handle = gb_drm_gem_prime_fd_to_handle,
	.gem_prime_export = gb_drm_gem_prime_export,
	.gem_prime_import = gb_drm_gem_prime_import,
	.gem_prime_get_sg_table = drm_gem_shmem_get_sg_table,
	.gem_prime_vmap = drm_gem_shmem_vmap,
	.gem_prime_vunmap = drm_gem_shmem_vunmap,
	.gem_prime_import_sg_table = drm_gem_shmem_prime_import_sg_table,
#else
	.prime_handle_to_fd = gb_drm_gem_prime_handle_to_fd,
	.prime_fd_to_handle = drm_gem_prime_fd_to_handle,
	.gem_prime_import = drm_gem_prime_import,
#if LINUX_VERSION_CODE <= KERNEL_VERSION(5, 3, 0)
	.gem_prime_export = drm_gem_prime_export,
	.gem_prime_res_obj = gb_gem_prime_res_obj,
#endif
#endif
	.name			= DRIVER_NAME,
	.desc			= DRIVER_DESC,
	.date			= DRIVER_DATE,
	.major			= DRIVER_MAJOR,
	.minor			= DRIVER_MINOR,
	.patchlevel		= DRIVER_PATCHLEVEL,
};

static struct gb_asic_dev_info genbu_glb_asic_dev_info[GENBU_TYPE_MAX] = {
	{GENBU_FPGA_DEV_INFO},
	{GENBU_01_DEV_INFO},
	{GENBU_02_DEV_INFO},
};

static struct gb_asic_dev_info *gb_get_dev_info(enum genbu_asic_type gb_type)
{
	if (gb_type >= GENBU_TYPE_MAX) {
		gb_printf(KERN_ERR, "%s fail!gbtype:%d\n", __func__, gb_type);
		return NULL;
	}

	return &(genbu_glb_asic_dev_info[gb_type]);
}

static int gb_set_devinfo(struct pci_dev *pdev,
		struct gpu_info *devinfo, struct gb_device *gbdev,
		struct gb_asic_dev_info *dtype_info)
{
	int vram_id = 0;
	char *devmodel = "GenBu";
	char *devname = "GenBu02";
	char *devproduct = "Sietium";
	char *drv_info	= GB_PCI_NAME;

	strncpy(devinfo->devmodel, devmodel, min(strlen(devmodel),
			 sizeof(devinfo->devmodel) - 1));
	strncpy(devinfo->devname, devname, min(strlen(devname),
			 sizeof(devinfo->devname) - 1));
	strncpy(devinfo->product, devproduct, min(strlen(devproduct),
			 sizeof(devinfo->product) - 1));

	vram_id = get_genbu_ddr_bar_id(dtype_info);
	devinfo->vram = gbdev->gb_pcie->pci_bars[vram_id].len>>20;
	devinfo->clk = 33;
	devinfo->bitwidth = 64;

	if (pdev->bus->cur_bus_speed == PCIE_SPEED_8_0GT)
		devinfo->bandwidth = 8;
	else if (pdev->bus->cur_bus_speed == PCIE_SPEED_5_0GT)
		devinfo->bandwidth = 5;
	else if (pdev->bus->cur_bus_speed == PCIE_SPEED_2_5GT)
		devinfo->bandwidth = 2;

	strncpy(devinfo->driver_info, drv_info, min(strlen(drv_info),
			 sizeof(devinfo->driver_info) - 1));

	if (get_genbu_gb_type(dtype_info) == GENBU_01) {
		devinfo->vender_id = GENBU01_PCI_VENDOR_ID;
		devinfo->device_id = GENBU01_PCI_DEVICE_ID;
	} else if (get_genbu_gb_type(dtype_info) == GENBU_FPGA) {
		devinfo->vender_id = GENBU_FPGA_PCI_VENDOR_ID;
		devinfo->device_id = GENBU_FPGA_PCI_DEVICE_ID;
	} else if (get_genbu_gb_type(dtype_info) == GENBU_02) {
		devinfo->vender_id = GENBU02_PCI_VENDOR_ID;
		devinfo->device_id = GENBU02_PCI_DEVICE_ID;
	}

	sprintf(devinfo->bus_info, "pci@%4d:%2d.%d", pdev->bus->number,
		((pdev->devfn)>>3) & 0x1f, (pdev->devfn) & 0x7);

	gbdev->gpu_device_info = devinfo;

	return 0;
}

irqreturn_t gb_gpu_common_irq_handler(int irq, void *data)
{
	return gb_common_irq_handler(irq, data);
}
#define GB_PAL_IRQ
#if 1
enum gb_board_type gb_get_board_type(struct gb_pcie_info *gpi)
{
	unsigned int val, ret;
	if (gpi->board_id >= 0x0 && gpi->board_id <= 0x1f)
		val = gpi->board_id;
	else {
		iowrite32(0x60000000, gpi->pci_bars[4].mmio + 0xd0c);
		val = ioread32(gpi->pci_bars[0].mmio + 0x210);
		gpi->board_id = val;
	}

	/* gb_printf(KERN_ERR, "%s %d board_id:%u\n", __func__, __LINE__, val); */
	switch (val) {
	case PCIE_LPDDR4_VAL:
		ret = PCIE_LPDDR4;
		break;
	case PCIE_CQ2040_P21_VAL:
	case PCIE_C0_200_VAL:
		ret = PCIE_C0_200;
		break;
	case FULL_LPDDR4_VAL:
		ret = PCIE_FULL_LPDDR4;
		break;
	case PCIE_M6FL8G_LPDDR4_VAL:
		ret = PCIE_M6FL8G_LPDDR4;
		break;
	case PCIE_HIE1LP4_LPDDR4_VAL:
	case PCIE_HIE1LP42_LPDDR4_VAL:
		ret = PCIE_HIE1LP4_LPDDR4;
		break;
	case PCIE_M4HL8G_LPDDR4_VAL:
		ret = PCIE_M4HL8G_LPDDR4;
		break;
	case CQ2040_MXM_M60_VAL:
		ret = CQ2040_MXM_M60;
		break;
	default:
		ret = PCIE_FULL_FUNC_DDR4;
		break;
	}
	return ret;

}

bool gb_connector_is_vga(int connector_id)
{
	bool is_vga = false;

	if ((PCIE_LPDDR4 == gb_get_board_type(get_pcie_gl_info()) ||
			PCIE_C0_200 == gb_get_board_type(get_pcie_gl_info()) ||
			CQ2040_MXM_M60 == gb_get_board_type(get_pcie_gl_info())) &&
		connector_id == 3)
		is_vga = true;
	else if (PCIE_FULL_LPDDR4 == gb_get_board_type(get_pcie_gl_info()) &&
                connector_id == 4)
                is_vga = true;

	return is_vga;
}
static void gb_hdmi_reset(struct gb_pcie_info *gpi, int HDMI_reset_flag)
{
	int type = gb_get_board_type(gpi);

	iowrite32(0x60000000, gpi->pci_bars[4].mmio + 0xd0c);
	if ((HDMI_reset_flag == HIBERNATE_SLEEP) ||
		(HDMI_reset_flag == SYSTEM_WORKING)) {
		if (type == PCIE_LPDDR4 || type == PCIE_C0_200)
			HDMI_power_reset(gpi, 2);
		else if (type == PCIE_FULL_LPDDR4)
			HDMI_power_reset(gpi, 5);
		else if (type == PCIE_M6FL8G_LPDDR4) {
			HDMI_power_reset(gpi, 1);
			HDMI_power_reset(gpi, 3);
			HDMI_power_reset(gpi, 4);
			HDMI_power_reset(gpi, 5);
		} else if (type == PCIE_HIE1LP4_LPDDR4) {
			HDMI_power_reset(gpi, 0);
			HDMI_power_reset(gpi, 2);
			HDMI_power_reset(gpi, 3);
			HDMI_power_reset(gpi, 5);
		} else if (type == PCIE_M4HL8G_LPDDR4) {
			HDMI_power_reset(gpi, 0);
			HDMI_power_reset(gpi, 2);
			HDMI_power_reset(gpi, 3);
			HDMI_power_reset(gpi, 5);
		}
	}
}
static void gb_sysctl_cfg(struct gb_pcie_info *gpi, int HDMI_reset_flag)
{
	iowrite32(0x60000000, gpi->pci_bars[4].mmio + 0xd0c);
	/* DC read QOS 7 */
	iowrite32(0x1c0000, gpi->pci_bars[0].mmio + 0x1b4);
}
#endif

struct gb_pcie_info *get_pcie_gl_info(void)
{
	return gl_gbdev->gb_pcie;
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 7, 0))
int gb_drm_get_pci_dev(struct pci_dev *pdev, const struct pci_device_id *ent,
		struct drm_driver *kms_driver)
{
	struct drm_device *dev;
	int ret;

	dev = drm_dev_alloc(kms_driver, &pdev->dev);
	if (IS_ERR(dev))
		return PTR_ERR(dev);

	ret = pci_enable_device(pdev);
	if (ret)
		goto err_free;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 7, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0))
	dev->pdev = pdev;
#endif
	pci_set_drvdata(pdev, dev);
	ret = drm_dev_register(dev, ent->driver_data);
	if (ret)
		goto err_agp;

	return 0;

err_agp:
	pci_disable_device(pdev);
err_free:
	drm_dev_put(dev);
	return ret;
}
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0)
static int gb_kick_out_firmware_fb(struct pci_dev *pdev, struct drm_driver *driver)
#else
static int gb_kick_out_firmware_fb(struct pci_dev *pdev,int ddr_bar_id)
#endif
{
	bool primary = false;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0)
	struct apertures_struct *ap;

	ap = alloc_apertures(1);
	if (!ap)
		return -ENOMEM;

	ap->ranges[0].base = pci_resource_start(pdev, ddr_bar_id);
	ap->ranges[0].size = pci_resource_len(pdev, ddr_bar_id);
#endif

#ifdef CONFIG_X86
	primary = pdev->resource[PCI_ROM_RESOURCE].flags & IORESOURCE_ROM_SHADOW;
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 0, 0)
	drm_aperture_remove_framebuffers(driver);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 14, 0)
	drm_aperture_remove_framebuffers(primary, driver);
#else
	drm_fb_helper_remove_conflicting_framebuffers(ap, "gbdrmfb", primary);
	kfree(ap);
#endif


	return 0;
}
static int gb_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
	struct gb_device *gbdev;
	struct pci_device_id kms_pci_id;
	int err, i;
	enum genbu_asic_type gb_type = (enum genbu_asic_type)(ent->driver_data);
	struct gb_asic_dev_info *gb_dev_info = gb_get_dev_info(gb_type);
	struct gpu_info *info_handle;
#ifdef	GB_AUDIO
	struct pcie_info_str apcie_info;
#endif
	int ret,ddr_bar_id;
	enum gb_board_type board_type;

	/* set the default log level, 0:quite mode,
	 * 3:only print KERN_ERR, 7:print all log */
	atomic_set(&log_en, 3);
	ddr_bar_id = get_genbu_ddr_bar_id(gb_dev_info);

/* Get rid of things like offb */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0)
	ret = gb_kick_out_firmware_fb(pdev,	&gb_drm_driver);
#else
	ret = gb_kick_out_firmware_fb(pdev, ddr_bar_id);
#endif
	gb_printf(KERN_INFO, "%s,kick out fb ,ret %d\n",__func__, ret);

	err = pci_enable_device(pdev);
	if (err) {
		dev_err(&pdev->dev, "%s Fatal error during PCI device init:%d\n",
			__func__, err);
		return err;
	}

	gbdev = devm_kzalloc(&pdev->dev, sizeof(*gbdev), GFP_KERNEL);
	if (!gbdev)
		return -ENOMEM;

	gbdev->gb_pcie =
	 devm_kzalloc(&pdev->dev, sizeof(*(gbdev->gb_pcie)), GFP_KERNEL);
	if (!gbdev->gb_pcie)
		return -ENOMEM;

	gbdev->dev = &pdev->dev;
	gbdev->gb_pcie->pdev = pdev;
	gbdev->gb_pcie->gbdev = gbdev;
	gl_gbdev = gbdev;
	gbdev->gb_pcie->gb_dev_info = gb_dev_info;

	for (i = 0; i < get_genbu_pci_bar_cnt(gb_dev_info); i++) {
		bool wc;

		if(get_genbu_ddr_bar_id(gb_dev_info) == i)
			gb_resize_pcie_bar2(gbdev->gb_pcie->pdev);

		gbdev->gb_pcie->pci_bars[i].base = pci_resource_start(pdev, i);
		gbdev->gb_pcie->pci_bars[i].len = pci_resource_len(pdev, i);

		if (gbdev->gb_pcie->pci_bars[i].len == 0) {
			gb_printf(KERN_INFO, "%s i: %d\n", __func__, i);
			continue;
		}

		wc = (get_genbu_ddr_bar_id(gb_dev_info) == i) ? true : false;
		if (gb_pci_bar_map(&gbdev->gb_pcie->pci_bars[i], wc)) {
			dev_err(&pdev->dev, "Fatal error during PCI bar ioremap\n");
			return -EFAULT;
		}

		if (get_genbu_ddr_bar_id(gb_dev_info) == i)
			ddr_va = gbdev->gb_pcie->pci_bars[i].mmio;

		dev_info(&pdev->dev, "GB PCI BAR[%d] base 0x%llx, iomap 0x%llx, size 0x%llx\n",
			i, gbdev->gb_pcie->pci_bars[i].base,
			(long long)gbdev->gb_pcie->pci_bars[i].mmio,
			gbdev->gb_pcie->pci_bars[i].len);
	}

#ifdef GB_PERIPHERALS
	gb_peripherals_init(gbdev);
#endif
	gbdev->gb_pcie->board_id = -1;
	gb_sysctl_cfg(gbdev->gb_pcie, SYSTEM_WORKING);
	HDMI_power_reset_all(gbdev->gb_pcie);

#ifdef	GB_AUDIO
	i = get_genbu_ddr_bar_id(gb_dev_info);
	apcie_info.ddrwr_va = ddr_va = gbdev->gb_pcie->pci_bars[i].mmio;
	apcie_info.ddrwr_pa = gBaseDDRHw = gbdev->gb_pcie->pci_bars[i].base;
	apcie_info.ddrwr_len = gBaseDDRLen = gbdev->gb_pcie->pci_bars[i].len;
	i = get_genbu_dc_reg_bar_id(gb_dev_info);
	apcie_info.hdwr_va = hd_va = gbdev->gb_pcie->pci_bars[i].mmio;
	apcie_info.hdwr_pa = gBaseHdwr = gbdev->gb_pcie->pci_bars[i].base;
	apcie_info.hdwr_len = gBaseLen = gbdev->gb_pcie->pci_bars[i].len;
	i = get_genbu_peri_base_bar_id(gb_dev_info);
	apcie_info.base_switch = (u64)gbdev->gb_pcie->pci_bars[i].mmio;
	i = get_genbu_mcu_peri_bar_id(gb_dev_info);
	apcie_info.pll_va = (u64)gbdev->gb_pcie->pci_bars[i].mmio;
	init_audio_infos();
	gb_alsa_audio_init(&apcie_info);
	gb_printf(KERN_INFO, "-------&&&&&&&&&&&&&&------ddr va =%pK--hd va=%pK--\n",
	 ddr_va, hd_va);
#endif

	pci_set_master(pdev);
	info_handle = get_gbinfo_handle();
	gb_set_devinfo(pdev, info_handle, gbdev, gb_dev_info);
	dev_info(gbdev->dev, "VRAM: %lluM \n", (unsigned long long)(info_handle->vram));
	/* It's needed to enable msi, or the interrupt handler
	 * can't be triggerred.
	 */
#ifndef GPU_MIRQ_ENABLED
	if (pci_enable_msi(pdev))
		dev_err(&pdev->dev, "Don't support MSI");
#endif
	gb_printf(KERN_INFO, "-------irq init----\n");
	err = gb_pcie_enable_msi(gbdev->gb_pcie);
	if (err) {
		dev_err(&pdev->dev, "gb_pcie_enable_msi failed!\n");
		return err;
	}

	err = gb_pcie_init_irqs(gbdev->gb_pcie);
	if (err) {
		dev_err(&pdev->dev, "gb_pcie_init_irqs failed!\n");
		return err;
	}
	gbip_offset_init(gbdev);

	if (gb_type == GENBU_02)
		gb_init_comp_irq(gbdev->gb_pcie);

#ifdef GB_DC_DP
	board_type = gb_get_board_type(gbdev->gb_pcie);
	for (i = 0; i < GB02_CONNECTER_NUM; i++) {
		bool is_edp = false;
		if (gb_port_check_nouse(board_type, i))
			continue;

		if ((CQ2040_MXM_M60 == board_type) && ((0 == i) || (1 == i)))
			is_edp = true;
		err |= gb_dp_init(i, gbdev->gb_pcie, is_edp);
		if (err) {
			gb_printf(KERN_ERR, "%s %d dp init %d error:%d!\n",
				__func__, __LINE__, i, err);
		}
	}
#endif

#ifdef GB_GPU
	err = pcie_hdma_init(gbdev->gb_pcie);
	if (err) {
		gb_printf(KERN_ERR, "%s %d hdma init error:%d!\n",
			__func__, __LINE__, err);
	}

	err = gb_device_init(gbdev, gb_dev_info);
	if (err) {
		if (err != -EPROBE_DEFER)
			dev_err(&pdev->dev, "Fatal error during GPU init\n");
		goto err_out0;
	}
#endif

	atomic_set(&gbdev->reset.pending, GB_RST_IDLE);
	pm_runtime_set_active(gbdev->dev);
	pm_runtime_mark_last_busy(gbdev->dev);
	pm_runtime_enable(gbdev->dev);
	pm_runtime_set_autosuspend_delay(gbdev->dev, 50);
	pm_runtime_use_autosuspend(gbdev->dev);
	kms_pci_id.driver_data = (unsigned long)gbdev;

	INIT_LIST_HEAD(&gb_list_head);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 7, 0))
	err = drm_get_pci_dev(pdev, &kms_pci_id,
			&gb_drm_driver);
#else
	err = gb_drm_get_pci_dev(pdev, &kms_pci_id,
			&gb_drm_driver);
#endif

	if (err) {
		dev_err(&pdev->dev, "%s %d:drm_get_pci_dev failed!:%d\n",
			__func__, __LINE__, err);
		goto err_out1;
	}
#if 0
	ret |= gb_hdmac_init(gb_pcie_priv);
#endif
#ifdef GB_AUDIO
	gb_snd_device_init(&pdev->dev);
	gb_audio_create_proc_entry();
#endif
	err |= gb_v2vdma_init(gbdev->gb_pcie);
	if (err) {
		gb_printf(KERN_ERR, "%s %d v2vdma init error:%d!\n",
			__func__, __LINE__, err);
	}
	/* MCU firmware upgrade init */
	err = gb_firmware_upgrade_init(gbdev, gb_dev_info);
	if (err)
		dev_err(&pdev->dev, "firmware upgrade init failed:%d\n", err);

#ifdef GB_MEM_TOOL
	gb_dump_init(gbdev);
#endif
	err = gb_create_gpuinfo(info_handle);
	if (err) {
		dev_err(&pdev->dev, "gb_create_gpuinfo failed!\n");
		return err;
	}

	err = gb_gpu_sysfs_init(gbdev);
	if (err) {
		dev_err(&pdev->dev, "gb_gpu_sysfs_init failed!\n");
		return err;
	}

	gb_debugfs_init(gbdev);
	return err;
err_out1:
	pm_runtime_disable(gbdev->dev);
#ifdef GB_GPU
	gb_device_release(gbdev);
#endif

#ifdef GB_PERIPHERALS
	gb_peripherals_exit(gbdev);
#endif
err_out0:
	return err;
}

static void gb_shutdown(struct pci_dev *pdev)
{
	struct gb_device *gbdev = NULL;

	gbdev = gl_gbdev;
	HDMI_power_down(gbdev->gb_pcie);

	gb_printf(KERN_INFO, "%s-%d: gb_shutdown\n",
	 __func__, __LINE__ );
}

static void gb_remove(struct pci_dev *pdev)
{
	struct drm_device *ddev = pci_get_drvdata(pdev);
	struct gb_device *gbdev = NULL;
	enum genbu_asic_type gb_type;
	int i;

	if (ddev == NULL) {
		gb_printf(KERN_ERR, "%s %d:Error: drm_device is NULL\n",
			__func__, __LINE__);
		/* return; */
	}

	gb_remove_gpuinfo();
	/* gbdev = (struct gb_device *)ddev->dev_private; */
	gbdev = gl_gbdev;
	if (gbdev == NULL) {
		gb_printf(KERN_ERR, "%s %d:Error: gb_device is NULL\n",
			__func__, __LINE__);
		return;
	}

	gb_gpu_sysfs_fini(gbdev);
	gb_debugfs_exit(gbdev);

	pm_runtime_get_sync(gbdev->dev);
#ifdef GB_PERIPHERALS
	gb_peripherals_exit(gbdev);
#endif
#ifdef GB_GPU
	gb_device_release(gbdev);
#endif
	pm_runtime_put_sync_suspend(gbdev->dev);
	pm_runtime_disable(gbdev->dev);

	for (i = 0; i < get_genbu_pci_bar_cnt(
		gbdev->gb_pcie->gb_dev_info); i++) {
		if (gbdev->gb_pcie->pci_bars[i].mmio)
			iounmap(gbdev->gb_pcie->pci_bars[i].mmio);

		release_mem_region(gbdev->gb_pcie->pci_bars[i].base,
		 gbdev->gb_pcie->pci_bars[i].len);
	}

	gb_type = get_genbu_gb_type(gbdev->gb_pcie->gb_dev_info);
	if (gb_type == GENBU_02)
		gb_release_comp_irq(gbdev->gb_pcie);
	/* gb_pcie_free_irqs(gb_pcie_priv);
	 * gb_pcie_disable_msi(gb_pcie_priv); */
	gb_pcie_free_irqs(gbdev->gb_pcie);
	gb_pcie_disable_msi(gbdev->gb_pcie);

	pci_disable_msi(pdev);
	pci_clear_master(pdev);
	pci_disable_device(pdev);

	drm_put_dev(ddev);
}

static struct pci_device_id gb_ids[] = {
	{
		PCI_DEVICE(GENBU_FPGA_PCI_VENDOR_ID,
			   GENBU_FPGA_PCI_DEVICE_ID),
		.class = 0,
		.class_mask = 0,
		.driver_data = GENBU_FPGA,
	},
	{ PCI_DEVICE(GENBU01_PCI_VENDOR_ID, GENBU01_PCI_DEVICE_ID),
		.class = (PCI_CLASS_DISPLAY_VGA << 8),
		.class_mask = 0xffff00,
		.driver_data = GENBU_01,
	},
	{
		PCI_DEVICE(GENBU02_PCI_VENDOR_ID, GENBU02_PCI_DEVICE_ID),
		.class = 0,
		.class_mask = 0,
		.driver_data = GENBU_02,
	},
	{}

};

static int gb_pmops_prepare(struct device *dev)
{
        gb_printf(KERN_INFO, "--enter -GB pm prepare--- \n");
        return 0;
}

static int gb_dp_suspend(struct drm_device *drm_dev)
{
	struct gb_device *gbdev = (struct gb_device *)drm_dev->dev_private;
	struct gb_pcie_info *gb_pcie = gbdev->gb_pcie;
	struct dptx *dptx_info = NULL;
	int i = 0;
	enum gb_board_type board_type = gb_get_board_type(gb_pcie);

	mdelay(100);
	for (i = 0; i < GB02_CONNECTER_NUM; i++) {
		if (gb_port_check_nouse(board_type, i) || !gb_pcie->dptx[i])
			continue;
		gb_dp_pm_suspend(i, gb_pcie, false);
		dptx_info = gb_pcie->dptx[i];
		dptx_info->edid_getted = false;
		dptx_info->link.trained = false;
		cancel_delayed_work_sync(&dptx_info->hotplug_work);
	}

	return 0;

}

static int gbdc_suspend_fb(struct drm_device *drmdev)
{
	struct drm_crtc *crtc = NULL;
	u64 left_up_fb = U64_MAX;
	unsigned long cur_fb_offset = 0;
	u64 save_fb_size = 0;
	int ret = 0;
	struct drm_plane_state *state;
	struct gbdc_crtc *gbdc_crtc;
	//dump_func_begin;

	drm_for_each_crtc(crtc, drmdev) {
		gbdc_crtc = to_gbdc_crtc(crtc);

		if (gbdc_crtc && crtc->enabled) {
			struct drm_framebuffer *fb;

			if (crtc->primary && crtc->primary->state) {
				state = crtc->primary->state;
				fb = state->fb;
				if (fb) {
					gb_printf(KERN_INFO, "crtc id %d,fb id %d\n",
							gbdc_crtc->crtc_id, fb->base.id);
					gb_printf(KERN_INFO, "cur_fb_offset 0x%lx\n",
							gbdc_crtc->cplanes[DC_PLANE_GRAPHIC].cur_fb_offset);
					save_fb_size = fb->width * fb->height * 4;
					cur_fb_offset = gbdc_crtc->cplanes[DC_PLANE_GRAPHIC].cur_fb_offset;
					if (left_up_fb > cur_fb_offset)
						left_up_fb = cur_fb_offset;
					}
			}

		}

	}

	gb_printf(KERN_INFO, "left_up_fb 0x%llx, FB SIZE 0x%llx\n", left_up_fb, save_fb_size);
	ret = gb_register_dump_block(left_up_fb, save_fb_size, GB_BLOCK_FB);
	//dump_func_end;

	return ret;
}
static int __maybe_unused gbdc_cursor_suspend(struct drm_device *drm_dev)
{
	int ret = 0;
	int cursor_save_index = GB_BLOCK_CURSOR_FIRST;
	struct drm_crtc *crtc;
	u64 cursor_size = 0;

	drm_for_each_crtc(crtc, drm_dev) {
		struct gbdc_crtc *gbdc_crtc = to_gbdc_crtc(crtc);

		if (crtc->enabled && gbdc_crtc->cursor_addr) {
			cursor_size = gbdc_crtc->cursor_height * gbdc_crtc->cursor_width * 4;

			gb_printf(KERN_INFO, "cursor_size 0x%llx,cursor_addr 0x%llx\n",
						cursor_size, gbdc_crtc->cursor_addr);
			gb_printf(KERN_INFO, "%s : gbcrtc id %d\n", __func__, gbdc_crtc->crtc_id);
			gbdc_crtc->cursor_resume = 0;
			ret = gb_register_dump_block(gbdc_crtc->cursor_addr, cursor_size,
							cursor_save_index);
			if (!ret)
				gb_printf(KERN_INFO, "cursor register block ok\n");
			WARN_ON((cursor_save_index++) > GB_BLOCK_MAX);

		}
	} //for each
	return ret;
}
static int __maybe_unused gbdc_cursor_resume(struct drm_device *drm_dev)
{
	struct drm_crtc *crtc;
	int ret = 0;

	drm_for_each_crtc(crtc, drm_dev) {
		struct gbdc_crtc *gbdc_crtc = to_gbdc_crtc(crtc);

		gb_printf(KERN_INFO, "crtc id %d, cursor_bo %p\n", gbdc_crtc->crtc_id,
				gbdc_crtc->cursor_bo);
		if (gbdc_crtc && gbdc_crtc->cursor_bo)
			gbdc_crtc->cursor_resume = 1;

	} //for each
	return ret;
}
static int gbdc_resume(struct drm_device *drm_dev)
{
	int ret = 0;

	//ret = gbdc_cursor_resume(drm_dev);
	gb_printf(KERN_INFO, "%s,ret =%d\n", __func__, ret);
	return ret;
}

static int gbdc_suspend(struct drm_device *drm_dev)
{
	int ret = 0;

	ret = gbdc_suspend_fb(drm_dev);
	//ret |= gbdc_cursor_suspend(drm_dev);

	gb_printf(KERN_INFO, "%s ret=%d\n", __func__, ret);

	return ret;
}

static int gb_dp_resume(struct drm_device *drm_dev)
{
	int i = 0;
	int err = 0;
	struct gb_device *gbdev = (struct gb_device *)drm_dev->dev_private;
	struct gb_pcie_info *gb_pcie = gbdev->gb_pcie;
	enum gb_board_type board_type = gb_get_board_type(gb_pcie);

	for (i = 0; i < GB02_CONNECTER_NUM; i++) {
		if (gb_port_check_nouse(board_type, i) || !gb_pcie->dptx[i])
			continue;

		err = gb_dp_pm_resume(i, gbdev->gb_pcie, false);
		if (err)
			gb_printf(KERN_ERR, "%s, %d dp init %d error:%d!\n",
				__func__, __LINE__, i, err);
	}

	return err;
}

static void set_system_sleep_status(struct gb_device *gbdev,
	enum system_working_status sleep_status)
{
	gbdev->sleep_status = sleep_status;
}

static int get_system_sleep_status(struct gb_device *gbdev)
{
	return gbdev->sleep_status;
}

static void dec_resume(void)
{
	dec0_resume();
	dec1_resume();
}

static void dec_suspend(struct drm_device *dev, struct gb_device *gbdev)
{
	dec0_suspend();
	dec1_suspend();
}

int gb_dev_resumes(struct drm_device *dev, bool resume)
{
	int ret = 0;
	struct gb_device *gbdev = (struct gb_device *)dev->dev_private;

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 19, 0)
	gb_pci_restore_rebar_state(dev->pdev);
#endif

	if (resume) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0)
		pci_set_power_state(dev->pdev, PCI_D0);
		pci_restore_state(dev->pdev);
		ret = pci_enable_device(dev->pdev);
		pci_set_master(dev->pdev);
#else
		pci_set_power_state(to_pci_dev(dev->dev), PCI_D0);
		pci_restore_state(to_pci_dev(dev->dev));
		ret = pci_enable_device(to_pci_dev(dev->dev));
		pci_set_master(to_pci_dev(dev->dev));
#endif
		set_firmware_verinfo(gbdev->gb_pcie->pci_bars[0].mmio, gbdev->gb_pcie->pci_bars[4].mmio, &gbdev->version);
		if (ret) {
			gb_printf(KERN_ERR, "pci_enable_device failed\n");
		}
	}

	gb_set_irq_mask_en();
	gb_sysctl_cfg(gbdev->gb_pcie, get_system_sleep_status(gbdev));
	gb_hdmi_reset(gbdev->gb_pcie, get_system_sleep_status(gbdev));
	gb_dp_resume(dev);
	gb_restore_vram_data();
	gbdc_resume(dev);
	//drm_helper_resume_force_mode(dev);
	ret = drm_mode_config_helper_resume(dev);
	dec_resume();
	enc_resume();
	gb_device_reset(gbdev);
	gbdev->mmu_mode->mmu_controll(gbdev, MMU_AS_ENABLED);
	gb_stage_sched_start(gbdev);
	gb_peripherals_resume(gbdev);
	return ret;
}

int gb_dev_suspend(struct drm_device *dev, bool suspend)
{
	struct gb_device *gbdev = (struct gb_device *)dev->dev_private;
	int ret = 0;

	gb_stage_sched_stop(gbdev);

	if (!gb_stage_is_idle(gbdev)) {
		gb_printf(KERN_ERR, "%s stage is not idle\n", __func__);
		return -EBUSY;
	}

	get_firmware_verinfo(gbdev->gb_pcie->pci_bars[0].mmio, &gbdev->version);
	set_system_sleep_status(gbdev, suspend);

	gb_peripherals_suspend(gbdev);
	gbdc_suspend(dev);
	gb_backup_vram_date(dev, gbdev);
	gb_dp_suspend(dev);
	dec_suspend(dev, gbdev);
	enc_suspend();
	gb_printf(KERN_INFO, "drm_mode_config_helper_suspend begin\n");
	ret = drm_mode_config_helper_suspend(dev);
	gb_printf(KERN_INFO, "ret %d\n", ret);
	if (ret)
		return ret;
	gb_printf(KERN_INFO, "drm_mode_config_helper_suspend done\n");
	gb_gpu_power_off(gbdev);
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0)
	pci_save_state(dev->pdev);
#else
	pci_save_state(to_pci_dev(dev->dev));
#endif
	if (suspend) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0)
		pci_disable_device(dev->pdev);
		gb_printf(KERN_INFO, "disable pci device\n");
#else
		pci_disable_device(to_pci_dev(dev->dev));
#endif
	}
	gb_printf(KERN_INFO, "suspend done\n");
	return 0;
}

static int gb_pmops_suspend(struct device *dev)
{
	struct pci_dev *pdev = to_pci_dev(dev);
	struct drm_device *drm_dev = pci_get_drvdata(pdev);

	return gb_dev_suspend(drm_dev, true);
}

static int gb_pmops_resume(struct device *dev)
{
	struct pci_dev *pdev = to_pci_dev(dev);
	struct drm_device *drm_dev = pci_get_drvdata(pdev);

	return gb_dev_resumes(drm_dev, true);
}

static int gb_pmops_freeze(struct device *dev)
{
	struct pci_dev *pdev = to_pci_dev(dev);
	struct drm_device *drm_dev = pci_get_drvdata(pdev);

	gb_printf(KERN_INFO, "--enter -gb freeze--- \n");

	return gb_dev_suspend(drm_dev, false);
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
static int gb_pmops_thaw(struct device *dev)
{
	struct pci_dev *pdev = to_pci_dev(dev);
	struct drm_device *drm_dev = pci_get_drvdata(pdev);

	gb_printf(KERN_INFO, "--enter -gb thaw--- \n");

	return gb_dev_resumes(drm_dev, false);
}
#endif

static int gb_pmops_restore(struct device *dev)
{
	struct pci_dev *pdev = to_pci_dev(dev);
	struct drm_device *drm_dev = pci_get_drvdata(pdev);

	gb_printf(KERN_INFO, "--enter -gb restore--- \n");

	return gb_dev_resumes(drm_dev, false);
}

static const struct dev_pm_ops gb_pmops = {
        .prepare = gb_pmops_prepare,
        .suspend = gb_pmops_suspend,
        .resume = gb_pmops_resume,
        .freeze = gb_pmops_freeze,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
        .thaw = gb_pmops_thaw,
#endif
        .restore = gb_pmops_restore,
};

static struct pci_driver gb_driver = {
	.name		= "gb",
	.id_table	= gb_ids,
	.probe		= gb_probe,
	.remove		= gb_remove,
	.shutdown	= gb_shutdown,
	.driver.pm  	= &gb_pmops,
};

static int __init gb_init(void)
{
	int err;

	find_kallsyms_lookup_name();

	err = pci_register_driver(&gb_driver);
	if (err) {
		gb_printf(KERN_ERR, "failed to register the PCI driver\n");
		return err;
	}

	gb_printf(KERN_INFO, "PCI driver registered\n");
	err = shell_init();
	if (err)
		gb_printf(KERN_ERR, "easy shell init failed");
	return 0;
}

static void gb_exit(void)
{
	pci_unregister_driver(&gb_driver);

	gb_printf(KERN_INFO, "PCI driver unregistered\n");
}

module_init(gb_init);
module_exit(gb_exit);
MODULE_AUTHOR("GB Project Developers");
MODULE_DESCRIPTION("GB DRM Driver");
MODULE_LICENSE("GPL v2");
MODULE_DEVICE_TABLE(pci, gb_ids);
