#ifndef __GB_FIRMWARE_UPGRADE_H
#define __GB_FIRMWARE_UPGRADE_H

#define PCI_BAR_CNT	6
//#define BAR_LEN	PCI_BAR_CNT
#define MCU_ACCESS_BAR	 		0
#define INT_CONTROLLER_BAR		1
#define GB02_DDR_BAR  			2
#define UPGRADE_CFG_REG_BAR		4
#define CFG_MCU_XIP_ADDR_OFFSET		0xd0c
#define CFG_SYS_PERI_ADDR_OFFSET	0xd10
#define MCU_SYSCTRL_BASE_ADDR	0x60000000
#define INT_CONTROLLER_BASE_ADDR	0x80000000
#define INT_CONTROLLER_OFFSET		0x403000

#define KERNEL_PAGE_SIZE					(0x1000) /* 4KB */

/* BAR0 register */
#define SYSCTL_BOOT_MODE_OFFSET	 0x14
#define BOOT_MODE_MCU2PCIE_SW_INT	BIT(4)
#define SYSCTL_SPARE0_OFFSET     0x200
#define SYSCTL_SPARE1_OFFSET     0x204
#define SYSCTL_SPARE2_OFFSET     0x208
#define SYSCTL_SPARE2_HOT_RESET     0x5a
#define SYSCTL_SPARE3_OFFSET     0x20c
#define SYSCTL_SPARE3_REBOOT_VALUE  0x5a
#define SYSCTL_SPARE4_OFFSET     0x210
#define SYSCTL_SPARE4_MCU_BOOT_FINISHED_VALUE     0x55
#define SYSCTL_SPARE5_OFFSET     0x214

/* BAR1 register */
#define INTC_IRQ_INTEN_L_OFFSET		0x0
#define INTC_IRQ_INTEN_H_OFFSET		0x4
#define INTC_IRQ_INTMASK_L_OFFSET	0x8
#define INTC_IRQ_INTMASK_H_OFFSET	0xc
#define INTC_IRT_RAWSTATUS_L_OFFSET	0x18
#define INTC_IRT_RAWSTATUS_H_OFFSET	0x1c
#define INTC_IRT_STATUS_L_OFFSET	0x20
#define INTC_IRT_STATUS_H_OFFSET	0x24

typedef enum {
	T_UPGRADE_MCU_RESULT_NO_UPGRADE = 0,	/* UPGRADE MCU failed */
	T_UPGRADE_MCU_RESULT_UPGRADING,			/* UPGRADE MCU is in progress */
	T_UPGRADE_MCU_RESULT_OK, 				/* UPGRADE MCU OK */
	T_UPGRADE_MCU_RESULT_MAX
} upgrade_mcu_result_e;

typedef enum {
	U_UPGRADE_TYPE_MCU_APP = 1,		/* upgrade MCU's APP image */
	U_UPGRADE_TYPE_UEFI_FW, 		/* upgrade UEFI firmware */
	U_UPGRADE_TYPE_MAX
} upgrade_mcu_sub_type_e;

struct upgrade_image_info_t {
	uint8_t filename[0x100];	/* sub image name : upgrade_app.bin */
	uint64_t addr; /* the address of the sub image in VRAM */
	uint32_t size; /* the size of the sub image */
	uint16_t crc16;
	uint8_t checksum;
	uint8_t type; /* MCU app, UEFI or others */
};

struct gb_fw_upgrade {
	struct device *dev;
	struct gb_device *gbdev;
	struct gb_asic_dev_info *dev_info;
	wait_queue_head_t wait;
	int upgrade_flag;
	struct proc_dir_entry *procdir;
	int upgrade_result;
};

int gb_firmware_upgrade_init(struct gb_device *gbdev, struct gb_asic_dev_info *gb_dev_info);
void gb_pci_upgrade_remove(struct gb_device *gbdev);
#endif
