/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __GB_GEM_H__
#define __GB_GEM_H__

#include <drm/drm_mm.h>
#if KERNEL_VERSION(5, 0, 0) < LINUX_VERSION_CODE
#include <drm/drm_gem_shmem_helper.h>
#endif
#include "common/gb_bo.h"
#include "common/gb_common.h"
#include "gb_mmu.h"

#define DOMAIN_LEFT 0
#define DOMAIN_RIGHT 1
#define IN_DOMAIN 2

#define GPU_ADDR_MODE 0
#define CPU_ADDR_MODE 1

extern struct list_head gb_list_head;
struct gb_mmu;
struct gb_device;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0)
extern const struct vm_operations_struct gb_vm_ops;
extern const struct drm_gem_object_funcs gb_gem_object_funcs;
#endif

struct gb_page {
	u64 gpu_phy;
	struct list_head lru;
	volatile u32 pfn;
	struct gb_device *gbdev;
};

static inline
struct  gb_gem_object *drm_mm_node_to_gb_bo(struct drm_mm_node *node)
{
	struct drm_gem_gbmem_object *gb_bo_base = container_of(node, struct drm_gem_gbmem_object, node);
	return gb_bo_base->bo;
}

struct drm_gem_object *gb_gpu_gem_create_object(struct drm_device *dev, size_t size);
void gb_gem_free_object(struct drm_gem_object *obj);

struct drm_gem_object *
gb_gem_prime_import_sg_table(struct drm_device *dev,
				   struct dma_buf_attachment *attach,
				   struct sg_table *sgt);

struct gb_gem_object *
gb_gpu_gem_ttm_create_with_handle(struct drm_file *file_priv,
	struct drm_device *dev, size_t size, int init_domain,
		u32 flags, bool kernel, uint32_t *handle);

struct gb_gem_object *
gb_gpu_gem_create_with_handle(struct drm_file *file_priv,
				struct drm_device *dev, size_t size,
				u32 flags,
				uint32_t *handle);

struct gb_gem_object *gb_gpu_ttm_bo_create(struct drm_device *dev,
				size_t size, int init_domain,
				u32 flags, bool kernel);

int gb_mmap(struct file *filp, struct vm_area_struct *vma);
int gb_gpu_mmap(struct gb_mmu_mode *mmu_mode,
			u64 vpfn, phys_addr_t *phys, size_t nr_pages, unsigned long flags, int dump);
int gb_gpu_munmap(struct gb_mmu_mode *mmu_mode, u64 vpfn, size_t nr);

int gb_gem_open(struct drm_gem_object *obj, struct drm_file *file_priv);
void gb_gem_close(struct drm_gem_object *obj, struct drm_file *file_priv);
struct reservation_object *gb_gem_prime_res_obj(struct drm_gem_object *obj);

struct rb_root *gb_get_rb_root(struct gb_file_priv *gb_priv, int mode);
struct gb_rb_node *gb_bo_rb_init(void);
struct gb_rb_node *gb_bo_rb_search(struct rb_root *rb_root, __u64 vaddr, int mode);
int gb_rb_clean(struct rb_root *rb_root, struct gb_rb_node *new);
int gb_bo_rb_insert(struct rb_root *rb_root, struct gb_rb_node *data, int mode);
void gb_bo_rb_create(struct gb_gem_object *bo, struct drm_gb_create_bo *args);
int gb_bo_rb_mmap(struct gb_gem_object *bo, struct vm_area_struct *vma);
void gb_rb_delete(struct gb_gem_object *bo, struct gb_file_priv *gb_priv);
void gb_bo_rb_delete(struct rb_root *rb_root, __u64 vaddr, int mode);
void gb_bo_rb_printk(struct rb_root *rb_root);

#define gb_err(dev, format, arg...) \
        dev_err(dev, "GB Error: "#format, ##arg)
#define gb_info(dev, format, arg...) \
        dev_info(dev, "GB INFO: "#format, ##arg)
#define gb_warn(dev, format, arg...) \
        dev_warn(dev, "GB WARN: "#format, ##arg)
#define gb_verb_printf printk

#endif /* __GB_GEM_H__ */
