/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __GB_MMU_H__
#define __GB_MMU_H__
#include <drm/drm_mm.h>
#define GB_HEAP_SIZE 		(2 * 1024 * 1024)
#define GB_MMU_PAGE_ENTRIES 512
#define GENBU_MMU_TOPLEVEL    0
#define GENBU_MMU_BOTTOMLEVEL 3

#define AS_TRANSCFG_PTW_MEMATTR_MASK (3 << 24)
#define AS_TRANSCFG_PTW_MEMATTR_NON_CACHEABLE (1 << 24)
#define AS_TRANSCFG_PTW_MEMATTR_WRITE_BACK (2 << 24)

/* AS<n>_MEMATTR values: */
/* Use GPU implementation-defined caching policy. */
#define AS_MEMATTR_IMPL_DEF_CACHE_POLICY 0x88ull
/* The attribute set to force all resources to be cached. */
#define AS_MEMATTR_FORCE_TO_CACHE_ALL    0x8Full
/* Inner write-alloc cache setup, no outer caching */
#define AS_MEMATTR_WRITE_ALLOC           0x8Dull
/* Set to implementation defined, outer caching */
#define AS_MEMATTR_AARCH64_OUTER_IMPL_DEF 0x88ull
/* Set to write back memory, outer caching */
#define AS_MEMATTR_AARCH64_OUTER_WA       0x8Dull

/* HW implementation defined caching */
#define AS_MEMATTR_INDEX_IMPL_DEF_CACHE_POLICY 0
/* Force cache on */
#define AS_MEMATTR_INDEX_FORCE_TO_CACHE_ALL    1
/* Write-alloc */
#define AS_MEMATTR_INDEX_WRITE_ALLOC           2
/* Outer coherent, inner implementation defined policy */
#define AS_MEMATTR_INDEX_OUTER_IMPL_DEF        3
/* Outer coherent, write alloc inner */
#define AS_MEMATTR_INDEX_OUTER_WA              4

#define AS_TRANSCFG_ADRMODE_LEGACY      0
#define AS_TRANSCFG_ADRMODE_UNMAPPED    1
#define AS_TRANSCFG_ADRMODE_IDENTITY    2
#define AS_TRANSCFG_ADRMODE_AARCH64_4K  6
#define AS_TRANSCFG_ADRMODE_AARCH64_64K 8

/*
 * Begin AARCH64 MMU TRANSTAB register values
 */
#define MMU_HW_OUTA_BITS 40
#define AS_TRANSTAB_BASE_MASK ((1ULL << MMU_HW_OUTA_BITS) - (1ULL << 4))

#define GB_PERF_SIZE 0x2600
#define PERF_CNT_GPU_VA_START	(512UL << 20) // 512MB
#define GPU_VA_START	(1ULL << 30)  // 1G
#define GPU_VA_LOW_MAX_SIZE	(3ULL << 30)  // 3G
#define GPU_VA_HIGH_START    (4ULL << 30)  // 4G
#define GPU_VA_MAX_SIZE ((1ULL << 48) - GPU_VA_HIGH_START) // 48 bit 256TB

#define GB_HEAP_DEBUG 1 << 0
#define GB_NORMAL_DEBUG 1 << 1

/* gpu mmu flag start */
/* Free region */
#define GB_REG_FREE              (1ul << 0)
/* CPU write access */
#define GB_REG_CPU_WR            (1ul << 1)
/* GPU write access */
#define GB_REG_GPU_WR            (1ul << 2)
/* No eXecute flag */
#define GB_REG_GPU_NX            (1ul << 3)
/* Is CPU cached? */
#define GB_REG_CPU_CACHED        (1ul << 4)
/* Is GPU cached? */
#define GB_REG_GPU_CACHED        (1ul << 5)

#define GB_REG_GROWABLE          (1ul << 6)
/* Can grow on pf? */
#define GB_REG_PF_GROW           (1ul << 7)
/* VA managed by us */
#define GB_REG_CUSTOM_VA         (1ul << 8)
#define GB_REG_SHARE_IN          (1ul << 9)
#define GB_REG_SHARE_BOTH        (1ul << 10)

/* Space for 4 different zones */
#define GB_REG_ZONE_MASK         (3ul << 11)
#define GB_REG_ZONE(x)           (((x) & 3) << 11)


/* GPU read access */
#define GB_REG_GPU_RD            (1ul<<13)
/* CPU read access */
#define GB_REG_CPU_RD            (1ul<<14)

/* Index of chosen MEMATTR for this region (0..7) */
#define GB_REG_MEMATTR_MASK      (7ul << 16)
#define GB_REG_MEMATTR_INDEX(x)  (((x) & 7) << 16)
#define GB_REG_MEMATTR_VALUE(x)  (((x) & GB_REG_MEMATTR_MASK) >> 16)

#define ENTRY_TYPE_MASK     3ULL
/* For valid ATEs bit 1 = (level == 3) ? 1 : 0.
 * The MMU is only ever configured by the driver so that ATEs
 * are at level 3, so bit 1 should always be set
 */
#define ENTRY_IS_ATE        3ULL
#define ENTRY_IS_INVAL      2ULL
#define ENTRY_IS_PTE        3ULL

#define ENTRY_ATTR_BITS (7ULL << 2)	/* bits 4:2 */
#define ENTRY_ACCESS_RW (1ULL << 6)     /* bits 6:7 */
#define ENTRY_ACCESS_RO (3ULL << 6)
#define ENTRY_SHARE_BITS (3ULL << 8)	/* bits 9:8 */
#define ENTRY_ACCESS_BIT (1ULL << 10)
#define ENTRY_NX_BIT (1ULL << 54)
#define MMU_AS_ENABLED 1
#define MMU_DISABLE 0
/* gpu mmu flag end */

struct gb_gem_object;
struct gb_file_priv;
struct gb_mmu;
struct gb_device;

void gb_mmu_controll_all(struct gb_device *gbdev, int enabled);
void gb_mmu_enable(struct gb_device *gbdev, int as_nr);
struct gb_gem_object *get_bo_by_addr(struct gb_device *gbdev, u64 addr);
void gb_mmu_reset(struct gb_device *gbdev);
void gb_mmu_release(struct gb_device *gbdev);
int gb_mmu_init(struct gb_device *gbdev);
int gb_mmu_as_get(void);

int gb_mmu_pgd_init(struct gb_device *gbdev);
void gb_mmu_pgd_release(struct gb_device *gbdev);
void gb_mmu_free_pg_table(struct gb_device *gbdev);
void gb_mmu_flush_range(struct gb_device *gbdev, int as, u64 iova, size_t size, bool sync);
int mmu_hw_do_operation(struct gb_device *gbdev, int as_nr, u64 iova, size_t size, u32 op);
int mmu_hw_do_operation_all(struct gb_device *gbdev, u64 iova, size_t size, u32 op);
struct gb_mmu_mode *gb_mmu_mode_get_64bit(void);

enum {
	GB_LOW_VA_MM,
	GB_HIGH_VA_MM,
	GB_MAX_VA_MM
};

struct gb_mmu_mode {
	phys_addr_t pgd;
	struct mutex mmu_lock;
	struct gb_device *gbdev;
	u64 *mmu_teardown_pages;
	struct drm_mm mm[GB_MAX_VA_MM];
	struct mutex mm_lock;
	void (*mmu_controll)(struct gb_device *gbdev, int enabled);
	void (*disable_as)(struct gb_device *gbdev, int as_nr);
	void (*enable_as)(struct gb_device *gbdev, int as_nr);
	phys_addr_t (*pte_to_phy_addr)(u64 entry);
	int (*ate_is_valid)(u64 ate);
	int (*pte_is_valid)(u64 pte);
	void (*entry_set_ate)(u64 *entry, phys_addr_t phy, unsigned long flags);
	void (*entry_set_pte)(u64 *entry, phys_addr_t phy);
	void (*entry_invalidate)(u64 *entry);
};

#endif
