/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/delay.h>
#ifndef _GENBU_REGMAP_H_
#define _GENBU_REGMAP_H_


// Vertices process shader: CPU --> CP --> TPU.computer --> PTU --> DDR  Action: primitive tiling     TPU meaning shader core  PTU meaning tiler core
// fragment process shader: CPU --> CP --> TPU.fragment --> DDR		 Action: rasterizer
// Stage slot change to Job slot

/*
 * Begin Register Offsets
 */

#define GPU_CONTROL_BASE        0x0000
#define GPU_CONTROL_REG(r)      (GPU_CONTROL_BASE + (r))

// GPU IRQ regs
#define GPU_ID                  0x020	/* (RO) GPU and revision identifier */
#define L2_FEATURES             0x024	/* (RO) Level 2 cache features */
#define SUSPEND_SIZE            0x008   /* (RO) Fixed-function suspend buffer */
// 0x010 --- 0x01C	GPU reserved

// GPU Config&Control regs
#define PTU_FEATURES			0x02C	/* (RO) PTU (primitive tiling unit, tiler core)Polygon list feature */
#define MEM_FEATURES			0x03C	/* (RO) GPU Memory system feature */
#define MMU_FEATURES			0x040	/* (RO) MMU feature */
#define AS_PRESENT				0x044	/* (RO) Address space slots present */
#define JS_PRESENT				0x048	/* (RO) Stage slots present */

//GB2 add features, just reserved now, not be accomplished yet
#define PMU_FEATURES			0x028	/* (RO) Power manager feature */
#define ROP_FEATURES			0x030	/* (RO) Rasterizer feature */
#define VRAM_FEATURES			0x034	/* (RO) GPU Vram feature */
#define TPA_FEATURES			0x038	/* (RO) TPU (tile processing unit, shader core) array */
#define GPU_COMMAND_FEATURES0	0x050	/* (WO) GPU command feature */

#define GPU_IRQ_RAWSTAT         0x000	/* (RO) GPU IRQ Raw status */
#define GPU_IRQ_CLEAR           0x004	/* (WO) GPU IRQ clear */
#define GPU_IRQ_MASK            0x008	/* (RW) GPU IRQ mask */
#define GPU_IRQ_STATUS          0x00C	/* (RO) GPU IRQ Raw & mask */


//Rename from TILER core to PTU
//Rename from SHADER core to TPU
//Rename from SS to JS
#define TILER_FEATURES			PTU_FEATURES
#define SS_PRESENT				JS_PRESENT

/* GPU_IRQ_RAWSTAT
 * GPU_IRQ_CLEAR
 * GPU_IRQ_MASK
 * GPU_IRQ_STATUS bitmap */
#define GPU_FAULT               (1 << 0)	/* A GPU Fault has occurred */
#define MULTIPLE_GPU_FAULTS     (1 << 7)	/* More than one GPU Fault occurred. */
#define RESET_COMPLETED         (1 << 8)	/* Set when a reset has completed. Intended to use with SOFT_RESET
						   commands which may take time. */
#define POWER_CHANGED_SINGLE    (1 << 9)	/* Set when a single core has finished powering up or down. */
#define POWER_CHANGED_ALL       (1 << 10)	/* Set when all cores have finished powering up or down
						   and the power manager is idle. */

#define PRFCNT_SAMPLE_COMPLETED (1 << 16)	/* Set when a performance count sample has completed. */
#define CLEAN_CACHES_COMPLETED  (1 << 17)	/* Set when a cache clean operation has completed. */

#define GPU_IRQ_REG_ALL (GPU_FAULT | MULTIPLE_GPU_FAULTS | RESET_COMPLETED \
			| POWER_CHANGED_ALL | PRFCNT_SAMPLE_COMPLETED)


// Command excute status
#define GPU_COMMAND_FEATURES0	0x050	/* (WO) GPU command feature */
#define GPU_COMMAND0		0x054	/* (WO) GPU control command */
#define GPU_STATUS0		0x058	/* (RO) GPU status */
#define GPU_LATEST_FLUSH0	0x05C	/* (RO) Latest flush ID */

#define GROUPS_L2_COHERENT      (1 << 0)	/* Cores groups are l2 coherent */
#define GPU_DBGEN               (1 << 8)	/* DBGEN wire status */
#define GPU_FAULTSTATUS0	0x060	/* (RO) GPU Fault status */
#define GPU_FAULTADDRESS_LO0	0x064	/* (RO) GPU Fault address low 32bit*/
#define GPU_FAULTADDRESS_HI0	0x068	/* (RO) GPU Fault address high 32bit*/

// total 8, GB02 used 1, others reseved
#define GPU_COMMANDS_STEP(c)	(0x01C * (c))
//#define GPU_COMMAND_FEATURES(c)	(GPU_COMMAND_FEATURES0 + GPU_COMMANDS_STEP(c))
//#define GPU_COMMAND(c)		(GPU_COMMAND0 + GPU_COMMANDS_STEP(c))
//#define GPU_STATUS(c)		(GPU_STATUS0 + GPU_COMMANDS_STEP(c))
//#define GPU_LATEST_FLUSH(c)	(GPU_LATEST_FLUSH0 + GPU_COMMANDS_STEP(c))
//#define GPU_FAULTSTATUS(c)	(GPU_FAULTSTATUS0 + GPU_COMMANDS_STEP(c))
//#define GPU_FAULTADDRESS_LO(c)	(GPU_FAULTADDRESS_LO0 + GPU_COMMANDS_STEP(c))
//#define GPU_FAULTADDRESS_HI(c)	(GPU_FAULTADDRESS_HI0 + GPU_COMMANDS_STEP(c))

#define GPU_COMMAND		(GPU_COMMAND0 + GPU_COMMANDS_STEP(0))
#define GPU_STATUS		(GPU_STATUS0 + GPU_COMMANDS_STEP(0))
#define LATEST_FLUSH		(GPU_LATEST_FLUSH0 + GPU_COMMANDS_STEP(0))
#define GPU_FAULTSTATUS		(GPU_FAULTSTATUS0 + GPU_COMMANDS_STEP(0))
#define GPU_FAULTADDRESS_LO	(GPU_FAULTADDRESS_LO0 + GPU_COMMANDS_STEP(0))
#define GPU_FAULTADDRESS_HI	(GPU_FAULTADDRESS_HI0 + GPU_COMMANDS_STEP(0))
// 0x130 --- 0x1FC 	GPU reserved

// Power manager
#define PWR_KEY                 0x200	/* (WO) Power manager key register */
#define PWR_OVERRIDE0           0x204	/* (RW) Power manager override settings */
#define PWR_OVERRIDE1           0x208	/* (RW) Power manager override settings */


// Performance counter
#define PRFCNT_BASE_LO          0x220	/* (RW) Performance counter memory region base address, low word */
#define PRFCNT_BASE_HI          0x224	/* (RW) Performance counter memory region base address, high word */
#define PRFCNT_CONFIG           0x228	/* (RW) Performance counter configuration */
#define PRFCNT_SM_EN            0x22C	/* (RW) Performance counter enable flags for Stage Manager */
#define PRFCNT_TPU_EN        	0x230	/* (RW) Performance counter enable flags for shader cores */
#define PRFCNT_PTU_EN         	0x234	/* (RW) Performance counter enable flags for tiler */
#define PRFCNT_MMU_L2_EN_OLD    0x238	/* old l2 en, now no used */
#define PRFCNT_MMU_L2_EN        0x23C	/* (RW) Performance counter enable flags for MMU/L2 cache */
#define PRFCNT_GPU_ACT_OFF		0x18
#define PRFCNT_GB2_GPU_FREQ		(800 << 20) /* GB2 GPU is 800MHz */

#define PRFCNT_SHADER_EN	PRFCNT_TPU_EN
#define PRFCNT_TILER_EN		PRFCNT_PTU_EN

// Other regs
#define CYCLE_COUNT_LO          0x250	/* (RO) Cycle counter, low word */
#define CYCLE_COUNT_HI          0x254	/* (RO) Cycle counter, high word */
#define TIMESTAMP_LO            0x258	/* (RO) Global time stamp counter, low word */
#define TIMESTAMP_HI            0x25C	/* (RO) Global time stamp counter, high word */

#define THREAD_MAX_THREADS		0x260	/* (RO) Maximum number of threads per core */
#define THREAD_MAX_WORKGROUP_SIZE 0x264	/* (RO) Maximum workgroup size */
#define THREAD_MAX_BARRIER_SIZE 0x268	/* (RO) Maximum threads waiting at a barrier */
#define THREAD_FEATURES         0x26C	/* (RO) Thread features */
#define TEXTURE_FEATURES_0      0x270	/* (RO) Support flags for indexed texture formats 0..31 */
#define TEXTURE_FEATURES_1      0x274	/* (RO) Support flags for indexed texture formats 32..63 */
#define TEXTURE_FEATURES_2      0x278	/* (RO) Support flags for indexed texture formats 64..95 */
#define TEXTURE_FEATURES_REG(n) GPU_CONTROL_REG(TEXTURE_FEATURES_0 + ((n) << 2))

// total 32 jobJs0 --- > Js2 used, other reserved
#define JS0_FEATURES		0x300	/* (RW) Slot support job type */
#define JS_FEATURES(slot)	(JS0_FEATURES + ((slot) << 2))
// 0x360 --- 0x3FC	GPU reserved

// User Define Regs
#define GPU_USER_IN0		0x400	/* (RO) Use for Read */
#define GPU_USER_IN47		0x4BC
#define GPU_USER_IN(c)		(GPU_USER_IN0 + ((c) << 2))
// 0x4BC --- 0x4FC	GPU reserved

#define GPU_USER_OUT0		0x500	/* (RW) Use for Write */
#define GPU_USER_OUT48		0x5BC
#define GPU_USER_OUT(c)		(GPU_USER_OUT0 + ((c) << 2))
// 0x5BC --- 0x5FC	GPU reserved

// Reserved
// 0x600 --- 0x7FC	GPU reserved


// Power Manager
#define TPU_PRESENT_LO		0x800
#define TPU_PRESENT_HI		0x804
#define PTU_PRESENT_LO		0x810
#define PTU_PRESENT_HI		0x814
#define L2_PRESENT_LO		0x820
#define L2_PRESENT_HI		0x824

#define TPC_PRESENT_LO		0x980
#define TPC_PRESENT_HI		0x984
#define TPU_READY_LO		0x840
#define TPU_READY_HI		0x844
#define PTU_READY_LO		0x850
#define PTU_READY_HI		0x854
#define L2_READY_LO			0x860
#define L2_READY_HI			0x864

#define TPC_READY_LO		0x990
#define TPC_READY_HI		0x994
#define TPU_PWRON_LO		0x880
#define TPU_PWRON_HI		0x884
#define PTU_PWRON_LO		0x890
#define PTU_PWRON_HI		0x894
#define L2_PWRON_LO			0x8A0
#define L2_PWRON_HI			0x8A4

#define TPC_PWRON_LO		0x9A0
#define TPC_PWRON_HI		0x9A4
#define TPU_PWROFF_LO		0x8C0
#define TPU_PWROFF_HI		0x8C4
#define PTU_PWROFF_LO		0x8D0
#define PTU_PWROFF_HI		0x8D4
#define L2_PWROFF_LO		0x8E0
#define L2_PWROFF_HI		0x8E4

#define TPC_PWROFF_LO		0x9B0
#define TPC_PWROFF_HI		0x9B4
#define TPU_PWRTRANS_LO		0x900
#define TPU_PWRTRANS_HI		0x904
#define PTU_PWRTRANS_LO		0x910
#define PTU_PWRTRANS_HI		0x914
#define L2_PWRTRANS_LO		0x920
#define L2_PWRTRANS_HI		0x924

#define TPC_PWRTRANS_LO		0x9C0
#define TPC_PWRTRANS_HI		0x9C4
#define TPU_PWRACTIVE_LO	0x940
#define TPU_PWRACTIVE_HI	0x944
#define PTU_PWRACTIVE_LO	0x950
#define PTU_PWRACTIVE_HI	0x954
#define L2_PWRACTIVE_LO		0x960
#define L2_PWRACTIVE_HI		0x964

#define COHERENCY_FEATURES  0x2C0	/* (RO) Coherency features present */
#define COHERENCY_ENABLE    0x2C4	/* (RW) Coherency enable */

#define GCC_CONFIG		0x280	/* (RW) reserved */
#define TPU_CONFIG		0x284	/* (RW) reserved */
#define PTU_CONFIG		0x288	/* (RW) reserved */
#define L2_CONFIG		0x28C	/* (RW) limited AXI request from GPU */
#define VRAM_CONFIG		0x290	/* (RW) reserved */
#define MMU_CONFIG		0x294	/* (RW) reserved */
#define REVIDR			0x298	/* (RW) reserved */

#define STAGE_CONTROL_BASE        0x2000
#define STAGE_CONTROL_REG(r)      (STAGE_CONTROL_BASE + (r))

#define STAGE_IRQ_RAWSTAT         (STAGE_CONTROL_BASE + 0x000) /* Raw interrupt status register */
#define STAGE_IRQ_CLEAR           (STAGE_CONTROL_BASE + 0x004) /* Interrupt clear register */
#define STAGE_IRQ_MASK            (STAGE_CONTROL_BASE + 0x008) /* Interrupt mask register */
#define STAGE_IRQ_STATUS          (STAGE_CONTROL_BASE + 0x00C) /* Interrupt status register */
#define STAGE_IRQ_SS_STATE        (STAGE_CONTROL_BASE + 0x010) /* status==active and _next == busy snapshot from last STAGE_IRQ_CLEAR */
#define STAGE_IRQ_THROTTLE        (STAGE_CONTROL_BASE + 0x014) /* cycles to delay delivering an interrupt externally. The STAGE_IRQ_STATUS is NOT affected by this, just the delivery of the interrupt.  */


// Slot0 ---> Slot2 Used, other reseved
#define STAGE_SLOT0               0x400 /* Configuration registers for stage slot 0 */
#define STAGE_SLOT1               0x480 /* Configuration registers for stage slot 1 */
#define STAGE_SLOT2               0x500 /* Configuration registers for stage slot 2 */

#define STAGE_SLOT_REG(n, r)      (STAGE_CONTROL_REG(STAGE_SLOT0 + ((n) << 7)) + (r))
#define SS_BASE                         (STAGE_CONTROL_BASE + STAGE_SLOT0)
#define SS_HEAD_LO(n)                   (SS_BASE + ((n) * 0x80) + 0x00)
#define SS_HEAD_HI(n)                   (SS_BASE + ((n) * 0x80) + 0x04)
#define SS_TAIL_LO(n)                   (SS_BASE + ((n) * 0x80) + 0x08)
#define SS_TAIL_HI(n)                   (SS_BASE + ((n) * 0x80) + 0x0c)
#define SS_AFFINITY_LO(n)               (SS_BASE + ((n) * 0x80) + 0x10)
#define SS_AFFINITY_HI(n)               (SS_BASE + ((n) * 0x80) + 0x14)
#define SS_CONFIG(n)                    (SS_BASE + ((n) * 0x80) + 0x18)
#define SS_XAFFINITY(n)                 (SS_BASE + ((n) * 0x80) + 0x1c)
#define SS_COMMAND(n)                   (SS_BASE + ((n) * 0x80) + 0x20)
#define SS_STATUS(n)                    (SS_BASE + ((n) * 0x80) + 0x24)
#define SS_HEAD_NEXT_LO(n)              (SS_BASE + ((n) * 0x80) + 0x40)
#define SS_HEAD_NEXT_HI(n)              (SS_BASE + ((n) * 0x80) + 0x44)
#define SS_AFFINITY_NEXT_LO(n)          (SS_BASE + ((n) * 0x80) + 0x50)
#define SS_AFFINITY_NEXT_HI(n)          (SS_BASE + ((n) * 0x80) + 0x54)
#define SS_CONFIG_NEXT(n)               (SS_BASE + ((n) * 0x80) + 0x58)
#define SS_COMMAND_NEXT(n)              (SS_BASE + ((n) * 0x80) + 0x60)
#define SS_FLUSH_ID_NEXT(n)             (SS_BASE + ((n) * 0x80) + 0x70)

#define SS_EVENT_MASK_NEXT        0x7C

/* End Register Offsets */




// Debug function Reg
#define GPU_DBG_LO		0xFE0
#define GPU_DBG_HI		0xFE4

/* End Register Offsets */



#define MEMORY_MANAGEMENT_BASE  0x1000
#define MMU_REG(r)              (MEMORY_MANAGEMENT_BASE + (r))

#define MMU_IRQ_RAWSTAT         (MEMORY_MANAGEMENT_BASE + 0x000)   /* (RW) Raw interrupt status register */
#define MMU_IRQ_CLEAR           (MEMORY_MANAGEMENT_BASE + 0x004)   /* (WO) Interrupt clear register */
#define MMU_IRQ_MASK            (MEMORY_MANAGEMENT_BASE + 0x008)   /* (RW) Interrupt mask register */
#define MMU_IRQ_STATUS          (MEMORY_MANAGEMENT_BASE + 0x00C)   /* (RO) Interrupt status register */

// AS0 --> AS7 Used, other reseved
#define MMU_AS0                 0x800   /* Configuration registers for address space 0 */
#define MMU_AS1                 0x840   /* Configuration registers for address space 1 */
#define MMU_AS2                 0x880   /* Configuration registers for address space 2 */
#define MMU_AS3                 0x8C0   /* Configuration registers for address space 3 */
#define MMU_AS4                 0x900   /* Configuration registers for address space 4 */
#define MMU_AS5                 0x940   /* Configuration registers for address space 5 */
#define MMU_AS6                 0x980   /* Configuration registers for address space 6 */
#define MMU_AS7                 0x9C0   /* Configuration registers for address space 7 */
#define MMU_AS8                 0xA00   /* Configuration registers for address space 8 */
#define MMU_AS9                 0xA40   /* Configuration registers for address space 9 */
#define MMU_AS10                0xA80   /* Configuration registers for address space 10 */
#define MMU_AS11                0xAC0   /* Configuration registers for address space 11 */
#define MMU_AS12                0xB00   /* Configuration registers for address space 12 */
#define MMU_AS13                0xB40   /* Configuration registers for address space 13 */
#define MMU_AS14                0xB80   /* Configuration registers for address space 14 */
#define MMU_AS15                0xBC0   /* Configuration registers for address space 15 */

#define MMU_AS_REG(n, r)        (MMU_REG(MMU_AS0 + ((n) << 6)) + (r))
#define MMU_AS(as)                      (MMU_REG(MMU_AS0) + ((as) << 6))
#define AS_TRANSTAB_LO(as)              (MMU_AS(as) + 0x00) /* (RW) Translation Table Base Address for address space n, low word */
#define AS_TRANSTAB_HI(as)              (MMU_AS(as) + 0x04) /* (RW) Translation Table Base Address for address space n, high word */
#define AS_MEMATTR_LO(as)               (MMU_AS(as) + 0x08) /* (RW) Memory attributes for address space n, low word. */
#define AS_MEMATTR_HI(as)               (MMU_AS(as) + 0x0C) /* (RW) Memory attributes for address space n, high word. */
#define AS_LOCKADDR_LO(as)              (MMU_AS(as) + 0x10) /* (RW) Lock region address for address space n, low word */
#define AS_LOCKADDR_HI(as)              (MMU_AS(as) + 0x14) /* (RW) Lock region address for address space n, high word */
#define AS_COMMAND(as)                  (MMU_AS(as) + 0x18) /* (WO) MMU command register for address space n */
#define AS_FAULTSTATUS(as)              (MMU_AS(as) + 0x1C) /* (RO) MMU fault status register for address space n */
#define AS_FAULTADDRESS_LO(as)          (MMU_AS(as) + 0x20) /* (RO) Fault Address for address space n, low word */
#define AS_FAULTADDRESS_HI(as)          (MMU_AS(as) + 0x24) /* (RO) Fault Address for address space n, high word */
#define AS_STATUS(as)                   (MMU_AS(as) + 0x28) /* (RO) Status flags for address space n */
#define AS_TRANSCFG_LO(as)              (MMU_AS(as) + 0x30) /* (RW) Translation table configuration for address space n, low word */
#define AS_TRANSCFG_HI(as)              (MMU_AS(as) + 0x34) /* (RW) Translation table configuration for address space n, high word */
#define AS_FAULTEXTRA_LO(as)            (MMU_AS(as) + 0x38) /* (RO) Secondary fault address for address space n, low word */
#define AS_FAULTEXTRA_HI(as)            (MMU_AS(as) + 0x3C) /* (RO) Secondary fault address for address space n, high word */


/* End Register Offsets */

#define SS0_FEATURES            JS0_FEATURES	/* (RO) Features of stage slot 0 */
#define SS_FEATURES_REG(n)      GPU_CONTROL_REG(SS0_FEATURES + ((n) << 2))

#define SHADER_PRESENT_LO       TPU_PRESENT_LO	/* (RO) Shader core present bitmap, low word */
#define SHADER_PRESENT_HI       TPU_PRESENT_HI	/* (RO) Shader core present bitmap, high word */
#define TILER_PRESENT_LO        PTU_PRESENT_LO	/* (RO) Tiler core present bitmap, low word */
#define TILER_PRESENT_HI        PTU_PRESENT_HI	/* (RO) Tiler core present bitmap, high word */
#define STACK_PRESENT_LO        TPC_PRESENT_LO   /* (RO) Core stack present bitmap, low word */
#define STACK_PRESENT_HI        TPC_PRESENT_HI   /* (RO) Core stack present bitmap, high word */

#define SHADER_READY_LO         TPU_READY_LO	/* (RO) Shader core ready bitmap, low word */
#define SHADER_READY_HI         TPU_READY_HI	/* (RO) Shader core ready bitmap, high word */
#define TILER_READY_LO          PTU_READY_LO	/* (RO) Tiler core ready bitmap, low word */
#define TILER_READY_HI          PTU_READY_HI	/* (RO) Tiler core ready bitmap, high word */
#define STACK_READY_LO          TPC_READY_LO   /* (RO) Core stack ready bitmap, low word */
#define STACK_READY_HI          TPC_READY_HI   /* (RO) Core stack ready bitmap, high word */

#define SHADER_PWRON_LO         TPU_PWRON_LO	/* (WO) Shader core power on bitmap, low word */
#define SHADER_PWRON_HI         TPU_PWRON_HI	/* (WO) Shader core power on bitmap, high word */
#define TILER_PWRON_LO          PTU_PWRON_LO	/* (WO) Tiler core power on bitmap, low word */
#define TILER_PWRON_HI          PTU_PWRON_HI	/* (WO) Tiler core power on bitmap, high word */
#define STACK_PWRON_LO          TPC_PWRON_LO   /* (RO) Core stack power on bitmap, low word */
#define STACK_PWRON_HI          TPC_PWRON_HI   /* (RO) Core stack power on bitmap, high word */

#define SHADER_PWROFF_LO        TPU_PWROFF_LO	/* (WO) Shader core power off bitmap, low word */
#define SHADER_PWROFF_HI        TPU_PWROFF_HI	/* (WO) Shader core power off bitmap, high word */
#define TILER_PWROFF_LO         PTU_PWROFF_LO	/* (WO) Tiler core power off bitmap, low word */
#define TILER_PWROFF_HI         PTU_PWROFF_LO	/* (WO) Tiler core power off bitmap, high word */
#define STACK_PWROFF_LO         TPC_PWROFF_LO   /* (RO) Core stack power off bitmap, low word */
#define STACK_PRWOFF_HI         TPC_PWROFF_LO   /* (RO) Core stack power off bitmap, high word */

#define SHADER_PWRTRANS_LO      TPU_PWRTRANS_LO 	/* (RO) Shader core power transition bitmap, low word */
#define SHADER_PWRTRANS_HI      TPU_PWRTRANS_HI	/* (RO) Shader core power transition bitmap, high word */
#define TILER_PWRTRANS_LO       PTU_PWRTRANS_LO	/* (RO) Tiler core power transition bitmap, low word */
#define TILER_PWRTRANS_HI       PTU_PWRTRANS_HI	/* (RO) Tiler core power transition bitmap, high word */
#define STACK_PWRTRANS_LO       TPC_PWRTRANS_LO   /* (RO) Core stack power transition bitmap, low word */
#define STACK_PRWTRANS_HI       TPC_PWRTRANS_HI   /* (RO) Core stack power transition bitmap, high word */

#define SHADER_PWRACTIVE_LO     TPU_PWRACTIVE_LO	/* (RO) Shader core active bitmap, low word */
#define SHADER_PWRACTIVE_HI     TPU_PWRACTIVE_HI	/* (RO) Shader core active bitmap, high word */
#define TILER_PWRACTIVE_LO      PTU_PWRACTIVE_LO	/* (RO) Tiler core active bitmap, low word */
#define TILER_PWRACTIVE_HI      PTU_PWRACTIVE_HI	/* (RO) Tiler core active bitmap, high word */

#define SM_CONFIG               GCC_CONFIG   /* (RW) Stage Manager configuration register (Implementation specific register) */
#define SHADER_CONFIG           TPU_CONFIG	/* (RW) Shader core configuration settings (Implementation specific register) */
#define TILER_CONFIG            PTU_CONFIG   /* (RW) Tiler core configuration settings (Implementation specific register) */
#define L2_MMU_CONFIG           L2_CONFIG	/* (RW) Configuration of the L2 cache and MMU (Implementation specific register) */

/*
 * MMU_IRQ_RAWSTAT register values. Values are valid also for
   MMU_IRQ_CLEAR, MMU_IRQ_MASK, MMU_IRQ_STATUS registers.
 */

#define MMU_PAGE_FAULT_FLAGS   16

/* Macros returning a bitmask to retrieve page fault or bus error flags from
 * MMU registers */
#define MMU_PAGE_FAULT(n)      (1UL << (n))
#define MMU_BUS_ERROR(n)       (1UL << ((n) + MMU_PAGE_FAULT_FLAGS))

/*
 * Begin LPAE MMU TRANSTAB register values
 */
#define AS_TRANSTAB_LPAE_ADDR_SPACE_MASK   			0xfffff000
#define AS_TRANSTAB_LPAE_ADRMODE_UNMAPPED  			(0u << 0)
#define AS_TRANSTAB_LPAE_ADRMODE_IDENTITY  			(1u << 1)
#define AS_TRANSTAB_LPAE_ADRMODE_TABLE     			(3u << 0)

#define AS_TRANSTAB_LPAE_READ_INNER        			(1u << 2)
#define AS_TRANSTAB_LPAE_SHARE_OUTER       			(1u << 4)

#define AS_TRANSTAB_LPAE_ADRMODE_MASK      			0x00000003
/*
 * Begin AARCH64 MMU TRANSTAB CFG register values
 */
#define MMU_HW_OUTA_BITS 40
#define AS_TRANSTAB_BASE_MASK 					((1ULL << MMU_HW_OUTA_BITS) - (1ULL << 4))


/*
 * Begin MMU STATUS register values
 */
#define AS_STATUS_AS_ACTIVE 0x01

#define AS_FAULTSTATUS_EXCEPTION_CODE_MASK                    	(0x7<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_TRANSLATION_FAULT       	(0x0<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_PERMISSION_FAULT        	(0x1<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_TRANSTAB_BUS_FAULT      	(0x2<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_ACCESS_FLAG             	(0x3<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_ADDRESS_SIZE_FAULT      	(0x4<<3)
#define AS_FAULTSTATUS_EXCEPTION_CODE_MEMORY_ATTRIBUTES_FAULT 	(0x5<<3)

#define FAULTSTATUS_ACCESS_TYPE_MASK	(3u << 8)
#define FAULTSTATUS_ACCESS_TYPE_UNKNOW	(0u << 8)
#define FAULTSTATUS_ACCESS_TYPE_EX	(1u << 8)
#define FAULTSTATUS_ACCESS_TYPE_READ	(2u << 8)
#define FAULTSTATUS_ACCESS_TYPE_WRITE	(3u << 8)
#define AS_FAULTSTATUS_ACCESS_TYPE_MASK				FAULTSTATUS_ACCESS_TYPE_MASK 
#define AS_FAULTSTATUS_ACCESS_TYPE_ATOMIC			FAULTSTATUS_ACCESS_TYPE_UNKNOW
#define AS_FAULTSTATUS_ACCESS_TYPE_EX 				FAULTSTATUS_ACCESS_TYPE_EX
#define AS_FAULTSTATUS_ACCESS_TYPE_READ				FAULTSTATUS_ACCESS_TYPE_READ
#define AS_FAULTSTATUS_ACCESS_TYPE_WRITE			FAULTSTATUS_ACCESS_TYPE_WRITE

#if 0
#define AS_TRANSCFG_ADRMODE_LEGACY				(0u << 0)
#define AS_TRANSCFG_ADRMODE_UNMAPPED			(1u << 0)
#define AS_TRANSCFG_ADRMODE_IDENTITY			(2u << 0)
#define AS_TRANSCFG_ADRMODE_AARCH64_4K			(6u << 0)
#define AS_TRANSCFG_ADRMODE_AARCH64_64K			(8u << 0)

#define AS_TRANSCFG_ADRMODE_MASK				(0xF << 0)
#define AS_TRANSCFG_PTW_MEMATTR_MASK			(3u << 24)
#define AS_TRANSCFG_PTW_MEMATTR_NON_CACHEABLE 	(1u << 24)
#define AS_TRANSCFG_PTW_MEMATTR_WRITE_BACK 		(2u << 24)
#endif

#define AS_TRANSCFG_PTW_SH_MASK 				(3u << 28)
#define AS_TRANSCFG_PTW_SH_OS 					(2u << 28)
#define AS_TRANSCFG_PTW_SH_IS 					(3u << 28)
/*
 * Begin Command Values
 */

/* SS_COMMAND register commands */
#define SS_COMMAND_NOP         					0x00	/* NOP Operation. Writing this value is ignored */
#define SS_COMMAND_START       					0x01	/* Start processing a stage chain. Writing this value is ignored */
#define SS_COMMAND_SOFT_STOP   					0x02	/* Gently stop processing a stage chain */
#define SS_COMMAND_HARD_STOP   					0x03	/* Rudely stop processing a stage chain */
#define SS_COMMAND_SOFT_STOP_0 					0x04	/* Execute SOFT_STOP if STAGE_CHAIN_FLAG is 0 */
#define SS_COMMAND_HARD_STOP_0 					0x05	/* Execute HARD_STOP if STAGE_CHAIN_FLAG is 0 */
#define SS_COMMAND_SOFT_STOP_1 					0x06	/* Execute SOFT_STOP if STAGE_CHAIN_FLAG is 1 */
#define SS_COMMAND_HARD_STOP_1 					0x07	/* Execute HARD_STOP if STAGE_CHAIN_FLAG is 1 */

#define SS_COMMAND_MASK        					0x07    /* Mask of bits currently in use by the HW */

/* AS_COMMAND register commands */
#define AS_COMMAND_NOP         					0x00	/* NOP Operation */
#define AS_COMMAND_UPDATE      					0x01	/* Broadcasts the values in AS_TRANSTAB and ASn_MEMATTR to all MMUs */
#define AS_COMMAND_LOCK        					0x02	/* Issue a lock region command to all MMUs */
#define AS_COMMAND_UNLOCK      					0x03	/* Issue a flush region command to all MMUs */
#define AS_COMMAND_FLUSH       					0x04	/* Flush all L2 caches then issue a flush region command to all MMUs
					 				(deprecated - only for use with T60x) */
#define AS_COMMAND_FLUSH_PT    					0x04	/* Flush all L2 caches then issue a flush region command to all MMUs */
#define AS_COMMAND_FLUSH_MEM   					0x05	/* Wait for memory accesses to complete, flush all the L1s cache then
				 					flush all L2 caches then issue a flush region command to all MMUs */

/* Possible values of SS_CONFIG and SS_CONFIG_NEXT registers */
#define SS_CONFIG_START_FLUSH_NO_ACTION        			(0u << 0)
#define SS_CONFIG_START_FLUSH_CLEAN            			(1u << 8)
#define SS_CONFIG_START_FLUSH_CLEAN_INVALIDATE 			(3u << 8)
#define SS_CONFIG_START_MMU                    			(1u << 10)
#define SS_CONFIG_STAGE_CHAIN_FLAG             			(1u << 11)
#define SS_CONFIG_END_FLUSH_NO_ACTION          			SS_CONFIG_START_FLUSH_NO_ACTION
#define SS_CONFIG_END_FLUSH_CLEAN              			(1u << 12)
#define SS_CONFIG_END_FLUSH_CLEAN_INVALIDATE   			(3u << 12)
#define SS_CONFIG_ENABLE_FLUSH_REDUCTION       			(1u << 14)
#define SS_CONFIG_DISABLE_DESCRIPTOR_WR_BK     			(1u << 15)
#define SS_CONFIG_THREAD_PRI(n)                			((n) << 16)

/* SS_XAFFINITY register values */
#define SS_XAFFINITY_XAFFINITY_ENABLE 				(1u << 0)
#define SS_XAFFINITY_TILER_ENABLE     				(1u << 8)
#define SS_XAFFINITY_CACHE_ENABLE     				(1u << 16)

/* SS_STATUS register values */

/* NOTE: Please keep this values in sync with enum gb_sd_event_code in btsgpu_kernel.h.
 * The values are separated to avoid dependency of userspace and kernel code.
 */

/* Group of values representing the stage status insead a particular fault */
#define SS_STATUS_NO_EXCEPTION_BASE   				0x00
#define SS_STATUS_INTERRUPTED         				(SS_STATUS_NO_EXCEPTION_BASE + 0x02)	/* 0x02 means INTERRUPTED */
#define SS_STATUS_STOPPED             				(SS_STATUS_NO_EXCEPTION_BASE + 0x03)	/* 0x03 means STOPPED */
#define SS_STATUS_TERMINATED          				(SS_STATUS_NO_EXCEPTION_BASE + 0x04)	/* 0x04 means TERMINATED */

/* General fault values */
#define SS_STATUS_FAULT_BASE          				0x40
#define SS_STATUS_CONFIG_FAULT        				(SS_STATUS_FAULT_BASE)	/* 0x40 means CONFIG FAULT */
#define SS_STATUS_POWER_FAULT         				(SS_STATUS_FAULT_BASE + 0x01)	/* 0x41 means POWER FAULT */
#define SS_STATUS_READ_FAULT          				(SS_STATUS_FAULT_BASE + 0x02)	/* 0x42 means READ FAULT */
#define SS_STATUS_WRITE_FAULT        			 	(SS_STATUS_FAULT_BASE + 0x03)	/* 0x43 means WRITE FAULT */
#define SS_STATUS_AFFINITY_FAULT      				(SS_STATUS_FAULT_BASE + 0x04)	/* 0x44 means AFFINITY FAULT */
#define SS_STATUS_BUS_FAULT           				(SS_STATUS_FAULT_BASE + 0x08)	/* 0x48 means BUS FAULT */

/* Instruction or data faults */
#define SS_STATUS_INSTRUCTION_FAULT_BASE  			0x50
#define SS_STATUS_INSTR_INVALID_PC        			(SS_STATUS_INSTRUCTION_FAULT_BASE)	/* 0x50 means INSTR INVALID PC */
#define SS_STATUS_INSTR_INVALID_ENC       			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x01)	/* 0x51 means INSTR INVALID ENC */
#define SS_STATUS_INSTR_TYPE_MISMATCH     			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x02)	/* 0x52 means INSTR TYPE MISMATCH */
#define SS_STATUS_INSTR_OPERAND_FAULT     			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x03)	/* 0x53 means INSTR OPERAND FAULT */
#define SS_STATUS_INSTR_TLS_FAULT         			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x04)	/* 0x54 means INSTR TLS FAULT */
#define SS_STATUS_INSTR_BARRIER_FAULT     			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x05)	/* 0x55 means INSTR BARRIER FAULT */
#define SS_STATUS_INSTR_ALIGN_FAULT       			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x06)	/* 0x56 means INSTR ALIGN FAULT */
/* NOTE: No fault with 0x57 code defined in spec. */
#define SS_STATUS_DATA_INVALID_FAULT      			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x08)	/* 0x58 means DATA INVALID FAULT */
#define SS_STATUS_TILE_RANGE_FAULT        			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x09)	/* 0x59 means TILE RANGE FAULT */
#define SS_STATUS_ADDRESS_RANGE_FAULT     			(SS_STATUS_INSTRUCTION_FAULT_BASE + 0x0A)	/* 0x5A means ADDRESS RANGE FAULT */

/* Other faults */
#define SS_STATUS_MEMORY_FAULT_BASE   				0x60
#define SS_STATUS_OUT_OF_MEMORY       				(SS_STATUS_MEMORY_FAULT_BASE)	/* 0x60 means OUT OF MEMORY */
#define SS_STATUS_UNKNOWN             				0x7F	/* 0x7F means UNKNOWN */

/* GPU_COMMAND values */
#define GPU_COMMAND_NOP                				0x00	/* No operation, nothing happens */
#define GPU_COMMAND_SOFT_RESET         				0x01	/* Stop all external bus interfaces, and then reset the entire GPU. */
#define GPU_COMMAND_HARD_RESET         				0x02	/* Immediately reset the entire GPU. */
#define GPU_COMMAND_PRFCNT_CLEAR       				0x03	/* Clear all performance counters, setting them all to zero. */
#define GPU_COMMAND_PRFCNT_SAMPLE      				0x04	/* Sample all performance counters, writing them out to memory */
#define GPU_COMMAND_CYCLE_COUNT_START  				0x05	/* Starts the cycle counter, and system timestamp propagation */
#define GPU_COMMAND_CYCLE_COUNT_STOP   				0x06	/* Stops the cycle counter, and system timestamp propagation */
#define GPU_COMMAND_CLEAN_CACHES       				0x07	/* Clean all caches */
#define GPU_COMMAND_CLEAN_INV_CACHES   				0x08	/* Clean and invalidate all caches */
#define GPU_COMMAND_SET_PROTECTED_MODE 				0x09	/* Places the GPU in protected mode */

/* End Command Values */

/* GPU_STATUS values */
#define GPU_STATUS_GPU_ACTIVE		   			(1 << 0)	/* Set if GPU active */
#define GPU_STATUS_PWR_ACTIVE		   			(1 << 1)	/* Set if Power manager enable */
#define GPU_STATUS_PRFCNT_ACTIVE           		(1 << 2)	/* Set if the performance counters are active. */
#define GPU_STATUS_JS_ACTIVE		   			(1 << 3)     /* Set if Slot enable */
#define GPU_STATUS_PAGE_FAULT		   			(1 << 4)	/* Set if Page fault enable */
#define GPU_STATUS_IRQ_ACTIVE		   			(1 << 5)	/* Set if IRQ enable */
#define GPU_STATUS_CYCLE_COUNT_ACTIVE	   		(1 << 6)     /* Set if timer counter enable */
#define GPU_STATUS_PROTECTED_MODE_ACTIVE   		(1 << 7)	/* Set if protected mode is active */
#define GPU_STATUS_DEBUG_ENABLED	   			(1 << 8)     /* Set if Debug mode is active */

/* PRFCNT_CONFIG register values */
#define PRFCNT_CONFIG_MODE_SHIFT      				0 /* Counter mode position. */
#define PRFCNT_CONFIG_AS_SHIFT        				4 /* Address space bitmap position. */
#define PRFCNT_CONFIG_SETSELECT_SHIFT 				8 /* Set select position. */

#define PRFCNT_CONFIG_MODE_OFF    				0 /* The performance counters are disabled. */
#define PRFCNT_CONFIG_MODE_MANUAL 				1 /* The performance counters are enabled, but are only written out when a PRFCNT_SAMPLE command is issued using the GPU_COMMAND register. */
#define PRFCNT_CONFIG_MODE_TILE   				2 /* The performance counters are enabled, and are written out each time a tile finishes rendering. */
#define PRFCNT_CONFIG_MODE_MANUAL_NO_CLEAR 		3 /* same as MODE_MANUAL but do not clear the count after sample. */

/* AS<n>_MEMATTR values: */
/* Use GPU implementation-defined caching policy. */
#define AS_MEMATTR_IMPL_DEF_CACHE_POLICY 			0x88ull
/* The attribute set to force all resources to be cached. */
#define AS_MEMATTR_FORCE_TO_CACHE_ALL    			0x8Full
/* Inner write-alloc cache setup, no outer caching */
#define AS_MEMATTR_WRITE_ALLOC           			0x8Dull

/* Set to implementation defined, outer caching */
#define AS_MEMATTR_AARCH64_OUTER_IMPL_DEF 			0x88ull
/* Set to write back memory, outer caching */
#define AS_MEMATTR_AARCH64_OUTER_WA       			0x8Dull

/* Use GPU implementation-defined  caching policy. */
#define AS_MEMATTR_LPAE_IMPL_DEF_CACHE_POLICY 			0x48ull
/* The attribute set to force all resources to be cached. */
#define AS_MEMATTR_LPAE_FORCE_TO_CACHE_ALL    			0x4Full
/* Inner write-alloc cache setup, no outer caching */
#define AS_MEMATTR_LPAE_WRITE_ALLOC           			0x4Dull
/* Set to implementation defined, outer caching */
#define AS_MEMATTR_LPAE_OUTER_IMPL_DEF        			0x88ull
/* Set to write back memory, outer caching */
#define AS_MEMATTR_LPAE_OUTER_WA              			0x8Dull

/* Symbol for default MEMATTR to use */

/* Default is - HW implementation defined caching */
#define AS_MEMATTR_INDEX_DEFAULT               			0
#define AS_MEMATTR_INDEX_DEFAULT_ACE          		 	3

/* HW implementation defined caching */
#define AS_MEMATTR_INDEX_IMPL_DEF_CACHE_POLICY 			0
/* Force cache on */
#define AS_MEMATTR_INDEX_FORCE_TO_CACHE_ALL    			1
/* Write-alloc */
#define AS_MEMATTR_INDEX_WRITE_ALLOC           			2
/* Outer coherent, inner implementation defined policy */
#define AS_MEMATTR_INDEX_OUTER_IMPL_DEF        			3
/* Outer coherent, write alloc inner */
#define AS_MEMATTR_INDEX_OUTER_WA              			4

/* SS<n>_FEATURES register */

#define SS_FEATURE_NULL_STAGE              			(1u << 1)
#define SS_FEATURE_SET_VALUE_STAGE         			(1u << 2)
#define SS_FEATURE_CACHE_FLUSH_STAGE       			(1u << 3)
#define SS_FEATURE_COMPUTE_STAGE           			(1u << 4)
#define SS_FEATURE_VERTEX_STAGE            			(1u << 5)
#define SS_FEATURE_GEOMETRY_STAGE          			(1u << 6)
#define SS_FEATURE_TILER_STAGE             			(1u << 7)
#define SS_FEATURE_FUSED_STAGE             			(1u << 8)
#define SS_FEATURE_FRAGMENT_STAGE          			(1u << 9)
#define SS_FEATURE_IDVS_STAGE		   				(1u << 10)

/* End SS<n>_FEATURES register */

/* L2_MMU_CONFIG register */
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_SHIFT        	(24)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_READS              	(0x0 << L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_OCTANT       	(0x1 << L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_QUARTER      	(0x2 << L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_HALF         	(0x3 << L2_MMU_CONFIG_LIMIT_EXTERNAL_READS_SHIFT)

#define L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_SHIFT       	(26)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES            	 	(0x0 << L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_OCTANT      	(0x1 << L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_QUARTER     	(0x2 << L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_SHIFT)
#define L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_HALF        	(0x3 << L2_MMU_CONFIG_LIMIT_EXTERNAL_WRITES_SHIFT)
/* End L2_MMU_CONFIG register */

/* THREAD_* registers */

/* THREAD_FEATURES IMPLEMENTATION_TECHNOLOGY values */
#define IMPLEMENTATION_UNSPECIFIED  				0
#define IMPLEMENTATION_SILICON      				1
#define IMPLEMENTATION_FPGA         				2
#define IMPLEMENTATION_MODEL        				3

/* Default values when registers are not supported by the implemented hardware */
#define THREAD_MT_DEFAULT     					256
#define THREAD_MWS_DEFAULT    					256
#define THREAD_MBS_DEFAULT    					256
#define THREAD_MR_DEFAULT     					1024
#define THREAD_MTQ_DEFAULT    					4
#define THREAD_MTGS_DEFAULT   					10

/* THREAD_FEATURS */
#define MAX_THREAD_GROUP_SPLIT					0x0A
#define MAX_THREAD_GROUP_SPLIT_MASK				0xFF000000
#define MAX_TASK_QUEUE							0x04
#define MAX_TASK_QUEUE_MASK						0xFF0000
#define MAX_REGISTERS							0x6000
#define MAX_REGISTERS_MASK						0xFFFF

/* End THREAD_* registers */

/* SHADER_CONFIG register */

#define SC_ALT_COUNTERS             		(1ul << 3)
#define SC_OVERRIDE_FWD_PIXEL_KILL  		(1ul << 4)
#define SC_SDC_DISABLE_OQ_DISCARD   		(1ul << 6)
#define SC_LS_ALLOW_ATTR_TYPES      		(1ul << 16)
#define SC_LS_PAUSEBUFFER_DISABLE   		(1ul << 16)
#define SC_LS_ATTR_CHECK_DISABLE    		(1ul << 18)
#define SC_ENABLE_TEXGRD_FLAGS      		(1ul << 25)
/* End SHADER_CONFIG register */

/* TILER_CONFIG register */

#define TC_CLOCK_GATE_OVERRIDE      		(1ul << 0)

/* End TILER_CONFIG register */

/* SM_CONFIG register */

#define SM_TIMESTAMP_OVERRIDE  			(1ul << 0)
#define SM_CLOCK_GATE_OVERRIDE 			(1ul << 1)
#define SM_STAGE_THROTTLE_ENABLE 		(1ul << 2)
#define SM_STAGE_THROTTLE_LIMIT_SHIFT 		(3)
#define SM_MAX_STAGE_THROTTLE_LIMIT 		(0x3F)
#define SM_FORCE_COHERENCY_FEATURES_SHIFT 	(2)
#define SM_IDVS_GROUP_SIZE_SHIFT 		(16)
#define SM_MAX_IDVS_GROUP_SIZE 			(0x3F)
#define SM_CONFIG_UNUSED 			(1ul << 31)
/* End SM_CONFIG register */

/* PWR_OVERRIDE0 */
#define THROTTLE_LIMIT_SHFIT			24u
#define THROTTLE_LIMIT_NONE			(0 << THROTTLE_LIMIT_SHFIT)
#define THROTTLE_LIMIT_INVERT			(1 << THROTTLE_LIMIT_SHFIT)
#define THROTTLE_LIMIT_ON			(2 << THROTTLE_LIMIT_SHFIT)
#define THROTTLE_LIMIT_OFF			(3 << THROTTLE_LIMIT_SHFIT)

#define THROTTLE_ENABLE				23u

#define PWRTRANS_LIMIT_SHFIT			16u
#define PWRTRANS_LIMIT_NONE			(0 << PWRTRANS_LIMIT_SHFIT)
#define PWRTRANS_LIMIT_INVERT			(1 << PWRTRANS_LIMIT_SHFIT)
#define PWRTRANS_LIMIT_ON			(2 << PWRTRANS_LIMIT_SHFIT)
#define PWRTRANS_LIMIT_OFF			(3 << PWRTRANS_LIMIT_SHFIT)

#define FUNC_ISO_ACK_OVERRIDE_SHIFT		14u
#define FUNC_ISO_ACK_OVERRIDE_NONE		(0 << FUNC_ISO_ACK_OVERRIDE_SHIFT)
#define FUNC_ISO_ACK_OVERRIDE_INVERT		(1 << FUNC_ISO_ACK_OVERRIDE_SHIFT)
#define FUNC_ISO_ACK_OVERRIDE_ON		(2 << FUNC_ISO_ACK_OVERRIDE_SHIFT)
#define FUNC_ISO_ACK_OVERRIDE_OFF		(3 << FUNC_ISO_ACK_OVERRIDE_SHIFT)

#define ISOLATE_ACK_OVERRIDE_SHIFT		10u
#define ISOLATE_ACK_OVERRIDE_NONE		(0 << ISOLATE_ACK_OVERRIDE_SHIFT)
#define ISOLATE_ACK_OVERRIDE_INVERT		(1 << ISOLATE_ACK_OVERRIDE_SHIFT)
#define ISOLATE_ACK_OVERRIDE_ON			(2 << ISOLATE_ACK_OVERRIDE_SHIFT)
#define ISOLATE_ACK_OVERRIDE_OFF		(3 << ISOLATE_ACK_OVERRIDE_SHIFT)

#define PWRUP_ACK_OVERRIDE_SHIFT		8u
#define PWRUP_ACK_OVERRIDE_NONE			(0 << PWRUP_ACK_OVERRIDE_SHIFT)
#define PWRUP_ACK_OVERRIDE_INVERT		(1 << PWRUP_ACK_OVERRIDE_SHIFT)
#define PWRUP_ACK_OVERRIDE_ON			(2 << PWRUP_ACK_OVERRIDE_SHIFT)
#define PWRUP_ACK_OVERRIDE_OFF			(3 << PWRUP_ACK_OVERRIDE_SHIFT)

#define RESET_OVERRIDE_SHIFT			4
#define RESET_OVERRIDE_NONE			(0 << RESET_OVERRIDE_SHIFT)
#define RESET_OVERRIDE_INVERT			(1 << RESET_OVERRIDE_SHIFT)
#define RESET_OVERRIDE_ON			(2 << RESET_OVERRIDE_SHIFT)
#define RESET_OVERRIDE_OFF			(3 << RESET_OVERRIDE_SHIFT)

#define ISOLATE_OVERRIDE_SHIFT			2
#define ISOLATE_OVERRIDE_NONE			(0 << ISOLATE_OVERRIDE_SHIFT)
#define ISOLATE_OVERRIDE_INVERT			(1 << ISOLATE_OVERRIDE_SHIFT)
#define ISOLATE_OVERRIDE_ON			(2 << ISOLATE_OVERRIDE_SHIFT)
#define ISOLATE_OVERRIDE_OFF			(3 << ISOLATE_OVERRIDE_SHIFT)

#define PWRUP_OVERRIDE_SHIFT			0
#define PWRUP_OVERRIDE_NONE			(0 << PWRUP_OVERRIDE_SHIFT)
#define PWRUP_OVERRIDE_INVERT			(1 << PWRUP_OVERRIDE_SHIFT)
#define PWRUP_OVERRIDE_ON			(2 << PWRUP_OVERRIDE_SHIFT)
#define PWRUP_OVERRIDE_OFF			(3 << PWRUP_OVERRIDE_SHIFT)

#endif /* _GENBU_REGMAP_H_ */
