/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __GB_JOB_H__
#define __GB_JOB_H__

#include <drm/gpu_scheduler.h>

extern int job_timeout_ms;
//#define DUMP_BO_VRAM_DATA
#define HW_LIMIT_NUM 	1
#define PROC_JOB_TIMEOUT_MS     4
#define GB_STAGE_IDEL_MAX_CNT		10
#define MK_SS_MASK(j)                   (0x10001 << (j))
#define STAGE_IRQ_MASK_ERR(j)             BIT((j) + 16)
#define STAGE_IRQ_MASK_DONE(j)            BIT(j)
#define STAGE_RESET_TIME_MAX	1

#define SLOTS_STA_MASK(NUM_JOB_SLOTS)	((1 << NUM_JOB_SLOTS) - 1)
#define GB_SYS_TIME_LEN                64

struct gb_device;
struct gb_gem_object;
struct gb_file_priv;

struct gb_stage {
	struct drm_sched_job base;

	struct kref refcount;

	struct gb_device *gbdev;
	struct gb_file_priv *file_priv;

	struct dma_fence **in_fences;
	u32 in_fence_count;

	struct dma_fence *irq_done_fence;
	int as;

	__u64 sc;
	__u32 slot_req;
	__u32 flush_id;

	struct dma_fence **implicit_fences;
	struct drm_gem_object **bos;
	u32 bo_count;

	struct dma_fence *render_done_fence;

	struct dma_fence *last_scheduled;
	u32 reset_time;

	u64 user_pid;
};

struct gb_remain_job {
	struct spsc_node queue_node;
	struct dma_fence_cb cb;
	struct drm_sched_job *job;
	struct task_struct *pcb;
	struct drm_gpu_scheduler *sched;
};

enum stage_run_err_t {
       OK_SUBMITED_JOB = 0,
       ERR_FINISHED_JOB = -1,
       ERR_PENDING_RST = -2,
       ERR_FENCE_CREATE = -3,
       ERR_STAGES_BUSY = -4,
       ERR_SUBMITTING_JOB = -5,
};
struct gb_queue_state {
	struct drm_gpu_scheduler sched;
	atomic_t status;
	struct mutex lock;
	u64 fence_context;
	u64 emit_seqno;
};

struct gb_stage_slot {
	struct gb_queue_state queue[NUM_JOB_SLOTS];
	spinlock_t fence_lock;
	spinlock_t stages_lock;
};

int sys_printtime(char *time_buf);
int gb_dump_current_bo_data(u64 size);
int gb_stage_init(struct gb_device *gbdev);
void gb_stage_release(struct gb_device *gbdev);
int gb_stage_open(struct gb_file_priv *gb_priv);
void gb_stage_close(struct gb_file_priv *gb_priv);
int gb_stage_push(struct gb_stage *stage);
void gb_stage_put(struct gb_stage *stage);
void gb_stage_enable_interrupts(struct gb_device *gbdev);
void gb_stage_clear_interrupts(struct gb_device *gbdev);
int gb_stage_is_idle(struct gb_device *gbdev);
void gb_wait_sched_complete(struct gb_file_priv *gb_priv);
void gb_stage_sched_start(struct gb_device *gbdev);
void gb_stage_sched_stop(struct gb_device *gbdev);
void gb_change_job_timeout_ms(int time_ms);
void gb_stage_enable_interrupts(struct gb_device *gbdev);
int dump_err_vram_buff(void);
int gb_stage_get_slot(struct gb_stage *stage);

#endif
