#ifndef __GB_TTM_H__
#define __GB_TTM_H__
#include <generated/uapi/linux/version.h>
#include "kms/gb_pcie_map.h"

#define	DRM_FILE_PAGE_OFFSET (0x100000000ULL >> PAGE_SHIFT)
#define MOVE_FOR_DMA 1

/* ttm */
struct ttm_info {
/*#ifndef NO_DRM_GLOBAL_REF*/
#if (!(defined CONFIG_CENTOS || defined SYS_CENTOS7_9_2009) && \
	defined SYS_CENTOS7_COMPILE_ENV) \
	|| (LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0) && \
	!defined CONFIG_CENTOS_OS)
	struct drm_global_reference mem_global_ref;
	struct ttm_bo_global_ref bo_global_ref;
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 13, 0)
	struct ttm_bo_device bdev;
#else
	struct ttm_device bdev;
#endif
	bool initialized;
};

struct gb_mm_info {
	struct ttm_info	ttm;
};
struct gb_ttm_range_manager {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0)
	struct ttm_resource_manager manager;
#endif
	struct drm_mm mm;
	spinlock_t lock;
};
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0)
static inline struct gb_ttm_range_manager *
to_range_manager(struct ttm_resource_manager *man)
{
	return container_of(man, struct gb_ttm_range_manager, manager);
}
#endif

int gb_ttm_mmap(struct file *filp, struct vm_area_struct *vma);
int gb_ttm_bo_create(struct drm_device *dev, size_t size, int align,
			  int init_domain, uint32_t flags,
			  bool kernel, struct gb_gem_ttm_bo *ttm_bo);
void gb_ttm_bo_unref(struct gb_gem_ttm_bo *ttm_bo);

u64 gb_ttm_bo_gpu_offset(struct gb_gem_ttm_bo *bo);
u64 gb_ttm_bo_mmap_offset(struct gb_gem_ttm_bo *bo);
int gb_ttm_get_bo_offset(struct gb_gem_ttm_bo *bo);

int gb_ttm_bo_reserve(struct gb_gem_ttm_bo *ttm_bo, bool no_intr, int domain);
void gb_ttm_bo_unreserve(struct gb_gem_ttm_bo *ttm_bo);

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)
int gb_ttm_bo_pin(struct gb_gem_ttm_bo *bo, u32 pl_flag, u64 *gpu_addr);
#else
int gb_ttm_bo_pin(struct gb_gem_ttm_bo *bo, u32 mem_type, u64 *gpu_addr);
#endif
int gb_ttm_bo_unpin(struct gb_gem_ttm_bo *bo);
int gb_ttm_mm_init(struct drm_device *drm_dev,
		struct gb_vram_info *vram_config, struct ttm_info *ttm);
void gb_ttm_mm_fini(struct gb_mm_info	*gb_mm);
int gb_ttm_create_heap_bo(struct gb_gem_object *bo, struct gb_device *gbdev,
	struct gb_file_priv *gb_priv, u64 addr, int clear_intr, int as);
extern int gb_get_mm_size_info(int mem_type);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
struct ttm_resource *
genbu_ttm_resource_manager_first(struct ttm_resource_manager *man,
			   struct ttm_resource_cursor *cursor);
struct ttm_resource *
genbu_ttm_resource_manager_next(struct ttm_resource_manager *man,
			  struct ttm_resource_cursor *cursor,
			  struct ttm_resource *res);

#define genbu_ttm_resource_manager_for_each_res(man, cursor, res)		\
	for (res = genbu_ttm_resource_manager_first(man, cursor); res;	\
	     res = genbu_ttm_resource_manager_next(man, cursor, res))
#endif

#endif
