#ifndef __GB_MEM_TOOL_H
#define __GB_MEM_TOOL_H
#include "gpu/gb_device.h"
#include "kms/gbdc_crtc.h"


enum gb_block_index {
	GB_BLOCK_MMU,
	GB_BLOCK_FB,
	GB_BLOCK_PRIV,
	GB_BLOCK_CURSOR_FIRST, //3
	GB_BLOCK_MAX = 6
};

typedef struct genbu_job_header {
   unsigned long exception_status : 32;
   unsigned long first_incomplete_task : 32;
   unsigned long fault_pointer : 64;
   unsigned long is_64b                : 1;
   unsigned long type                  : 7;
   unsigned long barrier               : 1;
   unsigned long invalidate_cache      : 1;
   unsigned long reserved_1            : 1;
   unsigned long suppress_prefetch     : 1;
   unsigned long enable_texture_mapper : 1;
   unsigned long reserved_2            : 1;
   unsigned long relax_dependency_1    : 1;
   unsigned long relax_dependency_2    : 1;
   unsigned long index                 : 16;
   unsigned long dependency_1 : 16;
   unsigned long dependency_2 : 16;
   unsigned long next : 64;
} genbu_job_header;

void gb_parse_fault_point(struct gb_stage *stage);

#define GB_MEM_TOOL
#ifdef GB_MEM_TOOL
int gb_dump_to_file(char *path, unsigned int size, u32 *buff);
int gb_dump_vram_to_file(char *path, unsigned int size, u64 start_offset,
    int debug);
int gb_dump_tool(unsigned long long offset, int size, int debug);
int gb_register_dump_block(u64 gb_addr, u64 size, int block_index);
void gb_set_fb_block_info(u64 fb_addr, u64 size);
int gb_restore_vram_data(void);
int gb_backup_vram_date(struct drm_device *dev, struct gb_device *gbdev);
int gb_dump_init(struct gb_device *gb_dev);
int gb_dump_fb(struct gbdc_crtc *gbdc_crtc, int debug);
void gb_dump_vram(unsigned int size, u64 start_offset, u32 *buff, int debug);
#else
static inline void gb_set_fb_block_info(u64 fb_addr, u64 size) {}
static inline int gb_dump_vram_to_file(char *path, unsigned int size,
    u64 start_offset, int debug)
{
    return 0;
}

static inline int gb_dump_tool(unsigned long long offset, int size, int debug)
{
    return 0;
}

static inline int
gb_register_dump_block(u64 gb_addr, u64 size, int block_index)
{
    return 0;
}

static inline int gb_restore_vram_data(void)
{
    return 0;
}

int gb_backup_vram_date(struct drm_device *dev, struct gb_device *gbdev)
{
    return 0;
}

static inline int gb_dump_init(struct gb_device *gb_dev)
{
    return 0;
}
#endif
#endif
