/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _EDMA_REG_H
#define _EDMA_REG_H

/* EDMA */
#define GB_HDMA_MAX_CH (4) /* max 4 channel for both read & write*/
/* from vram to ram, DMA can transfer 256B one step in ll mode */
#define LL_WRITE_STEP	(PAGE_SIZE)
#define LL_READ_STEP	(PAGE_SIZE)
#define EDMA_CFG_REG_OFFSET	0x60000
#define EDMA_CFG_REG_BAR4_OFFSET	0x1000
#define HDMAC_BAR1_BASE		0x800000
#define HDMAC_INT_MODE_OFFSET	0x21c4
#define HDMAC_INT_MODE_HDMA_INT_PASS_THROUGH_BIT	BIT(1)
#define HDMAC_INT_CLEAR_OFFSET	0x21c8
#define HDMAC_INT_CLEAR_BIT	BIT(0)

#define DMA_SINGLE_BLK_TS_SIZE				(0x4000) /* 16KB */
#define DWC_PCIE_DMA_LL_ELEMENT_LENGTH_U32	(24) /* 6 32-bit double words */
#define DWC_PCIE_DMA_LL_ELEMENT_NUM_MAX		(0x2000) /* maximum 8192 LL data element */
#define DWC_PCIE_DMA_LL_LINK_ELEMENT_NUM	(2) /* 2 link element */

#define COMMON_REG_LEN						(0)
#define CHAN_REG_LEN						(0x200)
#define WR_READ_CHAN_OFF					(0x100)

/* BAR0 register */
#define SYSCTL_SPARE0_OFFSET     0x200
#define SYSCTL_SPARE1_OFFSET     0x204
#define SYSCTL_SPARE2_OFFSET     0x208
#define SYSCTL_SPARE2_HOT_RESET     0x5a
#define SYSCTL_SPARE3_OFFSET     0x20c
#define SYSCTL_SPARE3_REBOOT_VALUE  0x5a
#define SYSCTL_SPARE4_OFFSET     0x210
#define SYSCTL_SPARE4_MCU_BOOT_FINISHED_VALUE     0x55

struct edma_ioctl_single_chnl_t {
	unsigned int dma_chnl; /* 0 --- 3 */
	unsigned int rd_wr_flag; /* 0 : write channel; 1 : read channel */
};

typedef enum {
    DMA_LL_MODE = 0,
    DMA_NON_LL_MODE,
    MULTI_CHNL_DMA_MODE_MAX
} multi_chnl_dma_mode_e;

struct pcie_edma_chan {
	void __iomem *chan_regs;
	u8 id;
	u8 rd_chnl_flag; /* 0 : write channel; 1 : read channel */
    struct hdma_ctrl_info *chip;
};

struct hdma_ctrl_info {
	struct pci_dev *pdev;
	unsigned long long *ll_structure[DWC_PCIE_DMA_LL_ELEMENT_NUM_MAX + DWC_PCIE_DMA_LL_LINK_ELEMENT_NUM];
	unsigned int ll_data_element_num_max;
	unsigned int ll_link_element_num;
	unsigned int dma_data_element_index;
	unsigned int irq_cnt;
	void __iomem *regs;
	void __iomem *hdmac_regs;
	struct pcie_edma_chan wr_chan[GB_HDMA_MAX_CH];
	struct pcie_edma_chan rd_chan[GB_HDMA_MAX_CH];
	struct task_struct *thread[2];
};

static inline u32 chan_reg_read32(struct pcie_edma_chan *chan, u32 offset)
{
	u32 val = 0;

	val = ioread32(chan->chan_regs + offset);
	/* gb_printf(KERN_DEBUG, "%s CHAN%d REG: read 0x%x value: 0x%x\n",
		chan->rd_chnl_flag ? "read" : "write", chan->id, offset, val); */

	return val;
}

static inline void chan_reg_write32(struct pcie_edma_chan *chan, u32 offset, u32 val)
{
	iowrite32(val, chan->chan_regs + offset);

	/* gb_printf(KERN_DEBUG, "%s CHAN%d REG: write 0x%x value: 0x%x\n",
		chan->rd_chnl_flag ? "read" : "write", chan->id, offset, val); */
}

static inline void chan_reg_write64(struct pcie_edma_chan *chan, u32 offset, u64 val)
{
	iowrite32(lower_32_bits(val), chan->chan_regs + offset);
	iowrite32(upper_32_bits(val), chan->chan_regs + offset + 4);

	gb_printf(KERN_DEBUG, "%s CHAN%d REG: write 0x%x value: 0x%llx\n",
		chan->rd_chnl_flag ? "read" : "write", chan->id, offset, val);
}

static inline void hdma_reg_write(u32 val, void __iomem *regbase, u32 offset)
{
	iowrite32(val, regbase + offset);
	/* gb_printf(KERN_DEBUG, "REG: write 0x%x value: 0x%x\n", offset, val); */
}

static inline u32 hdma_reg_read(void __iomem *regbase, u32 offset)
{
	u32 val;

	val = ioread32(regbase + offset);
	/* gb_printf(KERN_DEBUG, "REG: read 0x%x value: 0x%x\n", offset, val); */

	return val;
}

/* HDMA */
#define HDMA_CH_RESERVED			(0)
#define HDMA_CH_RUNNING				(1)
#define HDMA_CH_ABORT				(2)
#define HDMA_CH_STOPPED				(3)
#define HDMA_EN_OFF					(0x00)
#define HDMA_EN_ENABLE_BIT			(1 << 0)
#define HDMA_DOORBELL_OFF			(0x04)
#define HDMA_DOORBELL_START_BIT		(1 << 0)
#define HDMA_DOORBELL_STOP_BIT		(1 << 1)
#define HDMA_ELEM_PF_OFF			(0x08)
#define HDMA_HANDSHAKE_OFF			(0x0c)
#define HDMA_LLP_LOW_OFF			(0x10)
#define HDMA_LLP_HIGH_OFF			(0x14)
#define HDMA_CYCLE_OFF				(0x18)
#define HDMA_CYCLE_BIT_BIT			(1 << 0)
#define HDMA_CYCLE_STATE_BIT		(1 << 1)
#define HDMA_XFERSIZE_OFF			(0x1c)
#define HDMA_SAR_LOW_OFF			(0x20)
#define HDMA_SAR_HIGH_OFF			(0x24)
#define HDMA_DAR_LOW_OFF			(0x28)
#define HDMA_DAR_HIGH_OFF			(0x2c)
#define HDMA_WATERMARK_EN_OFF		(0x30)
#define HDMA_CONTROL1_OFF			(0x34)
#define HDMA_CONTROL1_LLEN_BIT		(1 << 0)
#define HDMA_CONTROL1_MEM_TYPE_BIT	(1 << 1)
#define HDMA_FUNC_NUM_OFF			(0x38)
#define HDMA_QOS_OFF				(0x3c)

#define HDMA_STATUS_OFF				(0x80)
#define HDMA_STATUS_MASK_BITS		(7 << 0)
#define HDMA_STATUS_RUNNING			(1)
#define HDMA_STATUS_ABORTED			(2)
#define HDMA_STATUS_STOPPED			(3)
#define HDMA_INT_STATUS_OFF			(0x84)
#define HDMA_INT_STATUS_STOP		BIT(0)
#define HDMA_INT_STATUS_WATERMARK	BIT(1)
#define HDMA_INT_STATUS_ABORT		BIT(2)
#define HDMA_INT_STATUS_ERROR		(0xf << 3)
#define HDMA_INT_SETUP_OFF			(0x88)
#define HDMA_INT_SETUP_STOP_MASK_BIT		(1 << 0)
#define HDMA_INT_SETUP_WATERMARK_MASK_BIT	(1 << 1)
#define HDMA_INT_SETUP_ABORT_MASK_BIT		(1 << 2)
#define HDMA_INT_SETUP_RSIE_BIT				(1 << 3)
#define HDMA_INT_SETUP_LSIE_BIT				(1 << 4)
#define HDMA_INT_SETUP_RAIE_BIT				(1 << 5)
#define HDMA_INT_SETUP_LAIE_BIT				(1 << 6)
#define HDMA_INT_CLEAR_OFF			(0x8c)
#define HDMA_MSI_STOP_LOW_OFF		(0x90)
#define HDMA_MSI_STOP_HIGH_OFF		(0x94)
#define HDMA_MSI_ABORT_LOW_OFF		(0x98)
#define HDMA_MSI_ABORT_HIGH_OFF		(0x9c)
#define HDMA_MSI_WATERMARK_LOW_OFF	(0xa0)
#define HDMA_MSI_WATERMARK_HIGH_OFF	(0xa4)
#define HDMA_MSI_MSGD_OFF			(0xa8)


/* PCIE DMA LL mode */
#define DMA_LL_MODE_ELEMENT_CTRL_OFF		(0x0)
#define DMA_LL_MODE_ELEMENT_XFER_SIZE_OFF	(0x4)
#define DMA_LL_MODE_ELEMENT_SAR_LOW_OFF		(0x8)
#define DMA_LL_MODE_ELEMENT_SAR_HIGH_OFF	(0xc)
#define DMA_LL_MODE_ELEMENT_DAR_LOW_OFF		(0x10)
#define DMA_LL_MODE_ELEMENT_DAR_HIGH_OFF	(0x14)
#define DMA_LL_MODE_ELEMENT_POINTER_LOW_OFF		(0x8)
#define DMA_LL_MODE_ELEMENT_POINTER_HIGH_OFF	(0xc)

#define DMA_LL_MODE_ELEMENT_CTRL_CB			(1 << 0)
#define DMA_LL_MODE_ELEMENT_CTRL_TCB		(1 << 1)
#define DMA_LL_MODE_ELEMENT_CTRL_LLP		(1 << 2)
#define DMA_LL_MODE_ELEMENT_CTRL_LIE		(1 << 3)
#define DMA_LL_MODE_ELEMENT_CTRL_RIE		(1 << 4)

#endif
