/* Copyright (c) 2016 Synopsys, Inc. */
/*
 * Synopsys DP TX Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 *
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 *
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/uaccess.h>
#include <asm/delay.h>
#include <linux/string.h>

#include "gb_ddr.h"
#include "common/gb_common.h"

#define CS_CNOC_DDR0CFG_PHY_BASE          0x90000000   //16
#define CS_CNOC_DDR0CFG_CTRL_BASE         0x92000000   //16
#define CS_CNOC_DDR0CFG_MISC_BASE         0x93000000   //16
#define CS_CNOC_DDR1CFG_PHY_BASE          0x94000000   //17
#define CS_CNOC_DDR1CFG_CTRL_BASE         0x96000000   //17
#define CS_CNOC_DDR1CFG_MISC_BASE         0x97000000   //17
#define CS_CNOC_DDR2CFG_PHY_BASE          0x98000000   //18
#define CS_CNOC_DDR2CFG_CTRL_BASE         0x9A000000   //18
#define CS_CNOC_DDR2CFG_MISC_BASE         0x9B000000   //18
#define CS_CNOC_DDR3CFG_PHY_BASE          0x9C000000   //19
#define CS_CNOC_DDR3CFG_CTRL_BASE         0x9E000000   //19
#define CS_CNOC_DDR3CFG_MISC_BASE         0x9F000000   //19

static void __iomem *sys_peri_bar;
static void __iomem *peri_base_bar;
static void __iomem *ddr_bar;
static void __iomem *mcu_peri_bar;
static char mdev;

static inline void config_ddr_cfg_base(u32 addr)
{
	iowrite32(addr, peri_base_bar + 0xD10);
}

static inline void write32_bar_map(u32 addr, int value)
{
	iowrite32(value, sys_peri_bar + addr);
	gb_printf(KERN_INFO, "write reg 0x%x\n", addr);
}

static inline uint32_t read32_bar_map(u32 addr)
{
	gb_printf(KERN_INFO, "read reg 0x%x\n", addr);
	return ioread32(sys_peri_bar + addr);
}

void check_ddr_access(void)
{
	int i, data;

	config_ddr_cfg_base(CS_CNOC_DDR0CFG_PHY_BASE);
	for (i = 0; i < 10; i++)
		gb_printf(KERN_INFO, "DDR0_CFG addr 0x%x data 0x%x\n",
		 i * 0x4, read32_bar_map(i * 0x4 + 0x2000000));

	config_ddr_cfg_base(CS_CNOC_DDR0CFG_PHY_BASE);
	for (i = 0; i < 10; i++)
		gb_printf(KERN_INFO, "DDR0_MISC addr 0x%x data 0x%x\n",
		 i * 0x4, read32_bar_map(i * 0x4 + 0x3000000));
}

static inline void ddr_reset(char value)
{
	u32 data;

	gb_printf(KERN_INFO, "sys_peri 0x%p, peri_base 0x%p, value %d\n",
	 sys_peri_bar, peri_base_bar, value);
	iowrite32(0x60000000, peri_base_bar + 0xD04);
	data = ioread32(mcu_peri_bar + 0x158);
	gb_printf(KERN_DEBUG, "kail1 read 0x158 = 0x%x\n", data);
	if (value == 0)
		iowrite32(0x7C000, mcu_peri_bar + 0x158);
	else if (value == 1) {
		switch (mdev) {
		case 0:
			iowrite32(0xFFFFFFF1, mcu_peri_bar + 0x158);
			break;
		case 1:
			iowrite32(0xFFFFFFF3, mcu_peri_bar + 0x158);
			break;
		case 2:
			iowrite32(0xFFFFFFF7, mcu_peri_bar + 0x158);
			break;
		case 3:
			iowrite32(0xFFFFFFFF, mcu_peri_bar + 0x158);
			break;
		default:
			iowrite32(0xFFFFFFFF, mcu_peri_bar + 0x158);
		}
	}
	data = ioread32(mcu_peri_bar + 0x158);
	gb_printf(KERN_DEBUG, "kail2 read 0x158 = 0x%x\n", data);
}

static inline void ddr_mc_clk_en(char value)
{
	u32 data;

	gb_printf(KERN_INFO, "sys_peri 0x%p, peri_base 0x%p, value %d\n",
	 sys_peri_bar, peri_base_bar, value);
	iowrite32(0x60000000, peri_base_bar + 0xD04);
	data = ioread32(mcu_peri_bar + 0x150);
	gb_printf(KERN_DEBUG, "kail1 read 0x150 = 0x%x\n", data);
	if (value == 0)
		iowrite32(data & ~0x780, mcu_peri_bar + 0x150);
	else if (value == 1)
		iowrite32(data | 0x780, mcu_peri_bar + 0x150);
	data = ioread32(mcu_peri_bar + 0x150);
	gb_printf(KERN_DEBUG, "kail2 read 0x150 = 0x%x\n", data);
}

static inline void ddr_pwrok(char value)
{
	u32 data;

	gb_printf(KERN_INFO, "sys_peri 0x%p, peri_base 0x%p, value %d\n",
	 sys_peri_bar, peri_base_bar, value);
	iowrite32(0x60000000, peri_base_bar + 0xD04);
	data = ioread32(mcu_peri_bar + 0x15c);
	gb_printf(KERN_INFO, "kail1 read 0x15c = 0x%x\n", data);
	iowrite32(0xF0F, mcu_peri_bar + 0x15c);
	data = ioread32(mcu_peri_bar + 0x15c);
	gb_printf(KERN_INFO, "kail2 read 0x15c = 0x%x\n", data);
}

static inline void ddr_phy_reset(char value)
{
	u32 data;

	iowrite32(0x60000000, peri_base_bar + 0xD04);
	data = ioread32(mcu_peri_bar + 0x15C);
	switch (mdev) {
	case 0:
		iowrite32(0xF1F, mcu_peri_bar + 0x15C);
		break;
	case 1:
		iowrite32(0xF3F, mcu_peri_bar + 0x15C);
		break;
	case 2:
		iowrite32(0xF7F, mcu_peri_bar + 0x15C);
		break;
	case 3:
		iowrite32(0xFFF, mcu_peri_bar + 0x15C);
		break;
	default:
		iowrite32(0xFFF, mcu_peri_bar + 0x15C);
	}
}

static inline void ddr_cap_type(int size, int interleave)
{
	iowrite32(0x60000000, peri_base_bar + 0xD04);
	iowrite32(interleave << 3 | size, mcu_peri_bar + 0x168);
}

void dwc_ddrc_apb_wr(int addr, int data)
{
	uint32_t val = 0;

	switch (mdev) {
	case 0:
		//Write DDR0 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR0CFG_PHY_BASE);
		write32_bar_map(addr + 0x2000000, data);

	#if 0
		val = read32_bar_map(addr + 0x2000000);
		if (val != data) {
			gb_printf(KERN_INFO, "addr = 0x%x, data = 0x%x, read val = 0x%x.\n",
			 addr, data, val);
	}
	#endif
		break;
	case 1:
		//Write DDR1 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR1CFG_PHY_BASE);
		write32_bar_map(addr + 0x2000000, data);
		break;

	case 2:
		//Write DDR2 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR2CFG_PHY_BASE);
		write32_bar_map(addr + 0x2000000, data);
		break;

	case 3:
		//Write DDR3 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR3CFG_PHY_BASE);
		write32_bar_map(addr + 0x2000000, data);
		break;
	default:
		break;
	}
}

int dwc_ddrc_apb_rd(int addr)
{
	int data;

	switch (mdev) {
	case 0:
		//Read DDR0 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR0CFG_PHY_BASE);
		data = read32_bar_map(addr + 0x2000000);
		break;

	case 1:
		//Read DDR1 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR1CFG_PHY_BASE);
		data = read32_bar_map(addr + 0x2000000);
		break;

	case 2:
		//Read DDR2 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR2CFG_PHY_BASE);
		data = read32_bar_map(addr + 0x2000000);
		break;

	case 3:
		//Read DDR3 Cfg
		config_ddr_cfg_base(CS_CNOC_DDR3CFG_PHY_BASE);
		data = read32_bar_map(addr + 0x2000000);
		break;

	default:
		break;
	}

	return data;
}


void dwc_ddrphy_apb_wr(int addr, int data)
{
	return 0;
	//write32(CS_CNOC_DDR0CFG_PHY_BASE + addr * 4, data);
	//write32(CS_CNOC_DDR1CFG_PHY_BASE + addr * 4, data);
	//write32(CS_CNOC_DDR2CFG_PHY_BASE + addr * 4, data);
	//write32(CS_CNOC_DDR3CFG_PHY_BASE + addr * 4, data);
}

int dwc_ddrphy_apb_rd(int addr)
{
	return 0;
	//int data;
	//data = read32(CS_CNOC_DDR0CFG_PHY_BASE + addr * 4);
	//data = read32(CS_CNOC_DDR1CFG_PHY_BASE + addr * 4);
	//data = read32(CS_CNOC_DDR2CFG_PHY_BASE + addr * 4);
	//data = read32(CS_CNOC_DDR3CFG_PHY_BASE + addr * 4);
	//return data;
}

int get_mail(void)
{
	int mail;

	while ((dwc_ddrphy_apb_rd(0xd0004) & 0x01) == 1) {

	}

	mail = dwc_ddrphy_apb_rd(0xd0032); // Get Major message
	mail = mail & 0xffff;

	if (mail == 0x08)
		// Stream Message
		mail |= (dwc_ddrphy_apb_rd(0xd0032) << 16);

	dwc_ddrphy_apb_wr(0x000d0031, 0);

	while ((dwc_ddrphy_apb_rd(0xd0004) & 0x01) == 0) {

	}

	dwc_ddrphy_apb_wr(0x000d0031, 1);
	return mail;
}

void dwc_ddrphy_phyinit_I_loadPIEImage(void)
{
	dwc_ddrphy_apb_wr(0xd0000, 0x0);
	dwc_ddrphy_apb_wr(0x90000, 0x10);
	dwc_ddrphy_apb_wr(0x90001, 0x400);
	dwc_ddrphy_apb_wr(0x90002, 0x10e);
	dwc_ddrphy_apb_wr(0x90003, 0x0);
	dwc_ddrphy_apb_wr(0x90004, 0x0);
	dwc_ddrphy_apb_wr(0x90005, 0x8);
	dwc_ddrphy_apb_wr(0x90029, 0xb);
	dwc_ddrphy_apb_wr(0x9002a, 0x480);
	dwc_ddrphy_apb_wr(0x9002b, 0x109);
	dwc_ddrphy_apb_wr(0x9002c, 0x8);
	dwc_ddrphy_apb_wr(0x9002d, 0x448);
	dwc_ddrphy_apb_wr(0x9002e, 0x139);
	dwc_ddrphy_apb_wr(0x9002f, 0x8);
	dwc_ddrphy_apb_wr(0x90030, 0x478);
	dwc_ddrphy_apb_wr(0x90031, 0x109);
	dwc_ddrphy_apb_wr(0x90032, 0x2);
	dwc_ddrphy_apb_wr(0x90033, 0x10);
	dwc_ddrphy_apb_wr(0x90034, 0x139);
	dwc_ddrphy_apb_wr(0x90035, 0xb);
	dwc_ddrphy_apb_wr(0x90036, 0x7c0);
	dwc_ddrphy_apb_wr(0x90037, 0x139);
	dwc_ddrphy_apb_wr(0x90038, 0x44);
	dwc_ddrphy_apb_wr(0x90039, 0x633);
	dwc_ddrphy_apb_wr(0x9003a, 0x159);
	dwc_ddrphy_apb_wr(0x9003b, 0x14f);
	dwc_ddrphy_apb_wr(0x9003c, 0x630);
	dwc_ddrphy_apb_wr(0x9003d, 0x159);
	dwc_ddrphy_apb_wr(0x9003e, 0x47);
	dwc_ddrphy_apb_wr(0x9003f, 0x633);
	dwc_ddrphy_apb_wr(0x90040, 0x149);
	dwc_ddrphy_apb_wr(0x90041, 0x4f);
	dwc_ddrphy_apb_wr(0x90042, 0x633);
	dwc_ddrphy_apb_wr(0x90043, 0x179);
	dwc_ddrphy_apb_wr(0x90044, 0x8);
	dwc_ddrphy_apb_wr(0x90045, 0xe0);
	dwc_ddrphy_apb_wr(0x90046, 0x109);
	dwc_ddrphy_apb_wr(0x90047, 0x0);
	dwc_ddrphy_apb_wr(0x90048, 0x7c8);
	dwc_ddrphy_apb_wr(0x90049, 0x109);
	dwc_ddrphy_apb_wr(0x9004a, 0x0);
	dwc_ddrphy_apb_wr(0x9004b, 0x1);
	dwc_ddrphy_apb_wr(0x9004c, 0x8);
	dwc_ddrphy_apb_wr(0x9004d, 0x0);
	dwc_ddrphy_apb_wr(0x9004e, 0x45a);
	dwc_ddrphy_apb_wr(0x9004f, 0x9);
	dwc_ddrphy_apb_wr(0x90050, 0x0);
	dwc_ddrphy_apb_wr(0x90051, 0x448);
	dwc_ddrphy_apb_wr(0x90052, 0x109);
	dwc_ddrphy_apb_wr(0x90053, 0x40);
	dwc_ddrphy_apb_wr(0x90054, 0x633);
	dwc_ddrphy_apb_wr(0x90055, 0x179);
	dwc_ddrphy_apb_wr(0x90056, 0x1);
	dwc_ddrphy_apb_wr(0x90057, 0x618);
	dwc_ddrphy_apb_wr(0x90058, 0x109);
	dwc_ddrphy_apb_wr(0x90059, 0x40c0);
	dwc_ddrphy_apb_wr(0x9005a, 0x633);
	dwc_ddrphy_apb_wr(0x9005b, 0x149);
	dwc_ddrphy_apb_wr(0x9005c, 0x8);
	dwc_ddrphy_apb_wr(0x9005d, 0x4);
	dwc_ddrphy_apb_wr(0x9005e, 0x48);
	dwc_ddrphy_apb_wr(0x9005f, 0x4040);
	dwc_ddrphy_apb_wr(0x90060, 0x633);
	dwc_ddrphy_apb_wr(0x90061, 0x149);
	dwc_ddrphy_apb_wr(0x90062, 0x0);
	dwc_ddrphy_apb_wr(0x90063, 0x4);
	dwc_ddrphy_apb_wr(0x90064, 0x48);
	dwc_ddrphy_apb_wr(0x90065, 0x40);
	dwc_ddrphy_apb_wr(0x90066, 0x633);
	dwc_ddrphy_apb_wr(0x90067, 0x149);
	dwc_ddrphy_apb_wr(0x90068, 0x10);
	dwc_ddrphy_apb_wr(0x90069, 0x4);
	dwc_ddrphy_apb_wr(0x9006a, 0x18);
	dwc_ddrphy_apb_wr(0x9006b, 0x0);
	dwc_ddrphy_apb_wr(0x9006c, 0x4);
	dwc_ddrphy_apb_wr(0x9006d, 0x78);
	dwc_ddrphy_apb_wr(0x9006e, 0x549);
	dwc_ddrphy_apb_wr(0x9006f, 0x633);
	dwc_ddrphy_apb_wr(0x90070, 0x159);
	dwc_ddrphy_apb_wr(0x90071, 0xd49);
	dwc_ddrphy_apb_wr(0x90072, 0x633);
	dwc_ddrphy_apb_wr(0x90073, 0x159);
	dwc_ddrphy_apb_wr(0x90074, 0x94a);
	dwc_ddrphy_apb_wr(0x90075, 0x633);
	dwc_ddrphy_apb_wr(0x90076, 0x159);
	dwc_ddrphy_apb_wr(0x90077, 0x441);
	dwc_ddrphy_apb_wr(0x90078, 0x633);
	dwc_ddrphy_apb_wr(0x90079, 0x149);
	dwc_ddrphy_apb_wr(0x9007a, 0x42);
	dwc_ddrphy_apb_wr(0x9007b, 0x633);
	dwc_ddrphy_apb_wr(0x9007c, 0x149);
	dwc_ddrphy_apb_wr(0x9007d, 0x1);
	dwc_ddrphy_apb_wr(0x9007e, 0x633);
	dwc_ddrphy_apb_wr(0x9007f, 0x149);
	dwc_ddrphy_apb_wr(0x90080, 0x0);
	dwc_ddrphy_apb_wr(0x90081, 0xe0);
	dwc_ddrphy_apb_wr(0x90082, 0x109);
	dwc_ddrphy_apb_wr(0x90083, 0xa);
	dwc_ddrphy_apb_wr(0x90084, 0x10);
	dwc_ddrphy_apb_wr(0x90085, 0x109);
	dwc_ddrphy_apb_wr(0x90086, 0x9);
	dwc_ddrphy_apb_wr(0x90087, 0x3c0);
	dwc_ddrphy_apb_wr(0x90088, 0x149);
	dwc_ddrphy_apb_wr(0x90089, 0x9);
	dwc_ddrphy_apb_wr(0x9008a, 0x3c0);
	dwc_ddrphy_apb_wr(0x9008b, 0x159);
	dwc_ddrphy_apb_wr(0x9008c, 0x18);
	dwc_ddrphy_apb_wr(0x9008d, 0x10);
	dwc_ddrphy_apb_wr(0x9008e, 0x109);
	dwc_ddrphy_apb_wr(0x9008f, 0x0);
	dwc_ddrphy_apb_wr(0x90090, 0x3c0);
	dwc_ddrphy_apb_wr(0x90091, 0x109);
	dwc_ddrphy_apb_wr(0x90092, 0x18);
	dwc_ddrphy_apb_wr(0x90093, 0x4);
	dwc_ddrphy_apb_wr(0x90094, 0x48);
	dwc_ddrphy_apb_wr(0x90095, 0x18);
	dwc_ddrphy_apb_wr(0x90096, 0x4);
	dwc_ddrphy_apb_wr(0x90097, 0x58);
	dwc_ddrphy_apb_wr(0x90098, 0xb);
	dwc_ddrphy_apb_wr(0x90099, 0x10);
	dwc_ddrphy_apb_wr(0x9009a, 0x109);
	dwc_ddrphy_apb_wr(0x9009b, 0x1);
	dwc_ddrphy_apb_wr(0x9009c, 0x10);
	dwc_ddrphy_apb_wr(0x9009d, 0x109);
	dwc_ddrphy_apb_wr(0x9009e, 0x5);
	dwc_ddrphy_apb_wr(0x9009f, 0x7c0);
	dwc_ddrphy_apb_wr(0x900a0, 0x109);
	dwc_ddrphy_apb_wr(0x900a1, 0x0);
	dwc_ddrphy_apb_wr(0x900a2, 0x8140);
	dwc_ddrphy_apb_wr(0x900a3, 0x10c);
	dwc_ddrphy_apb_wr(0x900a4, 0x10);
	dwc_ddrphy_apb_wr(0x900a5, 0x8138);
	dwc_ddrphy_apb_wr(0x900a6, 0x104);
	dwc_ddrphy_apb_wr(0x900a7, 0x8);
	dwc_ddrphy_apb_wr(0x900a8, 0x448);
	dwc_ddrphy_apb_wr(0x900a9, 0x109);
	dwc_ddrphy_apb_wr(0x900aa, 0xf);
	dwc_ddrphy_apb_wr(0x900ab, 0x7c0);
	dwc_ddrphy_apb_wr(0x900ac, 0x109);
	dwc_ddrphy_apb_wr(0x900ad, 0x47);
	dwc_ddrphy_apb_wr(0x900ae, 0x630);
	dwc_ddrphy_apb_wr(0x900af, 0x109);
	dwc_ddrphy_apb_wr(0x900b0, 0x8);
	dwc_ddrphy_apb_wr(0x900b1, 0x618);
	dwc_ddrphy_apb_wr(0x900b2, 0x109);
	dwc_ddrphy_apb_wr(0x900b3, 0x8);
	dwc_ddrphy_apb_wr(0x900b4, 0xe0);
	dwc_ddrphy_apb_wr(0x900b5, 0x109);
	dwc_ddrphy_apb_wr(0x900b6, 0x0);
	dwc_ddrphy_apb_wr(0x900b7, 0x7c8);
	dwc_ddrphy_apb_wr(0x900b8, 0x109);
	dwc_ddrphy_apb_wr(0x900b9, 0x8);
	dwc_ddrphy_apb_wr(0x900ba, 0x8140);
	dwc_ddrphy_apb_wr(0x900bb, 0x10c);
	dwc_ddrphy_apb_wr(0x900bc, 0x0);
	dwc_ddrphy_apb_wr(0x900bd, 0x478);
	dwc_ddrphy_apb_wr(0x900be, 0x109);
	dwc_ddrphy_apb_wr(0x900bf, 0x0);
	dwc_ddrphy_apb_wr(0x900c0, 0x1);
	dwc_ddrphy_apb_wr(0x900c1, 0x8);
	dwc_ddrphy_apb_wr(0x900c2, 0x8);
	dwc_ddrphy_apb_wr(0x900c3, 0x4);
	dwc_ddrphy_apb_wr(0x900c4, 0x0);
	dwc_ddrphy_apb_wr(0x90006, 0x8);
	dwc_ddrphy_apb_wr(0x90007, 0x7c8);
	dwc_ddrphy_apb_wr(0x90008, 0x109);
	dwc_ddrphy_apb_wr(0x90009, 0x0);
	dwc_ddrphy_apb_wr(0x9000a, 0x400);
	dwc_ddrphy_apb_wr(0x9000b, 0x106);
	dwc_ddrphy_apb_wr(0xd00e7, 0x400);
	dwc_ddrphy_apb_wr(0x90017, 0x0);
	dwc_ddrphy_apb_wr(0x90026, 0x2a);
	dwc_ddrphy_apb_wr(0x2000b, 0x64);
	dwc_ddrphy_apb_wr(0x2000c, 0xc8);
	dwc_ddrphy_apb_wr(0x2000d, 0x7d0);

	dwc_ddrphy_apb_wr(0x2000e, 0x2c);
	dwc_ddrphy_apb_wr(0x9000c, 0x0);
	dwc_ddrphy_apb_wr(0x9000d, 0x173);
	dwc_ddrphy_apb_wr(0x9000e, 0x60);
	dwc_ddrphy_apb_wr(0x9000f, 0x6110);
	dwc_ddrphy_apb_wr(0x90010, 0x2152);
	dwc_ddrphy_apb_wr(0x90011, 0xdfbd);
	dwc_ddrphy_apb_wr(0x90012, 0xffff);
	dwc_ddrphy_apb_wr(0x90013, 0x6152);
	dwc_ddrphy_apb_wr(0x20089, 0x1);
	dwc_ddrphy_apb_wr(0x20088, 0x19);
	dwc_ddrphy_apb_wr(0xc0080, 0x0);
	dwc_ddrphy_apb_wr(0xd0000, 0x1);
}

void dwc_ddrphy_phyinit_G_execFW(void)
{
	int mail;

	dwc_ddrphy_apb_wr(0xd0000, 0x1);
	dwc_ddrphy_apb_wr(0xd0099, 0x9);
	dwc_ddrphy_apb_wr(0xd0099, 0x1);

	dwc_ddrphy_apb_wr(0xd0099, 0x0);

	do {
		mail = get_mail();
	} while ((mail != 0x07) & (mail != 0xFF));

	dwc_ddrphy_apb_wr(0xd0099, 0x1);
}

void dwc_ddrphy_phyinit_progCsrSkipTrain(void)
{
	dwc_ddrphy_apb_wr(0x10020, 0x3);
	dwc_ddrphy_apb_wr(0x11020, 0x3);
	dwc_ddrphy_apb_wr(0x12020, 0x3);
	dwc_ddrphy_apb_wr(0x13020, 0x3);
	dwc_ddrphy_apb_wr(0x14020, 0x3);
	dwc_ddrphy_apb_wr(0x15020, 0x3);
	dwc_ddrphy_apb_wr(0x16020, 0x3);
	dwc_ddrphy_apb_wr(0x17020, 0x3);
	dwc_ddrphy_apb_wr(0x20020, 0x3);

	dwc_ddrphy_apb_wr(0x100d0, 0x100);
	dwc_ddrphy_apb_wr(0x101d0, 0x100);
	dwc_ddrphy_apb_wr(0x110d0, 0x100);
	dwc_ddrphy_apb_wr(0x111d0, 0x100);
	dwc_ddrphy_apb_wr(0x120d0, 0x100);
	dwc_ddrphy_apb_wr(0x121d0, 0x100);
	dwc_ddrphy_apb_wr(0x130d0, 0x100);
	dwc_ddrphy_apb_wr(0x131d0, 0x100);
	dwc_ddrphy_apb_wr(0x140d0, 0x100);
	dwc_ddrphy_apb_wr(0x141d0, 0x100);
	dwc_ddrphy_apb_wr(0x150d0, 0x100);
	dwc_ddrphy_apb_wr(0x151d0, 0x100);
	dwc_ddrphy_apb_wr(0x160d0, 0x100);
	dwc_ddrphy_apb_wr(0x161d0, 0x100);
	dwc_ddrphy_apb_wr(0x170d0, 0x100);
	dwc_ddrphy_apb_wr(0x171d0, 0x100);


	dwc_ddrphy_apb_wr(0x100c0, 0x10);
	dwc_ddrphy_apb_wr(0x101c0, 0x10);
	dwc_ddrphy_apb_wr(0x102c0, 0x10);
	dwc_ddrphy_apb_wr(0x103c0, 0x10);
	dwc_ddrphy_apb_wr(0x104c0, 0x10);
	dwc_ddrphy_apb_wr(0x105c0, 0x10);
	dwc_ddrphy_apb_wr(0x106c0, 0x10);
	dwc_ddrphy_apb_wr(0x107c0, 0x10);
	dwc_ddrphy_apb_wr(0x108c0, 0x10);
	dwc_ddrphy_apb_wr(0x110c0, 0x10);
	dwc_ddrphy_apb_wr(0x111c0, 0x10);
	dwc_ddrphy_apb_wr(0x112c0, 0x10);
	dwc_ddrphy_apb_wr(0x113c0, 0x10);
	dwc_ddrphy_apb_wr(0x114c0, 0x10);
	dwc_ddrphy_apb_wr(0x115c0, 0x10);
	dwc_ddrphy_apb_wr(0x116c0, 0x10);
	dwc_ddrphy_apb_wr(0x117c0, 0x10);
	dwc_ddrphy_apb_wr(0x118c0, 0x10);
	dwc_ddrphy_apb_wr(0x120c0, 0x10);
	dwc_ddrphy_apb_wr(0x121c0, 0x10);
	dwc_ddrphy_apb_wr(0x122c0, 0x10);
	dwc_ddrphy_apb_wr(0x123c0, 0x10);
	dwc_ddrphy_apb_wr(0x124c0, 0x10);
	dwc_ddrphy_apb_wr(0x125c0, 0x10);
	dwc_ddrphy_apb_wr(0x126c0, 0x10);
	dwc_ddrphy_apb_wr(0x127c0, 0x10);
	dwc_ddrphy_apb_wr(0x128c0, 0x10);
	dwc_ddrphy_apb_wr(0x130c0, 0x10);
	dwc_ddrphy_apb_wr(0x131c0, 0x10);
	dwc_ddrphy_apb_wr(0x132c0, 0x10);
	dwc_ddrphy_apb_wr(0x133c0, 0x10);
	dwc_ddrphy_apb_wr(0x134c0, 0x10);
	dwc_ddrphy_apb_wr(0x135c0, 0x10);
	dwc_ddrphy_apb_wr(0x136c0, 0x10);
	dwc_ddrphy_apb_wr(0x137c0, 0x10);
	dwc_ddrphy_apb_wr(0x138c0, 0x10);
	dwc_ddrphy_apb_wr(0x140c0, 0x10);
	dwc_ddrphy_apb_wr(0x141c0, 0x10);
	dwc_ddrphy_apb_wr(0x142c0, 0x10);
	dwc_ddrphy_apb_wr(0x143c0, 0x10);
	dwc_ddrphy_apb_wr(0x144c0, 0x10);
	dwc_ddrphy_apb_wr(0x145c0, 0x10);
	dwc_ddrphy_apb_wr(0x146c0, 0x10);
	dwc_ddrphy_apb_wr(0x147c0, 0x10);
	dwc_ddrphy_apb_wr(0x148c0, 0x10);
	dwc_ddrphy_apb_wr(0x150c0, 0x10);
	dwc_ddrphy_apb_wr(0x151c0, 0x10);
	dwc_ddrphy_apb_wr(0x152c0, 0x10);
	dwc_ddrphy_apb_wr(0x153c0, 0x10);
	dwc_ddrphy_apb_wr(0x154c0, 0x10);
	dwc_ddrphy_apb_wr(0x155c0, 0x10);
	dwc_ddrphy_apb_wr(0x156c0, 0x10);
	dwc_ddrphy_apb_wr(0x157c0, 0x10);
	dwc_ddrphy_apb_wr(0x158c0, 0x10);
	dwc_ddrphy_apb_wr(0x160c0, 0x10);
	dwc_ddrphy_apb_wr(0x161c0, 0x10);
	dwc_ddrphy_apb_wr(0x162c0, 0x10);
	dwc_ddrphy_apb_wr(0x163c0, 0x10);
	dwc_ddrphy_apb_wr(0x164c0, 0x10);
	dwc_ddrphy_apb_wr(0x165c0, 0x10);
	dwc_ddrphy_apb_wr(0x166c0, 0x10);
	dwc_ddrphy_apb_wr(0x167c0, 0x10);
	dwc_ddrphy_apb_wr(0x168c0, 0x10);

	dwc_ddrphy_apb_wr(0x170c0, 0x10);
	dwc_ddrphy_apb_wr(0x171c0, 0x10);
	dwc_ddrphy_apb_wr(0x172c0, 0x10);
	dwc_ddrphy_apb_wr(0x173c0, 0x10);
	dwc_ddrphy_apb_wr(0x174c0, 0x10);
	dwc_ddrphy_apb_wr(0x175c0, 0x10);
	dwc_ddrphy_apb_wr(0x176c0, 0x10);
	dwc_ddrphy_apb_wr(0x177c0, 0x10);
	dwc_ddrphy_apb_wr(0x178c0, 0x10);
	dwc_ddrphy_apb_wr(0x10080, 0x100);
	dwc_ddrphy_apb_wr(0x10180, 0x100);
	dwc_ddrphy_apb_wr(0x11080, 0x100);
	dwc_ddrphy_apb_wr(0x11180, 0x100);
	dwc_ddrphy_apb_wr(0x12080, 0x100);
	dwc_ddrphy_apb_wr(0x12180, 0x100);
	dwc_ddrphy_apb_wr(0x13080, 0x100);
	dwc_ddrphy_apb_wr(0x13180, 0x100);
	dwc_ddrphy_apb_wr(0x14080, 0x100);
	dwc_ddrphy_apb_wr(0x14180, 0x100);
	dwc_ddrphy_apb_wr(0x15080, 0x100);
	dwc_ddrphy_apb_wr(0x15180, 0x100);
	dwc_ddrphy_apb_wr(0x16080, 0x100);
	dwc_ddrphy_apb_wr(0x16180, 0x100);
	dwc_ddrphy_apb_wr(0x17080, 0x100);
	dwc_ddrphy_apb_wr(0x17180, 0x100);
	dwc_ddrphy_apb_wr(0x20077, 0x2);
	dwc_ddrphy_apb_wr(0x2007c, 0x54);
	dwc_ddrphy_apb_wr(0x2007d, 0x3f2);
	dwc_ddrphy_apb_wr(0x400c0, 0x10f);
	dwc_ddrphy_apb_wr(0x200cb, 0x61f0);
}

void dwc_ddrphy_phyinit_C_initPhyConfig(void)
{
	dwc_ddrphy_apb_wr(0x1005f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1015f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1105f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1115f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1205f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1215f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1305f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1315f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1405f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1415f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1505f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1515f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1605f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1615f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1705f, 0x2ff);
	dwc_ddrphy_apb_wr(0x1715f, 0x2ff);
	dwc_ddrphy_apb_wr(0x55, 0x3ff);
	dwc_ddrphy_apb_wr(0x1055, 0x3ff);
	dwc_ddrphy_apb_wr(0x2055, 0x3ff);
	dwc_ddrphy_apb_wr(0x3055, 0x3ff);
	dwc_ddrphy_apb_wr(0x4055, 0xff);
	dwc_ddrphy_apb_wr(0x5055, 0xff);
	dwc_ddrphy_apb_wr(0x6055, 0x3ff);
	dwc_ddrphy_apb_wr(0x7055, 0x3ff);
	dwc_ddrphy_apb_wr(0x8055, 0x3ff);
	dwc_ddrphy_apb_wr(0x9055, 0x3ff);
	dwc_ddrphy_apb_wr(0x200c5, 0x19);
	dwc_ddrphy_apb_wr(0x2002e, 0x2);
	dwc_ddrphy_apb_wr(0x20024, 0x8);
	dwc_ddrphy_apb_wr(0x2003a, 0x2);
	dwc_ddrphy_apb_wr(0x2007d, 0x212);
	dwc_ddrphy_apb_wr(0x2007c, 0x61);
	dwc_ddrphy_apb_wr(0x20056, 0x7);
	dwc_ddrphy_apb_wr(0x1004d, 0x18);
	dwc_ddrphy_apb_wr(0x1014d, 0x18);
	dwc_ddrphy_apb_wr(0x1104d, 0x18);
	dwc_ddrphy_apb_wr(0x1114d, 0x18);
	dwc_ddrphy_apb_wr(0x1204d, 0x18);
	dwc_ddrphy_apb_wr(0x1214d, 0x18);
	dwc_ddrphy_apb_wr(0x1304d, 0x18);
	dwc_ddrphy_apb_wr(0x1314d, 0x18);
	dwc_ddrphy_apb_wr(0x1404d, 0x18);
	dwc_ddrphy_apb_wr(0x1414d, 0x18);
	dwc_ddrphy_apb_wr(0x1504d, 0x18);
	dwc_ddrphy_apb_wr(0x1514d, 0x18);
	dwc_ddrphy_apb_wr(0x1604d, 0x18);
	dwc_ddrphy_apb_wr(0x1614d, 0x18);
	dwc_ddrphy_apb_wr(0x1704d, 0x18);
	dwc_ddrphy_apb_wr(0x1714d, 0x18);
	dwc_ddrphy_apb_wr(0x10049, 0x618);
	dwc_ddrphy_apb_wr(0x10149, 0x618);
	dwc_ddrphy_apb_wr(0x11049, 0x618);
	dwc_ddrphy_apb_wr(0x11149, 0x618);
	dwc_ddrphy_apb_wr(0x12049, 0x618);
	dwc_ddrphy_apb_wr(0x12149, 0x618);
	dwc_ddrphy_apb_wr(0x13049, 0x618);
	dwc_ddrphy_apb_wr(0x13149, 0x618);
	dwc_ddrphy_apb_wr(0x14049, 0x618);
	dwc_ddrphy_apb_wr(0x14149, 0x618);
	dwc_ddrphy_apb_wr(0x15049, 0x618);
	dwc_ddrphy_apb_wr(0x15149, 0x618);
	dwc_ddrphy_apb_wr(0x16049, 0x618);
	dwc_ddrphy_apb_wr(0x16149, 0x618);
	dwc_ddrphy_apb_wr(0x17049, 0x618);
	dwc_ddrphy_apb_wr(0x17149, 0x618);
	dwc_ddrphy_apb_wr(0x43, 0x3ff);
	dwc_ddrphy_apb_wr(0x1043, 0x3ff);
	dwc_ddrphy_apb_wr(0x2043, 0x3ff);
	dwc_ddrphy_apb_wr(0x3043, 0x3ff);
	dwc_ddrphy_apb_wr(0x10043, 0x5b1);
	dwc_ddrphy_apb_wr(0x10143, 0x5b1);
	dwc_ddrphy_apb_wr(0x11043, 0x5b1);
	dwc_ddrphy_apb_wr(0x11143, 0x5b1);
	dwc_ddrphy_apb_wr(0x12043, 0x5b1);
	dwc_ddrphy_apb_wr(0x12143, 0x5b1);
	dwc_ddrphy_apb_wr(0x13043, 0x5b1);
	dwc_ddrphy_apb_wr(0x13143, 0x5b1);
	dwc_ddrphy_apb_wr(0x14043, 0x5b1);
	dwc_ddrphy_apb_wr(0x14143, 0x5b1);
	dwc_ddrphy_apb_wr(0x15043, 0x5b1);
	dwc_ddrphy_apb_wr(0x15143, 0x5b1);
	dwc_ddrphy_apb_wr(0x16043, 0x5b1);
	dwc_ddrphy_apb_wr(0x16143, 0x5b1);
	dwc_ddrphy_apb_wr(0x17043, 0x5b1);
	dwc_ddrphy_apb_wr(0x17143, 0x5b1);
	dwc_ddrphy_apb_wr(0x2005b, 0x7529);
	dwc_ddrphy_apb_wr(0x2005c, 0x0);
	dwc_ddrphy_apb_wr(0x200fa, 0x1);
	dwc_ddrphy_apb_wr(0x20019, 0x6);
	dwc_ddrphy_apb_wr(0x200f0, 0x5555);
	dwc_ddrphy_apb_wr(0x200f1, 0x5555);
	dwc_ddrphy_apb_wr(0x200f2, 0x5555);
	dwc_ddrphy_apb_wr(0x200f3, 0x5555);
	dwc_ddrphy_apb_wr(0x200f4, 0x5555);
	dwc_ddrphy_apb_wr(0x200f5, 0x5555);
	dwc_ddrphy_apb_wr(0x200f6, 0x5555);
	dwc_ddrphy_apb_wr(0x200f7, 0xf000);
	dwc_ddrphy_apb_wr(0x20025, 0x0);
	dwc_ddrphy_apb_wr(0x2002d, 0x1);
	dwc_ddrphy_apb_wr(0x2002c, 0x0);
}

void phy_init(void)
{
	dwc_ddrphy_phyinit_C_initPhyConfig();
	dwc_ddrphy_phyinit_progCsrSkipTrain();
	dwc_ddrphy_phyinit_G_execFW();
	dwc_ddrphy_phyinit_I_loadPIEImage();

	dwc_ddrphy_apb_wr(0x000d0000, 0x00000000);
	dwc_ddrphy_apb_wr(0x00020019, 0x00000004);
	dwc_ddrphy_apb_wr(0x000d0000, 0x00000001);
}

void ddr_crm(const void __iomem *mcu_peri_bar_p,
	const void __iomem *sys_peri_bar_p, const void __iomem *ddr_bar_p,
		const void __iomem *peri_base_bar_p)
{
	mcu_peri_bar = mcu_peri_bar_p;
	sys_peri_bar = sys_peri_bar_p;
	ddr_bar = ddr_bar_p;
	peri_base_bar = peri_base_bar_p;

	gb_printf(KERN_INFO, "sys_peri 0x%p, peri_base 0x%p\n",
		sys_peri_bar, peri_base_bar);

	// Palladium Support
	// Enable DDR Clocks
	ddr_mc_clk_en(1);
	ddr_reset(0);
	msleep(1);

	// Hardware pwrokin wire Tie to 1
	ddr_pwrok(1);

	/* Palladium support per DDR 4G*/
	ddr_cap_type(5, 0); //4G ,no interleave

}

void check_ddr0(void)
{
	int i, addr, value;

	iowrite32(0x00000000, peri_base_bar + 0xD14);
	for (i = 0; i < 0x20000000; i++) {
		addr = ddr_bar + i * 4;
		iowrite32(addr, 0x5a5a5a5a);
		value = ioread32(addr);
		if (value != 0x5a5a5a5a)
			gb_printf(KERN_ERR, "write 0x%x error, read 0x%x\n", addr, value);
	}
	gb_printf(KERN_INFO, "1st 0G-2G ok\n");

	iowrite32(0x00000010, peri_base_bar + 0xD14);
	for (i = 0; i < 0x20000000; i++) {
		addr = ddr_bar + i * 4;
		iowrite32(addr, 0x5a5a5a5a);
		value = ioread32(addr);
		if (value != 0x5a5a5a5a)
			gb_printf(KERN_ERR, "write 0x%x error, read 0x%x\n", addr, value);
	}
	gb_printf(KERN_INFO, "1st 2G-4G ok\n");

	iowrite32(0x00000020, peri_base_bar + 0xD14);
	for (i = 0; i < 0x20000000; i++) {
		addr = ddr_bar + i * 4;
		iowrite32(addr, 0x5a5a5a5a);
		value = ioread32(addr);
		if (value != 0x5a5a5a5a)
			gb_printf(KERN_ERR, "write 0x%x error, read 0x%x\n", addr, value);
	}
	gb_printf(KERN_INFO, "1st 4G-6G ok\n");

	iowrite32(0x00000030, peri_base_bar + 0xD14);
	for (i = 0; i < 0x20000000; i++) {
		addr = ddr_bar + i * 4;
		iowrite32(addr, 0x5a5a5a5a);
		value = ioread32(addr);
		if (value != 0x5a5a5a5a)
			gb_printf(KERN_ERR, "write 0x%x error, read 0x%x\n", addr, value);
	}
	gb_printf(KERN_INFO, "1st 6G-8G ok\n");
}

void ddr0_init(void)
{
	int rdata;

	mdev = 0;
	dwc_ddrc_apb_wr(0x00000304, 0x00000001); //  (DBG1) -- DIS DQ
	dwc_ddrc_apb_wr(0x00000030, 0x00000001); //  (PWRCTL) -- SREF EN

	rdata = dwc_ddrc_apb_rd(0x00000004); //  (STAT) EXP = 00000000 -- INIT
	while (rdata != 0x0000) {
		rdata = dwc_ddrc_apb_rd(0x00000004);
		msleep(1000);
	}
	gb_printf(KERN_INFO, "DDR in init status\n");

	// X16, 1RANK, 1:2, No Stack, BL8, Full DQ, NoGearDown,
	// 2T, No-BC, Seq Burst, DDR4
	dwc_ddrc_apb_wr(0x00000000, 0x41040410); //(0x81040410 MSTR)
	dwc_ddrc_apb_wr(0x00000010, 0x00003030); //(           MRCTRL0)
	dwc_ddrc_apb_wr(0x00000014, 0x00013edf); //(           MRCTRL1)
	dwc_ddrc_apb_wr(0x0000001c, 0xe2e762f6); //(           MRCTRL2)
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(PWRCTL) // entry sw-SREF
	dwc_ddrc_apb_wr(0x00000034, 0x00010c18); //(            PWRTMG)
	dwc_ddrc_apb_wr(0x00000038, 0x005d0001); //(           HWLPCTL)
	dwc_ddrc_apb_wr(0x00000050, 0x6041f060); //(          RFSHCTL0)
	dwc_ddrc_apb_wr(0x00000054, 0x002f0046); //(          RFSHCTL1)
	dwc_ddrc_apb_wr(0x00000060, 0x00000010); //(          RFSHCTL3)
	dwc_ddrc_apb_wr(0x00000064, 0x006100d0); //(           RFSHTMG)
	dwc_ddrc_apb_wr(0x00000068, 0x00350000); //(          RFSHTMG1)
	dwc_ddrc_apb_wr(0x00000070, 0x40391f80); //(ECCCFG0) // ECC Disable
	dwc_ddrc_apb_wr(0x00000074, 0x00000530); //(           ECCCFG1)
	dwc_ddrc_apb_wr(0x0000007c, 0x00000300); //(            ECCCTL)
	dwc_ddrc_apb_wr(0x000000b8, 0x00000290); //(    ECCPOISONADDR0)
	dwc_ddrc_apb_wr(0x000000bc, 0x140233d8); //(    ECCPOISONADDR1)
	dwc_ddrc_apb_wr(0x000000c0, 0x00000000); //(        CRCPARCTL0)
	dwc_ddrc_apb_wr(0x000000c4, 0x00000010); //(        CRCPARCTL1)
	//(0xc0030003 INIT0) // SKIP_DRAM_INIT='b11
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x000000d4, 0x00010008); //(             INIT1)
	dwc_ddrc_apb_wr(0x000000d8, 0x00001000); //(             INIT2)
	dwc_ddrc_apb_wr(0x000000dc, 0x0c540001); //(             INIT3)
	dwc_ddrc_apb_wr(0x000000e0, 0x10280440); //(             INIT4)
	dwc_ddrc_apb_wr(0x000000e4, 0x001101fb); //(             INIT5)
	dwc_ddrc_apb_wr(0x000000e8, 0x000004c0); //(  0x14c0     INIT6)
	dwc_ddrc_apb_wr(0x000000ec, 0x00001067); //(             INIT7)
	dwc_ddrc_apb_wr(0x000000f0, 0x00000030); //(           DIMMCTL)
	dwc_ddrc_apb_wr(0x000000f4, 0x0000edd5); //(           RANKCTL)
	dwc_ddrc_apb_wr(0x000000f8, 0x0000000d); //(          RANKCTL1)
	dwc_ddrc_apb_wr(0x00000100, 0x1911361a); //(          DRAMTMG0)
	dwc_ddrc_apb_wr(0x00000104, 0x00050624); //(          DRAMTMG1)
	dwc_ddrc_apb_wr(0x00000108, 0x080c0713); //(          DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000010c, 0x0000400c); //(          DRAMTMG3)
	dwc_ddrc_apb_wr(0x00000110, 0x0a04040b); //(          DRAMTMG4)
	dwc_ddrc_apb_wr(0x00000114, 0x0b090504); //(          DRAMTMG5)
	dwc_ddrc_apb_wr(0x00000118, 0x0c0c0005); //(          DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000011c, 0x0000090b); //(          DRAMTMG7)
	dwc_ddrc_apb_wr(0x00000120, 0x0707120b); //(          DRAMTMG8)
	dwc_ddrc_apb_wr(0x00000124, 0x0002020f); //(          DRAMTMG9)
	dwc_ddrc_apb_wr(0x00000128, 0x00100e0f); //(         DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000012c, 0x1a0b0117); //(         DRAMTMG11)
	dwc_ddrc_apb_wr(0x00000130, 0x0c000010); //(         DRAMTMG12)
	dwc_ddrc_apb_wr(0x00000134, 0x00000000); //(         DRAMTMG13)
	dwc_ddrc_apb_wr(0x00000138, 0x00000000); //(         DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000013c, 0x80000000); //(         DRAMTMG15)
	dwc_ddrc_apb_wr(0x00000180, 0x21000040); //(            ZQCTL0)
	dwc_ddrc_apb_wr(0x00000184, 0x00000070); //(            ZQCTL1)
	dwc_ddrc_apb_wr(0x00000188, 0x00000000); //(            ZQCTL2)
	//( 0x0693820b 0x06bf820b DFITMG0)
	dwc_ddrc_apb_wr(0x00000190, 0x0693820b);
	dwc_ddrc_apb_wr(0x00000194, 0x00080505); //(           DFITMG1)
	dwc_ddrc_apb_wr(0x00000198, 0x07b03030); //(         DFILPCFG0)
	dwc_ddrc_apb_wr(0x0000019c, 0x000000f1); //(         DFILPCFG1)
	dwc_ddrc_apb_wr(0x000001a0, 0xe0400018); //(           DFIUPD0)
	dwc_ddrc_apb_wr(0x000001a4, 0x003c0077); //(           DFIUPD1)
	dwc_ddrc_apb_wr(0x000001a8, 0x80000000); //(           DFIUPD2)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000051); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b4, 0x0000130b); //(0x130b 0x00003f2f DFITMG2)
	dwc_ddrc_apb_wr(0x000001b8, 0x00000008); //(           DFITMG3)
	//(  0x5  DBICTL) // {RD_DBI_EN, WR_DBI_EN, DM_EN}
	dwc_ddrc_apb_wr(0x000001c0, 0x00000001);
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000200, 0x001f1f1f); //(          ADDRMAP0)
	dwc_ddrc_apb_wr(0x00000204, 0x003f0A0A); //(          ADDRMAP1)
	dwc_ddrc_apb_wr(0x00000208, 0x02020200); //(          ADDRMAP2)
	dwc_ddrc_apb_wr(0x0000020c, 0x02020202); //(          ADDRMAP3)
	dwc_ddrc_apb_wr(0x00000210, 0x00001f1f); //(          ADDRMAP4)
	dwc_ddrc_apb_wr(0x00000214, 0x08080808); //(          ADDRMAP5)
	dwc_ddrc_apb_wr(0x00000218, 0x08080808); //(          ADDRMAP6)
	dwc_ddrc_apb_wr(0x0000021c, 0x00000f0f); //(          ADDRMAP7)
	dwc_ddrc_apb_wr(0x00000220, 0x00000101); //(0x00000101 ADDRMAP8)
	dwc_ddrc_apb_wr(0x00000224, 0x08080808); //(          ADDRMAP9)
	dwc_ddrc_apb_wr(0x00000228, 0x08080808); //(         ADDRMAP10)
	dwc_ddrc_apb_wr(0x0000022c, 0x001F1F08); //(         ADDRMAP11)
	dwc_ddrc_apb_wr(0x00000240, 0x061e0d7c); //(            ODTCFG)
	dwc_ddrc_apb_wr(0x00000244, 0x00003032); //(            ODTMAP)
	dwc_ddrc_apb_wr(0x00000250, 0x0125991c); //(             SCHED)
	dwc_ddrc_apb_wr(0x00000254, 0xc1001018); //(            SCHED1)
	dwc_ddrc_apb_wr(0x0000025c, 0x780048bb); //(          PERFHPR1)
	dwc_ddrc_apb_wr(0x00000264, 0xde006a71); //(          PERFLPR1)
	dwc_ddrc_apb_wr(0x0000026c, 0xd30011e2); //(           PERFWR1)
	dwc_ddrc_apb_wr(0x00000270, 0x04110101); //(            SCHED3)
	dwc_ddrc_apb_wr(0x00000274, 0x007a005d); //(            SCHED4)
	dwc_ddrc_apb_wr(0x00000278, 0x2000050a); //(            SCHED5)
	dwc_ddrc_apb_wr(0x00000300, 0x00000011); //(              DBG0)
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	dwc_ddrc_apb_wr(0x0000030c, 0x00000000); //(            DBGCMD)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	dwc_ddrc_apb_wr(0x00000328, 0x00000000); //(       SWCTLSTATIC)
	dwc_ddrc_apb_wr(0x0000036c, 0x00100011); //(         POISONCFG)
	dwc_ddrc_apb_wr(0x00000374, 0x0000012d); //(       ADVECCINDEX)
	dwc_ddrc_apb_wr(0x0000037c, 0x00000000); //(     ECCPOISONPAT0)
	dwc_ddrc_apb_wr(0x00000380, 0x00000000); //(     ECCPOISONPAT1)
	dwc_ddrc_apb_wr(0x00000384, 0x00000000); //(     ECCPOISONPAT2)
	dwc_ddrc_apb_wr(0x00000400, 0x00000100); //(             PCCFG)
	dwc_ddrc_apb_wr(0x00000404, 0x0000026e); //(           PCFGR_0)
	dwc_ddrc_apb_wr(0x00000408, 0x000010ed); //(           PCFGW_0)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(           PCTRL_0)
	dwc_ddrc_apb_wr(0x00000494, 0x00110003); //(        PCFGQOS0_0)
	dwc_ddrc_apb_wr(0x00000f24, 0x0003e662); //(            SBRCTL)
	dwc_ddrc_apb_wr(0x00000f2c, 0x4024fbb2); //(         SBRWDATA0)
	dwc_ddrc_apb_wr(0x00000f30, 0x87daa350); //(         SBRWDATA1)
	dwc_ddrc_apb_wr(0x00000f38, 0x07b4ad98); //(         SBRSTART0)
	dwc_ddrc_apb_wr(0x00000f3c, 0x00000000); //(         SBRSTART1)
	dwc_ddrc_apb_wr(0x00000f40, 0x07b4b90f); //(         SBRRANGE0)
	dwc_ddrc_apb_wr(0x00000f44, 0x00000000); //(         SBRRANGE1)
	dwc_ddrc_apb_wr(0x00002020, 0x00001206); //(    FREQ1.DERATEEN)
	dwc_ddrc_apb_wr(0x00002024, 0x18be8306); //(   FREQ1.DERATEINT)
	dwc_ddrc_apb_wr(0x00002034, 0x00010c18); //(      FREQ1.PWRTMG)
	dwc_ddrc_apb_wr(0x00002050, 0x3041f060); //(    FREQ1.RFSHCTL0)
	dwc_ddrc_apb_wr(0x00002064, 0x006180d0); //(     FREQ1.RFSHTMG)
	dwc_ddrc_apb_wr(0x00002068, 0x00350000); //(    FREQ1.RFSHTMG1)
	dwc_ddrc_apb_wr(0x000020dc, 0x0c540001); //(       FREQ1.INIT3)
	dwc_ddrc_apb_wr(0x000020e0, 0x10280440); //(       FREQ1.INIT4)
	dwc_ddrc_apb_wr(0x000020e8, 0x000014c0); //(       FREQ1.INIT6)
	dwc_ddrc_apb_wr(0x000020ec, 0x00001067); //(       FREQ1.INIT7)
	dwc_ddrc_apb_wr(0x000020f4, 0x00000dd5); //(     FREQ1.RANKCTL)
	dwc_ddrc_apb_wr(0x000020f8, 0x0000000d); //(    FREQ1.RANKCTL1)
	dwc_ddrc_apb_wr(0x00002100, 0x1911361a); //(    FREQ1.DRAMTMG0)
	dwc_ddrc_apb_wr(0x00002104, 0x00050624); //(    FREQ1.DRAMTMG1)
	dwc_ddrc_apb_wr(0x00002108, 0x080c0713); //(    FREQ1.DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000210c, 0x0000400c); //(    FREQ1.DRAMTMG3)
	dwc_ddrc_apb_wr(0x00002110, 0x0a04040b); //(    FREQ1.DRAMTMG4)
	dwc_ddrc_apb_wr(0x00002114, 0x0b090504); //(    FREQ1.DRAMTMG5)
	dwc_ddrc_apb_wr(0x00002118, 0x0c0c0005); //(    FREQ1.DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000211c, 0x0000090b); //(    FREQ1.DRAMTMG7)
	dwc_ddrc_apb_wr(0x00002120, 0x0707120b); //(    FREQ1.DRAMTMG8)
	dwc_ddrc_apb_wr(0x00002124, 0x0002020f); //(    FREQ1.DRAMTMG9)
	dwc_ddrc_apb_wr(0x00002128, 0x00100e0f); //(   FREQ1.DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000212c, 0x1a0b0117); //(   FREQ1.DRAMTMG11)
	dwc_ddrc_apb_wr(0x00002130, 0x0c000010); //(   FREQ1.DRAMTMG12)
	dwc_ddrc_apb_wr(0x00002134, 0x00000000); //(   FREQ1.DRAMTMG13)
	dwc_ddrc_apb_wr(0x00002138, 0x00000000); //(   FREQ1.DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000213c, 0x80000000); //(   FREQ1.DRAMTMG15)
	dwc_ddrc_apb_wr(0x00002180, 0x21000040); //(      FREQ1.ZQCTL0)
	dwc_ddrc_apb_wr(0x00002190, 0x0693820b); //(     FREQ1.DFITMG0)
	dwc_ddrc_apb_wr(0x00002194, 0x00080505); //(     FREQ1.DFITMG1)
	dwc_ddrc_apb_wr(0x000021b4, 0x0000130b); //(     FREQ1.DFITMG2)
	dwc_ddrc_apb_wr(0x000021b8, 0x00000008); //(     FREQ1.DFITMG3)
	dwc_ddrc_apb_wr(0x00002240, 0x061e0d7c); //(      FREQ1.ODTCFG)
	rdata = dwc_ddrc_apb_rd(0x00000060); //(  RFSHCTL3) EXP=0x00000010
	rdata = dwc_ddrc_apb_rd(0x00000030); //(  PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)

	//SYSCTRL->SWRST0.type.ddr0_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr1_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr2_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr3_swrst_n = 1;
	ddr_reset(1);

	//SYSCTRL->SWRST1.type.ddr0_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr1_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr2_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr3_phy_swrst_n = 1;
	ddr_phy_reset(1);

	gb_printf(KERN_INFO, "DDR amnd phy reset release\n");
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026

	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000024); //(PWRCTL) -- PDEN=0
	rdata = dwc_ddrc_apb_rd(0x000001c4);    //(DFIPHYMSTR) EXP=0x44000000
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000320, 0x00000000); //(SWCTL) // SW_DONE = 0
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	//(DFIMISC) // DFI_CMPLETE_EN=0, CTL_IDLE_EN=1
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010);
	dwc_ddrc_apb_wr(0x00000304, 0x00000002); //(DBG1) // DIS_DFI

	/*Skip Phy init step */
	//phy_init();

	rdata = dwc_ddrc_apb_rd(0x00000000);    //(MSTR) EXP=0x41040410
	rdata = dwc_ddrc_apb_rd(0x000001b0);    //(DFIMISC) EXP=0x00000010
	dwc_ddrc_apb_wr(0x000001b0, 0x00000030); //(           DFIMISC)

	gb_printf(KERN_INFO, "detect power status\n");
	while (dwc_ddrc_apb_rd(0x01bc) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001c4, 0x44000000);
	dwc_ddrc_apb_wr(0x00000320, 0x00000000);
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x00000320, 0x00000001);
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x00000304, 0x00000000);
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000006); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(PCTRL_0) // Enable Port 0

}

void ddr1_init(void)
{
	int rdata;

	mdev = 1;
	dwc_ddrc_apb_wr(0x00000304, 0x00000001); //  (DBG1) -- DIS DQ
	dwc_ddrc_apb_wr(0x00000030, 0x00000001); //  (PWRCTL) -- SREF EN

	rdata = dwc_ddrc_apb_rd(0x00000004); //  (STAT) EXP = 00000000 -- INIT
	while (rdata != 0x0000) {
		rdata = dwc_ddrc_apb_rd(0x00000004);
		msleep(1000);
	}
	gb_printf(KERN_INFO, "DDR in init status\n");

	// X8, 1RANK, 1:2, No Stack, BL8, Full DQ, NoGearDown,
	// 2T, No-BC, Seq Burst, DDR4
	dwc_ddrc_apb_wr(0x00000000, 0x41040410); //(              MSTR)
	dwc_ddrc_apb_wr(0x00000010, 0x00003030); //(           MRCTRL0)
	dwc_ddrc_apb_wr(0x00000014, 0x00013edf); //(           MRCTRL1)
	dwc_ddrc_apb_wr(0x0000001c, 0xe2e762f6); //(           MRCTRL2)
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(PWRCTL) // entry sw-SREF
	dwc_ddrc_apb_wr(0x00000034, 0x00010c18); //(            PWRTMG)
	dwc_ddrc_apb_wr(0x00000038, 0x005d0001); //(           HWLPCTL)
	dwc_ddrc_apb_wr(0x00000050, 0x6041f060); //(          RFSHCTL0)
	dwc_ddrc_apb_wr(0x00000054, 0x002f0046); //(          RFSHCTL1)
	dwc_ddrc_apb_wr(0x00000060, 0x00000010); //(          RFSHCTL3)
	dwc_ddrc_apb_wr(0x00000064, 0x006100d0); //(           RFSHTMG)
	dwc_ddrc_apb_wr(0x00000068, 0x00350000); //(          RFSHTMG1)
	dwc_ddrc_apb_wr(0x00000070, 0x40391f80); //(ECCCFG0) // ECC Disable
	dwc_ddrc_apb_wr(0x00000074, 0x00000530); //(           ECCCFG1)
	dwc_ddrc_apb_wr(0x0000007c, 0x00000300); //(            ECCCTL)
	dwc_ddrc_apb_wr(0x000000b8, 0x00000290); //(    ECCPOISONADDR0)
	dwc_ddrc_apb_wr(0x000000bc, 0x140233d8); //(    ECCPOISONADDR1)
	dwc_ddrc_apb_wr(0x000000c0, 0x00000000); //(        CRCPARCTL0)
	dwc_ddrc_apb_wr(0x000000c4, 0x00000010); //(        CRCPARCTL1)
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003); //(NIT0) // SKIP_DRAM_INIT='b11
	dwc_ddrc_apb_wr(0x000000d4, 0x00010008); //(             INIT1)
	dwc_ddrc_apb_wr(0x000000d8, 0x00001000); //(             INIT2)
	dwc_ddrc_apb_wr(0x000000dc, 0x0c540001); //(             INIT3)
	dwc_ddrc_apb_wr(0x000000e0, 0x10280440); //(             INIT4)
	dwc_ddrc_apb_wr(0x000000e4, 0x001101fb); //(             INIT5)
	dwc_ddrc_apb_wr(0x000000e8, 0x000004c0); //(  0x14c0     INIT6)
	dwc_ddrc_apb_wr(0x000000ec, 0x00001067); //(             INIT7)
	dwc_ddrc_apb_wr(0x000000f0, 0x00000030); //(           DIMMCTL)
	dwc_ddrc_apb_wr(0x000000f4, 0x0000edd5); //(           RANKCTL)
	dwc_ddrc_apb_wr(0x000000f8, 0x0000000d); //(          RANKCTL1)
	dwc_ddrc_apb_wr(0x00000100, 0x1911361a); //(          DRAMTMG0)
	dwc_ddrc_apb_wr(0x00000104, 0x00050624); //(          DRAMTMG1)
	dwc_ddrc_apb_wr(0x00000108, 0x080c0713); //(          DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000010c, 0x0000400c); //(          DRAMTMG3)
	dwc_ddrc_apb_wr(0x00000110, 0x0a04040b); //(          DRAMTMG4)
	dwc_ddrc_apb_wr(0x00000114, 0x0b090504); //(          DRAMTMG5)
	dwc_ddrc_apb_wr(0x00000118, 0x0c0c0005); //(          DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000011c, 0x0000090b); //(          DRAMTMG7)
	dwc_ddrc_apb_wr(0x00000120, 0x0707120b); //(          DRAMTMG8)
	dwc_ddrc_apb_wr(0x00000124, 0x0002020f); //(          DRAMTMG9)
	dwc_ddrc_apb_wr(0x00000128, 0x00100e0f); //(         DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000012c, 0x1a0b0117); //(         DRAMTMG11)
	dwc_ddrc_apb_wr(0x00000130, 0x0c000010); //(         DRAMTMG12)
	dwc_ddrc_apb_wr(0x00000134, 0x00000000); //(         DRAMTMG13)
	dwc_ddrc_apb_wr(0x00000138, 0x00000000); //(         DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000013c, 0x80000000); //(         DRAMTMG15)
	dwc_ddrc_apb_wr(0x00000180, 0x21000040); //(            ZQCTL0)
	dwc_ddrc_apb_wr(0x00000184, 0x00000070); //(            ZQCTL1)
	dwc_ddrc_apb_wr(0x00000188, 0x00000000); //(            ZQCTL2)
	dwc_ddrc_apb_wr(0x00000190, 0x0693820b); //(           DFITMG0)
	dwc_ddrc_apb_wr(0x00000194, 0x00080505); //(           DFITMG1)
	dwc_ddrc_apb_wr(0x00000198, 0x07b03030); //(         DFILPCFG0)
	dwc_ddrc_apb_wr(0x0000019c, 0x000000f1); //(         DFILPCFG1)
	dwc_ddrc_apb_wr(0x000001a0, 0xe0400018); //(           DFIUPD0)
	dwc_ddrc_apb_wr(0x000001a4, 0x003c0077); //(           DFIUPD1)
	dwc_ddrc_apb_wr(0x000001a8, 0x80000000); //(           DFIUPD2)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000051); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b4, 0x0000130b); //(           DFITMG2)
	dwc_ddrc_apb_wr(0x000001b8, 0x00000008); //(           DFITMG3)
	//(0x5 DBICTL) // {RD_DBI_EN, WR_DBI_EN, DM_EN}
	dwc_ddrc_apb_wr(0x000001c0, 0x00000001);
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000200, 0x001f1f1f); //(          ADDRMAP0)
	dwc_ddrc_apb_wr(0x00000204, 0x003f0A0A); //(          ADDRMAP1)
	dwc_ddrc_apb_wr(0x00000208, 0x02020200); //(          ADDRMAP2)
	dwc_ddrc_apb_wr(0x0000020c, 0x02020202); //(          ADDRMAP3)
	dwc_ddrc_apb_wr(0x00000210, 0x00001f1f); //(          ADDRMAP4)
	dwc_ddrc_apb_wr(0x00000214, 0x08080808); //(          ADDRMAP5)
	dwc_ddrc_apb_wr(0x00000218, 0x08080808); //(          ADDRMAP6)
	dwc_ddrc_apb_wr(0x0000021c, 0x00000f0f); //(          ADDRMAP7)
	dwc_ddrc_apb_wr(0x00000220, 0x00000101); //(          ADDRMAP8)
	dwc_ddrc_apb_wr(0x00000224, 0x08080808); //(          ADDRMAP9)
	dwc_ddrc_apb_wr(0x00000228, 0x08080808); //(         ADDRMAP10)
	dwc_ddrc_apb_wr(0x0000022c, 0x001F1F08); //(         ADDRMAP11)
	dwc_ddrc_apb_wr(0x00000240, 0x061e0d7c); //(            ODTCFG)
	dwc_ddrc_apb_wr(0x00000244, 0x00003032); //(            ODTMAP)
	dwc_ddrc_apb_wr(0x00000250, 0x0125991c); //(             SCHED)
	dwc_ddrc_apb_wr(0x00000254, 0xc1001018); //(            SCHED1)
	dwc_ddrc_apb_wr(0x0000025c, 0x780048bb); //(          PERFHPR1)
	dwc_ddrc_apb_wr(0x00000264, 0xde006a71); //(          PERFLPR1)
	dwc_ddrc_apb_wr(0x0000026c, 0xd30011e2); //(           PERFWR1)
	dwc_ddrc_apb_wr(0x00000270, 0x04110101); //(            SCHED3)
	dwc_ddrc_apb_wr(0x00000274, 0x007a005d); //(            SCHED4)
	dwc_ddrc_apb_wr(0x00000278, 0x2000050a); //(            SCHED5)
	dwc_ddrc_apb_wr(0x00000300, 0x00000011); //(              DBG0)
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	dwc_ddrc_apb_wr(0x0000030c, 0x00000000); //(            DBGCMD)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	dwc_ddrc_apb_wr(0x00000328, 0x00000000); //(       SWCTLSTATIC)
	dwc_ddrc_apb_wr(0x0000036c, 0x00100011); //(         POISONCFG)
	dwc_ddrc_apb_wr(0x00000374, 0x0000012d); //(       ADVECCINDEX)
	dwc_ddrc_apb_wr(0x0000037c, 0x00000000); //(     ECCPOISONPAT0)
	dwc_ddrc_apb_wr(0x00000380, 0x00000000); //(     ECCPOISONPAT1)
	dwc_ddrc_apb_wr(0x00000384, 0x00000000); //(     ECCPOISONPAT2)
	dwc_ddrc_apb_wr(0x00000400, 0x00000100); //(             PCCFG)
	dwc_ddrc_apb_wr(0x00000404, 0x0000026e); //(           PCFGR_0)
	dwc_ddrc_apb_wr(0x00000408, 0x000010ed); //(           PCFGW_0)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(           PCTRL_0)
	dwc_ddrc_apb_wr(0x00000494, 0x00110003); //(        PCFGQOS0_0)
	dwc_ddrc_apb_wr(0x00000f24, 0x0003e662); //(            SBRCTL)
	dwc_ddrc_apb_wr(0x00000f2c, 0x4024fbb2); //(         SBRWDATA0)
	dwc_ddrc_apb_wr(0x00000f30, 0x87daa350); //(         SBRWDATA1)
	dwc_ddrc_apb_wr(0x00000f38, 0x07b4ad98); //(         SBRSTART0)
	dwc_ddrc_apb_wr(0x00000f3c, 0x00000000); //(         SBRSTART1)
	dwc_ddrc_apb_wr(0x00000f40, 0x07b4b90f); //(         SBRRANGE0)
	dwc_ddrc_apb_wr(0x00000f44, 0x00000000); //(         SBRRANGE1)
	dwc_ddrc_apb_wr(0x00002020, 0x00001206); //(    FREQ1.DERATEEN)
	dwc_ddrc_apb_wr(0x00002024, 0x18be8306); //(   FREQ1.DERATEINT)
	dwc_ddrc_apb_wr(0x00002034, 0x00010c18); //(      FREQ1.PWRTMG)
	dwc_ddrc_apb_wr(0x00002050, 0x3041f060); //(    FREQ1.RFSHCTL0)
	dwc_ddrc_apb_wr(0x00002064, 0x006180d0); //(     FREQ1.RFSHTMG)
	dwc_ddrc_apb_wr(0x00002068, 0x00350000); //(    FREQ1.RFSHTMG1)
	dwc_ddrc_apb_wr(0x000020dc, 0x0c540001); //(       FREQ1.INIT3)
	dwc_ddrc_apb_wr(0x000020e0, 0x10280440); //(       FREQ1.INIT4)
	dwc_ddrc_apb_wr(0x000020e8, 0x000014c0); //(       FREQ1.INIT6)
	dwc_ddrc_apb_wr(0x000020ec, 0x00001067); //(       FREQ1.INIT7)
	dwc_ddrc_apb_wr(0x000020f4, 0x00000dd5); //(     FREQ1.RANKCTL)
	dwc_ddrc_apb_wr(0x000020f8, 0x0000000d); //(    FREQ1.RANKCTL1)
	dwc_ddrc_apb_wr(0x00002100, 0x1911361a); //(    FREQ1.DRAMTMG0)
	dwc_ddrc_apb_wr(0x00002104, 0x00050624); //(    FREQ1.DRAMTMG1)
	dwc_ddrc_apb_wr(0x00002108, 0x080c0713); //(    FREQ1.DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000210c, 0x0000400c); //(    FREQ1.DRAMTMG3)
	dwc_ddrc_apb_wr(0x00002110, 0x0a04040b); //(    FREQ1.DRAMTMG4)
	dwc_ddrc_apb_wr(0x00002114, 0x0b090504); //(    FREQ1.DRAMTMG5)
	dwc_ddrc_apb_wr(0x00002118, 0x0c0c0005); //(    FREQ1.DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000211c, 0x0000090b); //(    FREQ1.DRAMTMG7)
	dwc_ddrc_apb_wr(0x00002120, 0x0707120b); //(    FREQ1.DRAMTMG8)
	dwc_ddrc_apb_wr(0x00002124, 0x0002020f); //(    FREQ1.DRAMTMG9)
	dwc_ddrc_apb_wr(0x00002128, 0x00100e0f); //(   FREQ1.DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000212c, 0x1a0b0117); //(   FREQ1.DRAMTMG11)
	dwc_ddrc_apb_wr(0x00002130, 0x0c000010); //(   FREQ1.DRAMTMG12)
	dwc_ddrc_apb_wr(0x00002134, 0x00000000); //(   FREQ1.DRAMTMG13)
	dwc_ddrc_apb_wr(0x00002138, 0x00000000); //(   FREQ1.DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000213c, 0x80000000); //(   FREQ1.DRAMTMG15)
	dwc_ddrc_apb_wr(0x00002180, 0x21000040); //(      FREQ1.ZQCTL0)
	dwc_ddrc_apb_wr(0x00002190, 0x0693820b); //(     FREQ1.DFITMG0)
	dwc_ddrc_apb_wr(0x00002194, 0x00080505); //(     FREQ1.DFITMG1)
	dwc_ddrc_apb_wr(0x000021b4, 0x0000130b); //(     FREQ1.DFITMG2)
	dwc_ddrc_apb_wr(0x000021b8, 0x00000008); //(     FREQ1.DFITMG3)
	dwc_ddrc_apb_wr(0x00002240, 0x061e0d7c); //(      FREQ1.ODTCFG)
	rdata = dwc_ddrc_apb_rd(0x00000060);    //(RFSHCTL3) EXP=0x00000010
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)

	//SYSCTRL->SWRST0.type.ddr0_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr1_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr2_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr3_swrst_n = 1;
	ddr_reset(1);

	//SYSCTRL->SWRST1.type.ddr0_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr1_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr2_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr3_phy_swrst_n = 1;
	ddr_phy_reset(1);

	gb_printf(KERN_INFO, "DDR amnd phy reset release\n");
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026

	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000024); //(PWRCTL) -- PDEN=0
	rdata = dwc_ddrc_apb_rd(0x000001c4);    //(DFIPHYMSTR) EXP=0x44000000
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000320, 0x00000000); //(SWCTL) // SW_DONE = 0
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC) //
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC
	//(DFIMISC) //DFI_CMPLETE_EN=0, CTL_IDLE_EN=1
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010);
	dwc_ddrc_apb_wr(0x00000304, 0x00000002); //(DBG1) // DIS_DFI

	/*Skip Phy init step */
	//phy_init();

	rdata = dwc_ddrc_apb_rd(0x00000000);    //(MSTR) EXP=0x41040410
	rdata = dwc_ddrc_apb_rd(0x000001b0);    //(DFIMISC) EXP=0x00000010
	dwc_ddrc_apb_wr(0x000001b0, 0x00000030); //(           DFIMISC)

	gb_printf(KERN_INFO, "detect power status\n");
	while (dwc_ddrc_apb_rd(0x01bc) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001c4, 0x44000000);
	dwc_ddrc_apb_wr(0x00000320, 0x00000000);
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x00000320, 0x00000001);
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x00000304, 0x00000000);
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000006); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(PCTRL_0) // Enable Port 0

}

void ddr2_init(void)
{
	int rdata;

	mdev = 2;
	dwc_ddrc_apb_wr(0x00000304, 0x00000001); //  (DBG1) -- DIS DQ
	dwc_ddrc_apb_wr(0x00000030, 0x00000001); //  (PWRCTL) -- SREF EN

	rdata = dwc_ddrc_apb_rd(0x00000004); //(STAT) EXP = 00000000 -- INIT
	while (rdata != 0x0000) {
		rdata = dwc_ddrc_apb_rd(0x00000004);
		msleep(1000);
	}
	gb_printf(KERN_INFO, "DDR in init status\n");

	// X8, 1RANK, 1:2, No Stack, BL8, Full DQ, NoGearDown,
	// 2T, No-BC, Seq Burst, DDR4
	dwc_ddrc_apb_wr(0x00000000, 0x41040410); //(              MSTR)
	dwc_ddrc_apb_wr(0x00000010, 0x00003030); //(           MRCTRL0)
	dwc_ddrc_apb_wr(0x00000014, 0x00013edf); //(           MRCTRL1)
	dwc_ddrc_apb_wr(0x0000001c, 0xe2e762f6); //(           MRCTRL2)
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(PWRCTL) // entry sw-SREF
	dwc_ddrc_apb_wr(0x00000034, 0x00010c18); //(            PWRTMG)
	dwc_ddrc_apb_wr(0x00000038, 0x005d0001); //(           HWLPCTL)
	dwc_ddrc_apb_wr(0x00000050, 0x6041f060); //(          RFSHCTL0)
	dwc_ddrc_apb_wr(0x00000054, 0x002f0046); //(          RFSHCTL1)
	dwc_ddrc_apb_wr(0x00000060, 0x00000010); //(          RFSHCTL3)
	dwc_ddrc_apb_wr(0x00000064, 0x006100d0); //(           RFSHTMG)
	dwc_ddrc_apb_wr(0x00000068, 0x00350000); //(          RFSHTMG1)
	dwc_ddrc_apb_wr(0x00000070, 0x40391f80); //(ECCCFG0) // ECC Disable
	dwc_ddrc_apb_wr(0x00000074, 0x00000530); //(           ECCCFG1)
	dwc_ddrc_apb_wr(0x0000007c, 0x00000300); //(            ECCCTL)
	dwc_ddrc_apb_wr(0x000000b8, 0x00000290); //(    ECCPOISONADDR0)
	dwc_ddrc_apb_wr(0x000000bc, 0x140233d8); //(    ECCPOISONADDR1)
	dwc_ddrc_apb_wr(0x000000c0, 0x00000000); //(        CRCPARCTL0)
	dwc_ddrc_apb_wr(0x000000c4, 0x00000010); //(        CRCPARCTL1)
	//(INIT0) // SKIP_DRAM_INIT='b11
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x000000d4, 0x00010008); //(             INIT1)
	dwc_ddrc_apb_wr(0x000000d8, 0x00001000); //(             INIT2)
	dwc_ddrc_apb_wr(0x000000dc, 0x0c540001); //(             INIT3)
	dwc_ddrc_apb_wr(0x000000e0, 0x10280440); //(             INIT4)
	dwc_ddrc_apb_wr(0x000000e4, 0x001101fb); //(             INIT5)
	dwc_ddrc_apb_wr(0x000000e8, 0x000004c0); //(  0x14c0           INIT6)
	dwc_ddrc_apb_wr(0x000000ec, 0x00001067); //(             INIT7)
	dwc_ddrc_apb_wr(0x000000f0, 0x00000030); //(           DIMMCTL)
	dwc_ddrc_apb_wr(0x000000f4, 0x0000edd5); //(           RANKCTL)
	dwc_ddrc_apb_wr(0x000000f8, 0x0000000d); //(          RANKCTL1)
	dwc_ddrc_apb_wr(0x00000100, 0x1911361a); //(          DRAMTMG0)
	dwc_ddrc_apb_wr(0x00000104, 0x00050624); //(          DRAMTMG1)
	dwc_ddrc_apb_wr(0x00000108, 0x080c0713); //(          DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000010c, 0x0000400c); //(          DRAMTMG3)
	dwc_ddrc_apb_wr(0x00000110, 0x0a04040b); //(          DRAMTMG4)
	dwc_ddrc_apb_wr(0x00000114, 0x0b090504); //(          DRAMTMG5)
	dwc_ddrc_apb_wr(0x00000118, 0x0c0c0005); //(          DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000011c, 0x0000090b); //(          DRAMTMG7)
	dwc_ddrc_apb_wr(0x00000120, 0x0707120b); //(          DRAMTMG8)
	dwc_ddrc_apb_wr(0x00000124, 0x0002020f); //(          DRAMTMG9)
	dwc_ddrc_apb_wr(0x00000128, 0x00100e0f); //(         DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000012c, 0x1a0b0117); //(         DRAMTMG11)
	dwc_ddrc_apb_wr(0x00000130, 0x0c000010); //(         DRAMTMG12)
	dwc_ddrc_apb_wr(0x00000134, 0x00000000); //(         DRAMTMG13)
	dwc_ddrc_apb_wr(0x00000138, 0x00000000); //(         DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000013c, 0x80000000); //(         DRAMTMG15)
	dwc_ddrc_apb_wr(0x00000180, 0x21000040); //(            ZQCTL0)
	dwc_ddrc_apb_wr(0x00000184, 0x00000070); //(            ZQCTL1)
	dwc_ddrc_apb_wr(0x00000188, 0x00000000); //(            ZQCTL2)
	dwc_ddrc_apb_wr(0x00000190, 0x0693820b); //(           DFITMG0)
	dwc_ddrc_apb_wr(0x00000194, 0x00080505); //(           DFITMG1)
	dwc_ddrc_apb_wr(0x00000198, 0x07b03030); //(         DFILPCFG0)
	dwc_ddrc_apb_wr(0x0000019c, 0x000000f1); //(         DFILPCFG1)
	dwc_ddrc_apb_wr(0x000001a0, 0xe0400018); //(           DFIUPD0)
	dwc_ddrc_apb_wr(0x000001a4, 0x003c0077); //(           DFIUPD1)
	dwc_ddrc_apb_wr(0x000001a8, 0x80000000); //(           DFIUPD2)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000051); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b4, 0x0000130b); //(           DFITMG2)
	dwc_ddrc_apb_wr(0x000001b8, 0x00000008); //(           DFITMG3)
	//(  0x5          DBICTL) // {RD_DBI_EN, WR_DBI_EN, DM_EN}
	dwc_ddrc_apb_wr(0x000001c0, 0x00000001);
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000200, 0x001f1f1f); //(          ADDRMAP0)
	dwc_ddrc_apb_wr(0x00000204, 0x003f0A0A); //(          ADDRMAP1)
	dwc_ddrc_apb_wr(0x00000208, 0x02020200); //(          ADDRMAP2)
	dwc_ddrc_apb_wr(0x0000020c, 0x02020202); //(          ADDRMAP3)
	dwc_ddrc_apb_wr(0x00000210, 0x00001f1f); //(          ADDRMAP4)
	dwc_ddrc_apb_wr(0x00000214, 0x08080808); //(          ADDRMAP5)
	dwc_ddrc_apb_wr(0x00000218, 0x08080808); //(          ADDRMAP6)
	dwc_ddrc_apb_wr(0x0000021c, 0x00000f0f); //(          ADDRMAP7)
	dwc_ddrc_apb_wr(0x00000220, 0x00000101); //(          ADDRMAP8)
	dwc_ddrc_apb_wr(0x00000224, 0x08080808); //(          ADDRMAP9)
	dwc_ddrc_apb_wr(0x00000228, 0x08080808); //(         ADDRMAP10)
	dwc_ddrc_apb_wr(0x0000022c, 0x001F1F08); //(         ADDRMAP11)
	dwc_ddrc_apb_wr(0x00000240, 0x061e0d7c); //(            ODTCFG)
	dwc_ddrc_apb_wr(0x00000244, 0x00003032); //(            ODTMAP)
	dwc_ddrc_apb_wr(0x00000250, 0x0125991c); //(             SCHED)
	dwc_ddrc_apb_wr(0x00000254, 0xc1001018); //(            SCHED1)
	dwc_ddrc_apb_wr(0x0000025c, 0x780048bb); //(          PERFHPR1)
	dwc_ddrc_apb_wr(0x00000264, 0xde006a71); //(          PERFLPR1)
	dwc_ddrc_apb_wr(0x0000026c, 0xd30011e2); //(           PERFWR1)
	dwc_ddrc_apb_wr(0x00000270, 0x04110101); //(            SCHED3)
	dwc_ddrc_apb_wr(0x00000274, 0x007a005d); //(            SCHED4)
	dwc_ddrc_apb_wr(0x00000278, 0x2000050a); //(            SCHED5)
	dwc_ddrc_apb_wr(0x00000300, 0x00000011); //(              DBG0)
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	dwc_ddrc_apb_wr(0x0000030c, 0x00000000); //(            DBGCMD)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	dwc_ddrc_apb_wr(0x00000328, 0x00000000); //(       SWCTLSTATIC)
	dwc_ddrc_apb_wr(0x0000036c, 0x00100011); //(         POISONCFG)
	dwc_ddrc_apb_wr(0x00000374, 0x0000012d); //(       ADVECCINDEX)
	dwc_ddrc_apb_wr(0x0000037c, 0x00000000); //(     ECCPOISONPAT0)
	dwc_ddrc_apb_wr(0x00000380, 0x00000000); //(     ECCPOISONPAT1)
	dwc_ddrc_apb_wr(0x00000384, 0x00000000); //(     ECCPOISONPAT2)
	dwc_ddrc_apb_wr(0x00000400, 0x00000100); //(             PCCFG)
	dwc_ddrc_apb_wr(0x00000404, 0x0000026e); //(           PCFGR_0)
	dwc_ddrc_apb_wr(0x00000408, 0x000010ed); //(           PCFGW_0)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(           PCTRL_0)
	dwc_ddrc_apb_wr(0x00000494, 0x00110003); //(        PCFGQOS0_0)
	dwc_ddrc_apb_wr(0x00000f24, 0x0003e662); //(            SBRCTL)
	dwc_ddrc_apb_wr(0x00000f2c, 0x4024fbb2); //(         SBRWDATA0)
	dwc_ddrc_apb_wr(0x00000f30, 0x87daa350); //(         SBRWDATA1)
	dwc_ddrc_apb_wr(0x00000f38, 0x07b4ad98); //(         SBRSTART0)
	dwc_ddrc_apb_wr(0x00000f3c, 0x00000000); //(         SBRSTART1)
	dwc_ddrc_apb_wr(0x00000f40, 0x07b4b90f); //(         SBRRANGE0)
	dwc_ddrc_apb_wr(0x00000f44, 0x00000000); //(         SBRRANGE1)
	dwc_ddrc_apb_wr(0x00002020, 0x00001206); //(    FREQ1.DERATEEN)
	dwc_ddrc_apb_wr(0x00002024, 0x18be8306); //(   FREQ1.DERATEINT)
	dwc_ddrc_apb_wr(0x00002034, 0x00010c18); //(      FREQ1.PWRTMG)
	dwc_ddrc_apb_wr(0x00002050, 0x3041f060); //(    FREQ1.RFSHCTL0)
	dwc_ddrc_apb_wr(0x00002064, 0x006180d0); //(     FREQ1.RFSHTMG)
	dwc_ddrc_apb_wr(0x00002068, 0x00350000); //(    FREQ1.RFSHTMG1)
	dwc_ddrc_apb_wr(0x000020dc, 0x0c540001); //(       FREQ1.INIT3)
	dwc_ddrc_apb_wr(0x000020e0, 0x10280440); //(       FREQ1.INIT4)
	dwc_ddrc_apb_wr(0x000020e8, 0x000014c0); //(       FREQ1.INIT6)
	dwc_ddrc_apb_wr(0x000020ec, 0x00001067); //(       FREQ1.INIT7)
	dwc_ddrc_apb_wr(0x000020f4, 0x00000dd5); //(     FREQ1.RANKCTL)
	dwc_ddrc_apb_wr(0x000020f8, 0x0000000d); //(    FREQ1.RANKCTL1)
	dwc_ddrc_apb_wr(0x00002100, 0x1911361a); //(    FREQ1.DRAMTMG0)
	dwc_ddrc_apb_wr(0x00002104, 0x00050624); //(    FREQ1.DRAMTMG1)
	dwc_ddrc_apb_wr(0x00002108, 0x080c0713); //(    FREQ1.DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000210c, 0x0000400c); //(    FREQ1.DRAMTMG3)
	dwc_ddrc_apb_wr(0x00002110, 0x0a04040b); //(    FREQ1.DRAMTMG4)
	dwc_ddrc_apb_wr(0x00002114, 0x0b090504); //(    FREQ1.DRAMTMG5)
	dwc_ddrc_apb_wr(0x00002118, 0x0c0c0005); //(    FREQ1.DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000211c, 0x0000090b); //(    FREQ1.DRAMTMG7)
	dwc_ddrc_apb_wr(0x00002120, 0x0707120b); //(    FREQ1.DRAMTMG8)
	dwc_ddrc_apb_wr(0x00002124, 0x0002020f); //(    FREQ1.DRAMTMG9)
	dwc_ddrc_apb_wr(0x00002128, 0x00100e0f); //(   FREQ1.DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000212c, 0x1a0b0117); //(   FREQ1.DRAMTMG11)
	dwc_ddrc_apb_wr(0x00002130, 0x0c000010); //(   FREQ1.DRAMTMG12)
	dwc_ddrc_apb_wr(0x00002134, 0x00000000); //(   FREQ1.DRAMTMG13)
	dwc_ddrc_apb_wr(0x00002138, 0x00000000); //(   FREQ1.DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000213c, 0x80000000); //(   FREQ1.DRAMTMG15)
	dwc_ddrc_apb_wr(0x00002180, 0x21000040); //(      FREQ1.ZQCTL0)
	dwc_ddrc_apb_wr(0x00002190, 0x0693820b); //(     FREQ1.DFITMG0)
	dwc_ddrc_apb_wr(0x00002194, 0x00080505); //(     FREQ1.DFITMG1)
	dwc_ddrc_apb_wr(0x000021b4, 0x0000130b); //(     FREQ1.DFITMG2)
	dwc_ddrc_apb_wr(0x000021b8, 0x00000008); //(     FREQ1.DFITMG3)
	dwc_ddrc_apb_wr(0x00002240, 0x061e0d7c); //(      FREQ1.ODTCFG)
	rdata = dwc_ddrc_apb_rd(0x00000060);    //(RFSHCTL3) EXP=0x00000010
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)

	//SYSCTRL->SWRST0.type.ddr0_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr1_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr2_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr3_swrst_n = 1;
	ddr_reset(1);

	//SYSCTRL->SWRST1.type.ddr0_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr1_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr2_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr3_phy_swrst_n = 1;
	ddr_phy_reset(1);

	gb_printf(KERN_INFO, "DDR amnd phy reset release\n");
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026

	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000024); //(  PWRCTL) -- PDEN=0
	rdata = dwc_ddrc_apb_rd(0x000001c4);    //(DFIPHYMSTR) EXP=0x44000000
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000320, 0x00000000); //(SWCTL) // SW_DONE = 0
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC) //
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	//(DFIMISC) // DFI_CMPLETE_EN=0, CTL_IDLE_EN=1
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010);
	dwc_ddrc_apb_wr(0x00000304, 0x00000002); //( DBG1) // DIS_DFI

	/*Skip Phy init step */
	//phy_init();

	rdata = dwc_ddrc_apb_rd(0x00000000);    //(MSTR) EXP=0x41040410
	rdata = dwc_ddrc_apb_rd(0x000001b0);    //(DFIMISC) EXP=0x00000010
	dwc_ddrc_apb_wr(0x000001b0, 0x00000030); //(           DFIMISC)

	gb_printf(KERN_INFO, "detect power status\n");
	while (dwc_ddrc_apb_rd(0x01bc) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001c4, 0x44000000);
	dwc_ddrc_apb_wr(0x00000320, 0x00000000);
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x00000320, 0x00000001);
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x00000304, 0x00000000);
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000006); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(PCTRL_0) // Enable Port 0

}

void ddr3_init(void)
{
	int rdata;

	mdev = 3;
	dwc_ddrc_apb_wr(0x00000304, 0x00000001); //  (  DBG1) -- DIS DQ
	dwc_ddrc_apb_wr(0x00000030, 0x00000001); //  (  PWRCTL) -- SREF EN

	rdata = dwc_ddrc_apb_rd(0x00000004); //  (STAT) EXP = 00000000 -- INIT
	while (rdata != 0x0000) {
		rdata = dwc_ddrc_apb_rd(0x00000004);
		msleep(1000);
	}
	gb_printf(KERN_INFO, "DDR in init status\n");

	// X8, 1RANK, 1:2, No Stack, BL8, Full DQ, NoGearDown,
	// 2T, No-BC, Seq Burst, DDR4
	dwc_ddrc_apb_wr(0x00000000, 0x41040410); //(              MSTR)
	dwc_ddrc_apb_wr(0x00000010, 0x00003030); //(           MRCTRL0)
	dwc_ddrc_apb_wr(0x00000014, 0x00013edf); //(           MRCTRL1)
	dwc_ddrc_apb_wr(0x0000001c, 0xe2e762f6); //(           MRCTRL2)
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(PWRCTL) // entry sw-SREF
	dwc_ddrc_apb_wr(0x00000034, 0x00010c18); //(            PWRTMG)
	dwc_ddrc_apb_wr(0x00000038, 0x005d0001); //(           HWLPCTL)
	dwc_ddrc_apb_wr(0x00000050, 0x6041f060); //(          RFSHCTL0)
	dwc_ddrc_apb_wr(0x00000054, 0x002f0046); //(          RFSHCTL1)
	dwc_ddrc_apb_wr(0x00000060, 0x00000010); //(          RFSHCTL3)
	dwc_ddrc_apb_wr(0x00000064, 0x006100d0); //(           RFSHTMG)
	dwc_ddrc_apb_wr(0x00000068, 0x00350000); //(          RFSHTMG1)
	dwc_ddrc_apb_wr(0x00000070, 0x40391f80); //(ECCCFG0) // ECC Disable
	dwc_ddrc_apb_wr(0x00000074, 0x00000530); //(           ECCCFG1)
	dwc_ddrc_apb_wr(0x0000007c, 0x00000300); //(            ECCCTL)
	dwc_ddrc_apb_wr(0x000000b8, 0x00000290); //(    ECCPOISONADDR0)
	dwc_ddrc_apb_wr(0x000000bc, 0x140233d8); //(    ECCPOISONADDR1)
	dwc_ddrc_apb_wr(0x000000c0, 0x00000000); //(        CRCPARCTL0)
	dwc_ddrc_apb_wr(0x000000c4, 0x00000010); //(        CRCPARCTL1)
	//(INIT0) // SKIP_DRAM_INIT='b11
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x000000d4, 0x00010008); //(             INIT1)
	dwc_ddrc_apb_wr(0x000000d8, 0x00001000); //(             INIT2)
	dwc_ddrc_apb_wr(0x000000dc, 0x0c540001); //(             INIT3)
	dwc_ddrc_apb_wr(0x000000e0, 0x10280440); //(             INIT4)
	dwc_ddrc_apb_wr(0x000000e4, 0x001101fb); //(             INIT5)
	dwc_ddrc_apb_wr(0x000000e8, 0x000004c0); //(  0x14c0     INIT6)
	dwc_ddrc_apb_wr(0x000000ec, 0x00001067); //(             INIT7)
	dwc_ddrc_apb_wr(0x000000f0, 0x00000030); //(           DIMMCTL)
	dwc_ddrc_apb_wr(0x000000f4, 0x0000edd5); //(           RANKCTL)
	dwc_ddrc_apb_wr(0x000000f8, 0x0000000d); //(          RANKCTL1)
	dwc_ddrc_apb_wr(0x00000100, 0x1911361a); //(          DRAMTMG0)
	dwc_ddrc_apb_wr(0x00000104, 0x00050624); //(          DRAMTMG1)
	dwc_ddrc_apb_wr(0x00000108, 0x080c0713); //(          DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000010c, 0x0000400c); //(          DRAMTMG3)
	dwc_ddrc_apb_wr(0x00000110, 0x0a04040b); //(          DRAMTMG4)
	dwc_ddrc_apb_wr(0x00000114, 0x0b090504); //(          DRAMTMG5)
	dwc_ddrc_apb_wr(0x00000118, 0x0c0c0005); //(          DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000011c, 0x0000090b); //(          DRAMTMG7)
	dwc_ddrc_apb_wr(0x00000120, 0x0707120b); //(          DRAMTMG8)
	dwc_ddrc_apb_wr(0x00000124, 0x0002020f); //(          DRAMTMG9)
	dwc_ddrc_apb_wr(0x00000128, 0x00100e0f); //(         DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000012c, 0x1a0b0117); //(         DRAMTMG11)
	dwc_ddrc_apb_wr(0x00000130, 0x0c000010); //(         DRAMTMG12)
	dwc_ddrc_apb_wr(0x00000134, 0x00000000); //(         DRAMTMG13)
	dwc_ddrc_apb_wr(0x00000138, 0x00000000); //(         DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000013c, 0x80000000); //(         DRAMTMG15)
	dwc_ddrc_apb_wr(0x00000180, 0x21000040); //(            ZQCTL0)
	dwc_ddrc_apb_wr(0x00000184, 0x00000070); //(            ZQCTL1)
	dwc_ddrc_apb_wr(0x00000188, 0x00000000); //(            ZQCTL2)
	dwc_ddrc_apb_wr(0x00000190, 0x0693820b); //(           DFITMG0)
	dwc_ddrc_apb_wr(0x00000194, 0x00080505); //(           DFITMG1)
	dwc_ddrc_apb_wr(0x00000198, 0x07b03030); //(         DFILPCFG0)
	dwc_ddrc_apb_wr(0x0000019c, 0x000000f1); //(         DFILPCFG1)
	dwc_ddrc_apb_wr(0x000001a0, 0xe0400018); //(           DFIUPD0)
	dwc_ddrc_apb_wr(0x000001a4, 0x003c0077); //(           DFIUPD1)
	dwc_ddrc_apb_wr(0x000001a8, 0x80000000); //(           DFIUPD2)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000051); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b4, 0x0000130b); //(           DFITMG2)
	dwc_ddrc_apb_wr(0x000001b8, 0x00000008); //(           DFITMG3)
	//(  0x5          DBICTL) // {RD_DBI_EN, WR_DBI_EN, DM_EN}
	dwc_ddrc_apb_wr(0x000001c0, 0x00000001);
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000200, 0x001f1f1f); //(          ADDRMAP0)
	dwc_ddrc_apb_wr(0x00000204, 0x003f0A0A); //(          ADDRMAP1)
	dwc_ddrc_apb_wr(0x00000208, 0x02020200); //(          ADDRMAP2)
	dwc_ddrc_apb_wr(0x0000020c, 0x02020202); //(          ADDRMAP3)
	dwc_ddrc_apb_wr(0x00000210, 0x00001f1f); //(          ADDRMAP4)
	dwc_ddrc_apb_wr(0x00000214, 0x08080808); //(          ADDRMAP5)
	dwc_ddrc_apb_wr(0x00000218, 0x08080808); //(          ADDRMAP6)
	dwc_ddrc_apb_wr(0x0000021c, 0x00000f0f); //(          ADDRMAP7)
	dwc_ddrc_apb_wr(0x00000220, 0x00000101); //(          ADDRMAP8)
	dwc_ddrc_apb_wr(0x00000224, 0x08080808); //(          ADDRMAP9)
	dwc_ddrc_apb_wr(0x00000228, 0x08080808); //(         ADDRMAP10)
	dwc_ddrc_apb_wr(0x0000022c, 0x001F1F08); //(         ADDRMAP11)
	dwc_ddrc_apb_wr(0x00000240, 0x061e0d7c); //(            ODTCFG)
	dwc_ddrc_apb_wr(0x00000244, 0x00003032); //(            ODTMAP)
	dwc_ddrc_apb_wr(0x00000250, 0x0125991c); //(             SCHED)
	dwc_ddrc_apb_wr(0x00000254, 0xc1001018); //(            SCHED1)
	dwc_ddrc_apb_wr(0x0000025c, 0x780048bb); //(          PERFHPR1)
	dwc_ddrc_apb_wr(0x00000264, 0xde006a71); //(          PERFLPR1)
	dwc_ddrc_apb_wr(0x0000026c, 0xd30011e2); //(           PERFWR1)
	dwc_ddrc_apb_wr(0x00000270, 0x04110101); //(            SCHED3)
	dwc_ddrc_apb_wr(0x00000274, 0x007a005d); //(            SCHED4)
	dwc_ddrc_apb_wr(0x00000278, 0x2000050a); //(            SCHED5)
	dwc_ddrc_apb_wr(0x00000300, 0x00000011); //(              DBG0)
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	dwc_ddrc_apb_wr(0x0000030c, 0x00000000); //(            DBGCMD)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	dwc_ddrc_apb_wr(0x00000328, 0x00000000); //(       SWCTLSTATIC)
	dwc_ddrc_apb_wr(0x0000036c, 0x00100011); //(         POISONCFG)
	dwc_ddrc_apb_wr(0x00000374, 0x0000012d); //(       ADVECCINDEX)
	dwc_ddrc_apb_wr(0x0000037c, 0x00000000); //(     ECCPOISONPAT0)
	dwc_ddrc_apb_wr(0x00000380, 0x00000000); //(     ECCPOISONPAT1)
	dwc_ddrc_apb_wr(0x00000384, 0x00000000); //(     ECCPOISONPAT2)
	dwc_ddrc_apb_wr(0x00000400, 0x00000100); //(             PCCFG)
	dwc_ddrc_apb_wr(0x00000404, 0x0000026e); //(           PCFGR_0)
	dwc_ddrc_apb_wr(0x00000408, 0x000010ed); //(           PCFGW_0)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(           PCTRL_0)
	dwc_ddrc_apb_wr(0x00000494, 0x00110003); //(        PCFGQOS0_0)
	dwc_ddrc_apb_wr(0x00000f24, 0x0003e662); //(            SBRCTL)
	dwc_ddrc_apb_wr(0x00000f2c, 0x4024fbb2); //(         SBRWDATA0)
	dwc_ddrc_apb_wr(0x00000f30, 0x87daa350); //(         SBRWDATA1)
	dwc_ddrc_apb_wr(0x00000f38, 0x07b4ad98); //(         SBRSTART0)
	dwc_ddrc_apb_wr(0x00000f3c, 0x00000000); //(         SBRSTART1)
	dwc_ddrc_apb_wr(0x00000f40, 0x07b4b90f); //(         SBRRANGE0)
	dwc_ddrc_apb_wr(0x00000f44, 0x00000000); //(         SBRRANGE1)
	dwc_ddrc_apb_wr(0x00002020, 0x00001206); //(    FREQ1.DERATEEN)
	dwc_ddrc_apb_wr(0x00002024, 0x18be8306); //(   FREQ1.DERATEINT)
	dwc_ddrc_apb_wr(0x00002034, 0x00010c18); //(      FREQ1.PWRTMG)
	dwc_ddrc_apb_wr(0x00002050, 0x3041f060); //(    FREQ1.RFSHCTL0)
	dwc_ddrc_apb_wr(0x00002064, 0x006180d0); //(     FREQ1.RFSHTMG)
	dwc_ddrc_apb_wr(0x00002068, 0x00350000); //(    FREQ1.RFSHTMG1)
	dwc_ddrc_apb_wr(0x000020dc, 0x0c540001); //(       FREQ1.INIT3)
	dwc_ddrc_apb_wr(0x000020e0, 0x10280440); //(       FREQ1.INIT4)
	dwc_ddrc_apb_wr(0x000020e8, 0x000014c0); //(       FREQ1.INIT6)
	dwc_ddrc_apb_wr(0x000020ec, 0x00001067); //(       FREQ1.INIT7)
	dwc_ddrc_apb_wr(0x000020f4, 0x00000dd5); //(     FREQ1.RANKCTL)
	dwc_ddrc_apb_wr(0x000020f8, 0x0000000d); //(    FREQ1.RANKCTL1)
	dwc_ddrc_apb_wr(0x00002100, 0x1911361a); //(    FREQ1.DRAMTMG0)
	dwc_ddrc_apb_wr(0x00002104, 0x00050624); //(    FREQ1.DRAMTMG1)
	dwc_ddrc_apb_wr(0x00002108, 0x080c0713); //(    FREQ1.DRAMTMG2)
	dwc_ddrc_apb_wr(0x0000210c, 0x0000400c); //(    FREQ1.DRAMTMG3)
	dwc_ddrc_apb_wr(0x00002110, 0x0a04040b); //(    FREQ1.DRAMTMG4)
	dwc_ddrc_apb_wr(0x00002114, 0x0b090504); //(    FREQ1.DRAMTMG5)
	dwc_ddrc_apb_wr(0x00002118, 0x0c0c0005); //(    FREQ1.DRAMTMG6)
	dwc_ddrc_apb_wr(0x0000211c, 0x0000090b); //(    FREQ1.DRAMTMG7)
	dwc_ddrc_apb_wr(0x00002120, 0x0707120b); //(    FREQ1.DRAMTMG8)
	dwc_ddrc_apb_wr(0x00002124, 0x0002020f); //(    FREQ1.DRAMTMG9)
	dwc_ddrc_apb_wr(0x00002128, 0x00100e0f); //(   FREQ1.DRAMTMG10)
	dwc_ddrc_apb_wr(0x0000212c, 0x1a0b0117); //(   FREQ1.DRAMTMG11)
	dwc_ddrc_apb_wr(0x00002130, 0x0c000010); //(   FREQ1.DRAMTMG12)
	dwc_ddrc_apb_wr(0x00002134, 0x00000000); //(   FREQ1.DRAMTMG13)
	dwc_ddrc_apb_wr(0x00002138, 0x00000000); //(   FREQ1.DRAMTMG14)
	dwc_ddrc_apb_wr(0x0000213c, 0x80000000); //(   FREQ1.DRAMTMG15)
	dwc_ddrc_apb_wr(0x00002180, 0x21000040); //(      FREQ1.ZQCTL0)
	dwc_ddrc_apb_wr(0x00002190, 0x0693820b); //(     FREQ1.DFITMG0)
	dwc_ddrc_apb_wr(0x00002194, 0x00080505); //(     FREQ1.DFITMG1)
	dwc_ddrc_apb_wr(0x000021b4, 0x0000130b); //(     FREQ1.DFITMG2)
	dwc_ddrc_apb_wr(0x000021b8, 0x00000008); //(     FREQ1.DFITMG3)
	dwc_ddrc_apb_wr(0x00002240, 0x061e0d7c); //(      FREQ1.ODTCFG)
	rdata = dwc_ddrc_apb_rd(0x00000060);    //(RFSHCTL3) EXP=0x00000010
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(  PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)

	//SYSCTRL->SWRST0.type.ddr0_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr1_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr2_swrst_n = 1;
	//SYSCTRL->SWRST0.type.ddr3_swrst_n = 1;
	ddr_reset(1);

	//SYSCTRL->SWRST1.type.ddr0_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr1_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr2_phy_swrst_n = 1;
	//SYSCTRL->SWRST1.type.ddr3_phy_swrst_n = 1;
	ddr_phy_reset(1);

	gb_printf(KERN_INFO, "DDR amnd phy reset release\n");
	dwc_ddrc_apb_wr(0x00000304, 0x00000000); //(              DBG1)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(PWRCTL) EXP=0x00000026

	dwc_ddrc_apb_wr(0x00000030, 0x00000026); //(            PWRCTL)
	rdata = dwc_ddrc_apb_rd(0x00000030);    //(   PWRCTL) EXP=0x00000026
	dwc_ddrc_apb_wr(0x00000030, 0x00000024); //(     PWRCTL) -- PDEN=0
	rdata = dwc_ddrc_apb_rd(0x000001c4);    //(DFIPHYMSTR) EXP=0x44000000
	dwc_ddrc_apb_wr(0x000001c4, 0x44000000); //(        DFIPHYMSTR)
	dwc_ddrc_apb_wr(0x00000320, 0x00000000); //(     SWCTL) // SW_DONE = 0
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC) //
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	 //(DFIMISC) // DFI_CMPLETE_EN=0, CTL_IDLE_EN=1
	dwc_ddrc_apb_wr(0x000001b0, 0x00000010);
	dwc_ddrc_apb_wr(0x00000304, 0x00000002); //(DBG1) // DIS_DFI

/*Skip Phy init step */
	//phy_init();

	rdata = dwc_ddrc_apb_rd(0x00000000);    //( MSTR) EXP=0x41040410
	rdata = dwc_ddrc_apb_rd(0x000001b0);    //( DFIMISC) EXP=0x00000010
	dwc_ddrc_apb_wr(0x000001b0, 0x00000030); //(           DFIMISC)

	gb_printf(KERN_INFO, "detect power status\n");
	while (dwc_ddrc_apb_rd(0x01bc) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001b0, 0x00000010); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x000001b0, 0x00000011); //(           DFIMISC)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000320, 0x00000001); //(             SWCTL)
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x000001c4, 0x44000000);
	dwc_ddrc_apb_wr(0x00000320, 0x00000000);
	dwc_ddrc_apb_wr(0x000000d0, 0x00030003);
	dwc_ddrc_apb_wr(0x00000320, 0x00000001);
	while (dwc_ddrc_apb_rd(0x0324) == 0)
		msleep(100);

	dwc_ddrc_apb_wr(0x00000304, 0x00000000);
	dwc_ddrc_apb_wr(0x00000030, 0x00000004); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000030, 0x00000006); //(            PWRCTL)
	dwc_ddrc_apb_wr(0x00000490, 0x00000001); //(PCTRL_0) // Enable Port 0

}

void gb02_ddr_init(const void __iomem *mcu_peri_bar_p,
	const void __iomem *sys_peri_bar_p, const void __iomem *ddr_bar_p,
	const void __iomem *peri_base_bar_p)
{
	ddr_crm(mcu_peri_bar_p, sys_peri_bar_p, ddr_bar_p, peri_base_bar_p);
	ddr0_init();
	//check_ddr0();
	ddr1_init();
	//check_ddr1();
	ddr2_init();
	//check_ddr2();
	ddr3_init();
	//check_ddr3();
}
