
#ifndef __GB02_DP_H__
#define __GB02_DP_H__

#include <linux/types.h>
#include "gb_dp_dptx.h"
#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
#include <drm/drm_dp_helper.h>
#else
#include <drm/display/drm_dp.h>
#endif

#define DCDP_CTL_OFFSET  0x00000
#define DCDP_TCA_OFFSET  0x10000
#define DCDP_CR_OFFSET   0x20000
#define DCDP_CFG_OFFSET  0x30000
#define DCDP_DC_OFFSET   0x40000
#define DCDP_DBG_OFFSET  0x50000
#define DCDP_VG_OFFSET   0x60000

#define AUX_CMD_TYPE_I2C_W              (0x0<<28)
#define AUX_CMD_TYPE_I2C_WT             (0x1<<28)
#define AUX_CMD_TYPE_WRITE_ST_UPD       (0x2<<28)
#define AUX_CMD_TYPE_MOT_W              (0x4<<28)
#define AUX_CMD_TYPE_MOT_R              (0x5<<28)
#define AUX_CMD_TYPE_MOT_WRITE_ST_UPD   (0x6<<28)
#define AUX_CMD_TYPE_NATIVE_W           (0x8<<28)
#define AUX_CMD_TYPE_NATIVE_R           (0x9<<28)
#define AUX_CMD_TYPE_OFFSET		28

#define AUX_CMD_WRITE			0x0
#define AUX_CMD_READ			0x1

#define DPRX_DPCD_DEV_FW_VER_MAJ	0x040A
#define DPRX_DPCD_DEV_FW_VER_MIN	0x040B
#define HDMI_VERSION_INFO		0x1
#define VGA_VERSION_INFO		0x2

#ifndef fallthrough
#define fallthrough do {} while (0)
#endif

typedef union {
	uint32_t val;
	struct {
		uint32_t en:1;
		uint32_t bypass:1;
		uint32_t dsmen:1;
		uint32_t refdiv:6;
		uint32_t postdiv2:3;
		uint32_t postdiv1:3;
		uint32_t fbdiv:12;
		uint32_t extdiv:4;
		uint32_t extdiven:1;
	} data;
} tDCDP_PLLP_CFG0;

typedef union {
	uint32_t val;
	struct {
		uint32_t frac:24;
		uint32_t update:1;
		uint32_t rsvd:7;
	} data;
} tDCDP_PLLP_CFG1;

typedef union {
	uint32_t val;
	struct {
		uint32_t en:1;
		uint32_t bypass:1;
		uint32_t dsmen:1;
		uint32_t refdiv:6;
		uint32_t postdiv2:3;
		uint32_t postdiv1:3;
		uint32_t fbdiv:12;
		uint32_t rsvd:5;
	} data;
} tDCDP_PLLM_CFG0;

typedef union {
	uint32_t val;
	struct {
		uint32_t frac:24;
		uint32_t update:1;
		uint32_t rsvd:7;
	} data;
} tDCDP_PLLM_CFG1;

typedef union {
	uint32_t val;
	struct {
		uint32_t en:1;
		uint32_t bypass:1;
		uint32_t refdiv:6;
		uint32_t postdiv2:3;
		uint32_t postdiv1:3;
		uint32_t fbdiv:12;
		uint32_t pllbw:1;
		uint32_t rsvd:5;
	} data;
} tDCDP_PLLD_CFG0;

typedef union {
	uint32_t val;
	struct {
		uint32_t frac:24;
		uint32_t update:1;
		uint32_t rsvd:7;
	} data;
} tDCDP_PLLD_CFG1;

typedef union {
	uint32_t val;
	struct {
		uint32_t plld_locked:1;
		uint32_t pllm_locked:1;
		uint32_t pllp_locked:1;
		uint32_t rsvd:29;
	} data;
} tDCDP_PLL_STAT;

enum DPTX_AUDIO_STATUS {
	DPTX_AUDIO_START_PLAY = 0,
	DPTX_AUDIO_EDID_CONFIGURE,
	DPTX_AUDIO_MAX_VAL,
};

extern int gb_dp_mode_change(const struct drm_display_mode *mode,
	int dp_index);
extern bool gb_dp_get_hpd_status(int dp_index);
extern int gb_dp_irq_enable(int dp_index,
	struct drm_device *drm);
extern int gb_dp_pllm_disable(int dp_index);
extern int gb_dp_pllm_enable(int dp_index);
extern void gb_dptx_hpd_work_func(struct work_struct *work);
extern int gb_dp_link_train(struct dptx *dptx_info);
extern unsigned char *gb_dp_get_edid(int dp_index);
extern bool gb_dp_edid_getted(int dp_index);
extern ssize_t dm_dp_aux_transfer(struct drm_dp_aux *aux,
                                  struct drm_dp_aux_msg *msg);
void set_dp_audio_param(struct dptx *dptx_info);
int dptx_config_audio_based_on_edid(struct dptx *dptx, int set_info);
int get_conversion_chip_version(struct gb_pcie_info *gb_pcie);

#endif
