/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/debugfs.h>
#include <linux/uaccess.h>
#include <linux/kthread.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/delay.h>
#include <linux/pci.h>
#include <linux/miscdevice.h>
#include <linux/interrupt.h>
#include <linux/sysfs.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/string.h>
#include <linux/gfp.h>
#include <linux/mm_types.h>
#include <linux/mm.h>
#include <linux/highmem.h>
#include <linux/slab.h>
#include "common/gb_common.h"
#include "common/xt.h"
#include "gb_hdmac.h"
#include "gb_hdmac_regmap.h"
#include "common/gb_pcie_info.h"

#if 0
#define DMA_TEST_CMD_BASE 'M'
#define HDMAC_TEST _IOW(DMA_TEST_CMD_BASE, 1u, unsigned long long)
#define HDMAC_TEST_MMU_SOFT_RESET\
	_IOW(DMA_TEST_CMD_BASE, 2u, unsigned long long)
#define HDMAC_TEST_HDMAC_SOFT_RESET\
	_IOW(DMA_TEST_CMD_BASE, 3u, unsigned long long)
#endif

static DEFINE_MUTEX(hdmac_channel_id_lock);
struct hdmac_ctrl_info *dma_infos;
int hdmac_channel_id;

static struct task_struct *gb_hdmac_thread_0;
static struct task_struct *gb_hdmac_thread_1;
static struct task_struct *gb_hdmac_thread_2;
static struct task_struct *gb_hdmac_thread_3;
static int gb02_hdmac_init_gbctx(struct gb_hdmac_context *gbctx,
	int channel_id, phys_addr_t *cpu_phys, phys_addr_t *gpu_phys,
	struct page **cpu_pages, u64 *cpu_virt, u64 *gpu_virt, u64 pg_cnt);

static int gb_hdmac_thread_proc_0(void *arg)
{
	int ret = 0;
	u64 size = 0x2000000;
	int channel_id = 0;
	u64 pg_cnt = size >> GPU_PAGE_SHIFT;
	phys_addr_t *cpu_phys = NULL;
	phys_addr_t *gpu_phys = NULL;
	struct page **cpu_pages = NULL;
	u64 cpu_virt, gpu_virt;
	struct gb_hdmac_context gbctx;

	gb_printf(KERN_ERR, "=========Enter %s, channel_id:%d, size:0x%llx==========\n",
		__func__, channel_id, size);

	cpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	gpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	cpu_pages =
	 (struct page **)kmalloc(sizeof(struct page **) * pg_cnt, GFP_KERNEL);

	if (gb02_hdmac_init_gbctx(&gbctx, channel_id, cpu_phys, gpu_phys,
			cpu_pages, &cpu_virt, &gpu_virt, pg_cnt)) {
		gb_printf(KERN_ERR, "init gbctx failed!!channel_id:%d, size:0x%llx\n",
		 channel_id, size);
		ret = -1;
		goto free;
	}

	gb_hdmac_dump_pg_table();
	while (!kthread_should_stop()) {
		mdelay(1);
		/* DMA READ TEST */
		ret = gb_hdmac_test(size, cpu_phys, gpu_phys, true,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac read test ERR chan_id:%d size:0x%llx\n",
			 channel_id, size);
		mdelay(1);
		/* DMA WRITE TEST */
		ret = gb_hdmac_test(size, gpu_phys, cpu_phys, false,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac write test ERR chan_id:%d size:0x%llx\n",
			 channel_id, size);
	}
free:
	kfree(cpu_phys);
	kfree(gpu_phys);
	kfree(cpu_pages);

	return 0;
}

static int gb_hdmac_thread_proc_1(void *arg)
{
	int ret = 0;
	u64 size = 0x2000000;
	int channel_id = 1;
	u64 pg_cnt = size >> GPU_PAGE_SHIFT;
	phys_addr_t *cpu_phys = NULL;
	phys_addr_t *gpu_phys = NULL;
	struct page **cpu_pages = NULL;
	u64 cpu_virt, gpu_virt;
	struct gb_hdmac_context gbctx;

	gb_printf(KERN_ERR, "=========Enter %s, channel_id:%d, size:0x%llx==========\n",
		__func__, channel_id, size);

	cpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	gpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	cpu_pages =
	 (struct page **)kmalloc(sizeof(struct page **) * pg_cnt, GFP_KERNEL);

	if (gb02_hdmac_init_gbctx(&gbctx, channel_id, cpu_phys, gpu_phys,
			cpu_pages, &cpu_virt, &gpu_virt, pg_cnt)) {
		gb_printf(KERN_ERR, "init gbctx failed!!channel_id:%d, size:0x%llx\n",
		 channel_id, size);
		ret = -1;
		goto free;
	}

	gb_hdmac_dump_pg_table();
	while (!kthread_should_stop()) {
		mdelay(1);
		/* DMA READ TEST */
		ret = gb_hdmac_test(size, cpu_phys, gpu_phys, true,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac read test ERR!!!chan_id:%d size:0x%llx\n",
			 channel_id, size);
		mdelay(1);
		/* DMA WRITE TEST */
		ret = gb_hdmac_test(size, gpu_phys, cpu_phys, false, channel_id,
			true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac write test ERR chan_id:%d size:0x%llx\n",
			 channel_id, size);
	}
free:
	kfree(cpu_phys);
	kfree(gpu_phys);
	kfree(cpu_pages);

	return 0;
}

static int gb_hdmac_thread_proc_2(void *arg)
{
	int ret = 0;
	u64 size = 0x2000000;
	int channel_id = 2;
	u64 pg_cnt = size >> GPU_PAGE_SHIFT;
	phys_addr_t *cpu_phys = NULL;
	phys_addr_t *gpu_phys = NULL;
	struct page **cpu_pages = NULL;
	u64 cpu_virt, gpu_virt;
	struct gb_hdmac_context gbctx;

	gb_printf(KERN_ERR, "=========Enter %s, channel_id:%d, size:0x%llx==========\n",
		__func__, channel_id, size);

	cpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	gpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	cpu_pages =
	 (struct page **)kmalloc(sizeof(struct page **) * pg_cnt, GFP_KERNEL);

	if (gb02_hdmac_init_gbctx(&gbctx, channel_id, cpu_phys, gpu_phys,
			cpu_pages, &cpu_virt, &gpu_virt, pg_cnt)) {
		gb_printf(KERN_ERR, "init gbctx failed!!channel_id:%d, size:0x%llx\n",
		 channel_id, size);
		ret = -1;
		goto free;
	}

	gb_hdmac_dump_pg_table();

	while (!kthread_should_stop()) {
		mdelay(1);
		/* DMA READ TEST */
		ret = gb_hdmac_test(size, cpu_phys, gpu_phys, true,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac read test ERR chan_id:%d size:0x%llx\n",
			 channel_id, size);
		mdelay(1);
		/* DMA WRITE TEST */
		ret = gb_hdmac_test(size, gpu_phys, cpu_phys, false,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac write test ERR chan_id:%d size:0x%llx\n",
			 channel_id, size);
	}
free:
	kfree(cpu_phys);
	kfree(gpu_phys);
	kfree(cpu_pages);

	return 0;
}

static int gb_hdmac_thread_proc_3(void *arg)
{
	int ret = 0;
	u64 size = 0x2000000;
	int channel_id = 3;
	u64 pg_cnt = size >> GPU_PAGE_SHIFT;
	phys_addr_t *cpu_phys = NULL;
	phys_addr_t *gpu_phys = NULL;
	struct page **cpu_pages = NULL;
	u64 cpu_virt, gpu_virt;
	struct gb_hdmac_context gbctx;

	gb_printf(KERN_ERR, "=========Enter %s, channel_id:%d, size:0x%llx========\n",
		__func__, channel_id, size);

	cpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	gpu_phys =
	 (phys_addr_t *)kmalloc(sizeof(phys_addr_t *) * pg_cnt, GFP_KERNEL);
	cpu_pages =
	 (struct page **)kmalloc(sizeof(struct page **) * pg_cnt, GFP_KERNEL);

	if (gb02_hdmac_init_gbctx(&gbctx, channel_id, cpu_phys, gpu_phys,
			cpu_pages, &cpu_virt, &gpu_virt, pg_cnt)) {
		gb_printf(KERN_ERR, "init gbctx failed!!channel_id:%d, size:0x%llx\n",
		 channel_id, size);
		ret = -1;
		goto free;
	}

	gb_hdmac_dump_pg_table();

	while (!kthread_should_stop()) {
		mdelay(1);
		/* DMA READ TEST */
		ret = gb_hdmac_test(size, cpu_phys, gpu_phys, true,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac read test ERR chan_id:%d size:0x%llx\n",
				channel_id, size);
		mdelay(1);
		/* DMA WRITE TEST */
		ret = gb_hdmac_test(size, gpu_phys, cpu_phys, false,
		 channel_id, true, pg_cnt - 1, cpu_virt, gpu_virt);
		if (ret)
			gb_printf(KERN_ERR, "hdmac write test ERR chan_id:%d size:0x%llx\n",
				channel_id, size);
	}
free:
	kfree(cpu_phys);
	kfree(gpu_phys);
	kfree(cpu_pages);

	return 0;
}

static int gb_hdmac_kthread_create(void)
{
	gb_hdmac_thread_0 =
	 kthread_run(gb_hdmac_thread_proc_0, NULL, "gb_hdmac_thread_0");
	gb_hdmac_thread_1 =
	 kthread_run(gb_hdmac_thread_proc_1, NULL, "gb_hdmac_thread_1");
	gb_hdmac_thread_2 =
	 kthread_run(gb_hdmac_thread_proc_2, NULL, "gb_hdmac_thread_2");
	gb_hdmac_thread_3 =
	 kthread_run(gb_hdmac_thread_proc_3, NULL, "gb_hdmac_thread_3");
	if (IS_ERR(gb_hdmac_thread_0) | IS_ERR(gb_hdmac_thread_1) |
		IS_ERR(gb_hdmac_thread_2) | IS_ERR(gb_hdmac_thread_3)) {
		gb_printf(KERN_ERR, "%s-%d create thread fail 0x%p, 0x%p, 0x%p, 0x%p\n",
		 __func__, __LINE__, gb_hdmac_thread_0,
		  gb_hdmac_thread_1, gb_hdmac_thread_2,
		   gb_hdmac_thread_3);
		return -EFAULT;
	}
	return 0;
}

static void gb_hdmac_kthread_release(void)
{
	if (gb_hdmac_thread_0)
		kthread_stop(gb_hdmac_thread_0);
	if (gb_hdmac_thread_1)
		kthread_stop(gb_hdmac_thread_1);
	if (gb_hdmac_thread_2)
		kthread_stop(gb_hdmac_thread_2);
	if (gb_hdmac_thread_3)
		kthread_stop(gb_hdmac_thread_3);
}

static void gb02_hdmac_init_pgd(struct gb_hdmac_context *gbctx, int as_nr)
{
	/* init gbctx */
	memset(gbctx, 0, sizeof(struct gb_hdmac_context));
	gbctx->mmu_mode = get_mmu_mode();
	gbctx->as_nr = as_nr;
	gbctx->pgd = gb_hdmac_mmu_alloc_pgd(gbctx);

	gb_printf(KERN_ERR, "%s as_nr:%d, pgd:0x%llx\n",
		__func__, gbctx->as_nr, gbctx->pgd);
	/* config mmu hw */
	gbctx->mmu_mode->update(gbctx);
}

static int gb02_hdmac_init_gbctx(struct gb_hdmac_context *gbctx,
	int channel_id, phys_addr_t *cpu_phys, phys_addr_t *gpu_phys,
	struct page **cpu_pages, u64 *cpu_virt, u64 *gpu_virt, u64 pg_cnt)
{
	int i;

	/*
	 * we have 8 AddrSpaces and 8 channels, one as for one channel,
	 * so treat channel_id as as_id
	 */
	gb02_hdmac_init_pgd(gbctx, channel_id);

	for (i = 0; i < pg_cnt; i++) {
		cpu_pages[i] = alloc_page(GFP_KERNEL);
		cpu_phys[i] = page_to_phys(cpu_pages[i]);
		hdmac_debug("cpu_phy[%d]:0x%llx\n", i, cpu_phys[i]);
	}

	if (gb_hdmac_gpu_mmap(gbctx, pg_cnt, false, cpu_phys, cpu_virt)) {
		gb_printf(KERN_ERR, "map cpu failed!pg_cnt:%llu\n", pg_cnt);
		return -1;
	}

	for (i = 0; i < pg_cnt; i++)
		gpu_phys[i] = gb_hdmac_alloc_gpu_page();

	if (gb_hdmac_gpu_mmap(gbctx, pg_cnt, true, gpu_phys, gpu_virt)) {
		gb_printf(KERN_ERR, "map gpu failed!pg_cnt:%llu\n", pg_cnt);
		return -1;
	}

	return 0;
}

irqreturn_t gb02_hdmac_irq_handler0(int irq, void *arg)
{
	u32 hdmac_int_status;
	u32 hdmac_abort_status;
	u32 hdmac_mmu_status;
	u64 fault_addr_lo, fault_addr_hi;
	u64 fault_addr;

	hdmac_int_status = gb_hdmac_reg_read(DMA_INT_STATUS);
	hdmac_abort_status = gb_hdmac_reg_read(DMA_ABORT_STATUS);
	hdmac_mmu_status = gb_hdmac_reg_read(MMU_AS_REG(0, AS_FAULTSTATUS));

	if (hdmac_mmu_status) {
		fault_addr_lo = gb_hdmac_reg_read(MMU_AS_REG(0,
		 AS_FAULTADDRESS_LO));
		fault_addr_hi = gb_hdmac_reg_read(MMU_AS_REG(0,
		 AS_FAULTADDRESS_HI));
		fault_addr = fault_addr_lo | (fault_addr_hi << 32);
		hdmac_debug("mmu interrupt!status:0x%x, fault_addr:0x%llx\n",
			hdmac_mmu_status, fault_addr);
	}

	if (hdmac_abort_status)
		hdmac_debug("abort interrupt!status:0x%x\n",
		 hdmac_abort_status);

	if (hdmac_int_status)
		hdmac_debug("done interrupt!status:0x%x\n", hdmac_int_status);

	return IRQ_HANDLED;
}

int gb_hdmac_init(struct gb_pcie_info *pcie_info)
{
	int result = 0;
	struct hdmac_ctrl_info *dma_ctrl;
	int ddr_bar_id = get_genbu_ddr_bar_id(pcie_info->gb_dev_info);
	int reg_bar_id = get_genbu_gpu_reg_bar_id(pcie_info->gb_dev_info);

	dma_ctrl =
	 (struct hdmac_ctrl_info *)kzalloc(sizeof(struct hdmac_ctrl_info),
	  GFP_KERNEL);
	if (dma_ctrl == NULL) {
		gb_printf(KERN_ERR, "Error: %s, kmalloc failed\n", __func__);
		return -ENOMEM;
	}

	dma_ctrl->vram_addr = pcie_info->pci_bars[ddr_bar_id].mmio;
	dma_ctrl->regs = pcie_info->pci_bars[reg_bar_id].mmio
	 + CS_S2_HDMAC_BASE;
	memset_io(dma_ctrl->vram_addr + HDMAC_MMU_OFFSET, 0, HDMAC_MMU_LEN);

	dma_infos = dma_ctrl;

	hdmac_enable_mmu_irq();
	gb_hdmac_reg_write(DMA_CFG_PCIE_BASE_ADDR, 0x40000000);
	gb_hdmac_reg_write(DMA_IRQ_MODE, 0);
	gb_hdmac_reg_write(DMA_CLEAR_HDMA_IRQ, (1 << 0) | (1 << 31));

	result = gb_hdmac_kthread_create();
	if (result)
		gb_printf(KERN_ERR, "%s %d fail!\n", __func__, __LINE__);

	return result;
}

void gb_hdmac_release(void)
{
	gb_hdmac_kthread_release();
	kfree(dma_infos);
	dma_infos = NULL;
}
