/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "gb_device.h"
#include "gb_device_ctrl.h"
#include "mcu_peripherals/gpio/pinctrl-gb02-mcu.h"

#define PCIE_C0_200_GPIO_GSV_RESET 25
#define CQ2040_MXM_M60_GPIO_GSV_RESET 25
#define PCIE_M4HL8G_GSV0_RESET 27
#define PCIE_M4HL8G_GSV2_RESET 24
#define PCIE_M4HL8G_GSV3_RESET 26
#define PCIE_M4HL8G_GSV5_RESET 39
static bool pcie_c0_200_gpio_gsv_reset_request = false;

static int gb_pcie_gpio_request(int gpio, int dir)
{
	int ret = -1;

#ifdef CONFIG_MCU_GPIO
	if ((PCIE_C0_200_GPIO_GSV_RESET == gpio) && pcie_c0_200_gpio_gsv_reset_request)
		return 0;

	ret = gb02_gpio_request(gpio);
	if (ret) {
		gb_printf(KERN_ERR, "Failed to request  gpio %d\n", gpio);
		return ret;
	}
	ret = gb02_gpio_set_direction(gpio, dir);
	if (ret) {
		gb_printf(KERN_ERR, "Failed to set  gpio %d direction\n", gpio);
		return ret;
	}
	if (PCIE_C0_200_GPIO_GSV_RESET == gpio)
		pcie_c0_200_gpio_gsv_reset_request = true;
#endif

	return ret;
}

void gb_device_io_clear(void *gpio_addr, u32 gpio_bit)
{
    u32 val;
    val = ioread32((void __iomem *)gpio_addr);
    val &= (~gpio_bit);
    iowrite32(val, (void __iomem *)gpio_addr);
//    msleep(200); // the time must >= 200ms
}
void gb_device_io_set(void *gpio_addr, u32 gpio_bit)
{
    u32 val;
    val = ioread32((void __iomem *)gpio_addr);
    val |= gpio_bit;
    iowrite32(val, (void __iomem *)gpio_addr);
//    msleep(200); // the time must >= 200ms
}

void HDMI_power_down(struct gb_pcie_info *gpi)
{
	void __iomem *base = gpi->pci_bars[0].mmio;
	int type = gb_get_board_type(gpi);

	switch (type) {
	case PCIE_LPDDR4:
		gb_device_io_clear(base + GPIO24_31, 0x3);
		msleep(200); // the time must >= 200ms
		gb_device_io_set(base + GPIO24_31, 0x1);
		gb_device_io_set(base + GPIO24_31, 0x2);
		break;
	case PCIE_FULL_LPDDR4:
		gb_device_io_clear(base + GPIO24_31, 0x3);
		msleep(200); // the time must >= 200ms
		gb_device_io_set(base + GPIO24_31, 0x1);
		gb_device_io_set(base + GPIO24_31, 0x2);
		break;
	case PCIE_M6FL8G_LPDDR4:
		/*Change the GPIO mode of hdmi-1 */
		gb_device_io_set(base + 0x10, 0x40);
		msleep(200); // the time must >= 200ms
		/*Power down PDB&5v*/
		/*hdmi 1*/
		gb_device_io_clear(base + GPIO24_31, 0xc0);
		/*hdmi 3*/
		gb_device_io_clear(base + GPIO16_23, 0x40);
		gb_device_io_clear(base + GPIO24_31, 0x8);
		/*hdmi 4*/
		gb_device_io_clear(base + GPIO16_23, 0x80);
		gb_device_io_clear(base + GPIO24_31, 0x1);
		/*hdmi 5*/
		gb_device_io_clear(base + GPIO24_31, 0x6);
		msleep(200); // the time must >= 200ms
		/*Power on 5v firstly*/
		/*hdmi 1*/
		gb_device_io_set(base + GPIO24_31, 0x40);
		/*hdmi 3*/
		gb_device_io_set(base + GPIO16_23, 0x40);
		/*hdmi 4*/
		gb_device_io_set(base + GPIO16_23, 0x80);
		/*hdmi 5*/
		gb_device_io_set(base + GPIO24_31, 0x2);
		msleep(200); // the time must >= 200ms
		/*Power on PDB secondly*/
		/*hdmi 1*/
		gb_device_io_set(base + GPIO24_31, 0x80);
		/*hdmi 3*/
		gb_device_io_set(base + GPIO24_31, 0x8);
		/*hdmi 4*/
		gb_device_io_set(base + GPIO24_31, 0x1);
		/*hdmi 5*/
		gb_device_io_set(base + GPIO24_31, 0x4);
		gb_printf(KERN_INFO, "%s-%d: board_type = %d HDMI_power down\n",
		__func__, __LINE__, gb_get_board_type(gpi));
		break;
	case PCIE_HIE1LP4_LPDDR4:
		/*hdmi 5v ctrl & PDB PDB3 clear*/
		gb_device_io_clear(base + GPIO24_31, 0xb);
		/*hdmi PDB2 PDB4 clear*/
		gb_device_io_clear(base + GPIO16_23, 0xc0);
		msleep(200); // the time must >= 200ms
		/*hdmi 5v ctrl*/
		gb_device_io_set(base + GPIO24_31, 0x1);
		/*hdmi1 PDB*/
		gb_device_io_set(base + GPIO24_31, 0x2);
		/*hdmi3 PDB3*/
		gb_device_io_set(base + GPIO24_31, 0x8);
		/*hdmi2 PDB2*/
		gb_device_io_set(base + GPIO16_23, 0x40);
		/*hdmi4 PDB4*/
		gb_device_io_set(base + GPIO16_23, 0x80);
		gb_printf(KERN_INFO, "%s-%d: board_type = %d HDMI_power down\n",
		__func__, __LINE__, gb_get_board_type(gpi));
		break;
	case PCIE_C0_200:
		if (gb_pcie_gpio_request(PCIE_C0_200_GPIO_GSV_RESET,
					GB02_GPIO_LINE_DIRECTION_OUT) == 0)
			gb02_gpio_set(PCIE_C0_200_GPIO_GSV_RESET, 0);
		break;
	case PCIE_M4HL8G_LPDDR4:
		gb02_gpio_set(PCIE_M4HL8G_GSV0_RESET, 0);
		gb02_gpio_set(PCIE_M4HL8G_GSV2_RESET, 0);
		gb02_gpio_set(PCIE_M4HL8G_GSV3_RESET, 0);
		gb02_gpio_set(PCIE_M4HL8G_GSV5_RESET, 0);
		gb_printf(KERN_INFO, "%s-%d: board_type = %d HDMI_power down\n",
		__func__, __LINE__, gb_get_board_type(gpi));
		break;
	case CQ2040_MXM_M60:
		gb02_gpio_set(CQ2040_MXM_M60_GPIO_GSV_RESET, 0);
		break;
	default:
		break;

	}
}
void HDMI_power_reset(struct gb_pcie_info *gpi, int dp_index)
{
	void __iomem *base = gpi->pci_bars[0].mmio;
	int type = gb_get_board_type(gpi);

	switch (type) {
	case PCIE_LPDDR4:
		if (dp_index == 2) {
			gb_device_io_clear(base + GPIO24_31, 0x2);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x2);
		}
		gb_printf(KERN_INFO,
		"%s-%d: board_type = %d, dp_index = %dHDMI_power finish\n",
		__func__, __LINE__, gb_get_board_type(gpi), dp_index);
		break;
	case PCIE_FULL_LPDDR4:
		if (dp_index == 5) {
			gb_device_io_clear(base + GPIO24_31, 0x2);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x2);
		}
		gb_printf(KERN_INFO,
		"%s-%d: board_type = %d, dp_index = %d HDMI_power finish\n",
		__func__, __LINE__, gb_get_board_type(gpi), dp_index);
		break;
	case PCIE_M6FL8G_LPDDR4:
		if (dp_index == 1) {
			/*hdmi 1*/
			/*Change the GPIO mode of hdmi-1 */
			gb_device_io_set(base + 0x10, 0x40);
			msleep(200); // the time must >= 200ms
			/*Power down PDB*/
			gb_device_io_clear(base + GPIO24_31, 0x80);
			msleep(200); // the time must >= 200ms
			/*Power on PDB */
			gb_device_io_set(base + GPIO24_31, 0x80);
		} else if (dp_index == 3) {
			/*hdmi 3*/
			gb_device_io_clear(base + GPIO24_31, 0x8);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x8);
		} else if (dp_index == 4) {
			 /*hdmi 4*/
			gb_device_io_clear(base + GPIO24_31, 0x1);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x1);
		} else if (dp_index == 5) {
			/*hdmi 5*/
			gb_device_io_clear(base + GPIO24_31, 0x4);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x4);
		}
		gb_printf(KERN_INFO,
		"%s-%d: board_type = %d, dp_index = %d HDMI_power finish\n",
		__func__, __LINE__, gb_get_board_type(gpi), dp_index);
		break;
	case PCIE_HIE1LP4_LPDDR4:
		if (dp_index == 0) {
			/*hdmi 1*/
			gb_device_io_clear(base + GPIO24_31, 0x2);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x2);
		} else if (dp_index == 2) {
			/*hdmi 2*/
			gb_device_io_clear(base + GPIO16_23, 0x40);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO16_23, 0x40);
		} else if (dp_index == 3) {
			/*hdmi 3*/
			gb_device_io_clear(base + GPIO24_31, 0x8);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO24_31, 0x8);
		} else if (dp_index == 5) {
			/*hdmi 4*/
			gb_device_io_clear(base + GPIO16_23, 0x80);
			msleep(200); // the time must >= 200ms
			gb_device_io_set(base + GPIO16_23, 0x80);
		}
		gb_printf(KERN_INFO,
		"%s-%d: board_type = %d, dp_index = %d HDMI_power finish\n",
		__func__, __LINE__, gb_get_board_type(gpi), dp_index);

		break;
	case PCIE_C0_200:
#if 0
		if (dp_index == 2) {
			if (gb_pcie_gpio_request(PCIE_C0_200_GPIO_GSV_RESET,
					GB02_GPIO_LINE_DIRECTION_OUT) == 0) {
				gb02_gpio_set(PCIE_C0_200_GPIO_GSV_RESET, 0);
				msleep(100);
				gb02_gpio_set(PCIE_C0_200_GPIO_GSV_RESET, 1);
			}
		}
#endif
		break;
	default:
		break;
	}
}

void HDMI_power_reset_all(struct gb_pcie_info *gpi)
{
	void __iomem *base = gpi->pci_bars[0].mmio;
	int type = gb_get_board_type(gpi);

	switch (type) {
	case PCIE_LPDDR4:
		gb_device_io_clear(base + GPIO24_31, 0x3);
		msleep(200); // the time must >= 200ms
		gb_device_io_set(base + GPIO24_31, 0x1);
		gb_device_io_set(base + GPIO24_31, 0x2);
		break;
	case PCIE_FULL_LPDDR4:
		gb_device_io_clear(base + GPIO24_31, 0x3);
		msleep(200); // the time must >= 200ms
		gb_device_io_set(base + GPIO24_31, 0x1);
		gb_device_io_set(base + GPIO24_31, 0x2);
		break;
	case PCIE_M6FL8G_LPDDR4:
		/*Change the GPIO mode of hdmi-1 */
		gb_device_io_set(base + 0x10, 0x40);
		msleep(200); // the time must >= 200ms
		/*Power down PDB*/
		/*hdmi 1*/
		gb_device_io_clear(base + GPIO24_31, 0x80);
		/*hdmi 3*/
		gb_device_io_clear(base + GPIO24_31, 0x8);
		/*hdmi 4*/
		gb_device_io_clear(base + GPIO24_31, 0x1);
		/*hdmi 5*/
		gb_device_io_clear(base + GPIO24_31, 0x4);
		msleep(200); // the time must >= 200ms
		/*Power on PDB */
		/*hdmi 1*/
		gb_device_io_set(base + GPIO24_31, 0x80);
		/*hdmi 3*/
		gb_device_io_set(base + GPIO24_31, 0x8);
		/*hdmi 4*/
		gb_device_io_set(base + GPIO24_31, 0x1);
		/*hdmi 5*/
		gb_device_io_set(base + GPIO24_31, 0x4);
		gb_printf(KERN_INFO, "%s-%d: board_type = %d HDMI_power down\n",
		__func__, __LINE__, gb_get_board_type(gpi));
		break;
	case PCIE_HIE1LP4_LPDDR4:
		/*hdmi PDB PDB3 clear*/
		gb_device_io_clear(base + GPIO24_31, 0x2);
		gb_device_io_clear(base + GPIO24_31, 0x8);
		/*hdmi PDB2 PDB4 clear*/
		gb_device_io_clear(base + GPIO16_23, 0xc0);
		msleep(200); // the time must >= 200ms
		/*hdmi1 PDB*/
		gb_device_io_set(base + GPIO24_31, 0x2);
		/*hdmi3 PDB3*/
		gb_device_io_set(base + GPIO24_31, 0x8);
		/*hdmi2 PDB2*/
		gb_device_io_set(base + GPIO16_23, 0x40);
		/*hdmi4 PDB4*/
		gb_device_io_set(base + GPIO16_23, 0x80);
		gb_printf(KERN_INFO, "%s-%d: board_type = %d HDMI_power down\n",
		__func__, __LINE__, gb_get_board_type(gpi));
		break;
	case PCIE_C0_200:
		// if (gb_pcie_gpio_request(PCIE_C0_200_GPIO_GSV_RESET,
		// 			GB02_GPIO_LINE_DIRECTION_OUT) == 0) {
		// 	gb02_gpio_set(PCIE_C0_200_GPIO_GSV_RESET, 0);
		// 	msleep(1);
		// 	gb02_gpio_set(PCIE_C0_200_GPIO_GSV_RESET, 1);
		// }
		break;
	case PCIE_M4HL8G_LPDDR4:
		/*hdmi 0*/
		gb02_gpio_set(PCIE_M4HL8G_GSV0_RESET, 0);
		msleep(1);
		gb02_gpio_set(PCIE_M4HL8G_GSV0_RESET, 1);
		/*hdmi 2*/
		gb02_gpio_set(PCIE_M4HL8G_GSV2_RESET, 0);
		msleep(1);
		gb02_gpio_set(PCIE_M4HL8G_GSV2_RESET, 1);
		/*hdmi 3*/
		gb02_gpio_set(PCIE_M4HL8G_GSV3_RESET, 0);
		msleep(1);
		gb02_gpio_set(PCIE_M4HL8G_GSV3_RESET, 1);
		/*hdmi 5*/
		gb02_gpio_set(PCIE_M4HL8G_GSV5_RESET, 0);
		msleep(1);
		gb02_gpio_set(PCIE_M4HL8G_GSV5_RESET, 1);
		break;
	case CQ2040_MXM_M60:
		/*hdmi*/
		gb02_gpio_set(CQ2040_MXM_M60_GPIO_GSV_RESET, 0);
		msleep(1);
		gb02_gpio_set(CQ2040_MXM_M60_GPIO_GSV_RESET, 1);
		break;
	default:
		break;
	}
}

