/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0)
#include <drm/drmP.h>
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
#include <drm/ttm/ttm_bo_driver.h>
#else
#include <drm/ttm/ttm_bo.h>
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)
#include <drm/ttm/ttm_page_alloc.h>
#endif
#include <drm/drm_crtc_helper.h>
#include <drm/drm_gem.h>
#include <drm/drm_fb_helper.h>
#include "common/gb_common.h"
#include "gb_pcie_map.h"
#include "gbdc_fbdev.h"
#include "gbdc_pm.h"
#include "gbdc_mm.h"
#include "gbdc_connector.h"
#include "gb_dev_res.h"
#include "gb_kms.h"
#include "gbdc_drv.h"
#include "reg_ops.h"
#include "gb_ip.h"
#include "gb_hdmi_ddc.h"
#include "gbdc_device.h"

#define DDC_CI_ADDR     0x37
#define DDC_SEGMENT_ADDR    0x30

void gb_hdmi_writeb(struct gbdc_device *gb_dev, u32 val, int offset)
{
	struct gbip_config_regmap *ip_config = &gb_dev->pcie_info.ip_config;
	void __iomem *hdmi_base = ip_config->hdmi_config_base;

	__gbreg_write(hdmi_base, REG_OFFSET(offset), val);
}

uint32_t gb_hdmi_readb(struct gbdc_device *gb_dev, int offset)
{
	uint32_t val = 0;
	struct gbip_config_regmap *ip_config = &gb_dev->pcie_info.ip_config;
	void __iomem *hdmi_base = ip_config->hdmi_config_base;

	val = __gbreg_read(hdmi_base, REG_OFFSET(offset));
	return val;
}

int gb_hdmi_i2c_read(struct gbdc_connector *gbdc_conn,
			    unsigned char *buf, unsigned int length)
{
	struct gbgpu_i2c_chan *i2c = gbdc_conn->ddc_bus;
	struct drm_connector *connector = &gbdc_conn->base;
	struct drm_device *dev = connector->dev;
	struct gbdc_device *gb_dev = gb_get_gbdc_device(dev->dev_private);
	uint32_t stat = 0, i = 0;

	if (!i2c->is_regaddr) {
		dev_dbg(dev->dev, "set read register address to 0\n");
		i2c->slave_reg = 0x00;
		i2c->is_regaddr = true;
	}

	while (length--) {
		gb_hdmi_writeb(gb_dev, i2c->slave_reg++, HDMI_I2CM_ADDRESS);
		if (i2c->is_segment)
			gb_hdmi_writeb(gb_dev, HDMI_I2CM_OPERATION_READ_EXT,
				    HDMI_I2CM_OPERATION);
		else
			gb_hdmi_writeb(gb_dev, HDMI_I2CM_OPERATION_READ,
				    HDMI_I2CM_OPERATION);

		udelay(1000);
		for (i = 0; i < 100; i++) {
			stat = gb_hdmi_readb(gb_dev, HDMI_IH_I2CM_STAT0);
			i2c->stat = stat;

			if (i2c->stat & HDMI_IH_I2CM_STAT0_DONE) {
				gb_hdmi_writeb(gb_dev, HDMI_IH_I2CM_STAT0,
				    i2c->stat);
				break;
			}
			/* Check for error condition on the bus */
			if (i2c->stat & HDMI_IH_I2CM_STAT0_ERROR)
				return -EIO;

			udelay(100);
		}
		if (i >= 100)
			return -EIO;

		*buf++ = gb_hdmi_readb(gb_dev, HDMI_I2CM_DATAI);
	}
	i2c->is_segment = false;

	return 0;
}

int gb_hdmi_i2c_write(struct gbdc_connector *gbdc_conn,
			     unsigned char *buf, unsigned int length)
{
	struct gbgpu_i2c_chan *i2c = gbdc_conn->ddc_bus;
	struct drm_connector *connector = &gbdc_conn->base;
	struct drm_device *dev = connector->dev;
	struct gbdc_device *gb_dev = gb_get_gbdc_device(dev->dev_private);
	uint32_t stat = 0;

	if (!i2c->is_regaddr) {
		/* Use the first write byte as register address */
		i2c->slave_reg = buf[0];
		length--;
		buf++;
		i2c->is_regaddr = true;
	}

	while (length--) {
		gb_hdmi_writeb(gb_dev, *buf++, HDMI_I2CM_DATAO);
		gb_hdmi_writeb(gb_dev, i2c->slave_reg++, HDMI_I2CM_ADDRESS);
		gb_hdmi_writeb(gb_dev, HDMI_I2CM_OPERATION_WRITE,
			HDMI_I2CM_OPERATION);

		udelay(500);
		stat = gb_hdmi_readb(gb_dev, HDMI_IH_I2CM_STAT0);
		i2c->stat = stat;
		/* Check for error condition on the bus */
		if (i2c->stat & HDMI_IH_I2CM_STAT0_ERROR)
			return -EIO;
	}

	return 0;
}

int gb_hdmi_i2c_xfer(struct i2c_adapter *adap,
			    struct i2c_msg *msgs, int num)
{
	struct gbdc_connector *gbdc_conn = i2c_get_adapdata(adap);
	struct drm_connector *connector = &gbdc_conn->base;
	struct drm_device *dev = connector->dev;
	struct gbdc_device *gb_dev = gb_get_gbdc_device(dev->dev_private);
	struct gbgpu_i2c_chan *i2c = gbdc_conn->ddc_bus;
	u8 addr = msgs[0].addr;
	int i = 0, ret = 0;

	if (addr == DDC_CI_ADDR)
		/*
		 * The internal I2C controller does not support the multi-byte
		 * read and write operations needed for DDC/CI.
		 * TOFIX: Blacklist the DDC/CI address until we filter out
		 * unsupported I2C operations.
		 */
		return -EOPNOTSUPP;

	dev_dbg(dev->dev, "xfer: num: %d, addr: %#x\n", num, addr);

	for (i = 0; i < num; i++) {
		if (msgs[i].len == 0) {
			dev_dbg(dev->dev,
				"unsupported transfer %d/%d, no data\n",
				i + 1, num);
			return -EOPNOTSUPP;
		}
	}

	mutex_lock(&i2c->lock);

	/* Unmute DONE and ERROR interrupts */
	gb_hdmi_writeb(gb_dev, 0x00, HDMI_IH_MUTE_I2CM_STAT0);

	/* Set slave device address taken from the first I2C message */
	gb_hdmi_writeb(gb_dev, addr, HDMI_I2CM_SLAVE);

	/* Set slave device register address on transfer */
	i2c->is_regaddr = false;

	/* Set segment pointer for I2C extended read mode operation */
	i2c->is_segment = false;

	for (i = 0; i < num; i++) {
		dev_dbg(dev->dev, "xfer: num: %d/%d, len: %d, flags: %#x\n",
			i + 1, num, msgs[i].len, msgs[i].flags);
		if (msgs[i].addr == DDC_SEGMENT_ADDR && msgs[i].len == 1) {
			i2c->is_segment = true;
			gb_hdmi_writeb(gb_dev, DDC_SEGMENT_ADDR,
				HDMI_I2CM_SEGADDR);
			gb_hdmi_writeb(gb_dev, *msgs[i].buf, HDMI_I2CM_SEGPTR);
		} else {
			if (msgs[i].flags & I2C_M_RD)
				ret = gb_hdmi_i2c_read(gbdc_conn, msgs[i].buf,
						       msgs[i].len);
			else
				ret = gb_hdmi_i2c_write(gbdc_conn, msgs[i].buf,
							msgs[i].len);
		}
		if (ret < 0) {
			dev_err(dev->dev, "gb hdmi ddc access failed(%d)\n",
						ret);
			break;
		}
	}

	if (!ret)
		ret = num;

	/* Mute DONE and ERROR interrupts */
	gb_hdmi_writeb(gb_dev, HDMI_IH_I2CM_STAT0_ERROR | HDMI_IH_I2CM_STAT0_DONE,
		HDMI_IH_MUTE_I2CM_STAT0);

	mutex_unlock(&i2c->lock);

	return ret;
}

void gb_hdmi_i2c_init(struct gbdc_device *gb_dev)
{
	/* Software reset */
	gb_hdmi_writeb(gb_dev, 0x00, HDMI_I2CM_SOFTRSTZ);

	/* Set Standard Mode speed (determined to be 100KHz on iMX6) */
	gb_hdmi_writeb(gb_dev, 0x00, HDMI_I2CM_DIV);

	/* Set done, not acknowledged and arbitration interrupt polarities */
	gb_hdmi_writeb(gb_dev, HDMI_I2CM_INT_DONE_POL, HDMI_I2CM_INT);
	gb_hdmi_writeb(gb_dev, HDMI_I2CM_CTLINT_NAC_POL | HDMI_I2CM_CTLINT_ARB_POL,
		HDMI_I2CM_CTLINT);

	/* Clear DONE and ERROR interrupts */
	gb_hdmi_writeb(gb_dev, HDMI_IH_I2CM_STAT0_ERROR | HDMI_IH_I2CM_STAT0_DONE,
		HDMI_IH_I2CM_STAT0);

	/* Mute DONE and ERROR interrupts */
	gb_hdmi_writeb(gb_dev, HDMI_IH_I2CM_STAT0_ERROR | HDMI_IH_I2CM_STAT0_DONE,
		HDMI_IH_MUTE_I2CM_STAT0);
}

void gb_hdmi_setup_i2c(struct gbdc_device *gb_dev)
{
	gb_hdmi_writeb(gb_dev, HDMI_PHY_I2CM_INT_ADDR_DONE_POL,
		HDMI_PHY_I2CM_INT_ADDR);

	gb_hdmi_writeb(gb_dev, HDMI_PHY_I2CM_CTLINT_ADDR_NAC_POL |
		HDMI_PHY_I2CM_CTLINT_ADDR_ARBITRATION_POL,
		HDMI_PHY_I2CM_CTLINT_ADDR);
}
