/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/delay.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/debugfs.h>
#include <linux/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/delay.h>
#include <linux/hrtimer.h>
#include <linux/ktime.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0)
#include <drm/drmP.h>
#endif

#include "gb_ip.h"
#include "reg_ops.h"
#include "gbdc_regs.h"
#include "gbdc_edid.h"
#include "gbdc_ops.h"
#include "common/gb_common.h"

/* #define PLL_TEST_CONF */

/*
*		PCIE init
*
*/

void get_firmware_verinfo(void __iomem *cfgbar_mmio,
			struct version_str *gb_version)
{
	gb_version->mcu_vdata = __gbreg_read(cfgbar_mmio, MCU_VERDATE_REG);
	gb_version->mcu_vid = __gbreg_read(cfgbar_mmio, MCU_VERGID_REG);
	gb_printf(KERN_INFO, "mcu version:%x%x", gb_version->mcu_vdata,
					gb_version->mcu_vid);
	gb_version->bios_vdata = __gbreg_read(cfgbar_mmio, BIOS_VERDATE_REG);
	gb_version->bios_vid = __gbreg_read(cfgbar_mmio, BIOS_VERGID_REG);
	gb_printf(KERN_INFO, "bios version:%x%x", gb_version->bios_vdata,
					gb_version->bios_vid);
}

void set_firmware_verinfo(void __iomem *cfgbar_mmio,
            void __iomem *sysctl_cfg_base, struct version_str *gb_version)
{
    u32 sysctl_cfg;
    sysctl_cfg = __gbreg_read(sysctl_cfg_base, CFG_MCU_XIP_ADDR_OFFSET);
    if (sysctl_cfg != MCU_SYSCTRL_BASE_ADDR)
        __gbreg_write(sysctl_cfg_base, CFG_MCU_XIP_ADDR_OFFSET, MCU_SYSCTRL_BASE_ADDR);

    __gbreg_write(cfgbar_mmio, BIOS_VERDATE_REG, gb_version->bios_vdata);
    __gbreg_write(cfgbar_mmio, BIOS_VERGID_REG, gb_version->bios_vid);
    __gbreg_write(sysctl_cfg_base, CFG_MCU_XIP_ADDR_OFFSET, sysctl_cfg);
}

/*
*		HDMI init
*
*/
static bool hdmi_phy_i2c_wait_done(void __iomem  *hdmi_base, int msec)
{
	u32 val;

	while ((val = __gbreg_read(hdmi_base, REG_OFFSET(0x0108)) & 0x3) == 0) {
		if (msec-- == 0)
			return false;
		udelay(1000);
	}
	__gbreg_write(hdmi_base, REG_OFFSET(0x0108), val);

	return true;
}

static void hdmi_phy_i2c_write(void __iomem  *hdmi_base, u32 addr, u32 data)
{
	__gbreg_write(hdmi_base, REG_OFFSET(0x0108), 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3021), addr);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3022), (data & 0xFF00) >> 8);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3023), (data & 0xFF));
	__gbreg_write(hdmi_base, REG_OFFSET(0x3026), 0x10);
	hdmi_phy_i2c_wait_done(hdmi_base, 10);
}
static void hdmi_phy_i2c_read(void __iomem  *hdmi_base, u32 addr, u32 *data)
{
	if (NULL == data)
		return ;
	else
		*data = 0;
	__gbreg_write(hdmi_base, REG_OFFSET(0x0108), 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3021), addr);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3026), 0x01);
	hdmi_phy_i2c_wait_done(hdmi_base, 10);
	*data = __gbreg_read(hdmi_base, REG_OFFSET(0x3024)) << 8;
	*data |= __gbreg_read(hdmi_base, REG_OFFSET(0x3025));
}


__attribute__((unused)) static uint8_t  hdmi_ddc_i2c_read(
				void __iomem *hdmi_base, u32 addr)
{
	u8 data = 0;
	__gbreg_write(hdmi_base, REG_OFFSET(0x0185), 0x00);
	__gbreg_write(hdmi_base, REG_OFFSET(0x7e00), 0x50);
	__gbreg_write(hdmi_base, REG_OFFSET(0x7e01), addr);
	__gbreg_write(hdmi_base, REG_OFFSET(0x7e04), 0x01);

	msleep(1);
	data = __gbreg_read(hdmi_base, REG_OFFSET(0x7e03));
	__gbreg_write(hdmi_base, REG_OFFSET(0x0185), 0x03);
	return data;
}

__attribute__((unused)) static void hdmi_get_edid(void __iomem  *hdmi_base,
							uint8_t *edid)
{
	int i = 0;
	for (i = 0; i < 128; i++)
		edid[i] = hdmi_ddc_i2c_read(hdmi_base, i);
}
/* -----------------------------------------------------------------------------
 * Register access
 */
static void hdmi_config_av(void __iomem  *hdmi_base,
		const struct drm_display_mode *ptr)
{
	char sync_flag = 0x0;
	gb_printf(KERN_INFO, "hdmi config:%s\n", ptr->name);

	/* AV_CONFIG */
	if (ptr->flags & DRM_MODE_FLAG_PHSYNC)
		sync_flag |=
			HDMI_FC_INVIDCONF_HSYNC_IN_POLARITY_ACTIVE_HIGH;

	if (ptr->flags & DRM_MODE_FLAG_PVSYNC)
		sync_flag |=
			HDMI_FC_INVIDCONF_VSYNC_IN_POLARITY_ACTIVE_HIGH;

	if (hdmi_conn_sink_is_hdmi_get())
		sync_flag |= HDMI_FC_INVIDCONF_DVI_MODEZ_HDMI_MODE;

	sync_flag |=
		HDMI_FC_INVIDCONF_DE_IN_POLARITY_ACTIVE_HIGH;

	gb_printf(KERN_INFO, "---gb----sync flag = %x---\n", sync_flag);

	__gbreg_write(hdmi_base, REG_OFFSET(0x1000),
		sync_flag);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1001),
		(ptr->hdisplay) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1002),
		(ptr->hdisplay >> 8) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1003),
		((ptr->htotal-ptr->hdisplay)) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1004),
		((ptr->htotal-ptr->hdisplay) >> 8) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1005),
		(ptr->vdisplay) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1006),
		(ptr->vdisplay >> 8) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1007),
		((ptr->vtotal-ptr->vdisplay)) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1008),
		((ptr->hsync_start-ptr->hdisplay)) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1009),
		((ptr->hsync_start-ptr->hdisplay) >> 8) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x100A),
		((ptr->hsync_end-ptr->hsync_start)) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x100B),
		((ptr->hsync_end-ptr->hsync_start) >> 8) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x100C),
		((ptr->vsync_start-ptr->vdisplay)) & 0xFF);
	__gbreg_write(hdmi_base, REG_OFFSET(0x100D),
		((ptr->vsync_end-ptr->vsync_start) >> 8) & 0xFF);
}
static int hdmi_config_phy(void __iomem  *hdmi_base,
			const struct drm_display_mode *ptr)
{

	u32 divset = 0;
	u32 curset = 0;
	u32 gmpset = 0;

	/* PHY_CONFIG */
	if (ptr->clock <= 27000) {
		divset = 0x5E0;
		curset = 0x91C;
		gmpset = 0x00;
	} else if (ptr->clock <= 59400) {
		divset = 0x540;
		curset = 0x91C;
		gmpset = 0x05;
	} else if (ptr->clock <= 74250) {
		divset = 0x540;
		curset = 0x6DC;
		gmpset = 0x05;
	} else if (ptr->clock <= 144000) {
		divset = 0x4A0;
		curset = 0x91C;
		gmpset = 0x0A;
	} else if (ptr->clock <= 148500) {
		divset = 0x4A0;
		curset = 0x6DC;
		gmpset = 0x0A;
	} else if (ptr->clock <= 216000) {
		divset = 0x000;
		curset = 0x6DC;
		gmpset = 0x0F;
	}

	hdmi_phy_i2c_write(hdmi_base, 0x06, divset);
	hdmi_phy_i2c_write(hdmi_base, 0x10, curset);
	hdmi_phy_i2c_write(hdmi_base, 0x15, gmpset);

	hdmi_phy_i2c_write(hdmi_base, 0x17, 0x06);
	hdmi_phy_i2c_write(hdmi_base, 0x19, 0x04);
	hdmi_phy_i2c_write(hdmi_base, 0x09, 0x0d);
	hdmi_phy_i2c_write(hdmi_base, 0x0E, 0x318);
	hdmi_phy_i2c_write(hdmi_base, 0x05, 0x8000);


	hdmi_phy_i2c_read(hdmi_base, 0x10, &gmpset);
	if (gmpset != curset) {
		gb_printf(KERN_ERR, "%s value set failed!\r\n", __func__);
		return -1;
	}
	return 0;
}

/* -----------------------------------------------------------------------------
 * Register function
 */
int hdmi_power_off(void __iomem *hdmi_base)
{
	u32 val;
	int i = 0;

	/* 0x3000 bit3Åä0£¬poweroff */
	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val &= (~(1 << 3));
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);

	/*
	 * Wait for TX_PHY_LOCK to be deasserted to indicate that the PHY went
	 * to low power mode.
	 */
	for (i = 0; i < 5; ++i) {
		val = __gbreg_read(hdmi_base, REG_OFFSET(0x3004));
		if (!(val & 0x1))
			break;

		usleep_range(1000, 2000);
	}

	if (val & 0x1) {
		gb_printf(KERN_ERR, "%s: gb01 hdmi phy power off failed!\n", __func__);
		return -1;
	}

	/* 0x3000 bit4Åä1£¬PDDQ */
	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val |= (1 << 4);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);

	return 0;
}
int hdmi_power_on(void __iomem  *hdmi_base)
{
	u32 val;
	int i = 0;

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val |= (1 << 3);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val &= (~(1 << 4));
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);

	/* Wait for PHY PLL lock */
	for (i = 0; i < 5; ++i) {
		val = __gbreg_read(hdmi_base, REG_OFFSET(0x3004));
		if (val & 0x1)
			break;

		usleep_range(1000, 2000);
	}

	if (!(val & 0x1)) {
		gb_printf(KERN_ERR, "PHY PLL failed to lock\n");
		return -ETIMEDOUT;
	}

	return 0;
}

#ifdef HDMI_OUT_DEBUG
static void hdmi_dbg_config(void __iomem  *hdmi_dgb_base)
{
	//debug output 1920x1080
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7012), 0x0A);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7001), 0x07);
	/* bit[5:4]00:8bit colordepth 01:10bit colordepth
	*  10:12bit colordepth 11:16bit colordepth */
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7002), 0x00);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7003), 0x80);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7004), 0x07);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7005), 0x18);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7006), 0x01);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7007), 0x58);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7008), 0x00);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7009), 0x2C);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700A), 0x00);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700B), 0x38);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700C), 0x04);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700D), 0x2D);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700E), 0x04);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x700F), 0x05);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7011), 0x04);

	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7038), 0x01);
	__gbreg_write(hdmi_dgb_base, REG_OFFSET(0x7039), 0x01);
}
#endif

static int hdmi_sink_check(void __iomem  *hdmi_base)
{
	u32 type = 0;
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), 0x04);

	type = __gbreg_read(hdmi_base, REG_OFFSET(0x4));
	if (type & (1 << 3))
		gb_printf(KERN_INFO, "GB1 support HDMI v1.4\r\n");

	type = __gbreg_read(hdmi_base, REG_OFFSET(0x5));
	if (type & (1 << 5))
		gb_printf(KERN_INFO, "GB1 support HDMI v2.0\r\n");

	type = __gbreg_read(hdmi_base, REG_OFFSET(0x6));
	if (type == 0xF2)
		gb_printf(KERN_INFO, "PHY type = 0x%x\n", type);
	else {
		gb_printf(KERN_ERR, "%s TYPE check failed!\r\n", __func__);
		return -1;
	}

	return 0;
}

static void hdmi_phy_i2c_set_addr(void __iomem  *hdmi_base, u32 addr)
{
	u32 val;

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3001));
	val |= (1 << 5);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3001), val);

	__gbreg_write(hdmi_base, REG_OFFSET(0x3020), addr);

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3001));
	val &= (~(1 << 5));
	__gbreg_write(hdmi_base, REG_OFFSET(0x3001), val);
}

static void hdmi_envideo_path(void __iomem *hdmi_base)
{
	/* ENV_CONFIG */
	__gbreg_write(hdmi_base, REG_OFFSET(0x1011), 0x0C);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1012), 0x20);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1013), 0x01);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1014), 0x0B);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1015), 0x16);
	__gbreg_write(hdmi_base, REG_OFFSET(0x1016), 0x21);
	__gbreg_write(hdmi_base, REG_OFFSET(0x4001), 0x7C);
}

static int hdmi_phy_cofigure(void __iomem  *hdmi_base,
			const struct drm_display_mode *ptr)
{
	u32 val;

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val |= (1 << 1);
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);

	val = __gbreg_read(hdmi_base, REG_OFFSET(0x3000));
	val &= (~(1 << 0));
	__gbreg_write(hdmi_base, REG_OFFSET(0x3000), val);
	/* power off */
	if (hdmi_power_off(hdmi_base))
		return -1;

	/* reset phy */
	__gbreg_write(hdmi_base, REG_OFFSET(0x4005), 0x01);
	__gbreg_write(hdmi_base, REG_OFFSET(0x4005), 0x00);

	/* I2C PHY addr 0x69 */
	hdmi_phy_i2c_set_addr(hdmi_base, 0x69);

	hdmi_config_phy(hdmi_base, ptr);

	/* power on */
	return hdmi_power_on(hdmi_base);
}

static void hdmi_disable_overflow_interrupts(void __iomem  *hdmi_base)
{
	__gbreg_write(hdmi_base, REG_OFFSET(0x182), 0x3);
}

int hdmi_mode_set(void __iomem *hdmi_base, const struct drm_display_mode *ptr)
{
	int i = 0;

	gb_printf(KERN_INFO, "%s hdmi status:0x%x!\r\n", __func__,
		__gbreg_read(hdmi_base, REG_OFFSET(0x3007)));

#ifdef HDMI_OUT_DEBUG
	hdmi_dbg_config(hdmi_base + HDMI_CONFIG_OFFSET);
#endif
	if (hdmi_sink_check(hdmi_base)) {
		gb_printf(KERN_ERR, "%s:gb01 hdmi sink check failed!", __func__);
		return -1;
	}
	hdmi_disable_overflow_interrupts(hdmi_base);
	hdmi_config_av(hdmi_base, ptr);

	/* HDMI Phy spec says to do the phy initialization sequence twice */
	for (i = 0; i < 2; i++) {
		if (hdmi_phy_cofigure(hdmi_base, ptr)) {
			gb_printf(KERN_ERR, "%s: gb01 hdmi phy cofigure failed!\n", __func__);
			return -1;
		}
	}

	hdmi_envideo_path(hdmi_base);

	return 0;
}

void gb_mcu_shutdown(void __iomem *ip_base)
{
	__gbreg_write(ip_base + HDMI_CONFIG_OFFSET,
			REG_OFFSET(HDMI_MCU_CTRL), HDMI_MCU_CRTL_V);
	gb_printf(KERN_INFO, "%s", __func__);
}
static u8 gb_get_pix_clockin(void __iomem *_base)
{
	u8 clock = 0;
	clock = __gbreg_read(_base + HDMI_CONFIG_OFFSET,
			REG_OFFSET(PIX_CLOCK_REG));
	gb_printf(KERN_INFO, "%s = %d\n", __func__, clock);
	return clock;
}

static u8 gb_vga_conn_judge(void __iomem *_base)
{
	u8 dat = 0, ret = 0;

	dat = __gbreg_read(_base + HDMI_CONFIG_OFFSET,
			REG_OFFSET(GENBU_BOARD_TYPE_ADDR));
	if (dat & (1 << GENBU_VGA_CONNECTOR_SHIFT))
		ret = 1;

	return ret;
}

#define		DISABLE_VGA_PORT	0x03
int gb_vga_switch(void __iomem *hdmi_base, int vga_cmd)
{
	unsigned int vga_status = 0;

	__gbreg_write(hdmi_base, REG_OFFSET(VGA_PORT_SWITCH), vga_cmd);
#if 0
	vga_status = __gbreg_read(hdmi_base, REG_OFFSET(VGA_PORT_STATUS));
	if (vga_status)
		usleep_range(100, 1000);
#endif
	gb_printf(KERN_INFO, "--gb---vga--cmd=%d--\n", vga_cmd);
	return vga_status;
}

void gb_disable_port(void __iomem *ip_base)
{
	hdmi_power_off(ip_base + HDMI_CONFIG_OFFSET);
	gb_vga_switch(ip_base + HDMI_CONFIG_OFFSET, DISABLE_VGA_PORT);

}
/*
*		PLL init
*
*/

typedef enum {
	CLK_TYPE_FOR_252 = 0, /* 25.2MHz */
	CLK_TYPE_for_200, /* 20MHz */
	CLK_TYPE_for_200_VGA, /* 20MHz, for VGA */
	CLK_TYPE_MAX
} clk_type_e;
#ifdef PLL_TEST_CONF
static int gb01_get_config_val(char *cmd)
{
	char val[64] = {0};
	int v = 0;

	if (!cmd)
		return -1;
	config_file_strkv(cmd, val, sizeof(val));
	/* fixme: WARNING: simple_strtol is obsolete, use kstrtol instead */
	v = simple_strtol(val, NULL, 10);
	gb_printf(KERN_INFO, "%s = %d\n", cmd, v);

	return v;
}
static int __ref2pll_clock_config_file(void __iomem  *ppll_base,
	pll_module_enum xpll, u32 clk, u32 fin, u32 p)
{
	u32 fvco = 0;
	u32 fclk = 0;
	u32 pll_mi = 0;
	u32 pll_m = 0;
	u32 pll_n = 0x13;
	u32 pll_p = 0;
	u32 pll_vdiv = 0;
	u32 pll_pdiv = 0;
	int delta_clk = 0;
	u32 mfrac = 0;
	u32 pll_mfrac = 0;

#if 0
	if (pll_m * pll_n * pll_p < 1) {
		gb_printf(KERN_ERR, "input Invalid argument");
		return -EINVAL;
	}
#endif
	gb_printf(KERN_INFO, "---------------/etc/sietium.conf------------------");
	pll_mi = gb01_get_config_val("pll_mi");
	pll_p = gb01_get_config_val("pll_p");
	if (pll_p < 3)
		pll_p = p;
	gb_printf(KERN_INFO, "fin = %u\n", fin);
	pll_m = 100 * (clk * (pll_p + 1) * (pll_vdiv + 1) * (pll_pdiv + 1) *
			      (pll_n + 1)) / fin;
	gb_printf(KERN_INFO, "M = %u\n", pll_m);
	pll_m += pll_mi;
	fvco = (((pll_m / 100) * fin) / (pll_n + 1));
	fclk = fvco / (p + 1);
	gb_printf(KERN_INFO, "fvco = %u\n", fvco);
	gb_printf(KERN_INFO, "fclk = %u\n", fclk);
	delta_clk = fclk - clk;
	if ((delta_clk > 1000) || (delta_clk < -1000)) {
		gb_printf(KERN_ERR, "clk exceeds the threshold\n");
		/*
		 * return -EINVAL;
		 */
	}

	/*
	 * The mfrac coefficient value corresponding to each 0.01 is 10485.
	 *
	 * 2^-20 : 0x1
	 * 2^-1  : 0x80000 524288
	 * 0.5   : 0x80000 524288
	 * 0.01  : 0x28f5 10485
	 *
	 * Reference doc: 19602tqlp_A753-0_1G5-DPLL_databook_0v7.pdf
	 */
	mfrac = pll_m % 100;
	gb_printf(KERN_INFO, "mfrac = %u\n", mfrac);
	pll_mfrac = (mfrac * 10485);
	gb_printf(KERN_INFO, "pll_mfrac = %u\n", pll_mfrac);

	gb_printf(KERN_INFO, "pll_m = %u, pll_n = %u, pll_p = %u\n",
	       pll_m / 100, pll_n, pll_p);

	__gbreg_write(ppll_base, PLL_STAT(xpll), 0x00001000);
	__gbreg_write(ppll_base, PLL_CFG0(xpll), 0x04000000 | (pll_n & 0xFF));
	__gbreg_write(ppll_base, PLL_CFG1(xpll),
		      (((pll_m / 100) & 0xFFF) << 20) | (pll_mfrac & 0xFFFFF));
	__gbreg_write(ppll_base, PLL_CFG2(xpll),
		      0x02000000 | (pll_p & 0x3F) << 16);
	__gbreg_write(ppll_base, PLL_CFG3(xpll), 0x00000000);
	__gbreg_write(ppll_base, PLL_STAT(xpll), 0x00011000);
	gb_printf(KERN_INFO, "------------/etc/sietium.conf---end---------------");

	if (!(__gbreg_read(ppll_base, PLL_STAT(xpll)) & 0x01))
		return -1;
	return 0;
}
#endif

static int __ref2pll_clock_config(void __iomem  *ppll_base,
	pll_module_enum xpll, u32 clk, u32 fin, u32 p, int high_speed)
{
	u32 M = 0;
	u32 N = 0x0;
	u32 vdiv = 0;
	u32 pdiv = 0;
	u32 mfrac = 0;

	if (high_speed)
		vdiv = 1;
	/*
	 * fvco = (M / N) * fin;
	 * clk = (1 / (p * enp * prop)) * ((M / N) * fin);
	 * M  = (clk * p * vdivp * pdivp * N) / fin;
	 */
	M  = 100 * (clk * (p + 1) * (vdiv + 1) * (pdiv + 1) * (N + 1)) / fin;
	gb_printf(KERN_INFO, "M = %u, N = %u, p = %u, vdiv = %u, pdiv = %u, high_speed = %d\n",
	       M, N, p, vdiv, pdiv, high_speed);
	/*
	 * The mfrac coefficient value corresponding to each 0.01 is 10485.
	 *
	 * 2^-20 : 0x1
	 * 2^-1  : 0x80000 524288
	 * 0.5   : 0x80000 524288
	 * 0.01  : 0x28f5 10485
	 *
	 * Reference doc: 19602tqlp_A753-0_1G5-DPLL_databook_0v7.pdf
	 */
	mfrac = (M % 100) * 10485;
	gb_printf(KERN_INFO, "mfrac = %u\n", mfrac);

	__gbreg_write(ppll_base, PLL_STAT(xpll), 0x00001000);
	__gbreg_write(ppll_base, PLL_CFG0(xpll),
		      0x04000000 | (N & 0xFF) | (high_speed << 9));
	__gbreg_write(ppll_base, PLL_CFG1(xpll),
		      (((M / 100) & 0xFFF) << 20) | (mfrac & 0xFFFFF));
	__gbreg_write(ppll_base, PLL_CFG2(xpll), 0x02000000
			| (p & 0x3F) << 16 | ((vdiv) << 24) | ((pdiv) << 22));
	__gbreg_write(ppll_base, PLL_CFG3(xpll), 0x00000000);
	__gbreg_write(ppll_base, PLL_STAT(xpll), 0x00011000);

	if (!(__gbreg_read(ppll_base, PLL_STAT(xpll)) & 0x01))
		return -1;

	return 0;
}

#define MAX_HIGH_MODE_FVCO 1500000
#define MAX_LOW_MODE_FVCO 1000000
#define MIN_FVCO 500000
#define MAX_CRTC1_CLK 123500

static int _pll_pix_clock_set(void __iomem  *ppll_base,
	const struct drm_display_mode *ptr, pll_module_enum xpll)
{
	int clock_temp = 0;
	int clkp_temp = 0;
	int clkp = 0;
	int clkin = 0;
	int high_speed = 0;
	u32 max_fvco = MAX_HIGH_MODE_FVCO;

	clk_type_e clk_type = CLK_TYPE_for_200;
	clock_temp = ptr->clock;
	if (xpll == PIX1_PLL && ptr->clock >= MAX_CRTC1_CLK)
		clock_temp = MAX_CRTC1_CLK;

	clkin = gb_get_pix_clockin(ppll_base - CORE_CTRL_CONFIG_OFFSET);
	if (!clkin) {
		clkin = 252;
		clk_type = CLK_TYPE_FOR_252;
	}

	if ((CLK_TYPE_for_200 == clk_type) &&
		(gb_vga_conn_judge(ppll_base - CORE_CTRL_CONFIG_OFFSET)))
		clk_type = CLK_TYPE_for_200_VGA;

find_clkp:
	if (clock_temp != 0)
		clkp = max_fvco / clock_temp;
	else {
		gb_printf(KERN_ERR, "%s %d clock freq is 0!\n",
				__func__, __LINE__);
		return -1;
	}

	clkp_temp = clkp;
	while ((clkp_temp * clock_temp) % 1000) {
		clkp_temp--;
		if (clkp_temp * clock_temp < MIN_FVCO) {
			clkp_temp = clkp;
			break;
		}
	}

	if (max_fvco == MAX_HIGH_MODE_FVCO) {
		if (clkp_temp % 2) {
			if ((clkp_temp - 1) * clock_temp > MAX_LOW_MODE_FVCO)
				high_speed = 1;
		} else {
			if (clkp_temp * clock_temp > MAX_LOW_MODE_FVCO)
				high_speed = 1;
		}
	}

	if (high_speed)
		clkp = (clkp_temp / 2) - 1;
	else {
		if (max_fvco == MAX_HIGH_MODE_FVCO) {
			max_fvco = MAX_LOW_MODE_FVCO;
			high_speed = 0;
			goto find_clkp;
		}
		clkp = clkp_temp - 1;
	}

	__ref2pll_clock_config(ppll_base, xpll, clock_temp,
		clkin * 100, clkp, high_speed);
#ifdef PLL_TEST_CONF
	__ref2pll_clock_config_file(ppll_base, xpll, clock_temp,
		clkin * 100, clkp);
#endif
	mdelay(50);
	if (!(__gbreg_read(ppll_base, PLL_STAT(xpll)) & 0x01)) {
		gb_printf(KERN_ERR, "PLL%d set pll %dx%d %dkHz failed!\r\n",
				xpll, ptr->hdisplay, ptr->vdisplay, ptr->clock);
		return -1;
	}

	mdelay(50);
	gb_printf(KERN_INFO, "PLL%d set pll %dx%d %dkHz succeed!\r\n",
				xpll, ptr->hdisplay, ptr->vdisplay, ptr->clock);
	return 0;
}

int pll_clock_set(void __iomem  *ppll_base, const struct drm_display_mode *ptr,
			int crtc_id)
{
	if (crtc_id == CRTC_0)
		return _pll_pix_clock_set(ppll_base, ptr, PIX_PLL);
	else
		return _pll_pix_clock_set(ppll_base, ptr, PIX1_PLL);
}

void lt86102_reg_write(void __iomem  *base, u32 addr, u32 data)
{
	u32 val = data;

	__gbreg_write(base, REG_OFFSET(addr), val);
}

u32 lt86102_reg_read(void __iomem  *base, u32 addr)
{
	u32 val = 0;

	val = __gbreg_read(base, REG_OFFSET(addr));
	return val;
}

#define LT86102_ACCESS_MAX_TIMES    5
#define LT86102_ADDR_6066   0x6066
int lt86102_get_edid_prepare(void __iomem  *base)
{
	int val = 0, i = 0;

    /* 1st : select DDC channel of LT86102's TX downstream to be accessed */
	for (i = 0; i < LT86102_ACCESS_MAX_TIMES; i++) {
		val = lt86102_reg_read(base, LT86102_RX_EDID_CTL_ADDR);
		if ((val & BIT(7)) == 0)
			break;
		mdelay(2);
	}
	if (i >= LT86102_ACCESS_MAX_TIMES)
		return -1;

	lt86102_reg_write(base, LT86102_RX_EDID_ADDR_HIGH_ADDR,
			(LT86102_ADDR_6066 >> 8) & 0xff);
	lt86102_reg_write(base, LT86102_RX_EDID_ADDR_LOW_ADDR,
			LT86102_ADDR_6066 & 0xff);
	lt86102_reg_write(base, LT86102_RX_EDID_DATA_ADDR, 0x4);
	lt86102_reg_write(base, LT86102_RX_EDID_CTL_ADDR,
			1 << LT86102_RX_EDID_CTRL_SHIFT);

	/* waiting for MCU to configure LT86102 for accessing EDID shadow */
	for (i = 0; i < LT86102_ACCESS_MAX_TIMES; i++) {
		val = lt86102_reg_read(base, LT86102_RX_EDID_CTL_ADDR);
		if ((val & BIT(7)) == 0)
			break;
		mdelay(10);
	}
	if (i >= LT86102_ACCESS_MAX_TIMES)
		return -2;

	return 0;
}

void backlight_reg_write(void __iomem  *base, u32 addr, u32 data)
{
	u32 val = data;

	__gbreg_write(base, REG_OFFSET(addr), val);
}

u32 backlight_reg_read(void __iomem  *base, u32 addr)
{
	u32 val = 0;

	val = __gbreg_read(base, REG_OFFSET(addr));
	return val;
}

u32 backlight_brightness_get(void __iomem  *base)
{
	u32 val = 0;

	val = backlight_reg_read(base, BACKLIGHT_DUTY_DATA_ADDR);
	val &= 0xff;

	return val;
}

void backlight_brightness_get_action(void __iomem  *base)
{
	backlight_reg_write(base, BACKLIGHT_DUTY_CTL_ADDR,
		(1 << BACKLIGHT_DUTY_CTRL_SHIFT)
		 | (1 << BACKLIGHT_DUTY_RW_SHIFT));
}

void backlight_brightness_set(void __iomem  *base, int brightness)
{
	backlight_reg_write(base, BACKLIGHT_DUTY_DATA_ADDR, brightness);
}

void backlight_brightness_set_action(void __iomem  *base)
{
	backlight_reg_write(base, BACKLIGHT_DUTY_CTL_ADDR,
		(1 << BACKLIGHT_DUTY_CTRL_SHIFT));
}

#define BACKLIGHT_ACCESS_MAX_TIMES  5
int backlight_brightness_action_check(void __iomem  *base)
{
	int i = 0;
	u32 val = 0;

	for (i = 0; i < BACKLIGHT_ACCESS_MAX_TIMES; i++) {
		val = backlight_reg_read(base, BACKLIGHT_DUTY_CTL_ADDR);
		if ((val & BIT(7)) == 0)
			break;
		mdelay(5);
	}

	if (i >= BACKLIGHT_ACCESS_MAX_TIMES)
		return -1;
	else
		return 0;
}

/*
 * detect the board type according to the value of
 * register GENBU_BOARD_TYPE_ADDR, as follows:
 * val = 1   ===> MXM BOARD
 * otherwise ===> PCIE BOARD
 */
u32 gb_board_type_detect(void __iomem  *base)
{
	u32 val = 0;
#if 0
	val = __gbreg_read(base, REG_OFFSET(GENBU_BOARD_TYPE_ADDR));
	val &= GENBU_BOARD_TYPE_MASK;
#endif
	return val;
}

/************************ (C) COPYRIGHT SIETIUM *****END OF FILE****/

