/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/delay.h>
#include <drm/drm_modes.h>
#ifndef __GB_IP_H__
#define __GB_IP_H__

#define	REG_OFFSET(value)	(value << 2)

#define MCU_SYSCTRL_BASE_ADDR 0x60000000
#define CFG_MCU_XIP_ADDR_OFFSET 0xd0c

#define		ENABLE_CONN_PORT	0
#define		DISABLE_CONN_PORT	3

#define	REF_CLOCK_20		(20000)
#define	REF_CLOCK_25		(25200)

#define PLL_CFG0_REG		0x00
#define PLL_CFG1_REG		0x10
#define PLL_CFG2_REG		0x20
#define PLL_CFG3_REG		0x30
#define PLL_STAT_REG		0x40
#define PLL_REG_SIZE		0x50
#define	CRTC_0		0
#define	CRTC_1		1
#define VERSION_REG			0x08
#define DBI_WR_EN_REG		0x8bc

#define PLL_CFG0(x)		((x*PLL_REG_SIZE)+PLL_CFG0_REG)
#define PLL_CFG1(x)		((x*PLL_REG_SIZE)+PLL_CFG1_REG)
#define PLL_CFG2(x)		((x*PLL_REG_SIZE)+PLL_CFG2_REG)
#define PLL_CFG3(x)		((x*PLL_REG_SIZE)+PLL_CFG3_REG)
#define PLL_STAT(x)		((x*PLL_REG_SIZE)+PLL_STAT_REG)
enum {
	HDMI_FC_INVIDCONF_HDCP_KEEPOUT_MASK = 0x80,
	HDMI_FC_INVIDCONF_HDCP_KEEPOUT_ACTIVE = 0x80,
	HDMI_FC_INVIDCONF_HDCP_KEEPOUT_INACTIVE = 0x00,
	HDMI_FC_INVIDCONF_VSYNC_IN_POLARITY_MASK = 0x40,
	HDMI_FC_INVIDCONF_VSYNC_IN_POLARITY_ACTIVE_HIGH = 0x40,
	HDMI_FC_INVIDCONF_VSYNC_IN_POLARITY_ACTIVE_LOW = 0x00,
	HDMI_FC_INVIDCONF_HSYNC_IN_POLARITY_MASK = 0x20,
	HDMI_FC_INVIDCONF_HSYNC_IN_POLARITY_ACTIVE_HIGH = 0x20,
	HDMI_FC_INVIDCONF_HSYNC_IN_POLARITY_ACTIVE_LOW = 0x00,
	HDMI_FC_INVIDCONF_DE_IN_POLARITY_MASK = 0x10,
	HDMI_FC_INVIDCONF_DE_IN_POLARITY_ACTIVE_HIGH = 0x10,
	HDMI_FC_INVIDCONF_DE_IN_POLARITY_ACTIVE_LOW = 0x00,
	HDMI_FC_INVIDCONF_DVI_MODEZ_MASK = 0x8,
	HDMI_FC_INVIDCONF_DVI_MODEZ_HDMI_MODE = 0x8,
	HDMI_FC_INVIDCONF_DVI_MODEZ_DVI_MODE = 0x0,
	HDMI_FC_INVIDCONF_R_V_BLANK_IN_OSC_MASK = 0x2,
	HDMI_FC_INVIDCONF_R_V_BLANK_IN_OSC_ACTIVE_HIGH = 0x2,
	HDMI_FC_INVIDCONF_R_V_BLANK_IN_OSC_ACTIVE_LOW = 0x0,
	HDMI_FC_INVIDCONF_IN_I_P_MASK = 0x1,
	HDMI_FC_INVIDCONF_IN_I_P_INTERLACED = 0x1,
	HDMI_FC_INVIDCONF_IN_I_P_PROGRESSIVE = 0x0,
};
typedef enum _pll_module {
	BUS_PLL = 0,
	GPU_PLL = 1,
	DDR_PLL = 2,
	PIX_PLL = 3,
	PIX1_PLL = 4
} pll_module_enum;

struct version_str {
	int mcu_vdata;
	int mcu_vid;
	int drv_vdata;
	int drv_vid;
	int bios_vdata;
	int bios_vid;
};

struct clk_to_pval {
	unsigned int clk_val;
	unsigned int post_div;
};

void get_firmware_verinfo(void __iomem *cfgbar_mmio,
			struct version_str *gb_version);
void set_firmware_verinfo(void __iomem *cfgbar_mmio,
            void __iomem *sysctl_cfg_base, struct version_str *gb_version);
void config_gb_vga_ep(void __iomem *cfgbar_mmio);

int pll_clock_set(void __iomem *ip_base,
			const struct drm_display_mode *ptr, int crtc_id);
int hdmi_mode_set(void __iomem *hdmi_base,
			const struct drm_display_mode *ptr);
int hdmi_power_on(void __iomem *hdmi_base);
int hdmi_power_off(void __iomem *hdmi_base);
void gb_mcu_shutdown(void __iomem *ip_base);
int gb_vga_switch(void __iomem *hdmi_base, int vga_cmd);
void gb_disable_port(void __iomem *ip_base);
void lt86102_reg_write(void __iomem  *base, u32 addr, u32 data);
u32 lt86102_reg_read(void __iomem  *base, u32 addr);
int lt86102_get_edid_prepare(void __iomem  *base);
void backlight_reg_write(void __iomem  *base, u32 addr, u32 data);
u32 backlight_reg_read(void __iomem  *base, u32 addr);
u32 backlight_brightness_get(void __iomem  *base);
void backlight_brightness_get_action(void __iomem  *base);
void backlight_brightness_set(void __iomem  *base, int brightness);
void backlight_brightness_set_action(void __iomem  *base);
int backlight_brightness_action_check(void __iomem  *base);
u32 gb_board_type_detect(void __iomem  *base);
#endif
