/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/module.h>
#ifndef __GBDC_REGS_H__
#define __GBDC_REGS_H__

/*
 * abbreviations used:
 *    - DC - display core (general settings)
 *    - DE - display engine
 *    - SE - scaling engine
 */

/* interrupt bit masks */
#define GBDC_DE_IRQ_UNDERRUN			(1 << 0)

#define GBDC_DE_IRQ_SATURATION		(1 << 8)
#define GBDC_DE_IRQ_VSYNC			(1 << 12)
#define GBDC_DE_IRQ_PROG_LINE		(1 << 13)
#define GBDC_DE_IRQ_AXI_ERR		(1 << 16)
#define GBDC_SE_IRQ_EOW			(1 << 0)
#define GBDC_SE_IRQ_AXI_ERR		(1 << 16)
#define GBDC_DC_IRQ_CONF_VALID		(1 << 0)
#define GBDC_DC_IRQ_CONF_MODE		(1 << 4)
#define GBDC_DC_IRQ_CONF_ACTIVE		(1 << 16)
#define GBDC_DC_IRQ_DE			(1 << 20)
#define GBDC_DC_IRQ_SE			(1 << 24)

/* bit masks that are common between products */
#define   GBDC_CFG_VALID		(1 << 0)
#define   GBDC_DISP_FUNC_GAMMA	(1 << 0)
#define   GBDC_DISP_FUNC_CADJ		(1 << 4)
#define   GBDC_DISP_FUNC_ILACED	(1 << 8)

/* register offsets for IRQ management */
#define GBDC_REG_STATUS		0x00000
#define GBDC_REG_SETIRQ		0x00004
#define GBDC_REG_MASKIRQ		0x00008
#define GBDC_REG_CLEARIRQ		0x0000c

/* register offsets */
#define GBDC_DE_CORE_ID		0x00018
#define GBDC_DE_DISPLAY_FUNC		0x00020

/* these offsets are relative to GBDC5x0_TIMINGS_BASE */
#define GBDC_DE_H_TIMINGS		0x0
#define GBDC_DE_V_TIMINGS		0x4
#define GBDC_DE_SYNC_WIDTH		0x8
#define GBDC_DE_HV_ACTIVE		0xc

/* Stride register offsets relative to Lx_BASE */
#define GBDC_DE_LG_STRIDE		0x18
#define GBDC_DE_LV_STRIDE0		0x18
#define GBDC_DE_LS_R1_STRIDE	0x28

/* macros to set values into registers */
#define GBDC_DE_H_FRONTPORCH(x)	(((x) & 0xfff) << 0)
#define GBDC_DE_H_BACKPORCH(x)	(((x) & 0x3ff) << 16)
#define GBDC_DE_V_FRONTPORCH(x)	(((x) & 0xfff) << 0)
#define GBDC_DE_V_BACKPORCH(x)	(((x) & 0xff) << 16)
#define GBDC_DE_H_SYNCWIDTH(x)	(((x) & 0x3ff) << 0)
#define GBDC_DE_V_SYNCWIDTH(x)	(((x) & 0xff) << 16)
#define GBDC_DE_H_ACTIVE(x)		(((x) & 0x1fff) << 0)
#define GBDC_DE_V_ACTIVE(x)		(((x) & 0x1fff) << 16)

#define GBDC_PRODUCT_ID(__core_id) ((u32)(__core_id) >> 16)

/* register offsets relative to GBDC5x0_COEFFS_BASE */
#define GBDC_COLOR_ADJ_COEF		0x00000
#define GBDC_COEF_TABLE_ADDR		0x00030
#define GBDC_COEF_TABLE_DATA		0x00034

/* Scaling engine registers and masks. */
#define   GBDC_SE_SCALING_EN			(1 << 0)
#define   GBDC_SE_ALPHA_EN			(1 << 1)
#define   GBDC_SE_ENH_MASK			3
#define   GBDC_SE_ENH(x)			(((x) & GBDC_SE_ENH_MASK) << 2)
#define   GBDC_SE_RGBO_IF_EN			(1 << 4)
#define   GBDC_SE_CTL_SEL_MASK		7
#define   GBDC_SE_CTL_VCSEL(x) \
		(((x) & GBDC_SE_CTL_SEL_MASK) << 20)
#define   GBDC_SE_CTL_HCSEL(x) \
		(((x) & GBDC_SE_CTL_SEL_MASK) << 16)

/* Blocks with offsets from SE_CONTROL register. */
#define GBDC_SE_LAYER_CONTROL			0x14
#define   GBDC_SE_L0_IN_SIZE			0x00
#define   GBDC_SE_L0_OUT_SIZE			0x04
#define   GBDC_SE_SET_V_SIZE(x)		(((x) & 0x1fff) << 16)
#define   GBDC_SE_SET_H_SIZE(x)		(((x) & 0x1fff) << 0)
#define GBDC_SE_SCALING_CONTROL		0x24
#define   GBDC_SE_H_INIT_PH			0x00
#define   GBDC_SE_H_DELTA_PH			0x04
#define   GBDC_SE_V_INIT_PH			0x08
#define   GBDC_SE_V_DELTA_PH			0x0c
#define   GBDC_SE_COEFFTAB_ADDR		0x10
#define     GBDC_SE_COEFFTAB_ADDR_MASK	0x7f
#define     GBDC_SE_V_COEFFTAB		(1 << 8)
#define     GBDC_SE_H_COEFFTAB		(1 << 9)
#define     GBDC_SE_SET_V_COEFFTAB_ADDR(x) \
		(GBDC_SE_V_COEFFTAB | ((x) & GBDC_SE_COEFFTAB_ADDR_MASK))
#define     GBDC_SE_SET_H_COEFFTAB_ADDR(x) \
		(GBDC_SE_H_COEFFTAB | ((x) & GBDC_SE_COEFFTAB_ADDR_MASK))
#define   GBDC_SE_COEFFTAB_DATA		0x14
#define     GBDC_SE_COEFFTAB_DATA_MASK	0x3fff
#define     GBDC_SE_SET_COEFFTAB_DATA(x) \
		((x) & GBDC_SE_COEFFTAB_DATA_MASK)
/* Enhance coeffents reigster offset */
#define GBDC_SE_IMAGE_ENH			0x3C
/* ENH_LIMITS offset 0x0 */
#define     GBDC_SE_ENH_LOW_LEVEL		24
#define     GBDC_SE_ENH_HIGH_LEVEL		63
#define     GBDC_SE_ENH_LIMIT_MASK		0xfff
#define     GBDC_SE_SET_ENH_LIMIT_LOW(x) \
		((x) & GBDC_SE_ENH_LIMIT_MASK)
#define     GBDC_SE_SET_ENH_LIMIT_HIGH(x) \
		(((x) & GBDC_SE_ENH_LIMIT_MASK) << 16)
#define   GBDC_SE_ENH_COEFF0			0x04

#define BAR2_CFG_BASE			0x00200

#define GBDC_ADDR_SPACE_SIZE	0x10000
#define GBDC_DE_CONTROL		0x00010
#define GBDC_DE_LINE_COUNTER	0x00014
#define GBDC_DE_AXI_CONTROL	0x00018
#define GBDC_DE_QOS		0x0001c
#define GBDC_TIMINGS_BASE		0x00030
#define GBDC_HSYNCPOL		(1 << 12)
#define GBDC_VSYNCPOL		(1 << 28)

#define GBDC_DE_DISP_SIDEBAND	0x00040
#define GBDC_DE_BGND_COLOR		0x00044
#define GBDC_DE_OUTPUT_DEPTH	0x0004c
#define GBDC_COEFFS_BASE		0x00050
#define GBDC_COEFFTAB_ADDR		0x00080
#define GBDC_COEFFTAB_DATA		0x00084
#define GBDC_DE_LV1_BASE		0x00100
#define GBDC_DE_LV1_PTR_BASE	0x00124
#define GBDC_DE_LV2_BASE		0x00200
#define GBDC_DE_LV2_PTR_BASE	0x00224
#define GBDC_DE_LG_BASE		0x00300
#define GBDC_DE_LG_PTR_BASE	0x0031c
#define GBDC_DE_LS_BASE		0x00400
#define GBDC_DE_LS_PTR_BASE	0x0042c
#define GBDC_DE_PERF_BASE		0x00500
#define GBDC_SE_BASE		0x08000
#define GBDC_SE_CONTROL		0x08010
#define GBDC_DC_BASE		0x0c000
#define GBDC_DC_CONTROL		0x0c010
#define GBDC_DC_CONFIG_REQ	(1 << 16)
#define GBDC_DC_CONFIG_REST	(1 << 17)
#define GBDC_DC_CONFIG_SRST	(1 << 18)
#define GBDC_CONFIG_VALID		0x0c014
#define GBDC_CONFIG_ID		0x0ffd4



#define	DC_REG_OFFSET					(0x00100000UL)
#define	DC_OFFSET_STEP					(0x00020000UL)
#define PCIE_CONFIG_OFFSET              (0x00000000UL)
#define DDR_CONFIG_OFFSET               (0x00010000UL)
#define CORE_CTRL_CONFIG_OFFSET         (0x00020000UL)
#define HDMI_CONFIG_OFFSET              (0x00040000UL)
#define HDMI_DBG_CONFIG_OFFSET          (0x00080000UL)

#define	GB_DC_OFFSET_STEP					(0x00020000UL)
#define	FPGA_DC_OFFSET_STEP					(0x00020000UL)

#define	GBDC_DC_CONTROL_OFFSET	0x00010
#define	GBDC_DC_VALID_OFFSET	0X00014



/* Layer specific register offsets */
#define GBDC_LAYER_CONTROL		0x004
#define   LAYER_ENABLE			(1 << 0)
#define   LAYER_FLOWCFG_MASK		7
#define   LAYER_FLOWCFG(x)		(((x) & LAYER_FLOWCFG_MASK) << 1)
#define     LAYER_FLOWCFG_SCALE_SE	3
#define   LAYER_ROT_OFFSET		8
#define   LAYER_H_FLIP			(1 << 10)
#define	  PIXEL_ALPHA_COMP		(1 << 12)
#define   LAYER_V_FLIP			(1 << 11)
#define   LAYER_ROT_MASK		(0xf << 8)
#define   LAYER_COMP_MASK		(0x3 << 12)
#define   LAYER_ALPHA_MASK		(0xff << 16)
#define GBDC_LAYER_COMPOSE		0x008
#define GBDC_LAYER_SIZE		0x00c
#define   LAYER_H_VAL(x)		(((x) & 0x1fff) << 0)
#define   LAYER_V_VAL(x)		(((x) & 0x1fff) << 16)
#define GBDC_LAYER_COMP_SIZE		0x010
#define GBDC_LAYER_OFFSET		0x014
#define GBDC_LS_R1_IN_SIZE		0x020

#define GBDC_BGND_COLOR_R		0x000
#define GBDC_BGND_COLOR_G		0x000
#define GBDC_BGND_COLOR_B		0x000

/* REG */
#define HDMI_PHY_STAT0 0x3004
#define HDMI_ADV_STAT0 0x202
#define HDMI_PHY_HPD 0x02
#define HDMI_ADV_HPD 0x01
#define	HDMI_ADV_HPD_V	'h'
#define	HDMI_MCU_CRTL_V			'r'
#define	HDMI_MCU_CTRL			0x203
#define	PIX_CLOCK_REG			0x204
#define	VGA_PORT_SWITCH			0x205
#define	VGA_PORT_STATUS			0x206
#define	MCU_VERDATE_REG			0x218
#define	MCU_VERGID_REG			0x21c
#define	BIOS_VERDATE_REG		0x200
#define	BIOS_VERGID_REG			0x204

/*
 * bit7 : 1 = GB01 access; 0 = MCU access;
 * bit6 : 1 = GB01 read; 0 = GB01 write;
 */
#define LT86102_RX_EDID_CTL_ADDR		0x1201
#define LT86102_RX_EDID_CTRL_SHIFT		7
#define LT86102_RX_EDID_RW_SHIFT		6
#define LT86102_RX_EDID_DATA_ADDR		0x1202
#define LT86102_RX_EDID_ADDR_HIGH_ADDR	0x1203
#define LT86102_RX_EDID_ADDR_LOW_ADDR	0x1204

/*
 * bit7 : 1 = GB01 access; 0 = MCU access;
 * bit6 : 1 = GB01 read; 0 = GB01 write;
 */
#define BACKLIGHT_DUTY_CTL_ADDR			0x1205
#define BACKLIGHT_DUTY_CTRL_SHIFT		7
#define BACKLIGHT_DUTY_RW_SHIFT			6
#define BACKLIGHT_DUTY_DATA_ADDR		0x1206

/*
 * bit2 : 1 = VGA connector; 0 = others;
 * bit[1:0] : 1 = MXM BOARD; 0 = PCIE BOARD;
 */
#define GENBU_BOARD_TYPE_ADDR			0x1207
#define GENBU_BOARD_TYPE_MASK			0x3
#define GENBU_VGA_CONNECTOR_SHIFT		2

#endif				/* __GBDC_REGS_H__ */
