/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/module.h>
#include <drm/drm_mm.h>
#include <drm/drm_property.h>
//#include <ip/gb_edma.h>
//#include <ip/gb_v2vdma.h>
//#include "kms/device/gb_dev_res.h"
#include "gpu/gb_device.h"
#include "kms/gbdc_mode.h"
#ifndef		__GB_KMS_H__
#define		__GB_KMS_H__

struct gb_plane_res;
struct gb_kms_mode_ops {
	int (*enter_config)(void __iomem *dc_base, void __iomem *de_base);
	int (*config_mode)(void __iomem *de_base, struct videomode *vmode);
	int (*gamma_set)(void __iomem *dc_base,
			 void __iomem *de_base, struct drm_color_lut *lut);
	int (*config_base)(struct drm_crtc *dcrtc, 	struct gb_plane_res *plane, void __iomem *de_base);
	int (*config_base_atomic)(struct drm_crtc *dcrtc, struct gb_plane_res *plane, struct drm_plane_state *state, void __iomem *de_base);
	int (*get_resolution)(struct drm_display_mode *dmode);
	int (*config_plane_base)(struct drm_plane *dplane, struct gb_plane_res *plane, void __iomem *de_base);
	int (*query_hw)(void __iomem *dc_base);
	int (*save_csr_image)(struct gb_plane_res *plane,
	 void __iomem *dc_base, void __iomem *de_base, uint64_t csr_addr,
	  int width, int height);
	int (*config_csr_pos)(struct gb_plane_res *plane,
	 void __iomem *dc_base, void __iomem *de_base, int x, int y,
	  int width, int height, int stride);
	int (*hide_csr)(struct gb_plane_res *plane, void __iomem *dc_base,
	 void __iomem *de_base);
	int (*leave_config)(void __iomem *dc_base, void __iomem *de_base);
	int (*config_overly_plane)(struct drm_crtc *dcrtc, struct drm_plane *drm_planes,
		struct gb_plane_res *plane, struct drm_framebuffer *fb,
		void __iomem *de_base, int plane_id);
	int (*colse_overly_plane)(u16 base, void __iomem *dc_base,
		void __iomem *de_base);
	int (*config_virt_base_atomic)(struct drm_crtc *dcrtc,
		struct gb_plane_res *plane, struct drm_plane_state *state);
};

struct gb_kms_info{
	struct gbgpu_fbdev			fb_info;
	struct gb_kms_mode_ops			*kms_ops;
	struct gbdev_connector			*connector_res;
	struct gb_plane_res			*plane_res;
	struct gb_plane_cap 			*plane_cap;
	int					num_plane_cap;
	int					num_crtc;
	int 					num_connector;
	int					num_plane;
	bool 					mode_config_initialized;
	const struct gb_max_resolution		*max_res;
	bool infinity;
	struct gbdc_crtc *crtcs[GBDC_MAX_CRTCS + (GBDC_MAX_CRTCS / 2)];
	struct list_head  virt_crtc_head;
	struct list_head  virt_conn_head;
	int virt_crtcid;

	struct drm_property *infinity_set_property;//phyconn
	struct drm_property *virtual_flag_property;//phyconn

	/*virt*/
	struct drm_property *infinity_enable_property;
	struct drm_property *infinity_info_property;
	struct drm_property *infinity_adjust_property;

	struct list_head infinity_list;
	spinlock_t infinity_lock;
	struct delayed_work infinity_hotplug_work;
	bool connect_status_change;
	struct mutex gbdc_infinity_atomic_mutex;
};

int gbdc_kms_init(struct gb_kms_info *kms_info, struct drm_device *drm_dev, struct gb_vram_info *vram_config);
void gbdc_kms_fini(struct gb_kms_info *kms_info, struct drm_device *drm_dev);
int gb_edma_translate_to_ram(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
int gb_edma_translate_to_fb(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
int gb_v2vdma_translate(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
int gb_edma_offset_translate(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);

#define CRTC_ID_0 0
#define CRTC_ID_1 1
#define CRTC_ID_2 2
#define CRTC_ID_3 3
#define CRTC_ID_4 4
#define CRTC_ID_5 5
#define CRTC_ID_6 6
#define CRTC_ID_7 7
#define CRTC_ID_8 8
#define CRTC_ID_9 9
int get_crtc_id(enum gb_board_type gb_type, int id);
int get_virt_crtc_id(enum gb_board_type gb_type, int vidx);
int gb_port_check_nouse(enum gb_board_type gb_type, int id);
struct gb_kms_info *gb_get_kms_info(void);

#ifdef GB_ALLSCREEN
struct gb_config_full_screen {
		atomic_t flag;
		phys_addr_t fb_addr;
		u64 height;
		u64 width;
		u64 size;
		struct drm_mm_node node;
};
#define DC_FB 0
#define NEW_FRONT_FB 1
#define NEW_BACK_FB 2
#define MAX_FULL_FB 3
#define MAX_FB 6

struct gb_fb_info {
		int expend_flag;
		struct gb_plane_res *plane_res;
		struct gb_config_full_screen crtc_plane[MAX_FB];
		struct gb_config_full_screen info[MAX_FULL_FB];
};

struct gb_fb_info *gb_get_fb_info_point(void);
int gb_get_fb_base_info(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
int gb_switch_fb(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
int gb_config_fb(struct gbdc_device *gbdc_dev, struct gb_fb_info *fb_info,
	int fb_type, int expend_flag);
#endif
int gb_set_render_size(struct drm_device *drm_dev, void *data,
	struct drm_file *filp);
#endif
