/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef	__GBDC_INFINITY_H__
#define	__GBDC_INFINITY_H__

/*avoid memory overstep*/
#define INFINITY_MAX_SIZE GBDC_MAX_CRTCS
#define INFINITY_MAX_MODE_NUM 32

enum gbdc_infinity_info_type {
	INFINITY_ID,
	INFINITY_ENABLE,
	INFINITY_ROW,
	INFINITY_COL,
	INFINITY_CONN_ID,
	INFINITY_CONN_STAT,
	INFINITY_ORDER,
	INFINITY_GEOMETRY,
	INFINITY_PADDING,
};

struct gbdc_infinity_padding {
	unsigned int width;
	unsigned int height;
	unsigned int top;
	unsigned int right;
	unsigned int bottom;
	unsigned int left;
};

struct gbdc_infinity_order {
	unsigned int x;
	unsigned int y;
};

struct gbdc_infinity_geometry {
	unsigned int x;
	unsigned int y;
	unsigned int w;
	unsigned int h;
};

struct gbdc_infinity_connector_info {
	unsigned int connector_id;
	enum drm_connector_status connected;
	struct gbdc_infinity_order order;
	struct gbdc_infinity_geometry geometry;
	struct gbdc_infinity_padding padding[INFINITY_MAX_MODE_NUM];
};

struct gbdc_infinity_connector_adjust {
	unsigned int connector_id;
	struct gbdc_infinity_order order;
	struct gbdc_infinity_padding padding[INFINITY_MAX_MODE_NUM];
};

struct gbdc_infinity_connector_infos {
	unsigned int infinity_id;
	unsigned int infinity_enable;
	unsigned int infinity_row;
	unsigned int infinity_col;
	struct gbdc_infinity_connector_info infos[0];
};

struct gbdc_infinity_connector_adjusts {
	unsigned int infinity_row;
	unsigned int infinity_col;
	struct gbdc_infinity_connector_adjust adjusts[0];
};

struct gbdc_infinity_connector_set {
	unsigned int infinity_id;
	unsigned int infinity_row;
	unsigned int infinity_col;
	struct gbdc_infinity_order order;
	struct gbdc_infinity_padding padding[INFINITY_MAX_MODE_NUM];
};

struct st_info_changed{
	unsigned int id_changed : 1;
	unsigned int order_changed : 1;
	unsigned int padding_changed : 1;
};

union u_info_changed{
	unsigned int info_changed;
	struct st_info_changed st;
};

struct gbdc_infinity_instance {
	struct list_head head;
	unsigned int infinity_connector_id;
	enum drm_connector_status connected;

	struct gbdc_connector *gbdc_connector;

	bool created;
	bool changed;
	bool mode_changed;
	bool geometry_changed;
	bool padding_changed;
	bool recreated;
	bool destroyed;
	bool rebuild;
	bool tiny_recreated;
	bool crtc_need_disable;

	union u_info_changed *uinfo_changed;

	struct gbdc_infinity_connector_infos *infinity_infos;
};

#define INFINITY_INFO_MAX_SIZE \
	(sizeof(struct gbdc_infinity_connector_info) * INFINITY_MAX_SIZE)

#define INFINITY_INFOS_MAX_SIZE \
	(sizeof(struct gbdc_infinity_connector_infos) + INFINITY_INFO_MAX_SIZE)

#define INFINITY_ADJ_MAX_SIZE \
	(sizeof(struct gbdc_infinity_connector_adjust) * INFINITY_MAX_SIZE)

#define INFINITY_ADJS_MAX_SIZE \
	(sizeof(struct gbdc_infinity_connector_adjusts) + INFINITY_ADJ_MAX_SIZE)

#define gbdc_for_each_infinity(infinity, dev) \
	list_for_each_entry(infinity, \
		&gb_get_gbdc_device((dev)->dev_private)->kms_info.infinity_list, head)

#define gbdc_for_each_infinity_safe(infinity, dev) \
	list_for_each_entry_safe(infinity, n_##infinity,\
		&gb_get_gbdc_device((dev)->dev_private)->kms_info.infinity_list, head)

#define gbdc_for_each_gbdc_obj(name) \
	list_for_each_entry(gbdc_##name, \
		&gbdc_##name##_state->infinity_##name##_list->head, head)

#define gbdc_for_each_gbdc_obj_safe(name) \
	list_for_each_entry_safe(gbdc_##name, n_##name,\
		&gbdc_##name##_state->infinity_##name##_list->head, head)

#define GB_PRINT_INFO(fmt, ...) \
	gb_printf(KERN_INFO, fmt, ##__VA_ARGS__)

#define GB_PRINT_ERR(fmt, ...) \
	gb_printf(KERN_ERR, fmt, ##__VA_ARGS__)

#define GB_PRINT_DBG(fmt, ...) \
	gb_printf(KERN_DEBUG, fmt, ##__VA_ARGS__)

inline void gbdc_infinity_get_kms_mutex_lock(struct drm_device *dev);
inline void gbdc_infinity_get_kms_mutex_unlock(struct drm_device *dev);
int gbdc_infinity_check(struct drm_device *dev,
						struct drm_atomic_state *state);
int gbdc_infinity_commit(struct drm_device *dev,
						struct drm_atomic_state *state);
void gbdc_infinity_commit_tail(struct drm_atomic_state *state);
//enum drm_connector_status gbdc_infinity_get_connect_status(struct drm_connector *connector);
//bool gbdc_infinity_status();
bool gbdc_infinity_connector_is_contained(struct drm_connector *connector,
									struct gbdc_infinity_instance *infi, bool irq);
bool gbdc_infinity_connector_id_is_contained(struct drm_device *dev,
											int connector,
											struct gbdc_infinity_instance *infi, bool irq);
void gbdc_infinity_set_connector_status(struct drm_device *dev,
							struct drm_connector *connector);
void gbdc_infinity_set_connector_status_by_id(struct drm_device *dev, int connector);
int gbdc_connector_update_infinity_info(
						struct gbdc_connector *gbdc_connector,
						enum gbdc_infinity_info_type type,
						unsigned int pos,
						void *data);
int gbdc_infinity_get_virtid_from_phyid(struct drm_device *dev, int phy_id, bool irq);
void gbdc_infinity_hpd_work_func(struct work_struct *work);
ssize_t gbdc_infinity_info_show(struct drm_device *dev, char *buf);
ssize_t gbdc_infinity_state_show(struct drm_device *dev, char *buf);
ssize_t gbdc_infinity_prop_show(struct drm_device *dev, char *buf);
void gbdc_infinity_schedule_work(struct drm_device *dev);
int gbdc_connector_create_infinity_info_property(struct drm_connector *connector);
void gbdc_connector_update_infinity_info_property_values(struct drm_connector *connector,
									const struct gbdc_infinity_connector_infos *infos,
									unsigned int size);
void gbdc_infinity_replace_vblank(struct drm_device *dev, int crtc_id);
int gbdc_infinity_atomic_ioctl(struct drm_device *dev,
			  void *data, struct drm_file *file_priv);
void gbdc_atomic_helper_commit_modeset_disables(struct drm_device *dev,
					       struct drm_atomic_state *old_state);
struct mutex *gbdc_infinity_get_mutex_lock(struct drm_atomic_state *state);
struct mutex *gbdc_infinity_get_atomic_mutex_lock(struct drm_atomic_state *state);
void gbdc_infinity_fake_vblank(struct drm_atomic_state *old_state);
#endif