// SPDX-License-Identifier: GPL-2.0
#include "gpu/gb_device.h"
#include "gb-peripherals-common.h"
#include "mcu_peripherals/i2c/tmp102.h"
#include "mcu_peripherals/i2c/ina2xx.h"
#include "mcu_peripherals/i2c/ads1015.h"
#include "mcu_peripherals/i2c/ps8625.h"
#include "mcu_peripherals/uart/gb02_usart.h"
#include "mcu_peripherals/gpio/pinctrl-gb02-mcu.h"
#include "mcu_peripherals/spi/spi-gb02-mcu.h"
#include "mcu_peripherals/i2c/i2c-gb02-mcu.h"
#include "mcu_peripherals/pwm/gb_pwm.h"
#include "mcu_peripherals/pvt/gb02-pvt.h"
#include "mcu_peripherals/gpu_freq/gpu_freq.h"
#include "mcu_peripherals/fan/gb02_fan.h"
#include "mcu_peripherals/sn_info/gb_sn_info.h"
#include "mcu_peripherals/pcie_msg/gb02_pcie_msg.h"

#define PERI_IP_CNT (sizeof(mcu_periph_info) / sizeof(struct gb02_mcu_peripherals_info))
#define I2C_DEVICE_CNT (sizeof(mcu_i2c_device) / sizeof(struct gb02_i2c_device))
#define PWM_DEVICE_CNT (sizeof(mcu_pwm_device) / sizeof(struct gb02_pwm_device))

static struct gb02_mcu_peripherals_info mcu_periph_info[] = {
	/*UART*/
	{
		.mcu_peripherals_device_name = PLATFORM_UART_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.peri_offset_address = MCU_UART0_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_uart_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_uart_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_UART_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID1,
		.peri_offset_address = MCU_UART1_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_uart_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_uart_drv_exit,
	},
	/*SPI*/
	{
		.mcu_peripherals_device_name = PLATFORM_SPI_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.peri_offset_address = MCU_SPI_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_spi_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_spi_drv_exit,
	},
	/*I2C*/
	{
		.mcu_peripherals_device_name = PLATFORM_I2C_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.peri_offset_address = MCU_I2C0_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_i2c_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_i2c_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_I2C_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID1,
		.peri_offset_address = MCU_I2C1_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_i2c_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_i2c_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_I2C_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID2,
		.peri_offset_address = MCU_I2C2_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_i2c_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_i2c_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_I2C_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID3,
		.peri_offset_address = MCU_I2C3_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_i2c_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_i2c_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_LGPIO_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.peri_offset_address = MCU_LGPIO_PERI_ADDR_BASE,
		.gb02_peri_ip_drv_init = gb02_mcu_gpio_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_gpio_drv_exit,
	},
	/*GPIO_PWM*/
	{
		.mcu_peripherals_device_name = PLATFORM_GPIO_PWM_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.gpio_num = 40,
		.gb02_peri_ip_drv_init = gb02_mcu_gpio_pwm_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_gpio_pwm_drv_exit,
		.gb02_peri_ip_drv_resume = gb02_mcu_gpio_pwm_drv_resume,
	},
	/*PWM*/
	{
		.mcu_peripherals_device_name = PLATFORM_PWM_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID1,
		.peri_offset_address = MCU_PWM1_PERI_ADDR_BASE,
		.gpio_num = -1,
		.gb02_peri_ip_drv_init = gb02_mcu_pwm_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_pwm_drv_exit,
	},
	{
		.mcu_peripherals_device_name = PLATFORM_PWM_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.peri_offset_address = MCU_PWM0_PERI_ADDR_BASE,
		.gpio_num = -1,
		.gb02_peri_ip_drv_init = gb02_mcu_pwm_drv_init,
		.gb02_peri_ip_drv_exit = gb02_mcu_pwm_drv_exit,
	},
	/*PVT*/
	{
		.mcu_peripherals_device_name = PLATFORM_PVT_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.gb02_peri_ip_drv_init = gb02_pvt_drv_init,
		.gb02_peri_ip_drv_exit = gb02_pvt_drv_exit,
	},
	/*FREQ*/
	{
		.mcu_peripherals_device_name = PLATFORM_FREQ_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.gb02_peri_ip_drv_init = gb02_freq_drv_init,
		.gb02_peri_ip_drv_exit = gb02_freq_drv_exit,
	},
	/*FAN*/
	{
		.mcu_peripherals_device_name = PLATFORM_FAN_DEVICE_NAME,
		.mcu_peripherals_device_id = DEVICE_ID0,
		.gb02_peri_ip_drv_init = gb02_fan_drv_init,
		.gb02_peri_ip_drv_exit = gb02_fan_drv_exit,
		.gb02_peri_ip_drv_suspend = gb02_fan_drv_suspend,
		.gb02_peri_ip_drv_resume = gb02_fan_drv_resume,
	},
};

static struct gb02_i2c_device mcu_i2c_device[] = {
	{
		.board_info = {I2C_BOARD_INFO(INA219_DEVICE_NAME, INA219_ADDR),},
		.bus_id = INA219_BUS_ID,
		.i2c_driver_init = ina2xx_driver_init,
		.i2c_driver_exit = ina2xx_remove,
		.enable = false,
	},
	{
		.board_info = {I2C_BOARD_INFO(ADS1015_DEVICE0_NAME, ADS1015_BUS0_ADDR),},
		.bus_id = ADS1015_BUS0_ID,
		.i2c_driver_init = ads1015_driver_init,
		.i2c_driver_exit = ads1015_remove,
		.enable = false,
	},
	{
		.board_info = {I2C_BOARD_INFO(ADS1015_DEVICE1_NAME, ADS1015_BUS1_ADDR),},
		.bus_id = ADS1015_BUS1_ID,
		.i2c_driver_init = ads1015_driver_init,
		.i2c_driver_exit = ads1015_remove,
		.enable = false,
	},
	{
		.board_info = {I2C_BOARD_INFO(ADS1015_DEVICE2_NAME, ADS1015_BUS2_ADDR),},
		.bus_id = ADS1015_BUS2_ID,
		.i2c_driver_init = ads1015_driver_init,
		.i2c_driver_exit = ads1015_remove,
		.enable = false,
	},
};

static struct gb02_pwm_device mcu_pwm_device[] = {
	{
		.device_id = FAN_PWM_CTRL_NO,
		.name = GB02_PWM_FAN_DEV_NAME,
		.gpio_pwm = false, //for gpio pwm
		.pwm_driver_init = gb02_pwm_fan_init,
		.pwm_driver_exit = gb02_pwm_fan_exit,
		.pwm_driver_resume = pwm_fan_resume,
		.pwm_driver_suspend = pwm_fan_suspend,
		.enable = false,
	},
	{
		.device_id = DEVICE_ID0,
		.name = GB02_PWM_BL_DEV_NAME,
		.pwm_driver_init = gb02_pwm_backlight_init,
		.pwm_driver_exit = gb02_pwm_backlight_exit,
		.num_connector = 0,
		.enable = true,
	},
};

static void gb02_peri_drv_init(struct gb_device *gbdev, int device_num)
{
	int ret;
	struct gb02_mcu_peripherals_info *peri_info;

	peri_info = &gbdev->gb_peri[device_num];
	if (!peri_info || !peri_info->gb02_peri_ip_drv_init)
		return;

	ret = peri_info->gb02_peri_ip_drv_init(gbdev, peri_info);
	if (ret)
		gb_printf(KERN_ERR, "%s%d driver init failed\n",
			peri_info->mcu_peripherals_device_name, peri_info->mcu_peripherals_device_id);
}

static void gb02_peri_drv_exit(struct gb_device *gbdev, int device_num)
{
	struct gb02_mcu_peripherals_info *peri_info;

	peri_info = &gbdev->gb_peri[device_num];
	if (!peri_info || !peri_info->gb02_peri_ip_drv_exit)
		return;

	peri_info->gb02_peri_ip_drv_exit(peri_info);
}

static void gb02_peri_drv_suspend(struct gb_device *gbdev, int device_num)
{
	struct gb02_mcu_peripherals_info *peri_info;

	peri_info = &gbdev->gb_peri[device_num];
	if (!peri_info || !peri_info->gb02_peri_ip_drv_suspend)
		return;

	peri_info->gb02_peri_ip_drv_suspend(peri_info);
}

static void gb02_peri_drv_resume(struct gb_device *gbdev, int device_num)
{
	struct gb02_mcu_peripherals_info *peri_info;

	peri_info = &gbdev->gb_peri[device_num];
	if (!peri_info || !peri_info->gb02_peri_ip_drv_resume)
		return;

	peri_info->gb02_peri_ip_drv_resume(peri_info);
}

static struct gb02_mcu_peripherals_info *gb02_get_mcu_peripherals_info(struct gb_device *gbdev, int device_id, char *name)
{
	int i, ret;

	for (i = 0; i < PERI_IP_CNT; i++) {
		ret = strcmp(gbdev->gb_peri[i].mcu_peripherals_device_name, name);
		if (!ret) {
			if (gbdev->gb_peri[i].mcu_peripherals_device_id == device_id) {
				if (!gbdev->gb_peri[i].priv)
					return NULL;
				else
					return &gbdev->gb_peri[i];
			}
		}
	}

	return NULL;
}

static void gb02_i2c_device_init(struct gb_device *gbdev, struct gb02_i2c_device *i2c_device)
{
	struct i2c_adapter *adapter;
	struct gb02_mcu_i2c *mcu_i2c;
	struct gb02_mcu_peripherals_info *peri_info;
	int ret;

	if (!i2c_device->enable)
		return;
	peri_info = gb02_get_mcu_peripherals_info(gbdev, i2c_device->bus_id, PLATFORM_I2C_DEVICE_NAME);
	if (!peri_info)
		return;

	mcu_i2c = (struct gb02_mcu_i2c *)peri_info->priv;
	if (!mcu_i2c)
		return;
	adapter = &mcu_i2c->adapter.adap;
	if (!adapter)
		return;

	i2c_device->client = gb02_i2c_new_device(adapter, &i2c_device->board_info);
	if (!i2c_device->i2c_driver_init || !i2c_device->client)
		return;
	ret = i2c_device->i2c_driver_init(i2c_device->client);
	if (ret)
		gb_printf(KERN_ERR, "i2c_device:%s init failed\n", i2c_device->board_info.type);
}

static void gb02_i2c_device_exit(struct gb02_i2c_device *i2c_device)
{
	if (!i2c_device->enable)
		return;

	if (!i2c_device->i2c_driver_exit || !i2c_device->client)
		return;

	i2c_device->i2c_driver_exit(i2c_device->client);
}

static void gb02_pwm_device_init(struct gb_device *gbdev, struct gb02_pwm_device *pwm_dev)
{
	int ret;
	struct gb02_mcu_peripherals_info *peri_info;

	if (!pwm_dev->enable)
		return;

	if (pwm_dev->gpio_pwm)
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_GPIO_PWM_DEVICE_NAME);
	else
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_PWM_DEVICE_NAME);
	if (!peri_info || !pwm_dev->pwm_driver_init)
		return;

	ret = pwm_dev->pwm_driver_init(gbdev, peri_info, pwm_dev);
	if (ret)
		gb_printf(KERN_ERR, "pwm_dev:%s init failed\n", pwm_dev->name);
}

static void gb02_pwm_device_exit(struct gb_device *gbdev, struct gb02_pwm_device *pwm_dev)
{
	struct gb02_mcu_peripherals_info *peri_info;

	if (!pwm_dev->enable)
		return;

	if (!pwm_dev->gpio_pwm)
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_GPIO_PWM_DEVICE_NAME);
	else
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_PWM_DEVICE_NAME);
	if (!peri_info || !pwm_dev->pwm_driver_exit)
		return;
	pwm_dev->pwm_driver_exit(peri_info);
}

static int gb02_pwm_device_suspend(struct gb_device *gbdev, struct gb02_pwm_device *pwm_dev)
{
	int ret = -1;
	struct gb02_mcu_peripherals_info *peri_info;

	if (!pwm_dev->enable)
		return 0;

	if (!pwm_dev->gpio_pwm)
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_GPIO_PWM_DEVICE_NAME);
	else
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_PWM_DEVICE_NAME);
	if (!peri_info || !pwm_dev->pwm_driver_suspend)
		return ret;

	ret = pwm_dev->pwm_driver_suspend(peri_info);
	if (ret)
		gb_printf(KERN_ERR, "pwm_dev:%s suspend failed\n", pwm_dev->name);

	return ret;
}

static int gb02_pwm_device_resume(struct gb_device *gbdev, struct gb02_pwm_device *pwm_dev)
{
	int ret = -1;
	struct gb02_mcu_peripherals_info *peri_info;

	if (!pwm_dev->enable)
		return 0;

	if (!pwm_dev->gpio_pwm)
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_GPIO_PWM_DEVICE_NAME);
	else
		peri_info = gb02_get_mcu_peripherals_info(gbdev, pwm_dev->device_id, PLATFORM_PWM_DEVICE_NAME);
	if (!peri_info || !pwm_dev->pwm_driver_resume)
		return ret;

	ret = pwm_dev->pwm_driver_resume(gbdev, peri_info);
	if (ret)
		gb_printf(KERN_ERR, "pwm_dev:%s resume failed\n", pwm_dev->name);

	return ret;
}

void gb_peripherals_init(struct gb_device *gbdev)
{
	int i;

	memcpy(gbdev->gb_peri, mcu_periph_info, sizeof(mcu_periph_info));

	//for mcu ip driver init
	for (i = 0; i < PERI_IP_CNT; i++)
		gb02_peri_drv_init(gbdev, i);

	//for i2c device init
	for (i = 0; i < I2C_DEVICE_CNT; i++)
		gb02_i2c_device_init(gbdev, &mcu_i2c_device[i]);

	//for pwm device init
	for (i = 0; i < PWM_DEVICE_CNT; i++)
		gb02_pwm_device_init(gbdev, &mcu_pwm_device[i]);

	gb_sn_info_init(gbdev->gb_pcie->pdev);
	gb_pcie_msg_init(gbdev);
}

void gb_peripherals_exit(struct gb_device *gbdev)
{
	int i;

	//for pwm device exit
	for (i = 0; i < PWM_DEVICE_CNT; i++)
		gb02_pwm_device_exit(gbdev, &mcu_pwm_device[i]);

	//for i2c device exit
	for (i = 0; i < I2C_DEVICE_CNT; i++)
		gb02_i2c_device_exit(&mcu_i2c_device[i]);

	//for mcu ip driver exit
	for (i = 0; i < PERI_IP_CNT; i++)
		gb02_peri_drv_exit(gbdev, i);

	gb_pcie_msg_remove(gbdev);
}

int gb_peripherals_suspend(struct gb_device *gbdev)
{
	int i;

	//for pwm device suspend
	for (i = 0; i < PWM_DEVICE_CNT; i++)
		gb02_pwm_device_suspend(gbdev, &mcu_pwm_device[i]);
	//for mcu ip driver suspend
	for (i = 0; i < PERI_IP_CNT; i++)
		gb02_peri_drv_suspend(gbdev, i);
	return 0;
}

int gb_peripherals_resume(struct gb_device *gbdev)
{
	int i;

	//for mcu ip driver resume
	for (i = 0; i < PERI_IP_CNT; i++)
		gb02_peri_drv_resume(gbdev, i);
	//for pwm device resume
	for (i = 0; i < PWM_DEVICE_CNT; i++)
		gb02_pwm_device_resume(gbdev, &mcu_pwm_device[i]);

	return 0;
}
