/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef __PINCTRL_GB02_MCU_H
#define __PINCTRL_GB02_MCU_H

#define NUCLEI_GPIO_INPUT_VAL 0x00
#define NUCLEI_GPIO_INPUT_EN 0x04
#define NUCLEI_GPIO_MODE0 0x08
#define NUCLEI_GPIO_MODE1 0x0C
#define NUCLEI_GPIO_OUTPUT_VAL 0x10
#define NUCLEI_GPIO_RISE_IE 0x14
#define NUCLEI_GPIO_RISE_IP 0x18
#define NUCLEI_GPIO_FALL_IE 0x1C
#define NUCLEI_GPIO_FALL_IP 0x20
#define NUCLEI_GPIO_HIGH_IE 0x24
#define NUCLEI_GPIO_HIGH_IP 0x28
#define NUCLEI_GPIO_LOW_IE 0x2C
#define NUCLEI_GPIO_LOW_IP 0x30
#define NUCLEI_GPIO_OUTPUT_XOR 0x40

#define GB02_MCU_GPIO_PINS_PER_BANK 8
#define GB02_MCU_GPIO_BANKS_NUM 6
#define GB02_GPIO_IRQ_LINE 16
#define GB02_GPIO_LINE_DIRECTION_IN 0
#define GB02_GPIO_LINE_DIRECTION_OUT 1

#define MAX_FUNCS 2

#define PINCTRL_MCU_TEST_CMD_BASE 'P'
#define PINCTRL_MCU_CMD_GPIO40 _IOW(PINCTRL_MCU_TEST_CMD_BASE, 1u, int)
#define PINCTRL_MCU_CMD_GPIO41 _IOW(PINCTRL_MCU_TEST_CMD_BASE, 2u, int)
#define PINCTRL_MCU_CMD_GPIO42 _IOW(PINCTRL_MCU_TEST_CMD_BASE, 3u, int)
#define PINCTRL_MCU_CMD_GPIO43 _IOW(PINCTRL_MCU_TEST_CMD_BASE, 4u, int)

#define LGPIO5_IRQ_CLEAR_ENABLE
#define GPIO_IRQ_SUPPORT_GPIO_NO_MIN (40)
#define GPIO_IRQ_SUPPORT_GPIO_NO_MAX (43)

#ifdef CONFIG_MCU_GPIO
int gb02_mcu_gpio_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info);
void gb02_mcu_gpio_drv_exit(struct gb02_mcu_peripherals_info *peri_info);
void gb02_gpio_irq_enable(unsigned int gpio, bool enable);
int gb02_gpio_set(unsigned int gpio, int value);
int gb02_gpio_get(unsigned int gpio);
int gb02_gpio_set_direction(unsigned int gpio, bool direction);
int gb02_gpio_get_direction(unsigned int gpio);
int gb02_gpio_request(unsigned int gpio);
void gb02_gpio_free(unsigned int gpio);
bool gb02_gpio_is_valid(unsigned int gpio);
#else
static inline int gb02_mcu_gpio_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}

static inline void gb02_mcu_gpio_drv_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}

static inline void gb02_gpio_irq_enable(unsigned int gpio, bool enable)
{
	return;
}

static inline int gb02_gpio_set(unsigned int gpio, int value)
{
	return 0;
}

static inline int gb02_gpio_get(unsigned int gpio)
{
	return 0;
}

static inline int gb02_gpio_set_direction(unsigned int gpio, bool direction)
{
	return 0;
}

static inline int gb02_gpio_get_direction(unsigned int gpio)
{
	return 0;
}

static inline int gb02_gpio_request(unsigned int gpio)
{
	return 0;
}

static inline void gb02_gpio_free(unsigned int gpio)
{
	return;
}

static inline bool gb02_gpio_is_valid(unsigned int gpio)
{
	return false;
}
#endif
#endif /* __PINCTRL_GB02_MCU_H */
