// SPDX-License-Identifier: GPL-2.0-only
/*
 */

#include <linux/module.h>
#include "gb_governor.h"

static int devfreq_performance_func(struct gb_devfreq *df,
				    unsigned long *freq)
{
	/*
	 * target callback should be able to get floor value as
	 * said in devfreq.h
	 */
	*freq = GB_DEVFREQ_MAX_FREQ;
	return 0;
}

static int devfreq_performance_handler(struct gb_devfreq *devfreq,
				unsigned int event, void *data)
{
	int ret = 0;

	if (event == GB_DEVFREQ_GOV_START) {
		mutex_lock(&devfreq->lock);
		ret = update_gb_devfreq(devfreq);
		mutex_unlock(&devfreq->lock);
	}

	return ret;
}

struct gb_devfreq_governor devfreq_performance = {
	.name = GB_DEVFREQ_GOV_PERFORMANCE,
	.get_target_freq = devfreq_performance_func,
	.event_handler = devfreq_performance_handler,
};


