/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef __ADS1015_H
#define __ADS1015_H
//#include <linux/platform_data/ads1015.h>
#include "i2c-gb02-mcu.h"
/* ADS1015 registers */
enum {
	ADS1015_CONVERSION = 0,
	ADS1015_CONFIG = 1,
};

/* PGA fullscale voltages in mV */
static const unsigned int fullscale_table[8] = {
	6144, 4096, 2048, 1024, 512, 256, 256, 256
};

/* Data rates in samples per second */
static const unsigned int data_rate_table_1015[8] = {
	128, 250, 490, 920, 1600, 2400, 3300, 3300
};

static const unsigned int data_rate_table_1115[8] = {
	8, 16, 32, 64, 128, 250, 475, 860
};

#define ADS1015_DEFAULT_CHANNELS 0xff
#define ADS1015_DEFAULT_PGA 3
#define ADS1015_DEFAULT_DATA_RATE 4
#define ADS1015_CHANNELS 4
enum ads1015_chips {
	ads1015,
	ads1115,
};

struct ads1015_channel_data {
	bool enabled;
	unsigned int pga;
	unsigned int data_rate;
};

struct ads1015_data {
	struct device *hwmon_dev;
	struct mutex update_lock; /* mutex protect updates */
	struct ads1015_channel_data channel_data[ADS1015_CHANNELS];
	enum ads1015_chips id;
};

struct ads1015_platform_data {
	struct ads1015_channel_data channel_data[ADS1015_CHANNELS];
};

#define ADS1015_BUS0_ADDR  0x49
#define ADS1015_BUS0_ID  0
#define ADS1015_DEVICE0_NAME ("ADS1015-0")
#define ADS1015_BUS1_ADDR  0x48
#define ADS1015_BUS1_ID  1
#define ADS1015_DEVICE1_NAME ("ADS1015-1")
#define ADS1015_BUS2_ADDR  0x48
#define ADS1015_BUS2_ID  2
#define ADS1015_DEVICE2_NAME ("ADS1015-2")
#ifdef CONFIG_MCU_I2C
int ads1015_driver_init(struct i2c_client *client);
void ads1015_remove(struct i2c_client *client);
#else
static int ads1015_driver_init(struct i2c_client *client)
{
	return 0;
}
static void ads1015_remove(struct i2c_client *client)
{
	return;
}
#endif
#endif
