/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef __I2C_GB02_MCU_H
#define __I2C_GB02_MCU_H
#include "common/gb-peripherals-common.h"

#define NUCLEI_I2C_STATUS 0x24
#define NUCLEI_I2C_DIV_NUM 0x28
#define NUCLEI_I2C_SETUP 0x2C
#define NUCLEI_I2C_TXDATA 0x30
#define NUCLEI_I2C_RXDATA 0x34
#define NUCLEI_I2C_INT_IE 0x38
#define NUCLEI_I2C_INT_CLR 0x40
#define NUCLEI_I2C_TIMEOUT_NUM 0x48

#define NUCLEI_I2C_STATUS_BUSBUSY (1 << 0)
#define NUCLEI_I2C_STATUS_ARB_LOST (1 << 1)
#define NUCLEI_I2C_STATUS_ACK (1 << 2)
#define NUCLEI_I2C_STATUS_EOT (1 << 3)
#define NUCLEI_I2C_STATUS_BYTE_ON_GOING (1 << 4)
#define NUCLEI_I2C_STATUS_TIMEOUT (1 << 5)

#define NUCLEI_I2C_INT_IE_EOT (1 << 0)
#define NUCLEI_I2C_INT_IE_ARB_LOST (1 << 1)
#define NUCLEI_I2C_INT_IE_TX_EMPTY (1 << 2)
#define NUCLEI_I2C_INT_IE_RX_FULL (1 << 3)
#define NUCLEI_I2C_INT_IE_TIMEOUT (1 << 4)

#define NUCLEI_I2C_SETUP_DMA_EN (1 << 0)
#define NUCLEI_I2C_SETUP_ENABLE (1 << 1)
#define NUCLEI_I2C_SETUP_START (1 << 2)
#define NUCLEI_I2C_SETUP_STOP (1 << 3)
#define NUCLEI_I2C_SETUP_WRITE (1 << 4)
#define NUCLEI_I2C_SETUP_READ (1 << 5)
#define NUCLEI_I2C_SETUP_ACK (1 << 6)
#define NUCLEI_I2C_SETUP_SOFT_RESET (1 << 7)
#define NUCLEI_I2C_SETUP_TIMEOUT_ENABLE (1 << 8)
#define NUCLEI_I2C_SETUP_10BIT_ADDR_EN (1 << 9)
#define NUCLEI_I2C_SETUP_ROLE_MODE (1 << 11)
#define NUCLEI_I2C_SETUP_I3C_MODE (1 << 12)
#define NUCLEI_I2C_SETUP_PUSH_PULL (1 << 13)
#define NUCLEI_I2C_SETUP_SCL_PULLUP (1 << 14)
#define NUCLEI_I2C_SETUP_SDA_PULLUP (1 << 15)

#define I2C_SETUP_SCL_DUTY_SEL_MASK (0x3 << 18)

#define GB02_MCU_I2C_ADAPTER_MAX 4
#define GB02_I2C_STOP_TRY_MAX 10
// #define GB02_MCU_SYS_CLK		(3200 * 1000 / 12)	/* in KHz */
// #define GB02_MCU_SYS_CLK		(5333)				/* in KHz */

#define I2C_MCU_TEST_CMD_BASE 'I'
#define I2C_MCU_CMD_I2C0 _IOW(I2C_MCU_TEST_CMD_BASE, 1u, int)
#define I2C_MCU_CMD_I2C1 _IOW(I2C_MCU_TEST_CMD_BASE, 2u, int)
#define I2C_MCU_CMD_I2C2 _IOW(I2C_MCU_TEST_CMD_BASE, 3u, int)
#define I2C_MCU_CMD_I2C3 _IOW(I2C_MCU_TEST_CMD_BASE, 4u, int)
#define I2C_MCU_CMD_I2C0_RD _IOW(I2C_MCU_TEST_CMD_BASE, 5u, int)
#define I2C_MCU_CMD_I2C0_WR _IOW(I2C_MCU_TEST_CMD_BASE, 6u, int)

#define I2C_MCU_TEST_DEVICE_ADDR 0x50
#define I2C_MCU_TEST_DATA_LENGTH 256
#define ADDR_LEN 2
#define GB02_ADS1015_DEV_NAME "ads1015"
#define GB02_INA2XXX_DEV_NAME "ina2xx"
#define GB02_PS8625_DEV_NAME "ps8625"
enum I2C_DutyTypedef {
	I2C_30_DUTY                     = 0,/*!< I2C %30 duty */
	I2C_40_DUTY                     = 1,/*!< I2C %40 duty */
	I2C_50_DUTY                     = 2,/*!< I2C %50 duty */
};

struct gb02_i2c_pdata {
	unsigned int flags;
	unsigned int slave_addr;
	unsigned long frequency; /* in KHz */
	unsigned int sda_delay;
};

struct gb02_i2c_adapter {
	wait_queue_head_t wait;
	kernel_ulong_t quirks;
	struct i2c_msg *msg;
	unsigned int msg_num;
	unsigned int msg_idx;
	unsigned int msg_ptr;
	unsigned int tx_setup;
	unsigned int irq;
	void __iomem *regs;
	struct device *dev;
	struct i2c_adapter adap;
	void *driver_data;
	u32 bus_no;
	int stopped;
	unsigned int slave_addr;
};
struct gb02_mcu_i2c {
	void __iomem *regs;
	void __iomem *sysctl_iofunc_base; // bar0
	void __iomem *sysctl_cfg_base;    // bar4
	unsigned int i2c_bus_no;
	struct device *dev;

	struct gb02_i2c_adapter adapter; // todo

	const struct gb02_i2c_pdata *pdata;
	struct i2c_client *ads1015_i2c_client;
	struct i2c_client *ina219_i2c_client;
	struct i2c_client *tmp102_i2c_client;
	struct i2c_client *ps8625_i2c_client;
};

#ifdef CONFIG_MCU_I2C
int gb02_mcu_i2c_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info);
void gb02_mcu_i2c_drv_exit(struct gb02_mcu_peripherals_info *peri_info);
struct i2c_client *
gb02_i2c_new_device(struct i2c_adapter *adap, struct i2c_board_info const *info);
#else
static inline int gb02_mcu_i2c_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline void gb02_mcu_i2c_drv_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
static inline struct i2c_client *
gb02_i2c_new_device(struct i2c_adapter *adap, struct i2c_board_info const *info)
{
	return NULL;
}
#endif

#endif /* __I2C_GB02_MCU_H */
