/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __PCIE_MSG_H
#define __PCIE_MSG_H
#include "gb_device.h"

#define PCIE_MSG_BUFFER_SIZE (1024 * 1024 - 4 * 4) 

#define UPGRADE_MCU_TEST_CMD_BASE 'U'
#define UPGRADE_MCU_CMD_START _IOW(UPGRADE_MCU_TEST_CMD_BASE, 1u, int)
#define UPGRADE_MCU_CMD_GET_PROGRESS _IOW(UPGRADE_MCU_TEST_CMD_BASE, 2u, int)
#define UPGRADE_MCU_CMD_SET_IMAGE_INFO _IOW(UPGRADE_MCU_TEST_CMD_BASE, 3u, int)

#define PCIE_MSG_MAGIC_NUMBER  0x57140001
#define MCU_MSG_MAGIC_NUMBER   0x5714FFF0
#define PCIE_COMMAND_SUCCESS   0x1001
#define PCIE_COMMAND_FAIL      0x1002

#define MCU_RAM_ADDR_OFFSET (0x1000000 + 0x40000)

struct gb_pcie_msg {
	struct device *dev;
	struct gb_device *gbdev;
};

typedef struct
{
	uint32_t magic;
	uint32_t cmd;
	uint32_t len;
	char * data;

}pcie_msg_t;

enum pcie_msg_cmd_e
{
	PCIE_DDR_TEST,
	PCIE_DDR_READ,
	PCIE_DDR_WRITE,
	PCIE_ECC_SET,
	PCIE_ECC_GET,
};

typedef struct
{
	char * cmd_str;
	pcie_msg_t pcie_msg;

}pcie_cmd_t;

int gb_pcie_msg_init(struct gb_device *gbdev);
void gb_pcie_msg_remove(struct gb_device *gbdev);
u64 get_genbu_ecc_len(void);
#endif
