/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __GB02_PVT__
#define __GB02_PVT__
#include "gb-peripherals-common.h"

#ifndef BIT
#define BIT(x) (1 << (x))
#endif

#define CS_CNOC_S1_PVT_OFFSET 0x82400000
#define GB02_DEFAULT_CFG_SYS_PERI_ADDR 0x80000000

#define PVT_DEFAULT_BAR GB02_SYSTEM_PERIPHERAL_BAR
#define PVT_BASE CS_CNOC_S1_PVT_BASE // 0x82400000

/* 75 25 typical conditions steady DOUT values */
#define DOUT75std 2874LL
#define DOUT25std 2761LL

/* assumes 75 and 25 degrees actual conditions DOUT values, for now*/
#define DOUT75act 2902LL
#define DOUT25act 2784LL

#define DOUTstd 2067
#define Vref 1170 // mV
#define DOUT_MAX 3000
#define DOUT_MIN 2000
#define D1 ((3 * DOUT75act - DOUT25act) / 2)
#define D2 ((DOUT75act + DOUT25act) / 2)
#define D3 ((3 * DOUT25act - DOUT75act) / 2)
#define D4 (DOUT25act - (DOUT75act - DOUT25act) * 9 / 10)

#define PVT_REG_00 (0x00)

#define HIGH_TEMP_THER_CFG_NORMAL_OFFSET 20
#define HIGH_TEMP_THER_CFG_NORMAL(x) (((x)&0x7ff) << HIGH_TEMP_THER_CFG_NORMAL_OFFSET) // default 3000
#define LOW_TEMP_THER_CFG_NORMAL_OFFSET 8
#define LOW_TEMP_THER_CFG_NORMAL(x) (((x)&0x7ff) << LOW_TEMP_THER_CFG_NORMAL_OFFSET) // default 2500
#define PU_VTDC_MASK BIT(7)															 // 1: power up  0:power down --default 1
#define PVT_POWER_UP 1
#define PVT_POWER_DOWN 0
#define PVT_RESETn_MASK BIT(6) // low active ,default 0
#define PVT_REST 0
#define PVT_DERESET 1

#define TDC_MODE_MASK BIT(5) // 1:temperature 0 :voltage default 1
#define TEMP_MODE 1
#define VOLT_MODE 0
#define TEST_EN_MASK BIT(4)		   // IP test enable signal ,defaul 0
#define BAND_GAP_TRIM(x) ((x)&0x7) // bandgap trim signal ,default 4'b0111

#define PVT_REG_04 (0x04)

#define HIGH_TEMP_THER_CFG_FATAL_OFFSET 12
#define HIGH_TEMP_THER_CFG_FATAL(x) (((x)&0x7ff) << HIGH_TEMP_THER_CFG_FATAL_OFFSET) // default 3000
#define LOW_TEMP_THER_CFG_FATAL_OFFSET 0
#define LOW_TEMP_THER_CFG_FATAL(x) (((x)&0x7ff) << LOW_TEMP_THER_CFG_FATAL_OFFSET) // default 2500

#define PVT_REG_08 (0x08)
#define HIGH_VOLT_THER_CFG_NORMAL_OFFSET 12
#define HIGH_VOLT_THER_CFG_NORMAL(x) (((x)&0x7ff) << HIGH_VOLT_THER_CFG_NORMAL_OFFSET) // default 3000
#define LOW_VOLT_THER_CFG_NORMAL_OFFSET 0
#define LOW_VOLT_THER_CFG_NORMAL(x) (((x)&0x7ff) << LOW_VOLT_THER_CFG_NORMAL_OFFSET) // default 2500

#define PVT_REG_0C (0x0C)

#define DIV_PCLK_FREQ_OFFSET 24
#define DIV_PCLK_FREQ_CFG(x) (((x)&0xff) << DIV_PCLK_FREQ_OFFSET) // default 8'd31

#define HIGH_VOLT_THER_CFG_FATAL_OFFSET 12
#define HIGH_VOLT_THER_CFG_FATAL(x) (((x)&0x7ff) << HIGH_VOLT_THER_CFG_FATAL_OFFSET) // default 3000
#define LOW_VOLT_THER_CFG_FATAL_OFFSET 0
#define LOW_VOLT_THER_CFG_FATAL(x) (((x)&0x7ff) << LOW_VOLT_THER_CFG_FATAL_OFFSET) // default 2500

#define PVT_REG_10 (0x10)
#define TEMP_CLR_NORMAL_CLEAR BIT(3) // write 1 clear default 0
#define TEMP_CLR_FATAL_CLEAR BIT(2)	 // write 1 clear default 0
#define VOLT_CLR_NOAMAL_CLEAR BIT(1) // write 1 clear default 0
#define VOLT_CLR_FATAL_CLEAR BIT(0)	 // write 1 clear default 0

#define PVT_REG_14 (0x14)

#define HIGH_VOLT_FATAL_STATUS BIT(19)
#define LOW_VOLT_FATAL_STATUS BIT(18)
#define HIGH_VOLT_NAOMAL_STATUS BIT(17)
#define LOW_VOLT_MORMAL_STATUS BIT(16)
#define HIGH_TEMP_FATAL_STATUS BIT(15)
#define LOW_TEMP_FATAL_STATUS BIT(14)
#define HIGH_TEMP_NAOMAL_STATUS BIT(13)
#define LOW_TEMP_MORMAL_STATUS BIT(12)
#define DOUT(x) ((x)&0xfff)

#ifdef CONFIG_MCU_PVT
int gb02_pvt_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info);
void gb02_pvt_drv_exit(struct gb02_mcu_peripherals_info *peri_info);
int get_temp_info(void);
#else
static inline int gb02_pvt_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline void gb02_pvt_drv_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
static inline int get_temp_info(void)
{
	return 0;
}
#endif
#endif
