/* SPDX-License-Identifier: GPL-2.0 */

#include "gb_pwm.h"

int gb_pwm_config(struct gb_pwm_device *pwm, int duty_ns, int period_ns)
{
	int ret = 0;
	struct gb_pwm_chip *chip = pwm->chip;

	if (chip->ops->config) {
		ret = chip->ops->config(chip, pwm, duty_ns, period_ns);
		if (ret) {
			pr_err("Failed to configure PWM\n");
			return ret;
		}
	}

	return ret;	
}

int gb_pwm_apply(struct gb_pwm_device *pwm, struct gb_pwm_state *state)
{
	int ret = 0;
	struct gb_pwm_chip *chip = pwm->chip;

	if (chip->ops->apply) {
		ret = chip->ops->apply(chip, pwm, state);
		if (ret) {
			pr_err("Failed to configure PWM\n");
			return ret;
		}
	}

	return ret;
}

int gb_pwm_enable(struct gb_pwm_device *pwm)
{
	int ret = 0;
	struct gb_pwm_chip *chip = pwm->chip;

	if (chip->ops->enable) {
		ret = chip->ops->enable(chip, pwm);
		if (ret) {
			pr_err("Failed to enable PWM\n");
			return ret;
		}
	}

	return ret;
}

void gb_pwm_disable(struct gb_pwm_device *pwm)
{
	struct gb_pwm_chip *chip = pwm->chip;

	if (chip->ops->disable)
		chip->ops->disable(chip, pwm);

	return;
}

