/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __PWM_GB02_MCU_H
#define __PWM_GB02_MCU_H
#include <linux/pwm.h>
#include "gpu/gb_device.h"
#include "common/gb-peripherals-common.h"
/* timer */
#define TIM_CR1 0x00   /* Control Register 1      */
#define TIM_CR2 0x04   /* Control Register 2      */
#define TIM_SMCR 0x08  /* Slave mode control reg  */
#define TIM_DIER 0x0C  /* DMA/interrupt register  */
#define TIM_SR 0x10	   /* Status register     */
#define TIM_EGR 0x14   /* Event Generation Reg    */
#define TIM_CCMR1 0x18 /* Capt/Comp 1 Mode Reg    */
#define TIM_CCMR2 0x1C /* Capt/Comp 2 Mode Reg    */
#define TIM_CCER 0x20  /* Capt/Comp Enable Reg    */
#define TIM_CNT 0x24   /* Counter         */
#define TIM_PSC 0x28   /* Prescaler               */
#define TIM_ARR 0x2c   /* Auto-Reload Register    */
#define TIM_CCR1 0x34  /* Capt/Comp Register 1    */
#define TIM_CCR2 0x38  /* Capt/Comp Register 2    */
#define TIM_CCR3 0x3C  /* Capt/Comp Register 3    */
#define TIM_CCR4 0x40  /* Capt/Comp Register 4    */
#define TIM_BDTR 0x44  /* Break and Dead-Time Reg */
#define TIM_DCR 0x48   /* DMA control register    */
#define TIM_DMAR 0x4C  /* DMA register for transfer */

#define TIM_CR1_CEN BIT(0)						/* Counter Enable      */
#define TIM_CR1_DIR BIT(4)						/* Counter Direction       */
#define TIM_CR1_ARPE BIT(7)						/* Auto-reload Preload Ena */
#define TIM_CR2_MMS (BIT(4) | BIT(5) | BIT(6))	/* Master mode selection */
#define TIM_CR2_MMS2 GENMASK(23, 20)			/* Master mode selection 2 */
#define TIM_SMCR_SMS (BIT(0) | BIT(1) | BIT(2)) /* Slave mode selection */
#define TIM_SMCR_TS (BIT(4) | BIT(5) | BIT(6))	/* Trigger selection */
#define TIM_DIER_UIE BIT(0)						/* Update interrupt    */
#define TIM_DIER_UDE BIT(8)						/* Update DMA request Enable */
#define TIM_DIER_CC1DE BIT(9)					/* CC1 DMA request Enable  */
#define TIM_DIER_CC2DE BIT(10)					/* CC2 DMA request Enable  */
#define TIM_DIER_CC3DE BIT(11)					/* CC3 DMA request Enable  */
#define TIM_DIER_CC4DE BIT(12)					/* CC4 DMA request Enable  */
#define TIM_DIER_COMDE BIT(13)					/* COM DMA request Enable  */
#define TIM_DIER_TDE BIT(14)					/* Trigger DMA request Enable */
#define TIM_SR_UIF BIT(0)						/* Update interrupt flag   */
#define TIM_EGR_UG BIT(0)						/* Update Generation       */
#define TIM_CCMR_PE BIT(3)						/* Channel Preload Enable  */
#define TIM_CCMR_M1 (BIT(6) | BIT(5))			/* Channel PWM Mode 1 */
#define TIM_CCMR_CC1S (BIT(0) | BIT(1))			/* Capture/compare 1 sel */
#define TIM_CCMR_IC1PSC GENMASK(3, 2)			/* Input capture 1 prescaler */
#define TIM_CCMR_CC2S (BIT(8) | BIT(9))			/* Capture/compare 2 sel */
#define TIM_CCMR_IC2PSC GENMASK(11, 10)			/* Input capture 2 prescaler */
#define TIM_CCMR_CC1S_TI1 BIT(0)				/* IC1/IC3 selects TI1/TI3 */
#define TIM_CCMR_CC1S_TI2 BIT(1)				/* IC1/IC3 selects TI2/TI4 */
#define TIM_CCMR_CC2S_TI2 BIT(8)				/* IC2/IC4 selects TI2/TI4 */
#define TIM_CCMR_CC2S_TI1 BIT(9)				/* IC2/IC4 selects TI1/TI3 */
#define TIM_CCER_CC1E BIT(0)					/* Capt/Comp 1  out Ena    */
#define TIM_CCER_CC1P BIT(1)					/* Capt/Comp 1  Polarity   */
#define TIM_CCER_CC1NE BIT(2)					/* Capt/Comp 1N out Ena    */
#define TIM_CCER_CC1NP BIT(3)					/* Capt/Comp 1N Polarity   */
#define TIM_CCER_CC2E BIT(4)					/* Capt/Comp 2  out Ena    */
#define TIM_CCER_CC2P BIT(5)					/* Capt/Comp 2  Polarity   */
#define TIM_CCER_CC3E BIT(8)					/* Capt/Comp 3  out Ena    */
#define TIM_CCER_CC3P BIT(9)					/* Capt/Comp 3  Polarity   */
#define TIM_CCER_CC4E BIT(12)					/* Capt/Comp 4  out Ena    */
#define TIM_CCER_CC4P BIT(13)					/* Capt/Comp 4  Polarity   */
#define TIM_CCER_CCXE (BIT(0) | BIT(4) | BIT(8) | BIT(12))
#define TIM_BDTR_BKE BIT(12) /* Break input enable      */
#define TIM_BDTR_BKP BIT(13) /* Break input polarity    */
#define TIM_BDTR_AOE BIT(14) /* Automatic Output Enable */
#define TIM_BDTR_MOE BIT(15) /* Main Output Enable      */
#define TIM_BDTR_BKF (BIT(16) | BIT(17) | BIT(18) | BIT(19))
#define TIM_BDTR_BK2F (BIT(20) | BIT(21) | BIT(22) | BIT(23))
#define TIM_BDTR_BK2E BIT(24)	   /* Break 2 input enable    */
#define TIM_BDTR_BK2P BIT(25)	   /* Break 2 input polarity  */
#define TIM_DCR_DBA GENMASK(4, 0)  /* DMA base addr */
#define TIM_DCR_DBL GENMASK(12, 8) /* DMA burst len */

#define MAX_TIM_PSC 0xFFFF
#define MAX_TIM_ICPSC 0x3
#define TIM_CR2_MMS_SHIFT 4
#define TIM_CR2_MMS2_SHIFT 20
#define TIM_SMCR_TS_SHIFT 4
#define TIM_BDTR_BKF_MASK 0xF
#define TIM_BDTR_BKF_SHIFT 16
#define TIM_BDTR_BK2F_SHIFT 20

#define GB02_TIMERS_MAX_REGISTERS	0x3fc
#define PWM_DEVICE_NAME_LENGTH 64
#define GB02_PWM_BL_DEV_NAME "gb02-pwm-backlight"
#define BACKLIGHT_PWM_CHAN_NO 0

#define GB02_PWM_FAN_DEV_NAME "gb02-pwm-fan"
#ifdef CONFIG_GPIO_PWM
#define FAN_PWM_CTRL_NO 0
#else
#define FAN_PWM_CTRL_NO 1
#endif
#define FAN_PWM_CHAN_NO 0

enum gb02_timers_dmas {
	GB02_TIMERS_DMA_CH1,
	GB02_TIMERS_DMA_CH2,
	GB02_TIMERS_DMA_CH3,
	GB02_TIMERS_DMA_CH4,
	GB02_TIMERS_DMA_UP,
	GB02_TIMERS_DMA_TRIG,
	GB02_TIMERS_DMA_COM,
	GB02_TIMERS_MAX_DMAS,
};

enum gb_pwm_type {
	GB_MCU_PWM,
	GB_GPIO_PWM,
};

struct gb02_timers_dma {
	struct completion completion;
	phys_addr_t phys_base;
	struct mutex lock;
	struct dma_chan *chan;
	struct dma_chan *chans[GB02_TIMERS_MAX_DMAS];
};

struct gb_pwm_state {
	unsigned int period;
	unsigned int duty_cycle;
	enum pwm_polarity polarity;
	bool enable;
};

struct gb_pwm_device {
	unsigned int hwpwm;
	struct gb_pwm_chip *chip;
	void *chip_data;

	unsigned int period;
	struct gb_pwm_state state;
};

struct gb_pwm_capture {
	unsigned int duty_cycle;
	unsigned int period;
};

struct gb_pwm_ops {
	int (*enable)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm);
	void (*disable)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm);
	int (*set_polarity)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm, enum pwm_polarity polarity);
	int (*config)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm, int duty_ns, int period_ns);
	int (*apply)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm, struct gb_pwm_state *state);
	int (*capture)(struct gb_pwm_chip *chip, struct gb_pwm_device *pwm, struct gb_pwm_capture *result, unsigned long tmo_ms);
};

struct gb_pwm_chip {
	struct device *dev;
	enum gb_pwm_type pwm_type;
	struct mutex lock;
	unsigned int chip_id;
	unsigned int npwm;
	struct gb_pwm_device *pwms;
	struct gb_pwm_ops *ops;
	void *driver_data;
};

struct mcu_pwm_chip {
	struct gb_pwm_chip chip;
	void __iomem *pci_mmio_bar0;
	void __iomem *pci_mmio_bar1;
	void __iomem *pci_mmio_bar4;
	void __iomem *res;		  /* resources found */
	void __iomem *sysctl_iofunc_base;
	bool have_complementary_output;
	u32 capture[4] ____cacheline_aligned; /* DMA'able buffer */
	u32 max_arr;
	unsigned long clk_rate;
	struct gb02_timers_dma dma; /* Only to be used by the parent */
	struct resource *res_req; /* resources requested */
};

struct gpio_pwm_chip {
	struct gb_pwm_chip chip;
	struct hrtimer timer;
	int gpio;
	unsigned int on_time;
	unsigned int off_time;
	bool pin_on;
};

#ifdef CONFIG_MCU_PWM
int gb02_mcu_pwm_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info);
void gb02_mcu_pwm_drv_exit(struct gb02_mcu_peripherals_info *peri_info);
#else
static inline int gb02_mcu_pwm_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline void gb02_mcu_pwm_drv_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
#endif

#ifdef CONFIG_GPIO_PWM
int gb02_mcu_gpio_pwm_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info);
void gb02_mcu_gpio_pwm_drv_exit(struct gb02_mcu_peripherals_info *peri_info);
int gb02_mcu_gpio_pwm_drv_resume(struct gb02_mcu_peripherals_info *peri_info);
#else
static inline int gb02_mcu_gpio_pwm_drv_init(struct gb_device *gb_dev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline void gb02_mcu_gpio_pwm_drv_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
static inline int gb02_mcu_gpio_pwm_drv_resume(struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
#endif

#if defined (CONFIG_GPIO_PWM) || defined(CONFIG_MCU_PWM)
int gb02_pwm_backlight_init(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info, struct gb02_pwm_device *pwm_dev);
void gb02_pwm_backlight_exit(struct gb02_mcu_peripherals_info *peri_info);
struct bl_data *gb_get_backlight_data(int index);
int gb_backlight_update_status(struct bl_data *pb, int brightness);
int gb02_pwm_fan_init(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info, struct gb02_pwm_device *pwm_dev);
void gb02_pwm_fan_exit(struct gb02_mcu_peripherals_info *peri_info);
int pwm_fan_suspend(struct gb02_mcu_peripherals_info *peri_info);
int pwm_fan_resume(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info);
int pwm_backlight_suspend(struct gb02_mcu_peripherals_info *peri_info);
int pwm_backlight_resume(struct gb02_mcu_peripherals_info *peri_info);
int gb_pwm_config(struct gb_pwm_device *pwm, int duty_ns, int period_ns);
int gb_pwm_apply(struct gb_pwm_device *pwm, struct gb_pwm_state *state);
int gb_pwm_enable(struct gb_pwm_device *pwm);
void gb_pwm_disable(struct gb_pwm_device *pwm);
#else
static inline int gb02_pwm_backlight_init(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info, struct gb02_pwm_device *pwm_dev)
{
	return 0;
}
static inline void gb02_pwm_backlight_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
static inline struct bl_data *gb_get_backlight_data(int index)
{
	return NULL;
}
static inline int gb_backlight_update_status(struct bl_data *pb, int brightness)
{
	return 0;
}
static inline int gb02_pwm_fan_init(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info, struct gb02_pwm_device *pwm_dev)
{
	return 0;
}
static inline void gb02_pwm_fan_exit(struct gb02_mcu_peripherals_info *peri_info)
{
	return;
}
static inline int pwm_fan_suspend(struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline int pwm_fan_resume(struct gb_device *gbdev, struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline int pwm_backlight_suspend(struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
static inline int pwm_backlight_resume(struct gb02_mcu_peripherals_info *peri_info)
{
	return 0;
}
#endif
#endif
