#ifndef __GB_SN_INFO_H
#define __GB_SN_INFO_H
#include "mem_chip.h"
#include "param_def.h"

#define GB_SN_OFFSET 0x80000

#define SN_CONFIG_ENABLE 0
#define SN_DEFAULT "XT10121020512401010001"

#ifdef MEM_CHIP_DBG
#define SN_STR_MAX_LEN (27+10)
#else
#define SN_STR_MAX_LEN (32)
#endif

#ifdef MEM_CHIP_DBG
typedef struct {
	enum MemChipVendor		vendor;
	enum MemChipBitWidth 	bit_width;
	enum MemChipChnl 		channel;
	enum MemChipDensity 	density;
	enum MemChipDies		dies;
}mem_chip_t;
#endif

typedef  struct {
	bool snvaild;
	uint8_t vendor;
	uint8_t factory;
	uint8_t series;
	uint8_t interface;
	uint8_t gpu_type;
	uint8_t gpu_batch;
	uint8_t bin_level;
	date_info_t date_time;
	uint16_t lot;
	char sn_code[SN_STR_MAX_LEN];
	uint8_t check_passed;
} serial_number_info_t;

bool gb_get_sn_vaild(void);
int gb_get_vendor_form_sn(void);
int gb_get_factory_form_sn(void);
int gb_get_series_form_sn(void);
int gb_get_interface_form_sn(void);
int gb_get_gpu_type_form_sn(void);
int gb_get_gpu_batch_form_sn(void);
int gb_get_bin_level_form_sn(void);
date_info_t * gb_get_date_form_sn(void);
int gb_get_lot_info_form_sn(void);
serial_number_info_t * gb_get_sn_info_t(void);

char* product_info_create_sn(serial_number_info_t *p_info);
int product_info_parse_sn(char *sn, serial_number_info_t *p_info);
void product_info_display(serial_number_info_t *p_info);

void gb_sn_info_init(struct pci_dev *pdev);

#endif