#ifndef	__GB_VPU_H__
#define	__GB_VPU_H__
#include "vpu_list.h"
#include "common/gb_pcie_info.h"
#include "gpu/gb_ip_offset.h"
//#include "gpu/gb_device.h"

#ifdef HANTROVCMD_ENABLE_IP_SUPPORT
#define VCMD_REGISTER_INDEX_SW_INIT_CMD0        32
#define ASIC_VCMD_SWREG_AMOUNT                  64
#else
#define ASIC_VCMD_SWREG_AMOUNT                  27
#endif
#define MAX_SUBSYS_NUM 4
#define HXDEC_MAX_CORES MAX_SUBSYS_NUM

#define VPU_BO_PRIV_NUM 82
#define VPU_BO_HANDLE_NUM 3

enum vbo_type {
	VBO_OUTPP = 0,
	VBO_OUT,
	VBO_OUTPP_MID,
};
struct vpu_bo_file_priv {
	int vpu_part_bo_num[VPU_BO_HANDLE_NUM];
	u64 vpu_bo_priv_addr[VPU_BO_HANDLE_NUM][VPU_BO_PRIV_NUM];
	struct list_head vbo_list;
	struct drm_file *file_priv;
	struct vpu_bo_file_priv *vbo_priv;
};

struct vpu_vcmd_buff {
	uint64_t offset;
	void __iomem *vir_buff;
	uint64_t flp_offset;
	struct drm_gem_object *obj;
};
enum core_type {
	/* Decoder */
	HW_VC8000D = 0,
	HW_VC8000DJ,
	HW_BIGOCEAN,
	HW_VCMD,
	HW_MMU,
	HW_MMU_WR,
	HW_DEC400,
	HW_L2CACHE,
	HW_SHAPER,
	/* Encoder*/
	/* Auxiliary IPs */
	HW_AXIFE,
	HW_AFBC,
	HW_CORE_MAX
};
struct enc_vcmd_config {
	unsigned long vcmd_base_addr;
	u32 vcmd_iosize;
	int vcmd_irq;
	/*input vc8000e=0,IM=1,vc8000d=2,jpege=3,jpegd=4*/
	u32 sub_module_type;
	u16 submodule_main_addr;
	u16 submodule_dec400_addr;
	u16 submodule_L2Cache_addr;
	u16 submodule_MMU_addr[2];
	u16 submodule_axife_addr[2];
};

struct enc_hantrovcmd_dev {
	struct enc_vcmd_config enc_vcmd_core_cfg;
	u32 core_id;
	u32 sw_cmdbuf_rdy_num;
	spinlock_t *spinlock;
	wait_queue_head_t  *wait_queue;
	wait_queue_head_t  *wait_abort_queue;
	bi_list list_manager;

	volatile u8 *hwregs; /* IO mem base */
	u32 reg_mirror[ASIC_VCMD_SWREG_AMOUNT];
	u32 duration_without_int;
	volatile u8 working_state;
	u64 total_exe_time;
	u16 status_cmdbuf_id;
	u32 hw_version_id; /*megvii 0x43421001, later 0x43421102*/
	u32 *vcmd_reg_mem_virtual_address;
	size_t vcmd_reg_mem_bus_address;
	unsigned int mmu_vcmd_reg_mem_bus_address;
	u32  vcmd_reg_mem_size;
};

typedef struct {
	char *buffer;
	volatile unsigned int iosize[HXDEC_MAX_CORES];
	/* mapped address to different HW cores regs*/
	volatile u8 *hwregs[HXDEC_MAX_CORES][HW_CORE_MAX];
	/* mapped address to different HW cores regs*/
	volatile u8 *apbfilter_hwregs[HXDEC_MAX_CORES][HW_CORE_MAX];
	volatile int irq[HXDEC_MAX_CORES];
	int hw_id[HXDEC_MAX_CORES][HW_CORE_MAX];
	int client_type[HXDEC_MAX_CORES];
	int cores;
	struct fasync_struct *async_queue_dec;
	struct fasync_struct *async_queue_pp;
} hantrodec_t;

#define GB_VPU_MAX_HANDLES		30
struct dec_vcmd {
	bool			fw_header_present;
	struct gbvpu_bo	*vcpu_bo;
	/*virtual addr*/
	void			*cpu_addr;
	/*physical addr*/
	uint64_t		gpu_addr;
	uint32_t		max_handles;
	atomic_t		handles[GB_VPU_MAX_HANDLES];
	struct drm_file		*filp[GB_VPU_MAX_HANDLES];
	uint32_t		img_size[GB_VPU_MAX_HANDLES];
	struct delayed_work	idle_work;
};

struct enc_vcmd {
	struct gbvpu_bo	*vcpu_bo;
	uint64_t		gpu_addr;
	void			*cpu_addr;
	uint32_t		fw_version;
	uint32_t		fb_version;
	atomic_t		handles[GB_VPU_MAX_HANDLES];
	struct drm_file		*filp[GB_VPU_MAX_HANDLES];
	uint32_t		img_size[GB_VPU_MAX_HANDLES];
	struct delayed_work	idle_work;
	uint32_t		keyselect;
};

irqreturn_t dec0_isr(int irq);
irqreturn_t dec1_isr(int irq);
irqreturn_t enc_isr(int irq);
int dec_init(hantrodec_t *gbdec_data, void *pci_bars,
	struct gb_asic_dev_info *dev_info, struct vpu_vcmd_buff *vcmd_buff,
	struct vpu_vcmd_buff *vcmd_reg, struct vpu_offset_base vpu_offset);

int dec1_init(hantrodec_t *gbdec_data, void *pci_bars,
	      struct gb_asic_dev_info *dev_info,
	      struct vpu_vcmd_buff *vcmd_buff, struct vpu_vcmd_buff *vcmd_reg,
	      struct vpu_offset_base vpu_offset);

int enc_init(struct enc_hantrovcmd_dev *gbenc_data, void *pci_bars,
	struct gb_asic_dev_info *dev_info, struct vpu_vcmd_buff *vcmd_pool,
	struct vpu_vcmd_buff *vcmd_reg, struct vpu_offset_base vpu_offset);
#endif
