// SPDX-License-Identifier: GPL-3.0-or-later
/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * jessy 20221019 Sietium
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/moduleparam.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/semaphore.h>
#include <linux/spinlock.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/uaccess.h>
#include <linux/ioport.h>
#include <asm/irq.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/of_device.h>
#include <linux/platform_device.h>
#include <errno.h>
#include <drm/drm_file.h>
#include "common/gb_uk.h"
#include "vpu_vcmd.h"
#include "vpu_subsys.h"
#include "vpu_dec_irq.h"
#include "vpu/vpu_comm/vpu_list.h"
#include "vpu/vpu_comm/vpu_vcmd_registers.h"
#include "vpu/vpu_comm/vpu_comm.h"
#include "vpu/vpu_comm/vpu_basetype.h"
#include "vpu/vpu_comm/vpu_afe.h"
#include "common/gb_kernel_ver.h"
#include "gpu_test/gb_mem_tool.h"

// #define	GB_TIME_DEBUG
static void free_cmdbuf_node(bi_list_node *cmdbuf_node);
//hw_queue can be used for reserve cmdbuf memory
DECLARE_WAIT_QUEUE_HEAD(vcmd1_cmdbuf_memory_wait);
DEFINE_SPINLOCK(vcmd1_cmdbuf_alloc_lock);
DEFINE_SPINLOCK(vcmd1_process_manager_lock);

/* for all vcmds, the core info should be listed here for subsequent use */
struct vcmd_config vcmd_core_array1[MAX_SUBSYS_NUM] = {
	//decoder configuration
	{VCMD_DEC_IO_ADDR_0,
	VCMD_DEC_IO_SIZE_0,
	VCMD_DEC_INT_PIN_0,
	VCMD_DEC_MODULE_TYPE_0,
	VCMD_DEC_MODULE_MAIN_ADDR_0,
	VCMD_DEC_MODULE_DEC400_ADDR_0,
	VCMD_DEC_MODULE_L2CACHE_ADDR_0,
	VCMD_DEC_MODULE_MMU_ADDR_0},

	{VCMD_DEC_IO_ADDR_1,
	VCMD_DEC_IO_SIZE_1,
	VCMD_DEC_INT_PIN_1,
	VCMD_DEC_MODULE_TYPE_1,
	VCMD_DEC_MODULE_MAIN_ADDR_1,
	VCMD_DEC_MODULE_DEC400_ADDR_1,
	VCMD_DEC_MODULE_L2CACHE_ADDR_1,
	VCMD_DEC_MODULE_MMU_ADDR_1},
};

static struct hantrovcmd_dev *vcmd_manager[MAX_VCMD_TYPE][MAX_VCMD_NUMBER];

/* dynamic allocation*/
static struct hantrovcmd_dev *hantrovcmd_data;
static struct DEC_VCMD_PRIVATE_DATA *dec_vcmd_private_data;

struct DEC_VCMD_PRIVATE_DATA *get_dec1_vcmd_private_handle(void)
{
	return dec_vcmd_private_data;
}

/*
 * Function name   : vcmd_pcie_init
 * Description     : Initialize PCI Hw access
 * Return type     : int
 */
static int vcmd_init(int total_vcmd_core_num, u64 dec_offset)
{
	int i = 0;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();
	for (i = 0; i < total_vcmd_core_num; i++) {
		vcmd_core_array1[i].vcmd_base_addr  = dec_offset
			+ vcmd_core_array1[i].vcmd_base_addr;
	}

	return 0;
}

void vcmd1_delink_cmdbuf(struct hantrovcmd_dev *dev,
			 bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	while (1) {
		if (new_cmdbuf_node == NULL)
			break;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_data_linked)
			cmdbuf_obj->cmdbuf_data_linked = 0;
		else
			break;
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	dev->sw_cmdbuf_rdy_num = 0;
}

void vcmd1_reset_asic(struct hantrovcmd_dev  *dev, int total_vcmd_core_num)
{
	int i, n;
	u32 ret;

	for (n = 0; n < total_vcmd_core_num; n++) {
		if (dev[n].hwregs == NULL)
			continue;

		/* disable interrupt at first */
		vcmd_write_reg((const void *)dev[n].hwregs,
		 VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		/* reset all */
		vcmd_write_reg((const void *)dev[n].hwregs,
		 VCMD_REGISTER_CONTROL_OFFSET, 0x0002);
		/* read status register */
		ret = vcmd_read_reg((const void *)dev[n].hwregs,
		 VCMD_REGISTER_INT_STATUS_OFFSET);
		/* clean status register */
		vcmd_write_reg((const void *)dev[n].hwregs,
		 VCMD_REGISTER_INT_STATUS_OFFSET, ret);
		for (i = VCMD_REGISTER_CONTROL_OFFSET; i <
			dev[n].vcmd_core_cfg.vcmd_iosize; i += 4) {
			vcmd_write_reg((const void *)dev[n].hwregs, i, 0x0000);
		}
		/* enable all interrupt */
		vcmd_write_reg((const void *)dev[n].hwregs,
		 VCMD_REGISTER_INT_CTL_OFFSET, 0xffffffff);
		/* gate all external interrupts */
		vcmd_write_reg((const void *)dev[n].hwregs,
		VCMD_REGISTER_EXT_INT_GATE_OFFSET, 0xffffffff);
	}
}

void vcmd1_release_IO(struct hantrovcmd_dev *hantrovcmd_data,
	int total_vcmd_core_num)
{
	u32 i;
	unsigned long base_addr;
	u32 vcmd_iosize;

	for (i = 0; i < total_vcmd_core_num; i++) {
		if (hantrovcmd_data[i].hwregs) {
			base_addr =
			 hantrovcmd_data[i].vcmd_core_cfg.vcmd_base_addr;
			vcmd_iosize =
			 hantrovcmd_data[i].vcmd_core_cfg.vcmd_iosize;
			iounmap((void *) hantrovcmd_data[i].hwregs);
			release_mem_region(base_addr, vcmd_iosize);
			hantrovcmd_data[i].hwregs = NULL;
		}
	}
}

int vcmd1_reserve_IO(struct hantrovcmd_dev *hantrovcmd_data,
	int total_vcmd_core_num, u8 *vpu_vcmd_base)
{
	u32 hwid;
	int i;
	u32 found_hw = 0;
	unsigned long vcmd_base_addr;
	u32 vcmd_iosize;

	for (i = 0; i < total_vcmd_core_num; i++) {
		hantrovcmd_data[i].hwregs = NULL;

		vcmd_base_addr =
		 hantrovcmd_data[i].vcmd_core_cfg.vcmd_base_addr;
		vcmd_iosize = hantrovcmd_data[i].vcmd_core_cfg.vcmd_iosize;
		hantrovcmd_data[i].hwregs = vpu_vcmd_base;

		if (hantrovcmd_data[i].hwregs == NULL) {
			gb_printf(KERN_ERR, "%s: failed to ioremap HW regs is NULL\n",
			 __func__);
			continue;
		}
		/* read hwid and check validness and store it  */
		hwid = (u32)ioread32((void *)hantrovcmd_data[i].hwregs);
		gb_printf(KERN_INFO, "%s: hwid = 0x%08x\n", __func__, hwid);
		hantrovcmd_data[i].hw_version_id = hwid;

		/* check for vcmd HW ID */
		if (((hwid >> 16) & 0xFFFF) != VCMD_HW_ID) {
			gb_printf(KERN_ERR, "%s: HW not found at 0x%lx\n",
			 __func__, vcmd_base_addr);
			hantrovcmd_data[i].hwregs = NULL;
			continue;
		}

		found_hw = 1;
		gb_printf(KERN_INFO, "%s: HW base <0x%lx> with ID <0x%08x>\n", __func__,
		 vcmd_base_addr, hwid);
	}

	if (found_hw == 0) {
		gb_printf(KERN_ERR, "%s: NO ANY HW found!!\n", __func__);
		return -1;
	}
	gb_printf(KERN_INFO, "%s-%d: dec vcmd reserve_IO finish!!!\n", __func__, __LINE__);
	return 0;
}

/* cmdbuf pool management */
static int allocate_cmdbuf(struct noncache_mem *new_cmdbuf_addr,
	struct noncache_mem *new_status_cmdbuf_addr, struct vpu_vcmd_buff vcmd_buff,
	struct drm_gb_cmdbuf_resv *input_para)
{
	unsigned long flags;
	int cmdbuf_index, status_cmdbuf_index;
	struct noncache_mem *vcmd_buf_mem_pool = NULL;
	struct noncache_mem *vcmd_status_buf_mem_pool = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	spin_lock_irqsave(&vcmd1_cmdbuf_alloc_lock, flags);
	if (dec_data->cmdbuf_used_residual == 0) {
		spin_unlock_irqrestore(&vcmd1_cmdbuf_alloc_lock, flags);
		return 0;
	}

	vcmd_buf_mem_pool = &dec_data->dec_vcmd_mem_pool[0];
	vcmd_status_buf_mem_pool = &dec_data->dec_vcmd_mem_pool[1];
	/* there is one cmdbuf at least */
	while (1) {
		if (dec_data->cmdbuf_used[dec_data->cmdbuf_used_pos] == 0 &&
		 (dec_data->global_cmdbuf_node[dec_data->cmdbuf_used_pos]
		  == NULL)) {
			dec_data->cmdbuf_used[dec_data->cmdbuf_used_pos] = 1;
			dec_data->cmdbuf_used_residual -= 1;
			cmdbuf_index = (dec_data->cmdbuf_used_pos - 1) * 2;
			status_cmdbuf_index = (dec_data->cmdbuf_used_pos - 1) * 2 + 1;
			new_cmdbuf_addr->virtual_address =
				vcmd_buff.vir_buff + cmdbuf_index * CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->bus_address =
				vcmd_buff.offset + cmdbuf_index * CMDBUF_MAX_SIZE;

			new_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->cmdbuf_id = dec_data->cmdbuf_used_pos;

			new_status_cmdbuf_addr->virtual_address =
			 	vcmd_buff.vir_buff + status_cmdbuf_index * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->bus_address =
			 	vcmd_buff.offset + status_cmdbuf_index * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->cmdbuf_id =
			 dec_data->cmdbuf_used_pos;

			(dec_data->cmdbuf_used_pos)++;
			if (dec_data->cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
				dec_data->cmdbuf_used_pos = 0;

			input_para->cmdbuf_offset = cmdbuf_index * CMDBUF_MAX_SIZE;
			input_para->status_offset = status_cmdbuf_index * CMDBUF_MAX_SIZE;
			spin_unlock_irqrestore(&vcmd1_cmdbuf_alloc_lock, flags);
			return 1;
		}
		(dec_data->cmdbuf_used_pos)++;
		if (dec_data->cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
			dec_data->cmdbuf_used_pos = 0;
	}
	return 0;
}


static void free_cmdbuf_mem(u16 cmdbuf_id)
{
	unsigned long flags;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	spin_lock_irqsave(&vcmd1_cmdbuf_alloc_lock, flags);
	dec_data->cmdbuf_used[cmdbuf_id] = 0;
	dec_data->cmdbuf_used_residual += 1;
	spin_unlock_irqrestore(&vcmd1_cmdbuf_alloc_lock, flags);
	wake_up_interruptible_all(&vcmd1_cmdbuf_memory_wait);
}

static bi_list_node *get_cmdbuf_node_in_list_by_addr(size_t  cmdbuf_addr,
	bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();
	new_cmdbuf_node = list->head;
	while (1) {
		if (new_cmdbuf_node == NULL)
			return NULL;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (((cmdbuf_obj->cmdbuf_bus_address - dec_data->base_ddr_addr)
		 <= cmdbuf_addr) && (((cmdbuf_obj->cmdbuf_bus_address -
		  dec_data->base_ddr_addr + cmdbuf_obj->cmdbuf_size)
		   > cmdbuf_addr)))
			return new_cmdbuf_node;
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	return NULL;
}

static int find_empty_vcmd(struct hantrovcmd_dev *dev, bi_list *list,
	bi_list_node *new_cmdbuf_node, struct cmdbuf_obj *cmdbuf_obj)
{
	unsigned long flags = 0;
	int counter = 0;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		spin_lock_irqsave(dev->spinlock, flags);
		if (list->tail == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			dec_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				dec_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return 0;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;

		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return -1;
}

static int select_use_vcmd(struct hantrovcmd_dev *dev, bi_list *list,
	bi_list_node *new_cmdbuf_node, struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	unsigned long flags = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		spin_lock_irqsave(dev->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (curr_cmdbuf_node == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			dec_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				dec_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return 0;
		}
		cmdbuf_obj_temp = (struct cmdbuf_obj *)curr_cmdbuf_node->data;
		if (cmdbuf_obj_temp->cmdbuf_run_done == 1) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			dec_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				dec_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return 0;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type]
				= 0;
		counter++;

		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return -1;
}

static int vcmd_pend_handle(struct hantrovcmd_dev *dev, bi_list *list,
	bi_list_node *new_cmdbuf_node, struct cmdbuf_obj *cmdbuf_obj)
{
	u32 hw_rdy_cmdbuf_num = 0;
	unsigned long flags = 0;
	int counter = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		//read executing cmdbuf address
		if (dev->hw_version_id <= HW_ID_1_0_C)
			hw_rdy_cmdbuf_num =
			 vcmd_get_register_value((const void *)dev->hwregs,
			dev->reg_mirror, HWIF_VCMD_EXE_CMDBUF_COUNT);
		else {
			hw_rdy_cmdbuf_num =
			 *(dev->vcmd_reg_mem_virtual_address +
				VCMD_EXE_CMDBUF_COUNT);
			if (hw_rdy_cmdbuf_num != dev->sw_cmdbuf_rdy_num)
				hw_rdy_cmdbuf_num += 1;
		}
		spin_lock_irqsave(dev->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (curr_cmdbuf_node == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			dec_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				dec_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return 0;
		}

		if ((hw_rdy_cmdbuf_num == dev->sw_cmdbuf_rdy_num)) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			dec_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				dec_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return 0;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type]
				= 0;
		counter++;

		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return -1;
}

static int vcmd_calculate_execute_time(struct hantrovcmd_dev *dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	size_t exe_cmdbuf_addr = 0;
	unsigned long flags = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	u32 cmdbuf_id = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		//read executing cmdbuf address
		if (dev->hw_version_id <= HW_ID_1_0_C) {
			exe_cmdbuf_addr =
			 VCMDGetAddrRegisterValue((const void *)dev->hwregs,
			  dev->reg_mirror, HWIF_VCMD_EXECUTING_CMD_ADDR);
			list = &dev->list_manager;
			spin_lock_irqsave(dev->spinlock, flags);
			//get the executing cmdbuf node.
			curr_cmdbuf_node =
			 get_cmdbuf_node_in_list_by_addr(exe_cmdbuf_addr,
			  list);

			//calculate total execute time of this device
			dev->total_exe_time =
			 calculate_executing_time_after_node(curr_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
		} else {
			if (cmdbuf_obj->priority == CMDBUF_PRIORITY_NORMAL)
				cmdbuf_id =
				 *(dev->vcmd_reg_mem_virtual_address +
					EXECUTING_CMDBUF_ID_ADDR + 1);
			else	//CMDBUF_PRIORITY_HIGH
				cmdbuf_id =
				 *(dev->vcmd_reg_mem_virtual_address +
					EXECUTING_CMDBUF_ID_ADDR);
			spin_lock_irqsave(dev->spinlock, flags);
			if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM
			 || cmdbuf_id == 0) {
				gb_printf(KERN_ERR, "%s:%d--cmdbuf addr =%lx, priotiry=%d, cmdbuf_id %d greater than the ceiling !!\n",
						__func__, __LINE__, dev->vcmd_reg_mem_bus_address, cmdbuf_obj->priority, cmdbuf_id);
				spin_unlock_irqrestore(dev->spinlock, flags);
				return -2;
			}
			//get the executing cmdbuf node.
			curr_cmdbuf_node =
			 dec_data->global_cmdbuf_node[cmdbuf_id];
			if (curr_cmdbuf_node == NULL) {
				list = &dev->list_manager;
				curr_cmdbuf_node = list->head;
				while (1) {
					if (curr_cmdbuf_node == NULL)
						break;
					cmdbuf_obj_temp =
					 (struct cmdbuf_obj *)curr_cmdbuf_node->data;
					if (cmdbuf_obj_temp->cmdbuf_data_linked
					 && cmdbuf_obj_temp->cmdbuf_run_done
					  == 0)
						break;
					curr_cmdbuf_node =
					 curr_cmdbuf_node->next;
				}
			}
			//calculate total execute time of this device
			dev->total_exe_time =
			 calculate_executing_time_after_node(curr_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
		}
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return -1;
}

void find1_least_total_exe_time(struct hantrovcmd_dev *dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	unsigned long flags = 0;
	u64 executing_time = 0xffffffffffffffff;
	struct hantrovcmd_dev *smallest_dev = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		if (dev->total_exe_time <= executing_time) {
			executing_time = dev->total_exe_time;
			smallest_dev = dev;
		}
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	//insert list
	list = &smallest_dev->list_manager;
	spin_lock_irqsave(smallest_dev->spinlock, flags);
	bi_list_insert_node_tail(list, new_cmdbuf_node);
	spin_unlock_irqrestore(smallest_dev->spinlock, flags);
	cmdbuf_obj->core_id = smallest_dev->core_id;
}

static int wait_abort_rdy(struct hantrovcmd_dev *dev)
{
	return dev->working_state == WORKING_STATE_IDLE;
}

int vcmd1_find_smallest_device(struct hantrovcmd_dev *dev,
			       bi_list *list, bi_list_node *new_cmdbuf_node,
			       struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	unsigned long flags = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	u64 executing_time = 0xffffffffffffffff;
	struct hantrovcmd_dev *smallest_dev = NULL;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		dev =
		 vcmd_manager[cmdbuf_obj->module_type][dec_data->vcmd_position[cmdbuf_obj->module_type]];
		if (dev->total_exe_time <= executing_time) {
			executing_time = dev->total_exe_time;
			smallest_dev = dev;
		}
		dec_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (dec_data->vcmd_position[cmdbuf_obj->module_type] >=
			dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			dec_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 dec_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
    //abort the vcmd and wait
	vcmd_write_register_value((const void *)smallest_dev->hwregs,
	 smallest_dev->reg_mirror, HWIF_VCMD_START_TRIGGER, 0);
	if (wait_event_interruptible(*smallest_dev->wait_abort_queue,
		wait_abort_rdy(smallest_dev)))
		return -ERESTARTSYS;
	//CMDBUF_PRIORITY_HIGH
	spin_lock_irqsave(smallest_dev->spinlock, flags);
	curr_cmdbuf_node = smallest_dev->list_manager.head;
	while (1) {
		//if list is empty or tail,insert to tail
		if	(curr_cmdbuf_node == NULL)
			break;
		cmdbuf_obj_temp = (struct cmdbuf_obj *)curr_cmdbuf_node->data;
		if ((cmdbuf_obj_temp->priority == CMDBUF_PRIORITY_NORMAL)
			&& (cmdbuf_obj_temp->cmdbuf_run_done == 0))
			break;
		curr_cmdbuf_node = curr_cmdbuf_node->next;
	}
	bi_list_insert_node_before(list, curr_cmdbuf_node, new_cmdbuf_node);
	cmdbuf_obj->core_id = smallest_dev->core_id;
	spin_unlock_irqrestore(smallest_dev->spinlock, flags);

	return 0;
}

static int select_vcmd(bi_list_node *new_cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list *list = NULL;
	int ret = -1;
	struct hantrovcmd_dev *dev = NULL;

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;

	//there is an empty vcmd to be used
	ret = find_empty_vcmd(dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 != ret) {
		gb_printf(KERN_INFO, "%s-%d: find empty vcmd, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}

	ret = select_use_vcmd(dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 != ret) {
		gb_printf(KERN_INFO, "%s-%d: select use vcmd, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}

	ret = vcmd_pend_handle(dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 != ret) {
		gb_printf(KERN_INFO, "%s-%d: vcmd pend handle, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}

	if (cmdbuf_obj->priority == CMDBUF_PRIORITY_NORMAL) {
		//calculate total execute time of all devices
		ret = vcmd_calculate_execute_time(dev, list,
		 new_cmdbuf_node, cmdbuf_obj);
		if (-1 != ret) {
			gb_printf(KERN_ERR, "%s-%d: error, ret = %d\n",
			 __func__, __LINE__, ret);
			return ret;
		}

		//find the device with the least total_exe_time.
		find1_least_total_exe_time(dev, list,
		 new_cmdbuf_node, cmdbuf_obj);
		return 0;
	}
	//calculate total execute time of all devices
	ret = vcmd_calculate_execute_time(dev, list,
		new_cmdbuf_node, cmdbuf_obj);
	if (-1 != ret) {
		gb_printf(KERN_ERR, "%s-%d: error, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}

	// find the smallest device.
	vcmd1_find_smallest_device(dev, list, new_cmdbuf_node, cmdbuf_obj);
	return 0;
}

static void vcmd_the_first_cmdbuf(struct hantrovcmd_dev *dev,
	bi_list_node *new_cmdbuf_node)
{
	u32  *jmp_addr = NULL;
	u32 operation_code;
	struct cmdbuf_obj *cmdbuf_obj;

	if (new_cmdbuf_node == NULL)
		return;
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if ((cmdbuf_obj->cmdbuf_data_linked == 0)) {
		(dev->sw_cmdbuf_rdy_num)++;
		cmdbuf_obj->cmdbuf_data_linked = 1;
		dev->duration_without_int = 0;
		if (cmdbuf_obj->has_end_cmdbuf == 0) {
			if (cmdbuf_obj->no_normal_int_cmdbuf == 1) {
				dev->duration_without_int =
				 cmdbuf_obj->executing_time;
				//maybe nop is modified, so write back.
				if (dev->duration_without_int >=
				 INT_MIN_SUM_OF_IMAGE_SIZE) {
					jmp_addr =
					 cmdbuf_obj->cmdbuf_virtual_address +
					  (cmdbuf_obj->cmdbuf_size / 4);
					operation_code = *(jmp_addr - 4);
					operation_code =
					 JMP_IE_1 | operation_code;
					*(jmp_addr - 4) = operation_code;
					dev->duration_without_int = 0;
				}
			}
		}
	}
}

static void current_cmdbuf_link_tonext_cmdbuf(struct hantrovcmd_dev *dev,
	struct cmdbuf_obj *cmdbuf_obj, struct cmdbuf_obj *next_cmdbuf_obj)
{
	u32  *jmp_addr = NULL;
	u32 operation_code;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	gb_printf(KERN_INFO, "%s: Link cmdbuf %d to cmdbuf %d", __func__,
		cmdbuf_obj->cmdbuf_id, next_cmdbuf_obj->cmdbuf_id);
	jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
		(cmdbuf_obj->cmdbuf_size / 4);
	if (dev->hw_version_id > HW_ID_1_0_C) {
		//set next cmdbuf id
		*(jmp_addr - 1) = next_cmdbuf_obj->cmdbuf_id;
	}
	if (sizeof(size_t) == 8)
		*(jmp_addr - 2) =
		 (u32)((u64)(next_cmdbuf_obj->cmdbuf_bus_address
		  - dec_data->base_ddr_addr) >> 32);
	else
		*(jmp_addr - 2) = 0;

	*(jmp_addr - 3) = (u32)(next_cmdbuf_obj->cmdbuf_bus_address
		- dec_data->base_ddr_addr);
	operation_code = *(jmp_addr - 4);
	operation_code >>= 16;
	operation_code <<= 16;
	*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1
		| ((next_cmdbuf_obj->cmdbuf_size + 7) / 8));
	next_cmdbuf_obj->cmdbuf_data_linked = 1;
	(dev->sw_cmdbuf_rdy_num)++;
}

static void modify_nop_code_next_cmdbuf(struct hantrovcmd_dev *dev,
	struct cmdbuf_obj *next_cmdbuf_obj)
{
	u32  *jmp_addr = NULL;
	u32 operation_code;

	if (next_cmdbuf_obj->has_end_cmdbuf == 0) {
		if (next_cmdbuf_obj->no_normal_int_cmdbuf == 1) {
			dev->duration_without_int +=
			 next_cmdbuf_obj->executing_time;

	//maybe we see the modified nop before abort, so need to write back.
			if (dev->duration_without_int >=
			 INT_MIN_SUM_OF_IMAGE_SIZE) {
				jmp_addr =
				 next_cmdbuf_obj->cmdbuf_virtual_address +
				  (next_cmdbuf_obj->cmdbuf_size / 4);
				operation_code = *(jmp_addr - 4);
				operation_code = JMP_IE_1 | operation_code;
				*(jmp_addr - 4) = operation_code;
				dev->duration_without_int = 0;
			}
		}
	} else
		dev->duration_without_int = 0;
}

void vcmd1_reset_current_asic(struct hantrovcmd_dev  *dev)
{
	u32 result;

	if (dev->hwregs != NULL) {
		// disable interrupt at first
		vcmd_write_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		// reset all
		vcmd_write_reg((const void *)dev->hwregs,
			VCMD_REGISTER_CONTROL_OFFSET, 0x0002);
		// read status register
		result = vcmd_read_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET);
		// clean status register
		vcmd_write_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET, result);
	}
}

void vcmd1_link_cmdbuf(struct hantrovcmd_dev *dev,
	bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *next_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct cmdbuf_obj *next_cmdbuf_obj = NULL;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	//for the first cmdbuf.
	vcmd_the_first_cmdbuf(dev, new_cmdbuf_node);

	while (1) {
		if (new_cmdbuf_node == NULL)
			break;
		if (new_cmdbuf_node->next == NULL)
			break;
		next_cmdbuf_node = new_cmdbuf_node->next;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		next_cmdbuf_obj = (struct cmdbuf_obj *)next_cmdbuf_node->data;
		if (cmdbuf_obj->has_end_cmdbuf == 0
			&& !next_cmdbuf_obj->cmdbuf_run_done) {
			//need to link, current cmdbuf link to next cmdbuf
			current_cmdbuf_link_tonext_cmdbuf(dev,
			 cmdbuf_obj, next_cmdbuf_obj);

			//modify nop code of next cmdbuf
			modify_nop_code_next_cmdbuf(dev, next_cmdbuf_obj);
		}
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
}

static void printk_vcmd_register_debug(const void *hwregs, char *info)
{
#ifdef HANTRO_VCMD_DRIVER_DEBUG
	u32 i, fordebug;

	for (i = 0; i < ASIC_VCMD_SWREG_AMOUNT; i++) {
		fordebug = vcmd_read_reg((const void *)hwregs, i * 4);
		gb_printf(KERN_INFO, "%s: vcmd register %d:0x%x\n", info, i, fordebug);
	}
#endif
}

#ifdef HANTRO_VCMD_DRIVER_DEBUG
static void enable_AXIFE_by_VCMD(struct hantrovcmd_dev **device,
	u32 *mirror_index, u32 write_command)
{
	u32 register_index, tmp_mirror_index, register_value;
	struct hantrovcmd_dev *dev;

	dev = *device;
	tmp_mirror_index = *mirror_index;
	if	(dev->vcmd_core_cfg.submodule_axife_addr != 0xffff) {
		register_index = AXI_REG10_SW_FRONTEND_EN;
		register_value = 0x02;
		dev->reg_mirror[tmp_mirror_index++] = write_command
		 | (dev->vcmd_core_cfg.submodule_axife_addr
		  + register_index);
		dev->reg_mirror[tmp_mirror_index++] = register_value;

		register_index =  AXI_REG11_SW_WORK_MODE;
		register_value = 0x00;
		dev->reg_mirror[tmp_mirror_index++] = write_command
		 | (dev->vcmd_core_cfg.submodule_axife_addr
		  + register_index);
		dev->reg_mirror[tmp_mirror_index++] = register_value;
	}
	*mirror_index = tmp_mirror_index;
}

static void enable_MMU_by_VCMD(struct hantrovcmd_dev **device,
	u32 *mirror_index, u32 write_command)
{
	u64 address = 0;
	u16 mmu_offset_address = 0;
	u32 register_index, tmp_mirror_index, register_value, i;
	struct hantrovcmd_dev *dev;
	u64 address_ext = 0;

	dev = *device;
	tmp_mirror_index = *mirror_index;
	address = GetMMUAddress();
	gb_printf(KERN_INFO, "%s: address = 0x%llx", __func__, address);
	for (i = 0; i < 2; i++) {
		mmu_offset_address = (i == 0) ?
		 dev->vcmd_core_cfg.submodule_MMU_addr
			: dev->vcmd_core_cfg.submodule_MMUWrite_addr;
		if (mmu_offset_address != 0xffff) {
			register_index = MMU_REG_ADDRESS;
			register_value = address;
			dev->reg_mirror[tmp_mirror_index++] = write_command
				| (mmu_offset_address+register_index);
			dev->reg_mirror[tmp_mirror_index++] = register_value;

			register_index = MMU_REG_ADDRESS_MSB;
			register_value = address_ext;
			dev->reg_mirror[tmp_mirror_index++] = write_command
				| (mmu_offset_address+register_index);
			dev->reg_mirror[tmp_mirror_index++] = register_value;

			register_index =  MMU_REG_PAGE_TABLE_ID;
			register_value = 0x10000;
			dev->reg_mirror[tmp_mirror_index++] = write_command
				| (mmu_offset_address+register_index);
			dev->reg_mirror[tmp_mirror_index++] = register_value;

			register_index =  MMU_REG_PAGE_TABLE_ID;
			register_value = 0x00000;
			dev->reg_mirror[tmp_mirror_index++] = write_command
				| (mmu_offset_address+register_index);
			dev->reg_mirror[tmp_mirror_index++] = register_value;

			register_index =  MMU_REG_CONTROL;
			register_value = 1;
			dev->reg_mirror[tmp_mirror_index++] = write_command
				| (mmu_offset_address+register_index);
			dev->reg_mirror[tmp_mirror_index++] = register_value;
		}
	}
	*mirror_index = tmp_mirror_index;
}
#endif

static void ConfigAIIXFE_MMU_BYVCMD(struct hantrovcmd_dev **device)
{
#ifdef HANTROVCMD_ENABLE_IP_SUPPORT
	u32 i = 0;

	u32 mirror_index, register_index, register_value;
	u32 write_command = 0;
	struct hantrovcmd_dev *dev;

	if (!device)
		return;
	dev = *device;
	mirror_index = VCMD_REGISTER_INDEX_SW_INIT_CMD0;
	write_command = OPCODE_WREG|(1<<26)|(1<<16);
	//enable AXIFE by VCMD
	enable_AXIFE_by_VCMD(device, &mirror_index, write_command);

	//enable MMU by VCMD
	enable_MMU_by_VCMD(device, &mirror_index, write_command);

	//END command
	dev->reg_mirror[mirror_index++] = OPCODE_END;
	dev->reg_mirror[mirror_index] = 0x00;

	for (i = 0; i < mirror_index - VCMD_REGISTER_INDEX_SW_INIT_CMD0; i++) {
		register_index = (i + VCMD_REGISTER_INDEX_SW_INIT_CMD0) * 4;
		vcmd_write_reg((const void *)dev->hwregs, register_index,
			dev->reg_mirror[i + VCMD_REGISTER_INDEX_SW_INIT_CMD0]);
	}
#endif
}

void vcmd1_start(struct hantrovcmd_dev *dev,
	bi_list_node *first_linked_cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	gb_printf(KERN_INFO, "%s:%d, dev = 0x%px, cmdbuf_id = %d\n", __func__, __LINE__, dev,
		((struct cmdbuf_obj *)first_linked_cmdbuf_node->data)->cmdbuf_id);
	if (dev->working_state == WORKING_STATE_IDLE) {
		if ((first_linked_cmdbuf_node != NULL)
		 && dev->sw_cmdbuf_rdy_num) {
			cmdbuf_obj =
			 (struct cmdbuf_obj *)first_linked_cmdbuf_node->data;
			printk_vcmd_register_debug((const void *)dev->hwregs,
			 "func vcmd start exits");
			//0x40
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_AXI_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_CORE_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_ABORT_MODE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_RESET_CORE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_RESET_ALL, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_START_TRIGGER, 0);
			//0x48
			if (dev->hw_version_id <= HW_ID_1_0_C)
				vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_INTCMD_EN, 0xffff);
			else {
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_IRQ_JMPP_EN, 1);
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_IRQ_JMPD_EN, 1);
			}

			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_RESET_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_ABORT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_CMDERR_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_TIMEOUT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_BUSERR_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_ENDCMD_EN, 1);
			//0x4c
			vcmd_set_register_mirror_value(dev->reg_mirror,
			HWIF_VCMD_TIMEOUT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_TIMEOUT_CYCLES, 0x1dcd6500);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR,
				(u32)(cmdbuf_obj->cmdbuf_bus_address));
			if (sizeof(size_t) == 8)
				vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR_MSB,
				(u32)((u64)(cmdbuf_obj->cmdbuf_bus_address)
				 >> 32));
			else
				vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR_MSB, 0);

			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXE_CMDBUF_LENGTH,
				(u32)((cmdbuf_obj->cmdbuf_size + 7) / 8));
			vcmd_set_register_mirror_value(dev->reg_mirror,
			 HWIF_VCMD_RDY_CMDBUF_COUNT, dev->sw_cmdbuf_rdy_num);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MAX_BURST_LEN, 0x10);
			if (dev->hw_version_id > HW_ID_1_0_C)
				vcmd_write_register_value((const void *)dev->hwregs,
				dev->reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID,
				(u32)cmdbuf_obj->cmdbuf_id);

			vcmd_write_reg((const void *)dev->hwregs, 0x44,
				vcmd_read_reg((const void *)dev->hwregs, 0x44));
			vcmd_write_reg((const void *)dev->hwregs, 0x40,
				dev->reg_mirror[0x40 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x48,
				dev->reg_mirror[0x48 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x4c,
				dev->reg_mirror[0x4c / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x50,
				dev->reg_mirror[0x50 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x54,
				dev->reg_mirror[0x54 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x58,
				dev->reg_mirror[0x58 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x5c,
				dev->reg_mirror[0x5c / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x60,
				dev->reg_mirror[0x60 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x64,
				0xffffffff);//not interrupt cpu
			dev->working_state = WORKING_STATE_WORKING;

			ConfigAIIXFE_MMU_BYVCMD(&dev);

			//start
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_START_TRIGGER, 1);
			vcmd_write_reg((const void *)dev->hwregs, 0x40,
				dev->reg_mirror[0x40 / 4]);

			printk_vcmd_register_debug((const void *)dev->hwregs,
			 "func vcmd start exits");
		}
	}
}


static bi_list_node *create_cmdbuf_node(struct vpu_vcmd_buff vcmd_buff,
	struct drm_gb_cmdbuf_resv *input_para)
{
	bi_list_node *current_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct noncache_mem  new_cmdbuf_addr;
	struct noncache_mem  new_status_cmdbuf_addr;

	if (wait_event_interruptible(vcmd1_cmdbuf_memory_wait,
	 allocate_cmdbuf(&new_cmdbuf_addr, &new_status_cmdbuf_addr, vcmd_buff, input_para)))
		return NULL;
	cmdbuf_obj = create_cmdbuf_obj();
	if (cmdbuf_obj == NULL) {
		gb_printf(KERN_ERR, "%s: create_cmdbuf_obj fail!\n", __func__);
		free_cmdbuf_mem(new_cmdbuf_addr.cmdbuf_id);
		return NULL;
	}
	cmdbuf_obj->cmdbuf_bus_address = new_cmdbuf_addr.bus_address;
	cmdbuf_obj->cmdbuf_virtual_address = new_cmdbuf_addr.virtual_address;
	cmdbuf_obj->cmdbuf_size = new_cmdbuf_addr.size;
	cmdbuf_obj->cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;

	cmdbuf_obj->status_bus_address = new_status_cmdbuf_addr.bus_address;
	cmdbuf_obj->status_virtual_address =
		new_status_cmdbuf_addr.virtual_address;
	cmdbuf_obj->status_size = new_status_cmdbuf_addr.size;

	current_node = bi_list_create_node();
	if (current_node == NULL) {
		gb_printf(KERN_ERR, "%s: bi_list_create_node fail!\n", __func__);
		free_cmdbuf_mem(new_cmdbuf_addr.cmdbuf_id);
		free_cmdbuf_obj(cmdbuf_obj);
		return NULL;
	}
	current_node->data = (void *)cmdbuf_obj;
	current_node->next = NULL;
	current_node->previous = NULL;
	return current_node;
}


long reserve1_cmdbuf(struct drm_file *filp,
	struct drm_gb_cmdbuf_resv *input_para, bi_list *global_process_manager,
	bi_list_node **global_cmdbuf_node, struct vpu_vcmd_buff vcmd_buff)
{
	u32 module_type;
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *process_manager_node = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;
	struct process_manager_obj *process_manager_obj = NULL;
	unsigned long flags;

	input_para->cmdbuf_id = 0;
	dec_data = get_dec1_vcmd_private_handle();

	if (input_para->cmdbuf_size > 2 * CMDBUF_MAX_SIZE)
		return -1;

	spin_lock_irqsave(&vcmd1_process_manager_lock, flags);
	process_manager_node = global_process_manager->head;
	while (1) {
		if (process_manager_node == NULL) {
			//should not happen
			gb_printf(KERN_ERR, "hantrovcmd: ERROR process_manager_node !!\n");
			spin_unlock_irqrestore(&vcmd1_process_manager_lock,
			 flags);
			return -1;
		}
		process_manager_obj =
		 (struct process_manager_obj *)process_manager_node->data;
		gb_printf(KERN_INFO, "%s: node %p, filp %p\n", __func__,
		 (void *)process_manager_node,
		  (void *)process_manager_obj->filp);
		if (filp == process_manager_obj->filp)
			break;
		process_manager_node = process_manager_node->next;
	}
	spin_unlock_irqrestore(&vcmd1_process_manager_lock, flags);

	spin_lock_irqsave(&process_manager_obj->spinlock, flags);
	process_manager_obj->total_exe_time += input_para->executing_time;
	spin_unlock_irqrestore(&process_manager_obj->spinlock, flags);
	if (wait_event_interruptible(process_manager_obj->wait_queue,
		wait_process_resource_rdy(process_manager_obj))) {
		gb_printf(KERN_ERR, "%s:%d wait resource rdy fail----\n", __func__, __LINE__);
		return -1;
	}

	module_type = input_para->module_type;
	if (down_interruptible(&dec_data->vcmd_reserve_cmdbuf_sem[module_type]))
		return -ERESTARTSYS;
	new_cmdbuf_node = create_cmdbuf_node(vcmd_buff, input_para);
	if (new_cmdbuf_node == NULL) {
		gb_printf(KERN_ERR, "%s:%d create cmdbuf node fail----\n", __func__, __LINE__);
		return -1;
	}

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	cmdbuf_obj->module_type = input_para->module_type;
	cmdbuf_obj->priority = input_para->priority;
	cmdbuf_obj->executing_time = input_para->executing_time;
	cmdbuf_obj->cmdbuf_size = CMDBUF_MAX_SIZE;
	input_para->cmdbuf_size = CMDBUF_MAX_SIZE;
	cmdbuf_obj->filp = filp;
	cmdbuf_obj->process_manager_obj = process_manager_obj;

	input_para->cmdbuf_id = cmdbuf_obj->cmdbuf_id;
	global_cmdbuf_node[input_para->cmdbuf_id] = new_cmdbuf_node;
	up(&dec_data->vcmd_reserve_cmdbuf_sem[module_type]);

	return 0;
}

void set_address_vcmd1_registers(struct hantrovcmd_dev *dev, u32 *jmp_addr,
	 struct cmdbuf_obj *cmdbuf_obj)
{
	if (dev->hw_version_id > HW_ID_1_0_C) {
		jmp_addr = cmdbuf_obj->cmdbuf_virtual_address;

		if (sizeof(size_t) == 8)
			*(jmp_addr + 2) =
			 (u32)((u64)(dev->vcmd_reg_mem_bus_address +
			  (EXECUTING_CMDBUF_ID_ADDR + 1) * 4) >> 32);
		else
			*(jmp_addr + 2) = 0;

		*(jmp_addr + 1) = (u32)((dev->vcmd_reg_mem_bus_address +
			(EXECUTING_CMDBUF_ID_ADDR + 1) * 4));

		jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
			(cmdbuf_obj->cmdbuf_size / 4);

		if (sizeof(size_t) == 8)
			*(jmp_addr - 6) =
			 (u32)((u64)dev->vcmd_reg_mem_bus_address >> 32);
		else
			*(jmp_addr - 6) = 0;

		*(jmp_addr - 7) = (u32)(dev->vcmd_reg_mem_bus_address);
	}
}


long link_and_run_cmdbuf1(struct drm_file *filp,
	int cmdbuf_id, int cmdbuf_size, int *core_id)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *last_cmdbuf_node;
	u32 *jmp_addr = NULL;
	u32 opCode, opcode_bak;
	u32 tempOpcode;
	u32 record_last_cmdbuf_rdy_num;
	struct hantrovcmd_dev *dev = NULL;
	unsigned long flags;
	int return_value;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	new_cmdbuf_node = dec_data->global_cmdbuf_node[cmdbuf_id];
	if (new_cmdbuf_node == NULL) {
		//should not happen
		gb_printf(KERN_ERR, "hantrovcmd: ERROR cmdbuf_id !!\n");
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) {
		//should not happen
		gb_printf(KERN_ERR, "hantrovcmd: ERROR cmdbuf_id !!\n");
		return -1;
	}
	cmdbuf_obj->cmdbuf_data_loaded = 1;
	cmdbuf_obj->cmdbuf_size = cmdbuf_size;
	cmdbuf_obj->waited = 0;
	gb_printf(KERN_INFO, "%s:%d--cmdbuf size=0x%x, cmdbuf obj = 0x%llx-\n",
		  __func__, __LINE__, cmdbuf_size, (u64)cmdbuf_obj);
	//test nop and end opcode, then assign value.
	cmdbuf_obj->has_end_cmdbuf = 0; //0: has jmp opcode,1 has end code
	cmdbuf_obj->no_normal_int_cmdbuf = 0;
	jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
		(cmdbuf_obj->cmdbuf_size / 4);
	opcode_bak = opCode = tempOpcode = *(jmp_addr - 4);
	opCode >>= 27;
	opCode <<= 27;
	if (opCode == OPCODE_JMP) {
		//jmp
		opCode = tempOpcode;
		opCode &= 0x02000000;
		if (opCode == JMP_IE_1)
			cmdbuf_obj->no_normal_int_cmdbuf = 0;
		else
			cmdbuf_obj->no_normal_int_cmdbuf = 1;
	} else {
		gb_printf(KERN_ERR, "%s:%d--not support other opcode = 0x%x \n",
			  __func__, __LINE__, opCode);
		/* not support other opcode */
		return -1;
	}

	if (down_interruptible(&dec_data->vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]))
		return -ERESTARTSYS;

	return_value = select_vcmd(new_cmdbuf_node);
	if (return_value) {
		up(&dec_data->vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]);
		return return_value;
	}
	dev = &hantrovcmd_data[cmdbuf_obj->core_id];
	*core_id = cmdbuf_obj->core_id;
	gb_printf(KERN_INFO, "Allocate cmd buffer [%d]\n", cmdbuf_id);

	//set ddr address for vcmd registers copy.
	set_address_vcmd1_registers(dev, jmp_addr, cmdbuf_obj);

	//start to link and/or run
	spin_lock_irqsave(dev->spinlock, flags);
	last_cmdbuf_node = find_last_linked_cmdbuf(new_cmdbuf_node);
	record_last_cmdbuf_rdy_num = dev->sw_cmdbuf_rdy_num;
	vcmd1_link_cmdbuf(dev, last_cmdbuf_node);
	tempOpcode = *(jmp_addr - 4);
	*(jmp_addr - 4) = opcode_bak | (tempOpcode & 0x2000000);

	if (dev->working_state == WORKING_STATE_IDLE) {
		//run
		if (filp == NULL)
			((struct cmdbuf_obj *)last_cmdbuf_node->data)->\
				cmdbuf_run_done = 0;
		while (last_cmdbuf_node &&
		 ((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done) {
			gb_printf(KERN_INFO, "%s:%d-----while---\n", __func__, __LINE__);
			last_cmdbuf_node = last_cmdbuf_node->next;
		}

		if (last_cmdbuf_node && last_cmdbuf_node->data) {
			gb_printf(KERN_INFO,
				  "%s:%d---vcmd start for cmdbuf id %d, cmdbuf_run_done = %d\n",
				  __func__, __LINE__,
				  ((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_id,
				  ((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done);
		}
		vcmd1_start(dev, last_cmdbuf_node);
	} else {
		//just update cmdbuf ready number
		if	(record_last_cmdbuf_rdy_num != dev->sw_cmdbuf_rdy_num)
			vcmd_write_register_value((const void *)dev->hwregs,
			dev->reg_mirror, HWIF_VCMD_RDY_CMDBUF_COUNT,
			dev->sw_cmdbuf_rdy_num);
	}
	spin_unlock_irqrestore(dev->spinlock, flags);
	up(&dec_data->vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]);

	return 0;
}

/******************************************************************************/
static int check_cmdbuf_irq(struct hantrovcmd_dev *dev,
	struct cmdbuf_obj *cmdbuf_obj, u32 *irq_status_ret)
{
	int rdy = 0;
	unsigned long flags;

	spin_lock_irqsave(dev->spinlock, flags);
	if (cmdbuf_obj->cmdbuf_run_done) {
		rdy = 1;
		*irq_status_ret = cmdbuf_obj->executing_status;
		gb_printf(KERN_INFO, "%s: cmdbuf_id = %d, rdy = 1\n",
			  __func__, cmdbuf_obj->cmdbuf_id);
	}
	spin_unlock_irqrestore(dev->spinlock, flags);
	return rdy;
}

/******************************************************************************/
static int check_mc_cmdbuf_irq(struct drm_file *filp,
	struct cmdbuf_obj *cmdbuf_obj, u32 *irq_status_ret)
{
	int k;
	bi_list_node *new_cmdbuf_node = NULL;
	struct hantrovcmd_dev *dev = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	for (k = 0; k < TOTAL_DISCRETE_CMDBUF_NUM; k++) {
		new_cmdbuf_node = dec_data->global_cmdbuf_node[k];
		if (new_cmdbuf_node == NULL)
			continue;

		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (!cmdbuf_obj || cmdbuf_obj->filp != filp)
			continue;

		dev = &hantrovcmd_data[cmdbuf_obj->core_id];
		if (check_cmdbuf_irq(dev, cmdbuf_obj, irq_status_ret) == 1) {
			/* Return cmdbuf_id when ANY_CMDBUF_ID is used. */
			if (!cmdbuf_obj->waited) {
				*irq_status_ret = cmdbuf_obj->cmdbuf_id;
				cmdbuf_obj->waited = 1;
				return 1;
			}
		}
	}

	return 0;
}

unsigned int wait_cmdbuf1_ready(struct drm_file *filp,
	u32 cmdbuf_id, u32 *irq_status_ret)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	struct hantrovcmd_dev *dev = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	if (cmdbuf_id != ANY_CMDBUF_ID) {
		new_cmdbuf_node = dec_data->global_cmdbuf_node[cmdbuf_id];
		if (new_cmdbuf_node == NULL) {
			/* should not happen */
			gb_printf(KERN_ERR, "%s: ERROR cmdbuf_id !!\n", __func__);
			return -1;
		}
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->filp != filp) {
			/* should not happen */
			gb_printf(KERN_ERR, "%s: ERROR cmdbuf_id !!\n", __func__);
			return -1;
		}
		dev = &hantrovcmd_data[cmdbuf_obj->core_id];
		if (wait_event_interruptible(*dev->wait_queue,
			check_cmdbuf_irq(dev, cmdbuf_obj, irq_status_ret))) {
			gb_printf(KERN_ERR, "%s: vcmd_wait error interrupted\n", __func__);
			return -ERESTARTSYS;
		}
		return 0;
	}
	if (check_mc_cmdbuf_irq(filp, cmdbuf_obj, irq_status_ret)) {
		gb_printf(KERN_INFO, "%s:%d--1-end--wait event interrput-cmdbuf id=%d----\n", __func__, __LINE__, cmdbuf_id);
		return 0;
	}
	if (wait_event_interruptible(dec_data->mc_wait_queue,
		check_mc_cmdbuf_irq(filp, cmdbuf_obj, irq_status_ret))) {
		gb_printf(KERN_ERR, "%s: multicore error interrupt\n", __func__);
		return -ERESTARTSYS;
	}
	return 0;
}

static void create_read_all_registers_cmdbuf
(struct drm_gb_cmdbuf_resv *input_para, bi_list_node **global_cmdbuf_node)
{
	u32 register_range[] = { VCMD_ENCODER_REGISTER_SIZE,
	 VCMD_DECODER_REGISTER_SIZE,
	  VCMD_JPEG_ENCODER_REGISTER_SIZE, VCMD_JPEG_DECODER_REGISTER_SIZE};
	u32 counter_cmdbuf_size = 0;
	u32  *set_base_addr;
	ptr_t status_base_phy_addr;
	u32 offset_inc = 0;
	u16 submodule_main_addr =
	 vcmd_manager[input_para->module_type][0]->vcmd_core_cfg.submodule_main_addr;
	u16 submodule_L2Cache_addr =
	 vcmd_manager[input_para->module_type][0]->vcmd_core_cfg.submodule_L2Cache_addr;
	struct noncache_mem *vcmd_buf_mem_pool = NULL;
	struct noncache_mem *vcmd_status_buf_mem_pool = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;
	struct cmdbuf_obj *cmdbuf =
	 (struct cmdbuf_obj *)(global_cmdbuf_node[input_para->cmdbuf_id]->data);

	dec_data = get_dec1_vcmd_private_handle();
	vcmd_buf_mem_pool = &dec_data->dec_vcmd_mem_pool[0];
	vcmd_status_buf_mem_pool = &dec_data->dec_vcmd_mem_pool[1];

	set_base_addr = cmdbuf->cmdbuf_virtual_address;
	gb_printf(KERN_INFO, "%s:%d----cmdbuf bus addr = %lx, vaddr = %px-\n",
		  __func__, __LINE__, cmdbuf->cmdbuf_bus_address,
		  set_base_addr);
	status_base_phy_addr = cmdbuf->status_bus_address +
	(submodule_main_addr / 2 + 0);
	if (vcmd_manager[input_para->module_type][0]->hw_version_id >
	 HW_ID_1_0_C) {
		gb_printf(KERN_INFO, "%s:create cmdbuf data when hw_version_id = 0x%x\n",
		 __func__,
		  vcmd_manager[input_para->module_type][0]->hw_version_id);

		*(set_base_addr + 0) = (OPCODE_RREG) | (1 << 16)
			| (EXECUTING_CMDBUF_ID_ADDR * 4);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 1) = (u32)0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 2) = (u32)0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 3) = 0;		//alignment
		counter_cmdbuf_size += 4;

		//read main IP all registers
		*(set_base_addr + 4) = (OPCODE_RREG)
			| ((register_range[input_para->module_type] / 4) << 16)
			| (submodule_main_addr + 0);
		counter_cmdbuf_size += 4;

		*(set_base_addr + 5) = (u32)(status_base_phy_addr);
		counter_cmdbuf_size += 4;

		if (sizeof(size_t) == 8)
			*(set_base_addr + 6) =
			 (u32)((u64)(status_base_phy_addr) >> 32);
		else
			*(set_base_addr + 6) = 0;

		counter_cmdbuf_size += 4;

		//alignment
		*(set_base_addr + 7) = 0;
		counter_cmdbuf_size += 4;
		if (submodule_L2Cache_addr != 0xffff) {
			offset_inc = 4;
			status_base_phy_addr =
			 cmdbuf->status_bus_address +
			   (submodule_L2Cache_addr / 2 + 0);
			/* read L2cache IP first register */
			*(set_base_addr + 8) = (OPCODE_RREG) | (1 << 16)
						| (submodule_L2Cache_addr + 0);
			counter_cmdbuf_size += 4;
			*(set_base_addr + 9) = (u32)(status_base_phy_addr -
				dec_data->base_ddr_addr);

			counter_cmdbuf_size += 4;
			if (sizeof(size_t) == 8) {
				*(set_base_addr + 10) =
				 (u32)((u64)(status_base_phy_addr -
				  dec_data->base_ddr_addr) >> 32);
			} else {
				*(set_base_addr + 10) = 0;
			}

			counter_cmdbuf_size += 4;
			//alignment
			*(set_base_addr + 11) = 0;
			counter_cmdbuf_size += 4;
		}

		/* read vcmd registers to ddr */
		*(set_base_addr + 8 + offset_inc) =
		 (OPCODE_RREG) | (27 << 16) | (0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 9 + offset_inc) = (u32)0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 10 + offset_inc) = (u32)0;
		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 11 + offset_inc) = 0;
		counter_cmdbuf_size += 4;
		//JMP RDY = 0
		*(set_base_addr + 12 + offset_inc) =
		 (OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 13 + offset_inc) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 14 + offset_inc) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 15 + offset_inc) = input_para->cmdbuf_id;
		/*
		 * don't add the last alignment DWORD in order to
		 * identify END command or JMP command.
		 */
		input_para->cmdbuf_size = (16 + offset_inc) * 4;
	} else {
		gb_printf(KERN_INFO, "%s: create cmdbuf when hw_version_id = 0x%x\n",
		 __func__,
		  vcmd_manager[input_para->module_type][0]->hw_version_id);
		//read all registers
		*(set_base_addr + 0) = (OPCODE_RREG)
			| ((register_range[input_para->module_type] / 4) << 16)
			| (submodule_main_addr + 0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 1) =
		 (u32)(status_base_phy_addr - dec_data->base_ddr_addr);
		counter_cmdbuf_size += 4;

		if (sizeof(size_t) == 8)
			*(set_base_addr + 2) =
			 (u32)((u64)(status_base_phy_addr -
			  dec_data->base_ddr_addr) >> 32);
		else
			*(set_base_addr + 2) = 0;

		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 3) = 0;
		counter_cmdbuf_size += 4;
		//JMP RDY = 0
		*(set_base_addr + 4) = (OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 5) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 6) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 7) = input_para->cmdbuf_id;
		/*
		 * don't add the last alignment DWORD in order to
		 * identify END command or JMP command.
		 */
		input_para->cmdbuf_size = 8 * 4;
	}
}
static void read_main_module_all_registers(u32 main_module_type,
	struct hantrovcmd_dev *hantrovcmd_data)
{
	int ret;
	int core_id;
	struct drm_gb_cmdbuf_resv input_para;
	u32 irq_status_ret = 0;
	u32 *status_base_virt_addr;
	struct noncache_mem *vcmd_status_buf_mem_pool = NULL;
	u16 submodule_main_addr;
	struct dec_hw_info *hw_info;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();
	vcmd_status_buf_mem_pool = &dec_data->dec_vcmd_mem_pool[1];
	hw_info = &dec_data->hw_info;

	input_para.executing_time = 0;
	input_para.module_type = main_module_type;
	input_para.cmdbuf_size = 0;
	input_para.priority = CMDBUF_PRIORITY_NORMAL;
	submodule_main_addr =
	 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_main_addr;
	ret = reserve1_cmdbuf(NULL, &input_para,
		&dec_data->global_process_manager,
		 dec_data->global_cmdbuf_node, dec_data->vcmd_buff);
	vcmd_manager[main_module_type][0]->status_cmdbuf_id
	 = input_para.cmdbuf_id;
	create_read_all_registers_cmdbuf(&input_para,
			dec_data->global_cmdbuf_node);
	link_and_run_cmdbuf1(NULL, input_para.cmdbuf_id,
		input_para.cmdbuf_size, &core_id);
	if (hw_info->hw_id != 0)
		return;
	wait_cmdbuf1_ready(NULL, input_para.cmdbuf_id, &irq_status_ret);
	status_base_virt_addr = dec_data->vcmd_buff.vir_buff +
		input_para.cmdbuf_id * CMDBUF_MAX_SIZE +
		(submodule_main_addr / 2 + 0);
	hw_info->hw_id = *status_base_virt_addr;
	hw_info->build_id = *(status_base_virt_addr + 309);
	hw_info->synth_cfg = *(status_base_virt_addr + 50);
	hw_info->synth_cfg_2 = *(status_base_virt_addr + 54);
	hw_info->synth_cfg_3 = *(status_base_virt_addr + 56);
	hw_info->pp_synth_cfg = *(status_base_virt_addr + 60);
	hw_info->fuse_cfg = *(status_base_virt_addr + 57);
	hw_info->pp_cfg_stat = *(status_base_virt_addr + 260);

	gb_printf(KERN_INFO, "%s: dec reg 0:0x%x\n", __func__, *status_base_virt_addr);
	gb_printf(KERN_INFO, "%s: reg 50:0x%08x\n", __func__, *(status_base_virt_addr + 50));
	gb_printf(KERN_INFO, "%s: reg 54:0x%08x\n", __func__, *(status_base_virt_addr + 54));
	gb_printf(KERN_INFO, "%s: reg 56:0x%08x\n", __func__, *(status_base_virt_addr + 56));
	gb_printf(KERN_INFO, "%s: reg 309:0x%08x\n", __func__,
	 *(status_base_virt_addr + 309));

	gb_printf(KERN_INFO, "%s: hw id 0:0x%x\n", __func__, *status_base_virt_addr);
	gb_printf(KERN_INFO, "%s: synth cfg:0x%08x\n", __func__, *(status_base_virt_addr + 50));
	gb_printf(KERN_INFO, "%s: synth cfg2:0x%08x\n", __func__, *(status_base_virt_addr + 54));
	gb_printf(KERN_INFO, "%s: synth cfg3:0x%08x\n", __func__, *(status_base_virt_addr + 56));
	gb_printf(KERN_INFO, "%s: pp synth cfg:0x%08x\n", __func__, *(status_base_virt_addr + 60));
	gb_printf(KERN_INFO, "%s: fuse cfg:0x%08x\n", __func__, *(status_base_virt_addr + 57));
	gb_printf(KERN_INFO, "%s: build id:0x%08x\n", __func__,
	*(status_base_virt_addr + 309));
}

int dec1_vcmd_open(struct drm_file *filp, void **dec_priv, void *data)
{
	int result = 0;
	unsigned long flags;
	struct drm_gem_object *vcmd_obj = NULL;
	struct hantrovcmd_dev *dev = hantrovcmd_data;
	bi_list_node *process_manager_node;
	struct process_manager_obj *process_manager_obj = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;
	struct drm_gb_vpu_work *args = (struct drm_gb_vpu_work *)data;

	dec_data = get_dec1_vcmd_private_handle();
	vcmd_obj = dec_data->vcmd_pool->obj;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
	result = drm_vma_node_allow(&vcmd_obj->vma_node, filp);
#else
	result = drm_vma_node_allow(&vcmd_obj->vma_node, filp->filp);
#endif
	args->offset_va = dec_data->vcmd_pool->flp_offset;
	args->offset_pa = dec_data->vcmd_pool->offset;
	gb_printf(KERN_INFO, "%s:%d vcmdbuf offset_va=0x%llx offset_pa=0x%llx\n",
		__func__, __LINE__, args->offset_va, args->offset_pa);
/*
	if (filp->driver_priv != NULL)
		kfree(filp->driver_priv);
	filp->driver_priv = (void *) dev;
*/
	*dec_priv = (void *)dev;

	process_manager_node = create_process_manager_node();
	if (process_manager_node == NULL)
		return -1;
	process_manager_obj =
		(struct process_manager_obj *)process_manager_node->data;
	process_manager_obj->filp = filp;
	spin_lock_irqsave(&vcmd1_process_manager_lock, flags);
	bi_list_insert_node_tail(&dec_data->global_process_manager,
		process_manager_node);
	spin_unlock_irqrestore(&vcmd1_process_manager_lock, flags);

	gb_printf(KERN_INFO, "%s: finish process node %p for filp opened %p\n",
		__func__, (void *)process_manager_node, (void *)filp);
	return result;
}

void vcmd1_restart_handle(struct hantrovcmd_dev *dev,
	struct cmdbuf_obj *restart_cmdbuf, u32 core_id)
{
	if (restart_cmdbuf) {
		u32 irq_status1, irq_status2;
		struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

		dec_data = get_dec1_vcmd_private_handle();

		gb_printf(KERN_INFO, "%s: Restart from cmdbuf [%d] after aborting.\n",
			__func__, restart_cmdbuf->cmdbuf_id);
		irq_status1 = vcmd_read_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET);
		vcmd_write_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET, irq_status1);
		irq_status2 = vcmd_read_reg((const void *)dev->hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET);
		gb_printf(KERN_INFO, "%s: Clear irq status from 0x%0x -> 0x%0x\n",
			__func__, irq_status1, irq_status2);

		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_EXECUTING_CMD_ADDR,
			(u32)(restart_cmdbuf->cmdbuf_bus_address
			 - dec_data->base_ddr_addr));
		if (sizeof(size_t) == 8)
			vcmd_write_register_value((const void *)dev[core_id].hwregs,
			 dev[core_id].reg_mirror,
			  HWIF_VCMD_EXECUTING_CMD_ADDR_MSB,
			(u32)((u64)(restart_cmdbuf->cmdbuf_bus_address -
			dec_data->base_ddr_addr) >> 32));
		else
			vcmd_write_register_value((const void *)dev[core_id].hwregs,
			 dev[core_id].reg_mirror,
			  HWIF_VCMD_EXECUTING_CMD_ADDR_MSB, 0);

		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_EXE_CMDBUF_COUNT, 0);
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_EXE_CMDBUF_LENGTH,
			(u32)((restart_cmdbuf->cmdbuf_size + 7) / 8));
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID,
			restart_cmdbuf->cmdbuf_id);
		gb_printf(KERN_INFO, "%s: dev->sw_cmdbuf_rdy_num is %d\n", __func__,
			dev[core_id].sw_cmdbuf_rdy_num);
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_RDY_CMDBUF_COUNT,
			dev[core_id].sw_cmdbuf_rdy_num);
		printk_vcmd_register_debug((const void *)dev->hwregs,
			"before restart");
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
		 dev[core_id].reg_mirror, HWIF_VCMD_START_TRIGGER, 1);

		gb_printf(KERN_INFO, "%s: Restart from cmdbuf [%d] : start trigger = %d.\n",
		 __func__, restart_cmdbuf->cmdbuf_id,
		  vcmd_get_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_START_TRIGGER));
		gb_printf(KERN_INFO, "%s: dev state from %d -> WORKING.\n",
			__func__, dev[core_id].working_state);
		dev[core_id].working_state = WORKING_STATE_WORKING;
		printk_vcmd_register_debug((const void *)dev->hwregs,
			"after restart");
	} else {
		gb_printf(KERN_INFO, "%s: No more command buffer to be restarted!\n",
		 __func__);
	}
}

/* Update the last JMP cmd in cmdbuf_ojb in order to jump to next_cmdbuf_obj. */
static void cmdbuf_update_jmp_cmd(int hw_version_id,
	struct cmdbuf_obj *cmdbuf_obj, struct cmdbuf_obj *next_cmdbuf_obj,
	int jmp_IE_1)
{
	u32 *jmp_addr;
	u32 operation_code;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	if (!cmdbuf_obj)
		return;

	if (cmdbuf_obj->has_end_cmdbuf == 0) {
		//need to link, current cmdbuf link to next cmdbuf
		jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
			(cmdbuf_obj->cmdbuf_size / 4);
		if (!next_cmdbuf_obj) {
			// If next cmdbuf is not available, set the RDY to 0.
			operation_code = *(jmp_addr-4);
			operation_code >>= 16;
			operation_code <<= 16;
			*(jmp_addr - 4) = (u32)(operation_code & ~JMP_RDY_1);
		} else {
			if (hw_version_id > HW_ID_1_0_C) {
				//set next cmdbuf id
				*(jmp_addr - 1) = next_cmdbuf_obj->cmdbuf_id;
			}

			if (sizeof(size_t) == 8) {
				*(jmp_addr - 2) =
				 (u32)((u64)(next_cmdbuf_obj->cmdbuf_bus_address
				  - dec_data->base_ddr_addr) >> 32);
			} else {
				*(jmp_addr - 2) = 0;
			}
			*(jmp_addr - 3) =
			 (u32)(next_cmdbuf_obj->cmdbuf_bus_address -
			  dec_data->base_ddr_addr);

			operation_code = *(jmp_addr - 4);
			operation_code >>= 16;
			operation_code <<= 16;
			*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1
				| jmp_IE_1 |
				 ((next_cmdbuf_obj->cmdbuf_size + 7) / 8));
		}
	}
}

void vcmd_delink_rm_cmdbuf1(struct hantrovcmd_dev *dev,
	bi_list_node *cmdbuf_node)
{
	bi_list *list = &dev->list_manager;
	struct cmdbuf_obj *cmdbuf_obj =
		(struct cmdbuf_obj *)cmdbuf_node->data;
	bi_list_node *prev = cmdbuf_node->previous;
	bi_list_node *next = cmdbuf_node->next;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	gb_printf(KERN_INFO, "%s: Delink and remove cmdbuf [%d] from vcmd list.\n",
		__func__, cmdbuf_obj->cmdbuf_id);
	bi_list_remove_node(list, cmdbuf_node);
	dec_data->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
	free_cmdbuf_node(cmdbuf_node);
	cmdbuf_update_jmp_cmd(dev->hw_version_id, prev ? prev->data : NULL,
		next ? next->data : NULL,
		dev->duration_without_int > INT_MIN_SUM_OF_IMAGE_SIZE);
}

static long release_cmdbuf_node(bi_list *list, bi_list_node *cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	/*get cmdbuf object according to cmdbuf_id*/
	new_cmdbuf_node = cmdbuf_node;
	if (new_cmdbuf_node == NULL)
		return -1;
	//remove node from list
	new_cmdbuf_node = remove_cmdbuf_node_from_list(list, new_cmdbuf_node);
	if (new_cmdbuf_node) {
		//free node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		dec_data->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
		free_cmdbuf_node(new_cmdbuf_node);
		return 0;
	}
	return 1;
}

int vcmd_cmdbuf_obj_handle1(struct hantrovcmd_dev *dev,
	bi_list_node *new_cmdbuf_node, struct cmdbuf_obj *cmdbuf_obj_temp,
	struct cmdbuf_obj *restart_cmdbuf, u32 core_id)
{
	int vcmd_aborted = 0;
	long retVal = 0;
	unsigned long flags = 0;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	if (cmdbuf_obj_temp->cmdbuf_run_done) {
		cmdbuf_obj_temp->cmdbuf_need_remove = 1;
		retVal = release_cmdbuf_node(&dev[core_id].list_manager,
			new_cmdbuf_node);
		if (retVal == 1)
			cmdbuf_obj_temp->process_manager_obj = NULL;
	} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 0) {
		cmdbuf_obj_temp->cmdbuf_data_linked = 1;
		cmdbuf_obj_temp->cmdbuf_run_done = 1;
		cmdbuf_obj_temp->cmdbuf_need_remove = 1;
		retVal = release_cmdbuf_node(&dev[core_id].list_manager,
			new_cmdbuf_node);
		if (retVal == 1)
			cmdbuf_obj_temp->process_manager_obj = NULL;
	} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1
		&& dev[core_id].working_state == WORKING_STATE_IDLE) {
		vcmd_delink_rm_cmdbuf1(&dev[core_id], new_cmdbuf_node);
		if (restart_cmdbuf == cmdbuf_obj_temp)
			restart_cmdbuf =
				new_cmdbuf_node->next ?
				 new_cmdbuf_node->next->data : NULL;
		if (restart_cmdbuf) {
			gb_printf(KERN_INFO, "%s: Set restart cmdbuf [%d].\n",
				__func__, restart_cmdbuf->cmdbuf_id);
		} else {
			gb_printf(KERN_INFO, "%s: Set restart cmdbuf to NULL.\n", __func__);
		}
	} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1
		&& dev[core_id].working_state == WORKING_STATE_WORKING) {
		bi_list_node *last_cmdbuf_node = NULL;
		bi_list_node *done_cmdbuf_node = NULL;
		int abort_cmdbuf_id;
		int loop_count = 0;

		//abort the vcmd and wait
		gb_printf(KERN_INFO, "%s: Abort due linked cmdbuf %d of current process.\n",
			__func__, cmdbuf_obj_temp->cmdbuf_id);
		printk_vcmd_register_debug((const void *)dev->hwregs,
			"Before trigger to 0");

		// disable abort interrupt
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_START_TRIGGER, 0);
		vcmd_aborted = 1;

		printk_vcmd_register_debug((const void *)dev->hwregs,
			"After trigger to 0");
		// Wait vcmd core aborted and vcmd enters IDLE mode.
		while (vcmd_get_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_WORK_STATE)) {
			loop_count++;
			if (!(loop_count % 10)) {
				u32 irq_status =
				 vcmd_read_reg((const void *)dev->hwregs,
				  VCMD_REGISTER_INT_STATUS_OFFSET);

				gb_printf(KERN_INFO, "%s: expected idle state, irq status = 0x%0x\n",
					__func__, irq_status);
				gb_printf(KERN_INFO, "%s: vcmd current status is %d\n",
				 __func__,
				  vcmd_get_register_value((const void *)dev->hwregs,
					dev[core_id].reg_mirror,
					 HWIF_VCMD_WORK_STATE));
			}
			mdelay(10);  // wait 10ms
			if (loop_count > 100) { // too long
				gb_printf(KERN_ERR, "%s: long before vcmd core IDLE state\n",
				 __func__);
				spin_unlock_irqrestore(dev[core_id].spinlock,
				 flags);
				up(&dec_data->vcmd_reserve_cmdbuf_sem[dev->vcmd_core_cfg.sub_module_type]);
				return -ERESTARTSYS;
			}
		}
		dev[core_id].working_state = WORKING_STATE_IDLE;
		// clear interrupt & restore abort_e
		if (vcmd_get_register_value((const void *)dev[core_id].hwregs,
				dev[core_id].reg_mirror, HWIF_VCMD_IRQ_ABORT)) {
			gb_printf(KERN_INFO, "%s: Abort interrupt triggered, clear it\n",
			 __func__);
			vcmd_write_reg((const void *)dev->hwregs,
				VCMD_REGISTER_INT_STATUS_OFFSET, 0x1 << 4);
			gb_printf(KERN_INFO, "%s: Now irq status = 0x%0x.\n", __func__,
				vcmd_read_reg((const void *)dev->hwregs,
					VCMD_REGISTER_INT_STATUS_OFFSET));
		}

		abort_cmdbuf_id =
		 vcmd_get_register_value((const void *)dev[core_id].hwregs,
		  dev[core_id].reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID);
		gb_printf(KERN_INFO, "%s: Abort when executing cmd buf %d.\n",
			__func__, abort_cmdbuf_id);
		dev[core_id].sw_cmdbuf_rdy_num = 0;
		dev[core_id].duration_without_int = 0;
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_EXE_CMDBUF_COUNT, 0);
		vcmd_write_register_value((const void *)dev[core_id].hwregs,
			dev[core_id].reg_mirror, HWIF_VCMD_RDY_CMDBUF_COUNT, 0);

		/* Mark cmdbuf_run_done to 1 for all the cmd buf executed. */
		done_cmdbuf_node = dev[core_id].list_manager.head;
		while (done_cmdbuf_node) {
			if (!((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_run_done) {
				((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_run_done = 1;
				((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_data_linked = 0;
				gb_printf(KERN_INFO, "%s: Set cmdbuf [%d] run_done to 1.\n", __func__,
					((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id);
			}
			if (((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id
					== abort_cmdbuf_id)
				break;
			done_cmdbuf_node = done_cmdbuf_node->next;
		}
		if (cmdbuf_obj_temp->cmdbuf_run_done) {
			if (done_cmdbuf_node && done_cmdbuf_node->data)
				gb_printf(KERN_INFO, "%s: done_cmdbuf_node is cmdbuf [%d].\n",
				 __func__,
				  ((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id);
			if (done_cmdbuf_node) {
				done_cmdbuf_node = done_cmdbuf_node->next;
				if (done_cmdbuf_node)
					restart_cmdbuf = (struct cmdbuf_obj *)done_cmdbuf_node->data;
				if (restart_cmdbuf) {
					gb_printf(KERN_INFO, "%s: Set restart cmdbuf [%d] via if.\n",
						  __func__, restart_cmdbuf->cmdbuf_id);
				}
			}
		} else {
			last_cmdbuf_node = new_cmdbuf_node;
	/* cmd buf num from aborted cmd buf to current cmdbuf_obj_temp */
			if (cmdbuf_obj_temp->cmdbuf_id != abort_cmdbuf_id) {
				last_cmdbuf_node  = new_cmdbuf_node->previous;
				while (last_cmdbuf_node &&
					((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_id
						!= abort_cmdbuf_id) {
					restart_cmdbuf =
					 (struct cmdbuf_obj *)last_cmdbuf_node->data;
					last_cmdbuf_node
					 = last_cmdbuf_node->previous;
					dev[core_id].sw_cmdbuf_rdy_num++;
					dev[core_id].duration_without_int +=
						restart_cmdbuf->executing_time;
					gb_printf(KERN_INFO, "%s: Keep valid cmdbuf [%d] in the list.\n",
					 __func__, restart_cmdbuf->cmdbuf_id);
				}
			}
			if (restart_cmdbuf) {
				gb_printf(KERN_INFO, "%s: Set restart cmdbuf [%d] via else.\n",
					__func__, restart_cmdbuf->cmdbuf_id);
			}
		}
		// remove first linked cmdbuf from list
		vcmd_delink_rm_cmdbuf1(&dev[core_id], new_cmdbuf_node);
	}
	return vcmd_aborted;
}

int dec1_vcmd_release(struct drm_file *filp, void *dec_priv)
{
	struct hantrovcmd_dev *dev =
		(struct hantrovcmd_dev *) dec_priv;
		//(struct hantrovcmd_dev *) filp->driver_priv;
	u32 core_id = 0;
	unsigned long flags;
	u32 release_cmdbuf_num = 0;
	struct drm_gem_object *vcmd_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	bi_list_node *process_manager_node;
	struct process_manager_obj *process_manager_obj = NULL;
	int vcmd_aborted = 0;  /* vcmd is aborted in this function */
	struct cmdbuf_obj *restart_cmdbuf = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();
	vcmd_obj = dec_data->vcmd_pool->obj;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
	drm_vma_node_revoke(&vcmd_obj->vma_node, filp);
#else
	drm_vma_node_revoke(&vcmd_obj->vma_node, filp->filp);
#endif

	gb_printf(KERN_INFO, "%s: dev closed for process %p via\n",
	 	__func__, (void *)filp);
	if (down_interruptible(&dec_data->vcmd_reserve_cmdbuf_sem[dev->vcmd_core_cfg.sub_module_type]))
		return -ERESTARTSYS;

	for (core_id = 0;
	 core_id < dec_data->total_vcmd_core_num; core_id++) {
		if ((&dev[core_id]) == NULL)
			continue;
		spin_lock_irqsave(dev[core_id].spinlock, flags);
		new_cmdbuf_node = dev[core_id].list_manager.head;
		while (1) {
			if (new_cmdbuf_node == NULL)
				break;
			cmdbuf_obj_temp
			 = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			gb_printf(KERN_INFO, "%s: Process %p is releas: check cmdbuf %d\n",
				__func__, filp, cmdbuf_obj_temp->cmdbuf_id);
			if (dev[core_id].hwregs
			 && (cmdbuf_obj_temp->filp == filp)) {
				vcmd_aborted =
				 vcmd_cmdbuf_obj_handle1(dev, new_cmdbuf_node,
				  cmdbuf_obj_temp, restart_cmdbuf, core_id);
				release_cmdbuf_num++;
				gb_printf(KERN_INFO, "%s: release reserved cmdbuf\n",
				 __func__);
			} else if (vcmd_aborted
			 && !cmdbuf_obj_temp->cmdbuf_run_done) {
				if (!restart_cmdbuf)
					restart_cmdbuf = cmdbuf_obj_temp;
				dev[core_id].duration_without_int +=
					cmdbuf_obj_temp->executing_time;
				dev[core_id].sw_cmdbuf_rdy_num++;
			}

			if (new_cmdbuf_node == NULL)
				break;

			new_cmdbuf_node = new_cmdbuf_node->next;
		}

		vcmd1_restart_handle(dev, restart_cmdbuf, core_id);
		spin_unlock_irqrestore(dev[core_id].spinlock, flags);
		// VCMD aborted but not restarted, nedd to wake up
		if (vcmd_aborted && !restart_cmdbuf)
			wake_up_interruptible_all(dev[core_id].wait_queue);
	}

	if (release_cmdbuf_num)
		wake_up_interruptible_all(&vcmd1_cmdbuf_memory_wait);
	spin_lock_irqsave(&vcmd1_process_manager_lock, flags);
	process_manager_node = dec_data->global_process_manager.head;
	while (1) {
		if (process_manager_node == NULL)
			break;
		process_manager_obj =
		 (struct process_manager_obj *)process_manager_node->data;
		if (process_manager_obj->filp == filp)
			break;
		process_manager_node = process_manager_node->next;
	}
	//remove node from list
	gb_printf(KERN_INFO, "%s: process node %p for filp to be removed: %p\n",
	 __func__, (void *)process_manager_node,
	  (void *)process_manager_obj->filp);
	bi_list_remove_node(&dec_data->global_process_manager,
		process_manager_node);
	spin_unlock_irqrestore(&vcmd1_process_manager_lock, flags);
	free_process_manager_node(process_manager_node);
	up(&dec_data->vcmd_reserve_cmdbuf_sem[dev->vcmd_core_cfg.sub_module_type]);
	return 0;
}

void release_vcmd1_non_cachable_memory(void)
{

}

irqreturn_t dec1_isr(int irq)
{
	irqreturn_t ret = 0;
	ret = hantrovcmd_d1isr(irq, (void *)&hantrovcmd_data[0]);
	return ret;
}
int gb_vdec1_irq_handle(int irq_id)
{
	int i, ret = 0;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	/* get the IRQ line */
	for (i = 0; i < dec_data->total_vcmd_core_num; i++) {
		if (hantrovcmd_data[i].hwregs == NULL)
			continue;
		gb_printf(KERN_INFO, "%s-%d: dec vcmd_irq = %d\n", __func__,
		 __LINE__, hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq);
		if (hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq != -1) {
			ret = request_irq(irq_id, hantrovcmd_d1isr,
			#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
				SA_INTERRUPT | SA_SHIRQ,
			#else
				IRQF_SHARED,
			#endif
				"vc8000d_dec_vcmd",
				 (void *)&hantrovcmd_data[i]);
			if (ret == -EINVAL) {
				gb_printf(KERN_ERR, "%s: Bad vcmd_irq handler i = %d\n",
				 __func__, i);
				vcmd1_release_IO(hantrovcmd_data,
				 dec_data->total_vcmd_core_num);
				goto err;
			} else if (ret == -EBUSY) {
				gb_printf(KERN_ERR, "%s: i = %d, IRQ <%d> busy, change config\n",
				 __func__, i,
				  hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq);
				vcmd1_release_IO(hantrovcmd_data,
				 dec_data->total_vcmd_core_num);
				goto err;
			} else {
				gb_printf(KERN_INFO, "hantro_vcmd: request IRQ <%d> successfully for subsystem %d\n",
				hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq, i);
			}
		} else
			gb_printf(KERN_INFO, "%s: hantro_vcmd: IRQ not in use!\n", __func__);
	}
	return ret;
err:
	release_vcmd1_non_cachable_memory();
	if (hantrovcmd_data != NULL)
		vfree(hantrovcmd_data);
	if (dec_data != NULL)
		vfree(dec_data);
	return -1;
}

void set_vcmd1_init_status_addr(struct DEC_VCMD_PRIVATE_DATA *dec_data,
	struct vpu_pcie_info dec_pcie, struct vpu_vcmd_buff *vcmd_pool,
	struct vpu_vcmd_buff *vcmd_reg)
{
	dec_data->dec_vcmd_mem_pool[0].bus_address = vcmd_pool->flp_offset;
	dec_data->dec_vcmd_mem_pool[1].bus_address = vcmd_pool->flp_offset + CMDBUF_MAX_SIZE;
	dec_data->dec_vcmd_mem_pool[2].bus_address = vcmd_reg->flp_offset;

	dec_data->dec_vcmd_mem_pool[0].virtual_address = vcmd_pool->vir_buff;
	dec_data->dec_vcmd_mem_pool[1].virtual_address = vcmd_pool->vir_buff + CMDBUF_MAX_SIZE;
	dec_data->dec_vcmd_mem_pool[2].virtual_address = vcmd_reg->vir_buff;
	dec_data->base_ddr_addr = 0;
}
int hantrovcmd1_init(struct vpu_pcie_info dec_pcie, int total_vcmd_core_num,
	struct vpu_vcmd_buff *vcmd_pool, struct vpu_vcmd_buff *vcmd_reg,
	u64 dec_offset)
{
	int i, k, result;
	struct vpu_vcmd_buff *vcmd_pool_tmp = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;
	gb_printf(KERN_INFO, "%s-%d: dec vcmd start\n", __func__, __LINE__);
	result = vcmd_init(total_vcmd_core_num, dec_offset);
	if (result)
		goto err;

	for (i = 0; i < total_vcmd_core_num; i++)
		gb_printf(KERN_INFO, "%s: module init - vcmdcore[%d] addr =0x%lx\n",
		 __func__, i, vcmd_core_array1[i].vcmd_base_addr);
	hantrovcmd_data =
	 	(struct hantrovcmd_dev *)vmalloc(sizeof(struct hantrovcmd_dev)
	  	* total_vcmd_core_num);
	if (hantrovcmd_data == NULL)
		goto err;
	memset(hantrovcmd_data, 0,
		sizeof(struct hantrovcmd_dev) * total_vcmd_core_num);

	vcmd_pool_tmp =
		(struct vpu_vcmd_buff *)vmalloc(sizeof(struct vpu_vcmd_buff));
	if (vcmd_pool_tmp == NULL)
		goto err;
	memset(vcmd_pool_tmp, 0, sizeof(struct vpu_vcmd_buff));

	dec_data = (struct DEC_VCMD_PRIVATE_DATA *)
		vmalloc(sizeof(struct DEC_VCMD_PRIVATE_DATA));
	if (dec_data == NULL)
		goto err;
	memset(dec_data, 0, sizeof(struct DEC_VCMD_PRIVATE_DATA));

	memcpy(vcmd_pool_tmp, vcmd_pool, sizeof(struct vpu_vcmd_buff));
	dec_data->vcmd_pool = vcmd_pool_tmp;

	dec_vcmd_private_data = dec_data;
	for (k = 0; k < MAX_VCMD_TYPE; k++) {
		dec_data->vcmd_type_core_num[k] = 0;
		dec_data->vcmd_position[k] = 0;
		for (i = 0; i < MAX_VCMD_NUMBER; i++)
			vcmd_manager[k][i] = NULL;
	}
	dec_data->vcmd_buff.offset = vcmd_pool->offset;
	dec_data->vcmd_buff.vir_buff = vcmd_pool->vir_buff;
	dec_data->total_vcmd_core_num = total_vcmd_core_num;
	set_vcmd1_init_status_addr(dec_data, dec_pcie, vcmd_pool, vcmd_reg);
	init_bi_list(&dec_data->global_process_manager);
	for (i = 0; i < total_vcmd_core_num; i++) {
		hantrovcmd_data[i].vcmd_core_cfg = vcmd_core_array1[i];
		hantrovcmd_data[i].hwregs = NULL;
		hantrovcmd_data[i].core_id = i;
		hantrovcmd_data[i].working_state = WORKING_STATE_IDLE;
		hantrovcmd_data[i].sw_cmdbuf_rdy_num = 0;
		hantrovcmd_data[i].spinlock = &dec_data->owner_lock_vcmd[i];
		spin_lock_init(&dec_data->owner_lock_vcmd[i]);
		hantrovcmd_data[i].wait_queue = &dec_data->wait_queue_vcmd[i];
		init_waitqueue_head(&dec_data->wait_queue_vcmd[i]);
		hantrovcmd_data[i].wait_abort_queue
		 = &dec_data->abort_queue_vcmd[i];
		init_waitqueue_head(&dec_data->abort_queue_vcmd[i]);
		init_bi_list(&hantrovcmd_data[i].list_manager);
		hantrovcmd_data[i].duration_without_int = 0;
		vcmd_manager[vcmd_core_array1[i].sub_module_type][dec_data->vcmd_type_core_num[vcmd_core_array1[i].sub_module_type]] =
		&hantrovcmd_data[i];
		dec_data->vcmd_type_core_num[vcmd_core_array1[i].sub_module_type]++;

		hantrovcmd_data[i].vcmd_reg_mem_bus_address =
			vcmd_reg->offset + i * VCMD_REGISTER_SIZE;

		hantrovcmd_data[i].vcmd_reg_mem_virtual_address =
		 vcmd_reg->vir_buff + i * VCMD_REGISTER_SIZE;

		hantrovcmd_data[i].vcmd_reg_mem_size = VCMD_REGISTER_SIZE;
#ifndef CONFIG_SW64
		memset(hantrovcmd_data[i].vcmd_reg_mem_virtual_address, 0,
                        VCMD_REGISTER_SIZE);
#else
		memset_io(hantrovcmd_data[i].vcmd_reg_mem_virtual_address, 0,
                       VCMD_REGISTER_SIZE);
#endif

	}
	init_waitqueue_head(&dec_data->mc_wait_queue);
	result = vcmd1_reserve_IO(hantrovcmd_data, total_vcmd_core_num,
	 (u8 *)dec_pcie.vpu_vaddr_base);
	if (result < 0)
		goto err;
	vcmd1_reset_asic(hantrovcmd_data, total_vcmd_core_num);

	dec_data->cmdbuf_used_pos = 0;
	for (k = 0; k < TOTAL_DISCRETE_CMDBUF_NUM; k++) {
		dec_data->cmdbuf_used[k] = 0;
		dec_data->global_cmdbuf_node[k] = NULL;
	}
	//cmdbuf_used[0] not be used, because int vector must non-zero
	dec_data->cmdbuf_used_residual = TOTAL_DISCRETE_CMDBUF_NUM;
	dec_data->cmdbuf_used_pos = 1;
	dec_data->cmdbuf_used[0] = 1;
	dec_data->cmdbuf_used_residual -= 1;

	create_kernel_process_manager(&dec_data->global_process_manager);
	for (i = 0; i < MAX_VCMD_TYPE; i++) {
		if (dec_data->vcmd_type_core_num[i] == 0)
			continue;
		sema_init(&dec_data->vcmd_reserve_cmdbuf_sem[i], 1);
	}

	/*
	 * read all registers for each type of module for analyzing
	 * configuration in cwl
	 */
	for (i = 0; i < MAX_VCMD_TYPE; i++) {
		if (dec_data->vcmd_type_core_num[i] == 0)
			continue;
		read_main_module_all_registers(i, hantrovcmd_data);
	}

	return 0;
err:
	release_vcmd1_non_cachable_memory();
	if (hantrovcmd_data != NULL)
		vfree(hantrovcmd_data);
	if (dec_data != NULL)
		vfree(dec_data);
	if (vcmd_pool_tmp != NULL)
		vfree(vcmd_pool_tmp);
	return -1;
}

void dec1_resume(void)
{
	bi_list_node *cmdbuf_node = NULL;

	gb_printf(KERN_INFO, "%s:%d, dec1 resume in\n", __func__, __LINE__);

	cmdbuf_node = hantrovcmd_data[0].list_manager.tail;
	if (cmdbuf_node != NULL)
		((struct cmdbuf_obj *)cmdbuf_node->data)->cmdbuf_run_done = 1;

	hantrovcmd_data[0].working_state = WORKING_STATE_IDLE;
	hantrovcmd_data[0].sw_cmdbuf_rdy_num = 0;

	vcmd1_reset_asic(hantrovcmd_data, 1);

	gb_printf(KERN_INFO, "%s:%d, dec1 resume out\n", __func__, __LINE__);
}

void dec1_suspend(void)
{
	gb_printf(KERN_INFO, "%s:%d, dec1 suspend in\n", __func__, __LINE__);

	for(;;) {
		if (list_empty(&hantrovcmd_data[0].wait_queue->head))
			break;
	}

	gb_printf(KERN_INFO, "%s:%d, dec1 suspend out\n", __func__, __LINE__);
}

static void free_cmdbuf_node(bi_list_node *cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	if (cmdbuf_node == NULL) {
		gb_printf(KERN_ERR, "%s: remove_cmdbuf_node NULL\n", __func__);
		return;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)cmdbuf_node->data;
	//free cmdbuf mem in pool
	free_cmdbuf_mem(cmdbuf_obj->cmdbuf_id);
	//free struct cmdbuf_obj
	free_cmdbuf_obj(cmdbuf_obj);
	//free current cmdbuf_node entity.
	bi_list_free_node(cmdbuf_node);
}

static long release_cmdbuf_node_cleanup(bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	while (1) {
		new_cmdbuf_node = list->head;
		if (new_cmdbuf_node == NULL)
			return 0;
		/* remove node from list */
		bi_list_remove_node(list, new_cmdbuf_node);
		/* free node */
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		dec_data->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
		free_cmdbuf_node(new_cmdbuf_node);
	}
	return 0;
}

void hantrovcmd1_cleanup(int total_vcmd_core_num)
{
	int i = 0;
	u32 result;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	for (i = 0; i < total_vcmd_core_num; i++) {
		if (hantrovcmd_data[i].hwregs == NULL)
			continue;
		//disable interrupt at first
		vcmd_write_reg((const void *) hantrovcmd_data[i].hwregs,
			VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		//disable HW
		vcmd_write_reg((const void *) hantrovcmd_data[i].hwregs,
			VCMD_REGISTER_CONTROL_OFFSET, 0x0000);
		//read status register
		result = vcmd_read_reg((const void *) hantrovcmd_data[i].hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET);
		//clean status register
		vcmd_write_reg((const void *) hantrovcmd_data[i].hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET, result);

		/* free the vcmd IRQ */
		if (hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq != -1) {
			free_irq(hantrovcmd_data[i].vcmd_core_cfg.vcmd_irq,
				(void *)&hantrovcmd_data[i]);
		}
		release_cmdbuf_node_cleanup(&hantrovcmd_data[i].list_manager);
	}
	release_process_node_cleanup(&dec_data->global_process_manager);

	vcmd1_release_IO(hantrovcmd_data, total_vcmd_core_num);
	vfree(hantrovcmd_data);

	release_vcmd1_non_cachable_memory();
	vfree(dec_data);
	gb_printf(KERN_INFO, "%s: module removed finish!!\n", __func__);
}

long release_cmdbuf1(struct drm_file *filp, u16 cmdbuf_id)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *last_cmdbuf_node = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list *list = NULL;
	u32 module_type;
	unsigned long flags;
	struct hantrovcmd_dev *dev = NULL;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	/*get cmdbuf object according to cmdbuf_id*/
	new_cmdbuf_node = dec_data->global_cmdbuf_node[cmdbuf_id];
	if (new_cmdbuf_node == NULL) {
		//should not happen
		gb_printf(KERN_ERR, "%s: ERROR new_cmdbuf_node = NULL error!!\n", __func__);
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) {
		//should not happen
		gb_printf(KERN_ERR, "%s: ERROR cmdbuf_obj->filp != filp error!!\n",
		 __func__);
		return -1;
	}
	module_type = cmdbuf_obj->module_type;
	//TODO
	if (down_interruptible(&dec_data->vcmd_reserve_cmdbuf_sem[module_type]))
		return -ERESTARTSYS;
	dev = &hantrovcmd_data[cmdbuf_obj->core_id];

	list = &dev->list_manager;
	cmdbuf_obj->cmdbuf_need_remove = 1;
	last_cmdbuf_node = new_cmdbuf_node->previous;
	while (1) {
		//remove current node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_need_remove == 1) {
			new_cmdbuf_node = remove_cmdbuf_node_from_list(list,
				new_cmdbuf_node);
			if (new_cmdbuf_node) {
				//free node
				dec_data->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id]
				 = NULL;
				if (cmdbuf_obj->process_manager_obj) {
					spin_lock_irqsave(&cmdbuf_obj->process_manager_obj->spinlock, flags);
					cmdbuf_obj->process_manager_obj->total_exe_time
					 -= cmdbuf_obj->executing_time;
					spin_unlock_irqrestore(&cmdbuf_obj->process_manager_obj->spinlock, flags);
					wake_up_interruptible_all(&cmdbuf_obj->process_manager_obj->wait_queue);
				}
				free_cmdbuf_node(new_cmdbuf_node);
			}
		}
		if (last_cmdbuf_node == NULL)
			break;
		new_cmdbuf_node = last_cmdbuf_node;
		last_cmdbuf_node = new_cmdbuf_node->previous;
	}
	up(&dec_data->vcmd_reserve_cmdbuf_sem[module_type]);
	return 0;
}

int gbdec1_cmdbuf_reserve(struct drm_file *filp, void *data)
{
	int ret = -1;
	struct vpu_vcmd_buff vcmd_buff = {0};
	struct drm_gb_cmdbuf_resv *input_para =
		(struct drm_gb_cmdbuf_resv *)data;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();
	vcmd_buff.offset = dec_data->vcmd_pool->offset;
	vcmd_buff.vir_buff = dec_data->vcmd_pool->vir_buff;

	gb_printf(KERN_INFO, "%s:%d:-vcmd buff offset = 0x%llx,\
		vir_buf = 0x%llx--\n", __func__, __LINE__,
		vcmd_buff.offset, (long long)vcmd_buff.vir_buff);

	ret = reserve1_cmdbuf(filp, input_para,
		&dec_data->global_process_manager,
		dec_data->global_cmdbuf_node, vcmd_buff);
	if (ret == -1)
		gb_printf(KERN_INFO, "%s: dec VCMD Reserve %d\n", __func__,
		 	input_para->cmdbuf_id);

	return ret;
}

long hantrovcmd1_ioctl(struct drm_file *filp,
		unsigned int cmd, void *arg)
{
	int err = 0;
	static int last_polling_cmd;
	struct DEC_VCMD_PRIVATE_DATA *dec_data = NULL;

	dec_data = get_dec1_vcmd_private_handle();

	if (cmd != HANTRO_VCMD_IOCH_POLLING_CMDBUF) {
		last_polling_cmd = 0;
		gb_printf(KERN_INFO, "%s: if != ioctl cmd 0x%08x\n", __func__, cmd);
	} else {
		if (!last_polling_cmd)
			gb_printf(KERN_INFO, "%s: else == ioctl cmd 0x%08x\n",
			 __func__, cmd);
		last_polling_cmd = 1;
	}
	if (_IOC_TYPE(cmd) != HANTRO_VCMD_IOC_MAGIC)
		return -ENOTTY;
	if ((_IOC_TYPE(cmd) == HANTRO_VCMD_IOC_MAGIC
	 && _IOC_NR(cmd) > HANTRO_VCMD_IOC_MAXNR))
		return -ENOTTY;

	if (_IOC_DIR(cmd) & _IOC_READ)
		err = !gb_access_ok(VERIFY_WRITE, (void *) arg,
			_IOC_SIZE(cmd));
	if (_IOC_DIR(cmd) & _IOC_WRITE)
		err = !gb_access_ok(VERIFY_READ, (void *) arg,
			_IOC_SIZE(cmd));
	if (err)
		return -EFAULT;

	switch (cmd) {
	case HANTRO_VCMD_IOCH_GET_CMDBUF_PARAMETER: {
		struct cmdbuf_mem_parameter local_cmdbuf_mem_data;
		struct noncache_mem *vcmd_buf_mem_pool =
			&dec_data->dec_vcmd_mem_pool[0];
		struct noncache_mem *vcmd_status_buf_mem_pool =
			&dec_data->dec_vcmd_mem_pool[1];

		gb_printf(KERN_INFO, "%s: dec VCMD Reserve, cmd = %x\n", __func__, cmd);
		local_cmdbuf_mem_data.cmdbuf_unit_size = CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_unit_size
		 = CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.cmdbuf_total_size
		 = CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_total_size
		 = CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.phy_status_cmdbuf_addr =
			//vcmd_status_buf_mem_pool->bus_address;
			vcmd_buf_mem_pool->bus_address;
		gb_printf(KERN_INFO, "-------vcmd status pool 1 = %llx---\n", vcmd_status_buf_mem_pool->bus_address);
		local_cmdbuf_mem_data.virt_status_cmdbuf_addr =
			vcmd_status_buf_mem_pool->virtual_address;

		local_cmdbuf_mem_data.phy_cmdbuf_addr =
			vcmd_buf_mem_pool->bus_address;


		local_cmdbuf_mem_data.base_ddr_addr = dec_data->base_ddr_addr;
		return copy_to_user((struct cmdbuf_mem_parameter *)arg,
		 &local_cmdbuf_mem_data, sizeof(struct cmdbuf_mem_parameter));
	}
	case HANTRO_VCMD_IOCH_GET_VCMD_PARAMETER: {
		struct config_parameter input_para;
		int ret;

		gb_printf(KERN_INFO, "%s: dec vcmd config cmd = %x\n", __func__, cmd);
		ret = copy_from_user(&input_para,
		 (struct config_parameter *)arg,
		  sizeof(struct config_parameter));
		if (ret != 0)
			return -1;
		if (dec_data->vcmd_type_core_num[input_para.module_type]) {
			input_para.submodule_main_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_main_addr;
			input_para.submodule_dec400_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_dec400_addr;
			input_para.submodule_L2Cache_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_L2Cache_addr;
			input_para.submodule_MMU_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMU_addr;
			input_para.submodule_MMUWrite_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMUWrite_addr;
			input_para.submodule_axife_addr =
			 vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_axife_addr;
			input_para.config_status_cmdbuf_id =
			 vcmd_manager[input_para.module_type][0]->status_cmdbuf_id;
			input_para.vcmd_hw_version_id =
			 vcmd_manager[input_para.module_type][0]->hw_version_id;
			input_para.vcmd_core_num =
			 dec_data->vcmd_type_core_num[input_para.module_type];
		} else {
			input_para.submodule_main_addr = 0xffff;
			input_para.submodule_dec400_addr = 0xffff;
			input_para.submodule_L2Cache_addr = 0xffff;
			input_para.submodule_MMU_addr = 0xffff;
			input_para.submodule_MMUWrite_addr = 0xffff;
			input_para.submodule_axife_addr = 0xffff;
			input_para.config_status_cmdbuf_id = 0;
			input_para.vcmd_core_num = 0;
			input_para.vcmd_hw_version_id = HW_ID_1_0_C;
		}
		return copy_to_user((struct config_parameter *)arg,
			&input_para, sizeof(struct config_parameter));
	}
	case HANTRO_VCMD_IOCH_GET_HWINFO_FROM_VCMD: {
		struct dec_hw_info *hw_info  = &dec_data->hw_info;
		struct dec_hw_info uhw_info;

		memcpy(&uhw_info, hw_info, sizeof(struct dec_hw_info));

		gb_printf(KERN_INFO, "%s: dec VCMD get hw info from vcmd, cmd = %x\n", __func__, cmd);
		return copy_to_user((struct dec_hw_info *)arg,
		 &uhw_info, sizeof(struct dec_hw_info));
	}
	default:
		break;
	}
	return 0;
}


