/* SPDX-License-Identifier: GPL-3.0-or-later */
/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * jessy 20221019 Sietium
 */
#ifndef __VPU_VCMD1_H__
#define __VPU_VCMD1_H__

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/moduleparam.h>
#include <linux/interrupt.h>
#include <linux/semaphore.h>
#include <linux/spinlock.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/uaccess.h>
#include <linux/ioport.h>
#include <asm/irq.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/of_device.h>
#include <linux/platform_device.h>
#include <drm/drm_device.h>
#include <errno.h>
#include "common/gb_uk.h"
#include "vpu/vpu_comm/vpu_basetype.h"
#include "vpu/vpu_comm/vpu_list.h"
#include "vpu/vpu_comm/vpu_vcmd_registers.h"
#include "vpu_subsys.h"
#include "vpu/vpu_comm/vpu_comm.h"

int hantrovcmd1_init(struct vpu_pcie_info dec_pcie, int total_vcmd_core_num,
	struct vpu_vcmd_buff *vcmd_buff, struct vpu_vcmd_buff *vcmd_reg,
	u64 dec_offset);

int dec1_vcmd_open(struct drm_file *filp, void **dec_priv,
	void *data);
long release_cmdbuf1(struct drm_file *filp, u16 cmdbuf_id);

long hantrovcmd1_ioctl(struct drm_file *filp,
		unsigned int cmd, void *arg);
void hantrovcmd1_cleanup(int total_vcmd_core_num);
struct DEC_VCMD_PRIVATE_DATA *get_dec1_vcmd_private_handle(void);
int dec1_vcmd_release(struct drm_file *filp, void *dec_priv);
void vcmd1_reset_current_asic(struct hantrovcmd_dev *dev);
void vcmd1_start(struct hantrovcmd_dev *dev,
		 bi_list_node *first_linked_cmdbuf_node);
void vcmd1_link_cmdbuf(struct hantrovcmd_dev *dev,
		       bi_list_node *last_linked_cmdbuf_node);
void vcmd1_delink_cmdbuf(struct hantrovcmd_dev *dev,
			 bi_list_node *last_linked_cmdbuf_node);

long link_and_run_cmdbuf1(struct drm_file *filp,
	int cmdbuf_id, int cmdbuf_size, int *core_id);
unsigned int wait_cmdbuf1_ready(struct drm_file *filp,
	u32 cmdbuf_id, u32 *irq_status_ret);
int gbdec1_cmdbuf_reserve(struct drm_file *filp, void *data);
int gb_vdec1_irq_handle(int irq_id);
void vcmd1_release_IO(struct hantrovcmd_dev *hantrovcmd_data,
		      int total_vcmd_core_num);
int vcmd1_reserve_IO(struct hantrovcmd_dev *hantrovcmd_data,
		     int total_vcmd_core_num, u8 *vpu_vcmd_base);
void vcmd1_reset_asic(struct hantrovcmd_dev  *dev,
		      int total_vcmd_core_num);
void set_address_vcmd1_registers(struct hantrovcmd_dev *dev,
				 u32 *jmp_addr, struct cmdbuf_obj *cmdbuf_obj);
void set_vcmd1_init_status_addr(struct DEC_VCMD_PRIVATE_DATA *dec_data,
				struct vpu_pcie_info dec_pcie,
				struct vpu_vcmd_buff *vcmd_pool,
				struct vpu_vcmd_buff *vcmd_reg);
void release_vcmd1_non_cachable_memory(void);
void dec1_resume(void);
void dec1_suspend(void);

#endif
