// SPDX-License-Identifier: GPL-3.0-or-later
/*
 * Copyright (C) Xi'an Sietium Electronics Co.Ltd
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * jessy 20221019 Sietium
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/ioctl.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/semaphore.h>
#include <linux/spinlock.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/uaccess.h>
#include <linux/ioport.h>
#include <asm/irq.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include "gpu/gb_device.h"
#include "common/gb_common.h"
#include "vpu_vc8000E_vcmd.h"
#include "vpu_vc8000E_irq.h"
#include "vpu/vpu_comm/vpu_list.h"
#include "vpu/vpu_comm/vpu_vcmd_registers.h"
#include "vpu/vpu_comm/vpu_comm.h"
#include "vpu/vpu_comm/vpu_basetype.h"
#include "common/gb_kernel_ver.h"

static void enc_free_cmdbuf_node(bi_list_node *cmdbuf_node);
/*for all vcmds, the core info should be listed here for subsequent use*/
static struct enc_vcmd_config enc_vcmd_core_array[] = {
	#if defined(NETINT) || defined(OYB_VCE)
	//encoder configuration
	{VCMD_ENC_IO_ADDR_0,
	VCMD_ENC_IO_SIZE_0,
	VCMD_ENC_INT_PIN_0,
	VCMD_ENC_MODULE_TYPE_0,
	VCMD_ENC_MODULE_MAIN_ADDR_0,
	VCMD_ENC_MODULE_DEC400_ADDR_0,
	VCMD_ENC_MODULE_L2CACHE_ADDR_0,
	{VCMD_ENC_MODULE_MMU0_ADDR_0,
		VCMD_ENC_MODULE_MMU1_ADDR_0},
	{VCMD_ENC_MODULE_AXIFE0_ADDR_0,
		VCMD_ENC_MODULE_AXIFE1_ADDR_0}},
	#endif
};

#ifdef HANTROAXIFE_SUPPORT
#define AXIFE_SIZE (64*4)
volatile u8 *axife_hwregs[MAX_VCMD_NUMBER][2];
#endif

struct enc_hantrovcmd_dev *enc_vcmd_manager[MAX_VCMD_TYPE][MAX_VCMD_NUMBER];
/* dynamic allocation*/
struct enc_hantrovcmd_dev *hantrovcmd_enc_data;
static struct ENC_PRIVATE_DATA *enc_private_data;

//hw_queue can be used for reserve cmdbuf memory
DECLARE_WAIT_QUEUE_HEAD(enc_vcmd_cmdbuf_memory_wait);
DEFINE_SPINLOCK(enc_vcmd_cmdbuf_alloc_lock);
DEFINE_SPINLOCK(enc_vcmd_process_manager_lock);

static u32 vcmd_release_AXIFE_IO(int total_vcmd_core_num)
{
#ifdef HANTROAXIFE_SUPPORT
	int i = 0, j = 0;

	for (i = 0; i < total_vcmd_core_num; i++) {
		for (j = 0; j < 2; j++) {
			if (axife_hwregs[i][j] != NULL) {
				iounmap(axife_hwregs[i][j]);
				release_mem_region(enc_vcmd_core_array[i].vcmd_base_addr +
				 enc_vcmd_core_array[i].submodule_axife_addr[j],
					AXIFE_SIZE);
				axife_hwregs[i][j] = NULL;
			}
		}
	}
#endif
	return 0;
}

static u32 ConfigAXIFE(u32 mode, int total_vcmd_core_num)
{
#ifdef HANTROAXIFE_SUPPORT
	u32 i = 0;
	u8 sub_module_type = 0;

	for (i = 0; i < total_vcmd_core_num; i++) {
		sub_module_type = enc_vcmd_core_array[i].sub_module_type;
		gb_printf(KERN_INFO, "%s: sub_module_type is %d\n",
			__func__, sub_module_type);
		if (enc_vcmd_core_array[i].submodule_axife_addr[0] != 0xffff) {
			if (!request_mem_region(enc_vcmd_core_array[i].vcmd_base_addr
					+ enc_vcmd_core_array[i].submodule_axife_addr[0],
					AXIFE_SIZE, "vc8000E")) {
				gb_printf(KERN_ERR, "%s: error request_mem for axife_hwregs[%d][0]\n",
					__func__, i);
				return -EBUSY;
			}
			axife_hwregs[i][0] =
				(volatile u8 *)ioremap_nocache(enc_vcmd_core_array[i].vcmd_base_addr
				+ enc_vcmd_core_array[i].submodule_axife_addr[0],
				AXIFE_SIZE);
			if (axife_hwregs[i][0] != NULL) {
				gb_printf(KERN_INFO, "%s: axife_hwregs[%d][0] = %p\n",
				 __func__, i,
					axife_hwregs[i][0]);
				AXIFEEnable(axife_hwregs[i][0], mode);
			}
		}
		if (enc_vcmd_core_array[i].submodule_axife_addr[1] != 0xffff) {
			if (!request_mem_region(enc_vcmd_core_array[i].vcmd_base_addr +
				enc_vcmd_core_array[i].submodule_axife_addr[1],
				 AXIFE_SIZE, "vc8000E")) {
				gb_printf(KERN_ERR, "%s: error request_mem for axife_hwregs[%d][1]\n",
					__func__, i);
			}
			axife_hwregs[i][1]  =
				(volatile u8 *)ioremap_nocache(enc_vcmd_core_array[i].vcmd_base_addr
				+ enc_vcmd_core_array[i].submodule_axife_addr[1],
				AXIFE_SIZE);
			if (axife_hwregs[i][1]  != NULL) {
				gb_printf(KERN_INFO, "%s: axife_hwregs[%d][1] = %p\n",
				 __func__, i,
					axife_hwregs[i][1]);
				AXIFEEnable(axife_hwregs[i][1], mode);
			}
		}
	}
#endif
	return 0;
}

static void enc_create_registers_cmdbuf(struct drm_gb_cmdbuf_resv *input_para,
	struct noncache_mem *enc_vcmd_mem_pool,
	bi_list_node **global_cmdbuf_node)
{
	u32 register_range[] = {VCMD_ENCODER_REGISTER_SIZE,
		VCMD_IM_REGISTER_SIZE, VCMD_DECODER_REGISTER_SIZE,
		VCMD_JPEG_ENCODER_REGISTER_SIZE,
		 VCMD_JPEG_DECODER_REGISTER_SIZE};
	u32 counter_cmdbuf_size = 0;
	struct noncache_mem *vcmd_status_buf_mem_pool = &enc_vcmd_mem_pool[1];
	u32  *set_base_addr;
	u16 submodule_main_addr =
		enc_vcmd_manager[input_para->module_type][0]->enc_vcmd_core_cfg.submodule_main_addr;
	ptr_t status_base_phy_addr;
	u32 offset_inc = 0;
	u32 offset_inc_dec400 = 0;
	u32 hw_version_id =
		enc_vcmd_manager[input_para->module_type][0]->hw_version_id;
	u16 submodule_L2Cache_addr =
		enc_vcmd_manager[input_para->module_type][0]->enc_vcmd_core_cfg.submodule_L2Cache_addr;
	u16 submodule_dec400_addr =
		enc_vcmd_manager[input_para->module_type][0]->enc_vcmd_core_cfg.submodule_dec400_addr;
	size_t base_ddr_addr;
	struct ENC_PRIVATE_DATA *enc_data = NULL;
	struct cmdbuf_obj *cmdbuf =
	 (struct cmdbuf_obj *)(global_cmdbuf_node[input_para->cmdbuf_id]->data);

	enc_data = get_enc_private_handle();
	base_ddr_addr = enc_data->enc_base_ddr_addr;
	set_base_addr = cmdbuf->cmdbuf_virtual_address;
	status_base_phy_addr = cmdbuf->status_bus_address +
	       (submodule_main_addr / 2 + 0);

	if (hw_version_id > HW_ID_1_0_C) {
		gb_printf(KERN_INFO, "%s: > hw_version_id:0x%x\n", __func__, hw_version_id);

		*(set_base_addr + 0) = (OPCODE_RREG) | (1 << 16)
			| (EXECUTING_CMDBUF_ID_ADDR * 4);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 1) = (u32)0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 2) = (u32)0;
		counter_cmdbuf_size += 4;

		//alignment
		*(set_base_addr + 3) = 0;
		counter_cmdbuf_size += 4;

		//read main IP all registers
		*(set_base_addr + 4) = (OPCODE_RREG)
			| ((register_range[input_para->module_type] / 4) << 16)
			| (submodule_main_addr + 0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 5) = (u32)(status_base_phy_addr
		 - base_ddr_addr);
		counter_cmdbuf_size += 4;
		if (sizeof(size_t) == 8)
			*(set_base_addr + 6) =
			 (u32)((u64)(status_base_phy_addr -
				base_ddr_addr) >> 32);
		else
			*(set_base_addr + 6) = 0;

		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 7) = 0;
		counter_cmdbuf_size += 4;

		if (submodule_L2Cache_addr != 0xffff) {
			/* read L2 cache register */
			offset_inc = 4;
			status_base_phy_addr =
			 vcmd_status_buf_mem_pool->bus_address +
				input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
				(submodule_L2Cache_addr / 2 + 0);
			/* read L2cache IP first register */
			*(set_base_addr + 8) = (OPCODE_RREG) | (1 << 16)
				| (submodule_L2Cache_addr + 0);
			counter_cmdbuf_size += 4;

			*(set_base_addr + 9) = (u32)(status_base_phy_addr -
				base_ddr_addr);
			counter_cmdbuf_size += 4;

			if (sizeof(size_t) == 8)
				*(set_base_addr + 10) =
					(u32)((u64)(status_base_phy_addr -
					 base_ddr_addr) >> 32);
			else
				*(set_base_addr + 10) = 0;

			counter_cmdbuf_size += 4;

			*(set_base_addr + 11) = 0;
			counter_cmdbuf_size += 4;
		}
		if (submodule_dec400_addr != 0xffff) {
			/* read dec400 register */
			offset_inc_dec400 = 4;
			status_base_phy_addr =
			 vcmd_status_buf_mem_pool->bus_address +
				input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
				(submodule_dec400_addr / 2 + 0);
			//read DEC400 IP first register
			*(set_base_addr + 8 + offset_inc) = (OPCODE_RREG)
				| (0x2b << 16) | (submodule_dec400_addr + 0);
			counter_cmdbuf_size += 4;
			*(set_base_addr + 9 + offset_inc) =
				(u32)(status_base_phy_addr - base_ddr_addr);
			counter_cmdbuf_size += 4;

			if (sizeof(size_t) == 8)
				*(set_base_addr + 10 + offset_inc) =
					(u32)((u64)(status_base_phy_addr -
					 base_ddr_addr) >> 32);
			else
				*(set_base_addr + 10 + offset_inc) = 0;
			counter_cmdbuf_size += 4;
			//alignment
			*(set_base_addr + 11 + offset_inc) = 0;
			counter_cmdbuf_size += 4;
		}

		/* read vcmd registers to ddr */
		*(set_base_addr + 8 + offset_inc + offset_inc_dec400) =
			(OPCODE_RREG) | (27 << 16) | (0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 9 + offset_inc + offset_inc_dec400) =
		 (u32)0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 10 + offset_inc + offset_inc_dec400) =
		 (u32)0;
		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 11 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		//JMP RDY = 0
		*(set_base_addr + 12 + offset_inc + offset_inc_dec400) =
			(OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 13 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 14 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 15 + offset_inc + offset_inc_dec400) =
			input_para->cmdbuf_id;
		//counter_cmdbuf_size += 4;
		input_para->cmdbuf_size = (16 + offset_inc +
		 offset_inc_dec400) * 4;
	} else {
		gb_printf(KERN_INFO, "%s: < hw_version_id:0x%x\n", __func__, hw_version_id);
		//read all registers
		*(set_base_addr + 0) = (OPCODE_RREG)
			| ((register_range[input_para->module_type] / 4) << 16)
			| (submodule_main_addr + 0);
		counter_cmdbuf_size += 4;

		*(set_base_addr + 1) = (u32)(status_base_phy_addr
		 - base_ddr_addr);
		counter_cmdbuf_size += 4;

		if (sizeof(size_t) == 8)
			*(set_base_addr + 2) =
				(u32)((u64)(status_base_phy_addr -
				 base_ddr_addr) >> 32);
		else
			*(set_base_addr + 2) = 0;
		counter_cmdbuf_size += 4;

		//alignment
		*(set_base_addr + 3) = 0;
		counter_cmdbuf_size += 4;

		//JMP RDY = 0
		*(set_base_addr + 4) = (OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;

		*(set_base_addr + 5) = 0;
		counter_cmdbuf_size += 4;

		*(set_base_addr + 6) = 0;
		counter_cmdbuf_size += 4;

		*(set_base_addr + 7) = input_para->cmdbuf_id;
		//counter_cmdbuf_size += 4;
		input_para->cmdbuf_size = 8 * 4;
	}
}

static int enc_find_empty_vcmd(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	unsigned long flags = 0;
	int counter = 0;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		spin_lock_irqsave(dev->spinlock, flags);
		if (list->tail == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			enc_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (enc_data->vcmd_position[cmdbuf_obj->module_type]
			 >= enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				enc_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return -1;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type]
				= 0;
		counter++;

		if (counter >= enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return 0;
}

static int enc_select_use_vcmd(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	unsigned long flags = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		spin_lock_irqsave(dev->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (curr_cmdbuf_node == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			enc_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
				enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				enc_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return -1;
		}
		cmdbuf_obj_temp = (struct cmdbuf_obj *)curr_cmdbuf_node->data;
		if (cmdbuf_obj_temp->cmdbuf_run_done == 1) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			enc_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
				enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				enc_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return -1;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type]
				= 0;
		counter++;
		if (counter >=
		 enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return 0;
}

static int enc_vcmd_pend_handle(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	u32 hw_rdy_cmdbuf_num = 0;
	unsigned long flags = 0;
	int counter = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		list = &dev->list_manager;
		//read executing cmdbuf address
		if (dev->hw_version_id <= HW_ID_1_0_C)
			hw_rdy_cmdbuf_num =
				vcmd_get_register_value((const void *)dev->hwregs,
				dev->reg_mirror, HWIF_VCMD_EXE_CMDBUF_COUNT);
		else {
			hw_rdy_cmdbuf_num =
				*(dev->vcmd_reg_mem_virtual_address +
				VCMD_EXE_CMDBUF_COUNT);
			if (hw_rdy_cmdbuf_num != dev->sw_cmdbuf_rdy_num)
				hw_rdy_cmdbuf_num += 1;
		}
		spin_lock_irqsave(dev->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (curr_cmdbuf_node == NULL) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			enc_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
				enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				enc_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return -1;
		}

		if (dev->sw_cmdbuf_rdy_num == hw_rdy_cmdbuf_num) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
			enc_data->vcmd_position[cmdbuf_obj->module_type]++;
			if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
				enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
				enc_data->vcmd_position[cmdbuf_obj->module_type]
				 = 0;
			cmdbuf_obj->core_id = dev->core_id;
			return -1;
		}
		spin_unlock_irqrestore(dev->spinlock, flags);
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return 0;
}

static bi_list_node *enc_get_cmdbuf_node_in_list_by_addr(size_t  cmdbuf_addr,
	bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	size_t base_ddr_addr = 0;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	base_ddr_addr = enc_data->enc_base_ddr_addr;
	new_cmdbuf_node = list->head;
	while (1) {
		if (new_cmdbuf_node == NULL)
			return NULL;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (((cmdbuf_obj->cmdbuf_bus_address - base_ddr_addr)
		 <= cmdbuf_addr) && (((cmdbuf_obj->cmdbuf_bus_address
		  - base_ddr_addr + cmdbuf_obj->cmdbuf_size)
		   > cmdbuf_addr)))
			return new_cmdbuf_node;
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	return NULL;
}

static int enc_vcmd_calculate_execute_time(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node **enc_global_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	size_t exe_cmdbuf_addr = 0;
	unsigned long flags = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	u32 cmdbuf_id = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		/* read executing cmdbuf address */
		if (dev->hw_version_id <= HW_ID_1_0_C) {
			exe_cmdbuf_addr =
			 VCMDGetAddrRegisterValue((const void *)dev->hwregs,
				dev->reg_mirror, HWIF_VCMD_EXECUTING_CMD_ADDR);
			list = &dev->list_manager;
			spin_lock_irqsave(dev->spinlock, flags);
			//get the executing cmdbuf node.
			curr_cmdbuf_node =
			 enc_get_cmdbuf_node_in_list_by_addr(exe_cmdbuf_addr,
				 list);

			//calculate total execute time of this device
			dev->total_exe_time =
				calculate_executing_time_after_node(curr_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
		} else {
			if (cmdbuf_obj->priority == CMDBUF_PRIORITY_NORMAL)
				cmdbuf_id =
				 *(dev->vcmd_reg_mem_virtual_address +
					EXECUTING_CMDBUF_ID_ADDR + 1);
			else	/* CMDBUF_PRIORITY_HIGH */
				cmdbuf_id =
				 *(dev->vcmd_reg_mem_virtual_address +
					EXECUTING_CMDBUF_ID_ADDR);
			spin_lock_irqsave(dev->spinlock, flags);
			gb_printf(KERN_INFO, "%s:%d-----cmdbuf id =%x----\n",
				__func__, __LINE__, cmdbuf_id);
			if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM
			 || cmdbuf_id == 0) {
				gb_printf(KERN_ERR, "cmdbuf_id greater than the ceiling !!\n");
				spin_unlock_irqrestore(dev->spinlock, flags);
				return -1;
			}
			//get the executing cmdbuf node.
			curr_cmdbuf_node = enc_global_cmdbuf_node[cmdbuf_id];
			if (curr_cmdbuf_node == NULL) {
				list = &dev->list_manager;
				curr_cmdbuf_node = list->head;
				while (1) {
					if (curr_cmdbuf_node == NULL)
						break;
					cmdbuf_obj_temp =
						(struct cmdbuf_obj *)curr_cmdbuf_node->data;
					if (cmdbuf_obj_temp->cmdbuf_data_linked
						&& cmdbuf_obj_temp->cmdbuf_run_done == 0)
						break;
					curr_cmdbuf_node =
					 curr_cmdbuf_node->next;
				}
			}
			/* calculate total execute time of this device */
			dev->total_exe_time =
				calculate_executing_time_after_node(curr_cmdbuf_node);
			spin_unlock_irqrestore(dev->spinlock, flags);
		}
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	return 0;
}

static void enc_find_least_total_exe_time(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	unsigned long flags = 0;
	u32 executing_time = 0xffffffff;
	struct enc_hantrovcmd_dev *smallest_dev = NULL;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		if (dev->total_exe_time <= executing_time) {
			executing_time = dev->total_exe_time;
			smallest_dev = dev;
		}
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}

	list = &smallest_dev->list_manager;
	spin_lock_irqsave(smallest_dev->spinlock, flags);
	bi_list_insert_node_tail(list, new_cmdbuf_node);
	spin_unlock_irqrestore(smallest_dev->spinlock, flags);
	cmdbuf_obj->core_id = smallest_dev->core_id;

}

static int enc_wait_abort_rdy(struct enc_hantrovcmd_dev *dev)
{
	return dev->working_state == WORKING_STATE_IDLE;
}

static int enc_vcmd_find_smallest_device(struct enc_hantrovcmd_dev *dst_dev,
	bi_list *list, bi_list_node *new_cmdbuf_node,
	struct cmdbuf_obj *cmdbuf_obj)
{
	int counter = 0;
	unsigned long flags = 0;
	bi_list_node *curr_cmdbuf_node = NULL;
	u32 executing_time = 0xffffffff;
	struct enc_hantrovcmd_dev *smallest_dev = NULL;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	struct enc_hantrovcmd_dev *dev;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		dev = enc_vcmd_manager[cmdbuf_obj->module_type][enc_data->vcmd_position[cmdbuf_obj->module_type]];
		if (dev->total_exe_time <= executing_time) {
			executing_time = dev->total_exe_time;
			smallest_dev = dev;
		}
		enc_data->vcmd_position[cmdbuf_obj->module_type]++;
		if (enc_data->vcmd_position[cmdbuf_obj->module_type] >=
			enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			enc_data->vcmd_position[cmdbuf_obj->module_type] = 0;
		counter++;
		if (counter >=
		 enc_data->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}

	vcmd_write_register_value((const void *)smallest_dev->hwregs,
	 smallest_dev->reg_mirror, HWIF_VCMD_START_TRIGGER, 0);
	enc_data->software_triger_abort = 1;
	if (wait_event_interruptible(*smallest_dev->wait_abort_queue,
		enc_wait_abort_rdy(smallest_dev))) {
		enc_data->software_triger_abort = 0;
		return -ERESTARTSYS;
	}
		enc_data->software_triger_abort = 0;
	spin_lock_irqsave(smallest_dev->spinlock, flags);
	curr_cmdbuf_node = smallest_dev->list_manager.head;
	while (1) {
		/*if list is empty or tail,insert to tail*/
		if (curr_cmdbuf_node == NULL)
			break;
		cmdbuf_obj_temp = (struct cmdbuf_obj *)curr_cmdbuf_node->data;
		if ((cmdbuf_obj_temp->priority == CMDBUF_PRIORITY_NORMAL)
			&& (cmdbuf_obj_temp->cmdbuf_run_done == 0))
			break;
		curr_cmdbuf_node = curr_cmdbuf_node->next;
	}
	bi_list_insert_node_before(list, curr_cmdbuf_node, new_cmdbuf_node);
	cmdbuf_obj->core_id = smallest_dev->core_id;
	spin_unlock_irqrestore(smallest_dev->spinlock, flags);
	return 0;
}

static int enc_select_vcmd(struct enc_hantrovcmd_dev *dst_dev,
	bi_list_node *new_cmdbuf_node, bi_list_node **enc_global_cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list *list = NULL;
	int ret = -1;

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;

	/* there is an empty vcmd to be used */
	ret = enc_find_empty_vcmd(dst_dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 == ret) {
		gb_printf(KERN_INFO, "%s-%d: return, ret = %d\n", __func__, __LINE__, ret);
		return 0;
	}

	ret = enc_select_use_vcmd(dst_dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 == ret) {
		gb_printf(KERN_INFO, "%s-%d: return, ret = %d\n", __func__, __LINE__, ret);
		return 0;
	}

	ret =  enc_vcmd_pend_handle(dst_dev, list, new_cmdbuf_node, cmdbuf_obj);
	if (-1 == ret) {
		gb_printf(KERN_INFO, "%s-%d: return, ret = %d\n", __func__, __LINE__, ret);
		return 0;
	}

	if (cmdbuf_obj->priority == CMDBUF_PRIORITY_NORMAL) {
		/* calculate total execute time of all devices*/
		ret = enc_vcmd_calculate_execute_time(dst_dev, list,
			enc_global_cmdbuf_node, cmdbuf_obj);
		if (-1 == ret) {
			gb_printf(KERN_ERR, "%s-%d: error, ret = %d\n",
			 __func__, __LINE__, ret);
			return ret;
		}

		/* find the device with the least total_exe_time.*/
		enc_find_least_total_exe_time(dst_dev, list,
			new_cmdbuf_node, cmdbuf_obj);
		return 0;
	}
	/* calculate total execute time of all devices*/
	ret = enc_vcmd_calculate_execute_time(dst_dev, list,
		enc_global_cmdbuf_node, cmdbuf_obj);
	if (-1 == ret) {
		gb_printf(KERN_ERR, "%s-%d: error, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}

	/* find the smallest device. */
	ret = enc_vcmd_find_smallest_device(dst_dev, list,
		new_cmdbuf_node, cmdbuf_obj);
	if (ret != 0) {
		gb_printf(KERN_ERR, "%s-%d: error, ret = %d\n", __func__, __LINE__, ret);
		return ret;
	}
	return 0;
}

static void enc_vcmd_the_first_cmdbuf(struct enc_hantrovcmd_dev *dev,
	bi_list_node *new_cmdbuf_node)
{
	u32 *jmp_addr = NULL;
	u32 operation_code;
	struct cmdbuf_obj *cmdbuf_obj;

	if (new_cmdbuf_node == NULL)
		return;
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if ((cmdbuf_obj->cmdbuf_data_linked == 0)) {
		(dev->sw_cmdbuf_rdy_num)++;
		cmdbuf_obj->cmdbuf_data_linked = 1;
		dev->duration_without_int = 0;
		if (cmdbuf_obj->has_end_cmdbuf == 0) {
			if (cmdbuf_obj->no_normal_int_cmdbuf == 1) {
				dev->duration_without_int =
					cmdbuf_obj->executing_time;
				//maybe nop is modified, so write back.
				if (dev->duration_without_int >=
					INT_MIN_SUM_OF_IMAGE_SIZE) {
					jmp_addr =
					 cmdbuf_obj->cmdbuf_virtual_address
					  + (cmdbuf_obj->cmdbuf_size / 4);
					operation_code = *(jmp_addr - 4);
					operation_code = JMP_IE_1
						| operation_code;
					*(jmp_addr - 4) = operation_code;
					dev->duration_without_int = 0;
				}
			}
		}
	}
}

static void enc_cmdbuf_link_tonext_cmdbuf(struct enc_hantrovcmd_dev *dev,
	struct cmdbuf_obj *cmdbuf_obj, struct cmdbuf_obj *next_cmdbuf_obj)
{
	u32 *jmp_addr = NULL;
	u32 operation_code;
	size_t base_ddr_addr = 0;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	base_ddr_addr = enc_data->enc_base_ddr_addr;

	gb_printf(KERN_INFO, "%s: Link cmdbuf %d to cmdbuf %d", __func__,
		cmdbuf_obj->cmdbuf_id, next_cmdbuf_obj->cmdbuf_id);
	jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
		(cmdbuf_obj->cmdbuf_size / 4);
	if (dev->hw_version_id > HW_ID_1_0_C)
		*(jmp_addr - 1) =
		 next_cmdbuf_obj->cmdbuf_id; /* set next cmdbuf id */

	if (sizeof(size_t) == 8)
		*(jmp_addr - 2) =
		 (u32)((u64)(next_cmdbuf_obj->cmdbuf_bus_address
			- base_ddr_addr) >> 32);
	else
		*(jmp_addr - 2) = 0;

	*(jmp_addr - 3) =
		(u32)(next_cmdbuf_obj->cmdbuf_bus_address - base_ddr_addr);
	operation_code = *(jmp_addr - 4);
	operation_code >>= 16;
	operation_code <<= 16;
	*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1
		| ((next_cmdbuf_obj->cmdbuf_size + 7) / 8));
	next_cmdbuf_obj->cmdbuf_data_linked = 1;
	(dev->sw_cmdbuf_rdy_num)++;
}

static void enc_modify_nop_code_next_cmdbuf(struct enc_hantrovcmd_dev *dev,
	struct cmdbuf_obj *next_cmdbuf_obj)
{
	u32 *jmp_addr = NULL;
	u32 operation_code;

	if (next_cmdbuf_obj->has_end_cmdbuf == 0) {
		if (next_cmdbuf_obj->no_normal_int_cmdbuf == 1) {
			dev->duration_without_int +=
			 next_cmdbuf_obj->executing_time;

			if (dev->duration_without_int >=
			 INT_MIN_SUM_OF_IMAGE_SIZE) {
				jmp_addr =
				 next_cmdbuf_obj->cmdbuf_virtual_address +
					(next_cmdbuf_obj->cmdbuf_size / 4);
				operation_code = *(jmp_addr - 4);
				operation_code = JMP_IE_1 | operation_code;
				*(jmp_addr - 4) = operation_code;
				dev->duration_without_int = 0;
			}
		}
	} else
		dev->duration_without_int = 0;
}

void enc_vcmd_link_cmdbuf(struct enc_hantrovcmd_dev *dev,
	bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *next_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct cmdbuf_obj *next_cmdbuf_obj = NULL;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	/* for the first cmdbuf. */
	enc_vcmd_the_first_cmdbuf(dev, new_cmdbuf_node);

	while (1) {
		if (new_cmdbuf_node == NULL)
			break;
		if (new_cmdbuf_node->next == NULL)
			break;
		next_cmdbuf_node = new_cmdbuf_node->next;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		next_cmdbuf_obj = (struct cmdbuf_obj *)next_cmdbuf_node->data;
		if (cmdbuf_obj->has_end_cmdbuf == 0
			&& !next_cmdbuf_obj->cmdbuf_run_done) {
			//need to link, current cmdbuf link to next cmdbuf
			enc_cmdbuf_link_tonext_cmdbuf(dev, cmdbuf_obj,
			 next_cmdbuf_obj);

			//modify nop code of next cmdbuf
			enc_modify_nop_code_next_cmdbuf(dev, next_cmdbuf_obj);
		}
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
}

static void enc_ConfigAIIXFE_MMU_BYVCMD(struct enc_hantrovcmd_dev **device)
{
#ifdef HANTROVCMD_ENABLE_IP_SUPPORT
	u32 i = 0;
	u64 address = 0;
	u32 mirror_index, register_index, register_value;
	u32 write_command = 0;

	if (!device)
		return;
	struct enc_hantrovcmd_dev *dev = *device;

	mirror_index = VCMD_REGISTER_INDEX_SW_INIT_CMD0;
	write_command = OPCODE_WREG | (1 << 26) | (1 << 16);
#ifdef HANTROAXIFE_SUPPORT
	//enable AXIFE by VCMD
	for (i = 0; i < 2; i++) {
		if (dev->enc_vcmd_core_cfg.submodule_axife_addr[i] != 0xffff) {
			register_index = AXI_REG10_SW_FRONTEND_EN;
			register_value = 0x02;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_axife_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;

			register_index = AXI_REG11_SW_WORK_MODE;
			register_value = 0x00;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_axife_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;
		}
	}
#endif
#ifdef HANTROMMU_SUPPORT
	//enable MMU by VCMD
	address = GetMMUAddress();
	gb_printf(KERN_INFO, "%s: GetMMUAddress address = 0x%llx\n", __func__, address);
	for (i = 0; i < 2; i++) {
		if (dev->enc_vcmd_core_cfg.submodule_MMU_addr[i] != 0xffff) {
			register_index = MMU_REG_ADDRESS;
			register_value = address;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_MMU_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;

			register_index = MMU_REG_PAGE_TABLE_ID;
			register_value = 0x10000;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_MMU_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;

			register_index = MMU_REG_PAGE_TABLE_ID;
			register_value = 0x00000;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_MMU_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;

			register_index = MMU_REG_CONTROL;
			register_value = 1;
			dev->reg_mirror[mirror_index++] = write_command
				| (dev->enc_vcmd_core_cfg.submodule_MMU_addr[i] +
				register_index);
			dev->reg_mirror[mirror_index++] = register_value;
		}
	}
#endif
	//END command
	dev->reg_mirror[mirror_index++] = OPCODE_END;
	dev->reg_mirror[mirror_index] = 0x00;

	for (i = 0; i < mirror_index - VCMD_REGISTER_INDEX_SW_INIT_CMD0; i++) {
		register_index = (i + VCMD_REGISTER_INDEX_SW_INIT_CMD0) * 4;
		vcmd_write_reg((const void *)dev->hwregs, register_index,
			dev->reg_mirror[i + VCMD_REGISTER_INDEX_SW_INIT_CMD0]);
	}
#endif
}

void enc_vcmd_start(struct enc_hantrovcmd_dev *dev,
	bi_list_node *first_linked_cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	size_t base_ddr_addr = 0;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	base_ddr_addr = enc_data->enc_base_ddr_addr;

	if (dev->working_state == WORKING_STATE_IDLE) {
		if ((first_linked_cmdbuf_node != NULL)
		 && dev->sw_cmdbuf_rdy_num) {
			cmdbuf_obj =
			 (struct cmdbuf_obj *)first_linked_cmdbuf_node->data;
			/* 0x40 */
			#ifdef HANTROVCMD_ENABLE_IP_SUPPORT
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_INIT_MODE, 1);
			#endif
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_AXI_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_CORE_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_ABORT_MODE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_RESET_CORE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_RESET_ALL, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_START_TRIGGER, 0);
			/* 0x48 */
			if (dev->hw_version_id <= HW_ID_1_0_C)
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_IRQ_INTCMD_EN, 0xffff);
			else {
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_IRQ_JMPP_EN, 1);
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_IRQ_JMPD_EN, 1);
			}

			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_RESET_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_ABORT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_CMDERR_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_TIMEOUT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_BUSERR_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_IRQ_ENDCMD_EN, 1);
			//0x4c
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_TIMEOUT_EN, 1);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_TIMEOUT_CYCLES, 0x1dcd6500);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR,
				(u32)(cmdbuf_obj->cmdbuf_bus_address));
			gb_printf(KERN_INFO, "--%s:%d--cmdbuf bus addr = %lx-\n",
				__func__, __LINE__,
				cmdbuf_obj->cmdbuf_bus_address);
			if (sizeof(size_t) == 8)
				vcmd_set_register_mirror_value(dev->reg_mirror,
				 HWIF_VCMD_EXECUTING_CMD_ADDR_MSB,
				(u32)((u64)(cmdbuf_obj->cmdbuf_bus_address) >> 32));
			else
				vcmd_set_register_mirror_value(dev->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR_MSB, 0);

			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_EXE_CMDBUF_LENGTH,
				(u32)((cmdbuf_obj->cmdbuf_size + 7) / 8));
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_RDY_CMDBUF_COUNT,
				 dev->sw_cmdbuf_rdy_num);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MAX_BURST_LEN, 0x10);
			if (dev->hw_version_id > HW_ID_1_0_C)
				vcmd_write_register_value((const void *)dev->hwregs,
					dev->reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID,
					(u32)cmdbuf_obj->cmdbuf_id);

			vcmd_write_reg((const void *)dev->hwregs, 0x44,
				vcmd_read_reg((const void *)dev->hwregs, 0x44));
			vcmd_write_reg((const void *)dev->hwregs, 0x40,
				dev->reg_mirror[0x40 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x48,
				dev->reg_mirror[0x48 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x4c,
				dev->reg_mirror[0x4c / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x50,
				dev->reg_mirror[0x50 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x54,
				dev->reg_mirror[0x54 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x58,
				dev->reg_mirror[0x58 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x5c,
				dev->reg_mirror[0x5c / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x60,
				dev->reg_mirror[0x60 / 4]);
			vcmd_write_reg((const void *)dev->hwregs, 0x64,
			 0xffffffff);
			dev->working_state = WORKING_STATE_WORKING;

			enc_ConfigAIIXFE_MMU_BYVCMD(&dev);

			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(dev->reg_mirror,
				HWIF_VCMD_START_TRIGGER, 1);
			vcmd_write_reg((const void *)dev->hwregs, 0x40,
				dev->reg_mirror[0x40 / 4]);
		}
	}
}

int enc_link_and_run_cmdbuf(struct drm_file *filp,
	int cmdbuf_id, int cmdbuf_size, int *core_id)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *last_cmdbuf_node;
	u32 *jmp_addr = NULL;
	u32 opCode, opcode_bak;
	u32 tempOpcode;
	u32 record_last_cmdbuf_rdy_num;
	struct enc_hantrovcmd_dev *dev = NULL;
	unsigned long flags;
	int return_value;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	new_cmdbuf_node = enc_data->enc_global_cmdbuf_node[cmdbuf_id];
	if (new_cmdbuf_node == NULL) {	//should not happen
		gb_printf(KERN_ERR, "%s: ERROR new_cmdbuf_node = NULL--cmdbud id =%d !!\n",
			__func__, cmdbuf_id);
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) { //should not happen
		gb_printf(KERN_ERR, "%s: ERROR cmdbuf_obj->filp != filp !!\n", __func__);
		return -1;
	}
	cmdbuf_obj->cmdbuf_data_loaded = 1;
	cmdbuf_obj->cmdbuf_size = cmdbuf_size;
	//test nop and end opcode, then assign value.
	cmdbuf_obj->has_end_cmdbuf = 0; /* 0: has jmp opcode, 1 has end code */
	/* 0: interrupt when JMP, 1 not interrupt when JMP */
	cmdbuf_obj->no_normal_int_cmdbuf = 0;
	jmp_addr =
		cmdbuf_obj->cmdbuf_virtual_address +
		 (cmdbuf_obj->cmdbuf_size / 4);
	opcode_bak = opCode = tempOpcode = *(jmp_addr - 4);
	gb_printf(KERN_INFO, "%s:%d-cmdbuf vaddr= %llx-cmdbufsize=%x,opCode=%x\n",
	__func__, __LINE__,
	(long long)cmdbuf_obj->cmdbuf_virtual_address,
	cmdbuf_obj->cmdbuf_size, opCode);
	opCode >>= 27;
	opCode <<= 27;
	if (opCode == OPCODE_JMP) {
		//jmp
		opCode = tempOpcode;
		opCode &= 0x02000000;
		if (opCode == JMP_IE_1)
			cmdbuf_obj->no_normal_int_cmdbuf = 0;
		else
			cmdbuf_obj->no_normal_int_cmdbuf = 1;
	} else {
		gb_printf(KERN_ERR, "%s-%d: error, so quit!!\n", __func__, __LINE__);
		mdelay(200);
		return -1;
	}

	if (down_interruptible(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type])))
		return -ERESTARTSYS;

	dev = &hantrovcmd_enc_data[cmdbuf_obj->core_id]; /* 0 or 1 */

	return_value = enc_select_vcmd(dev, new_cmdbuf_node,
	 enc_data->enc_global_cmdbuf_node);
	if (return_value) {
		up(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]));
		return return_value;
	}
	//input_para->core_id = cmdbuf_obj->core_id;
	*core_id = cmdbuf_obj->core_id;
	gb_printf(KERN_INFO, "%s: Allocate cmd buffer [%d]\n",
		__func__, cmdbuf_id);
	//set ddr address for vcmd registers copy.
	if (dev->hw_version_id > HW_ID_1_0_C) {
		jmp_addr = cmdbuf_obj->cmdbuf_virtual_address;
		if (sizeof(size_t) == 8)
			*(jmp_addr + 2) =
			 (u32)((u64)(dev->vcmd_reg_mem_bus_address
				+ (EXECUTING_CMDBUF_ID_ADDR + 1) * 4) >> 32);
		else
			*(jmp_addr + 2) = 0;
		*(jmp_addr + 1) = (u32)((dev->vcmd_reg_mem_bus_address
			+ (EXECUTING_CMDBUF_ID_ADDR + 1) * 4));

		jmp_addr = cmdbuf_obj->cmdbuf_virtual_address +
			(cmdbuf_obj->cmdbuf_size / 4);

		if (sizeof(size_t) == 8)
			*(jmp_addr - 6) =
				(u32)((u64)dev->vcmd_reg_mem_bus_address >> 32);
		else
			*(jmp_addr - 6) = 0;

		*(jmp_addr - 7) = (u32)(dev->vcmd_reg_mem_bus_address);
	}

	/* start to link and/or run */
	spin_lock_irqsave(dev->spinlock, flags);
	last_cmdbuf_node = find_last_linked_cmdbuf(new_cmdbuf_node);
	record_last_cmdbuf_rdy_num = dev->sw_cmdbuf_rdy_num;
	enc_vcmd_link_cmdbuf(dev, last_cmdbuf_node);
	tempOpcode = *(jmp_addr - 4);
	*(jmp_addr - 4) = opcode_bak | (tempOpcode & 0x2000000);
	if (dev->working_state == WORKING_STATE_IDLE) {
		if (filp == NULL)
			((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done = 0;
		while (last_cmdbuf_node &&
			((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done) {
			gb_printf(KERN_INFO, "%s:%d-----while---\n", __func__, __LINE__);
			last_cmdbuf_node = last_cmdbuf_node->next;
		}

		if (last_cmdbuf_node && last_cmdbuf_node->data) {
			gb_printf(KERN_INFO, "%s:%d---vcmd start for cmdbuf id %d,\
				cmdbuf_run_done = %d\n", __func__, __LINE__,
			 	((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_id,
			  	((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done);
		}
		enc_vcmd_start(dev, last_cmdbuf_node);
	} else {
		/* just update cmdbuf ready number */
		if (record_last_cmdbuf_rdy_num != dev->sw_cmdbuf_rdy_num)
			vcmd_write_register_value((const void *)dev->hwregs,
				dev->reg_mirror, HWIF_VCMD_RDY_CMDBUF_COUNT,
				dev->sw_cmdbuf_rdy_num);
	}
	spin_unlock_irqrestore(dev->spinlock, flags);
	up(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]));

	return 0;
}

static int enc_check_cmdbuf_irq(struct enc_hantrovcmd_dev *dev,
	struct cmdbuf_obj *cmdbuf_obj, u32 *irq_status_ret)
{
	int rdy = 0;
	unsigned long flags;

	spin_lock_irqsave(dev->spinlock, flags);
	if (cmdbuf_obj->cmdbuf_run_done) {
		rdy = 1;
		*irq_status_ret = cmdbuf_obj->executing_status;
	}
	spin_unlock_irqrestore(dev->spinlock, flags);
	return rdy;
}

unsigned int enc_wait_cmdbuf_ready(struct drm_file *filp, u32 cmdbuf_id,
	u32 *irq_status_ret)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	struct enc_hantrovcmd_dev *dev = NULL;
	bi_list_node **global_cmdbuf_node = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	global_cmdbuf_node = &enc_data->enc_global_cmdbuf_node[0];
	gb_printf(KERN_INFO, "%s: wait_cmdbuf_ready\n", __func__);
	new_cmdbuf_node = global_cmdbuf_node[cmdbuf_id];
	if (new_cmdbuf_node == NULL) {
		/* should not happen */
		gb_printf(KERN_ERR, "%s: ERROR new_cmdbuf_node == NULL !!\n", __func__);
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) {
		/* should not happen */
		gb_printf(KERN_ERR, "%s: ERROR cmdbuf_obj->filp != filp\n", __func__);
		return -1;
	}
	dev = &hantrovcmd_enc_data[cmdbuf_obj->core_id];
	if (wait_event_interruptible(*dev->wait_queue,
		enc_check_cmdbuf_irq(dev, cmdbuf_obj, irq_status_ret))) {
		gb_printf(KERN_ERR, "%s: wait_event_interruptible error\n", __func__);
		/* abort the vcmd */
		return -ERESTARTSYS;
	}
	return 0;
}

static void enc_free_cmdbuf_mem(u16 cmdbuf_id)
{
	unsigned long flags;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	spin_lock_irqsave(&enc_vcmd_cmdbuf_alloc_lock, flags);
	enc_data->enc_cmdbuf_used[cmdbuf_id] = 0;
	enc_data->enc_cmdbuf_used_residual += 1;
	spin_unlock_irqrestore(&enc_vcmd_cmdbuf_alloc_lock, flags);
	wake_up_interruptible_all(&enc_vcmd_cmdbuf_memory_wait);
}

/* cmdbuf pool management */
static int enc_allocate_cmdbuf(struct noncache_mem *new_cmdbuf_addr,
	struct noncache_mem *new_status_cmdbuf_addr,
	bi_list_node **global_cmdbuf_node, struct noncache_mem *enc_vcmd_mem_pool,
	struct vpu_vcmd_buff vcmd_buff, struct drm_gb_cmdbuf_resv *input_para)
{
	unsigned long flags;
	int cmdbuf_index, status_cmdbuf_index;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	spin_lock_irqsave(&enc_vcmd_cmdbuf_alloc_lock, flags);
	if (enc_data->enc_cmdbuf_used_residual == 0) {
		spin_unlock_irqrestore(&enc_vcmd_cmdbuf_alloc_lock, flags);
		gb_printf(KERN_ERR, "%s:%d--no empty cmdbuf--\n",
			__func__, __LINE__);
		return 0;	//no empty cmdbuf
	}
	//there is one cmdbuf at least
	while (1) {
		if (enc_data->enc_cmdbuf_used[enc_data->enc_cmdbuf_used_pos] == 0
			&& (global_cmdbuf_node[enc_data->enc_cmdbuf_used_pos] == NULL)) {
			enc_data->enc_cmdbuf_used[enc_data->enc_cmdbuf_used_pos] = 1;
			enc_data->enc_cmdbuf_used_residual -= 1;
			cmdbuf_index = (enc_data->enc_cmdbuf_used_pos - 1) * 2;
			status_cmdbuf_index = (enc_data->enc_cmdbuf_used_pos - 1) * 2 + 1;
			new_cmdbuf_addr->virtual_address =
				vcmd_buff.vir_buff + cmdbuf_index * CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->bus_address =
				vcmd_buff.offset + cmdbuf_index * CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->cmdbuf_id = enc_data->enc_cmdbuf_used_pos;

			new_status_cmdbuf_addr->virtual_address =
			 	vcmd_buff.vir_buff + status_cmdbuf_index * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->bus_address =
			 	vcmd_buff.offset + status_cmdbuf_index * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->cmdbuf_id =
			 	enc_data->enc_cmdbuf_used_pos;
			(enc_data->enc_cmdbuf_used_pos)++;
			if (enc_data->enc_cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
				enc_data->enc_cmdbuf_used_pos = 0;

			input_para->cmdbuf_offset = cmdbuf_index * CMDBUF_MAX_SIZE;
			input_para->status_offset = status_cmdbuf_index * CMDBUF_MAX_SIZE;
			spin_unlock_irqrestore(&enc_vcmd_cmdbuf_alloc_lock, flags);
			return 1;
		}
		(enc_data->enc_cmdbuf_used_pos)++;
		if (enc_data->enc_cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
			enc_data->enc_cmdbuf_used_pos = 0;
	}
	return 0;
}

static bi_list_node *enc_create_cmdbuf_node(bi_list_node **global_cmdbuf_node,
	struct noncache_mem *enc_vcmd_mem_pool, struct vpu_vcmd_buff vcmd_buff,
	struct drm_gb_cmdbuf_resv *input_para)
{
	bi_list_node *current_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct noncache_mem  new_cmdbuf_addr;
	struct noncache_mem  new_status_cmdbuf_addr;
	int ret_allocate = -1;

	memset((void *)&new_cmdbuf_addr, 0, sizeof(struct noncache_mem));
	memset((void *)&new_status_cmdbuf_addr, 0, sizeof(struct noncache_mem));
	ret_allocate =
	 enc_allocate_cmdbuf(&new_cmdbuf_addr, &new_status_cmdbuf_addr,
	  global_cmdbuf_node, enc_vcmd_mem_pool, vcmd_buff, input_para);
	gb_printf(KERN_INFO, "%s:%d--enter--wait allocate event--\n",
			__func__, __LINE__);
	if (wait_event_interruptible(enc_vcmd_cmdbuf_memory_wait, ret_allocate)) {
		gb_printf(KERN_ERR, "%s:%d--wait event interruptible--fail--\n",
			__func__, __LINE__);
		return NULL;
	}
	cmdbuf_obj = create_cmdbuf_obj();
	if (cmdbuf_obj == NULL) {
		gb_printf(KERN_ERR, "%s: create_cmdbuf_obj fail!\n", __func__);
		enc_free_cmdbuf_mem(new_cmdbuf_addr.cmdbuf_id);
		return NULL;
	}
	cmdbuf_obj->cmdbuf_bus_address = new_cmdbuf_addr.bus_address;
	cmdbuf_obj->mmu_cmdbuf_bus_address = new_cmdbuf_addr.mmu_bus_address;
	cmdbuf_obj->cmdbuf_virtual_address = new_cmdbuf_addr.virtual_address;
	cmdbuf_obj->cmdbuf_size = new_cmdbuf_addr.size;
	cmdbuf_obj->cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;
	cmdbuf_obj->status_bus_address = new_status_cmdbuf_addr.bus_address;
	cmdbuf_obj->mmu_status_bus_address =
		new_status_cmdbuf_addr.mmu_bus_address;
	cmdbuf_obj->status_virtual_address =
		new_status_cmdbuf_addr.virtual_address;
	cmdbuf_obj->status_size = new_status_cmdbuf_addr.size;
	current_node = bi_list_create_node();
	if (current_node == NULL) {
		gb_printf(KERN_ERR, "%s: bi_list_create_node fail!\n", __func__);
		enc_free_cmdbuf_mem(new_cmdbuf_addr.cmdbuf_id);
		free_cmdbuf_obj(cmdbuf_obj);
		return NULL;
	}
	gb_printf(KERN_INFO, "%s:%d-cmdbuf paddr=%llx-vaddr=%llx,\
	status paddr=%lx, vaddr=%llx---\n",
	__func__, __LINE__,
	(long long)cmdbuf_obj->cmdbuf_bus_address,
	(long long)cmdbuf_obj->cmdbuf_virtual_address,
	cmdbuf_obj->status_bus_address,
	(long long)cmdbuf_obj->status_virtual_address);

	current_node->data = (void *)cmdbuf_obj;
	current_node->next = NULL;
	current_node->previous = NULL;
	return current_node;
}

static long enc_reserve_cmdbuf(struct drm_file *filp,
	struct drm_gb_cmdbuf_resv *input_para,
	bi_list *global_process_manager,
	bi_list_node **global_cmdbuf_node,
	struct noncache_mem *enc_vcmd_mem_pool,
	struct vpu_vcmd_buff vcmd_buff)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *process_manager_node = NULL;
	struct process_manager_obj *process_manager_obj = NULL;
	unsigned long flags, flags1;

	input_para->cmdbuf_id = 0;
	if (input_para->cmdbuf_size > 2 *CMDBUF_MAX_SIZE) {
		gb_printf(KERN_ERR, "%s:%d--cmdbuf size=%x  to big-\n",
		__func__, __LINE__, input_para->cmdbuf_size);
		return -1;
	}
	spin_lock_irqsave(&enc_vcmd_process_manager_lock, flags);
	process_manager_node = global_process_manager->head;
	while (1) {
		if (process_manager_node == NULL) {
			//should not happen
			gb_printf(KERN_ERR, "%s: ERROR process_manager_node !!\n", __func__);
			spin_unlock_irqrestore(&enc_vcmd_process_manager_lock,
			 flags);
			return -1;
		}
		process_manager_obj =
			(struct process_manager_obj *)process_manager_node->data;
		if (filp == process_manager_obj->filp) {
			spin_lock_irqsave(&process_manager_obj->spinlock,
				flags1);
			process_manager_obj->total_exe_time +=
				input_para->executing_time;
			spin_unlock_irqrestore(&process_manager_obj->spinlock,
				flags1);
			break;
		}
		process_manager_node = process_manager_node->next;
	}
	spin_unlock_irqrestore(&enc_vcmd_process_manager_lock, flags);
	gb_printf(KERN_INFO, "%s:%d----total exe time = %llx---\n",
		__func__, __LINE__,
		process_manager_obj->total_exe_time);
	if (wait_event_interruptible(process_manager_obj->wait_queue,
		wait_process_resource_rdy(process_manager_obj))) {
		gb_printf(KERN_ERR, "%s:%d--wait process resource rdy fail\n",
			__func__, __LINE__);
		return -1;
	}
	new_cmdbuf_node = enc_create_cmdbuf_node(global_cmdbuf_node,
		enc_vcmd_mem_pool, vcmd_buff, input_para);
	if (new_cmdbuf_node == NULL) {
		gb_printf(KERN_ERR, "%s:%d--create cmdbuf node fail---\n",
			__func__, __LINE__);
		return -1;
	}

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	cmdbuf_obj->module_type = input_para->module_type;
	cmdbuf_obj->priority = input_para->priority;
	cmdbuf_obj->executing_time = input_para->executing_time;
	cmdbuf_obj->cmdbuf_size = CMDBUF_MAX_SIZE;
	input_para->cmdbuf_size = CMDBUF_MAX_SIZE;
	cmdbuf_obj->filp = filp;
	cmdbuf_obj->process_manager_obj = process_manager_obj;

	input_para->cmdbuf_id = cmdbuf_obj->cmdbuf_id;
	global_cmdbuf_node[input_para->cmdbuf_id] = new_cmdbuf_node;
	return 0;
}

static void enc_read_main_module_all_registers(u32 main_module_type,
	struct enc_hantrovcmd_dev *hantrovcmd_enc_data)
{
	int ret;
	int core_id = 0;
	struct drm_gb_cmdbuf_resv input_para;
	u32 irq_status_ret = 0;
	u32 *status_base_virt_addr;
	struct noncache_mem *vcmd_status_buf_mem_pool;
	u16 submodule_main_addr;
	struct ENC_PRIVATE_DATA *enc_data = NULL;
	enc_data = get_enc_private_handle();
	vcmd_status_buf_mem_pool = &enc_data->enc_vcmd_mem_pool[1];
	input_para.executing_time = 0;
	input_para.priority = CMDBUF_PRIORITY_NORMAL;
	input_para.module_type = main_module_type;
	input_para.cmdbuf_size = 0;
	submodule_main_addr =
	 enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_main_addr;
	ret = enc_reserve_cmdbuf(NULL, &input_para,
	 &enc_data->enc_global_process_manager,
	  enc_data->enc_global_cmdbuf_node,
	   enc_data->enc_vcmd_mem_pool, enc_data->vcmd_buff);
	enc_vcmd_manager[main_module_type][0]->status_cmdbuf_id =
		input_para.cmdbuf_id;
	enc_create_registers_cmdbuf(&input_para, enc_data->enc_vcmd_mem_pool,
		       enc_data->enc_global_cmdbuf_node);
	enc_link_and_run_cmdbuf(NULL, input_para.cmdbuf_id, input_para.cmdbuf_size, &core_id);
	if (enc_data->hw_info.hw_id != 0)
		return;
	enc_wait_cmdbuf_ready(NULL, input_para.cmdbuf_id,
		&irq_status_ret);
	status_base_virt_addr = enc_data->vcmd_buff.vir_buff +
		input_para.cmdbuf_id * CMDBUF_MAX_SIZE +
		(submodule_main_addr / 2 + 0);

	enc_data->hw_info.hw_id = *status_base_virt_addr;
	enc_data->hw_info.cfg1val = *(status_base_virt_addr + 80);
	enc_data->hw_info.cfg2val = *(status_base_virt_addr + 214);
	enc_data->hw_info.cfg3val = *(status_base_virt_addr + 226);
	enc_data->hw_info.cfg4val = *(status_base_virt_addr + 287);
	enc_data->hw_info.cfg5val = *(status_base_virt_addr + 430);
	enc_data->hw_info.cfgaxi = *(status_base_virt_addr + 319);
	gb_printf(KERN_INFO, "%s: enc hwid:0x%x\n", __func__, enc_data->hw_info.hw_id);
	gb_printf(KERN_INFO, "%s: enc cfg1val 0x%x\n", __func__, enc_data->hw_info.cfg1val);
	gb_printf(KERN_INFO, "%s: enc cfg2val 0x%x\n", __func__, enc_data->hw_info.cfg2val);
	gb_printf(KERN_INFO, "%s: enc cfg3val 0x%x\n", __func__, enc_data->hw_info.cfg3val);
	gb_printf(KERN_INFO, "%s: enc cfg4val 0x%x\n", __func__, enc_data->hw_info.cfg4val);
	gb_printf(KERN_INFO, "%s: enc cfg5val 0x%x\n", __func__, enc_data->hw_info.cfg5val);
	gb_printf(KERN_INFO, "%s: enc cfgaxi 0x%x\n", __func__, enc_data->hw_info.cfgaxi);

	gb_printf(KERN_INFO, "%s: enc reg 0:0x%x\n", __func__, *status_base_virt_addr);
	gb_printf(KERN_INFO, "%s: reg 80:0x%x\n", __func__, *(status_base_virt_addr + 80));
	gb_printf(KERN_INFO, "%s: reg 214:0x%x\n", __func__,
	 *(status_base_virt_addr + 214));
	gb_printf(KERN_INFO, "%s: reg 226:0x%x\n", __func__,
	 *(status_base_virt_addr + 226));
	gb_printf(KERN_INFO, "%s: reg 287:0x%x\n", __func__,
	 *(status_base_virt_addr + 287));
}


static int enc_vcmd_init(int total_vcmd_core_num, u64 enc_offset)
{
	int i = 0;

	for (i = 0; i < total_vcmd_core_num; i++)
		enc_vcmd_core_array[i].vcmd_base_addr =
		 enc_offset + enc_vcmd_core_array[i].vcmd_base_addr;

	return 0;
}

static int enc_vcmd_reserve(struct enc_hantrovcmd_dev *hantrovcmd_enc_data,
	int total_vcmd_core_num, u8 *vpu_vcmd_base)
{
	u32 hwid = 0;
	int i;
	u32 found_hw = 0, vcmd_iosize;
	unsigned long vcmd_base_addr;

	gb_printf(KERN_INFO, "%s: total_vcmd_core_num = %d\n",
	 __func__, total_vcmd_core_num);
	for (i = 0; i < total_vcmd_core_num; i++) {
		hantrovcmd_enc_data[i].hwregs = NULL;
		vcmd_base_addr =
		 hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_base_addr;
		vcmd_iosize =
		 hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_iosize;
		hantrovcmd_enc_data[i].hwregs = vpu_vcmd_base;
		if (hantrovcmd_enc_data[i].hwregs == NULL) {
			gb_printf(KERN_ERR, "%s: failed to ioremap HW regs\n", __func__);
			continue;
		}
		gb_printf(KERN_INFO, "%s: hantroenc_data[%d].hwregs=0x%p, hwid=0x%08x\n",
		 __func__, i, hantrovcmd_enc_data[i].hwregs, hwid);

		/*read hwid and check validness and store it*/
		hwid = (u32)ioread32((void *)hantrovcmd_enc_data[i].hwregs);
		hantrovcmd_enc_data[i].hw_version_id = hwid;
		gb_printf(KERN_INFO, "%s: hantroenc_data[%d].hwregs=0x%p, hwid=0x%08x\n",
			__func__, i, hantrovcmd_enc_data[i].hwregs, hwid);

		/* check for vcmd HW ID */
		if (((hwid >> 16) & 0xFFFF) != VCMD_HW_ID) {
			gb_printf(KERN_ERR, "%s: err HW not found at 0x%lx\n", __func__,
				hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_base_addr);
			hantrovcmd_enc_data[i].hwregs = NULL;
			continue;
		}

		found_hw = 1;
	}

	if (found_hw == 0) {
		gb_printf(KERN_ERR, "%s NO ANY HW found!!\n", __func__);
		return -1;
	}
	gb_printf(KERN_INFO, "%s-%d: enc vcmd reserve finish!!!\n", __func__, __LINE__);
	return 0;
}

static void enc_vcmd_reset_asic(struct enc_hantrovcmd_dev *dev,
	int total_vcmd_core_num)
{
	int i, n;
	u32 result;

	for (n = 0; n < total_vcmd_core_num; n++) {
		if (dev[n].hwregs != NULL) {
			//disable interrupt at first
			vcmd_write_reg((const void *)dev[n].hwregs,
				VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
			//reset all
			vcmd_write_reg((const void *)dev[n].hwregs,
				VCMD_REGISTER_CONTROL_OFFSET, 0x0002);
			//read status register
			result = vcmd_read_reg((const void *)dev[n].hwregs,
				VCMD_REGISTER_INT_STATUS_OFFSET);
			//clean status register
			vcmd_write_reg((const void *)dev[n].hwregs,
				VCMD_REGISTER_INT_STATUS_OFFSET, result);
			for (i = VCMD_REGISTER_CONTROL_OFFSET;
				i < dev[n].enc_vcmd_core_cfg.vcmd_iosize;
				 i += 4) {
				//set all register 0
				vcmd_write_reg((const void *)dev[n].hwregs,
				 i, 0x0000);
			}
			//enable all interrupt
			vcmd_write_reg((const void *)dev[n].hwregs,
				VCMD_REGISTER_INT_CTL_OFFSET, 0xffffffff);
		}
	}
}

void enc_resume(void)
{
	bi_list_node *cmdbuf_node = NULL;
	gb_printf(KERN_INFO, "%s:%d, enc resume in\n", __func__, __LINE__);

	cmdbuf_node = hantrovcmd_enc_data[0].list_manager.tail;
	if (cmdbuf_node != NULL)
		((struct cmdbuf_obj *)cmdbuf_node->data)->cmdbuf_run_done = 1;

	hantrovcmd_enc_data[0].working_state = WORKING_STATE_IDLE;
	hantrovcmd_enc_data[0].sw_cmdbuf_rdy_num = 0;

	enc_vcmd_reset_asic(hantrovcmd_enc_data, 1);

	gb_printf(KERN_INFO, "%s:%d, enc resume out\n", __func__, __LINE__);
}
void enc_suspend(void)
{
	gb_printf(KERN_INFO, "%s:%d, enc suspend in\n", __func__, __LINE__);

	for(;;) {
		if (list_empty(&hantrovcmd_enc_data[0].wait_queue->head))
			break;
	}

	gb_printf(KERN_INFO, "%s:%d, enc suspend out\n", __func__, __LINE__);
}

static void enc_vcmd_release_IO(struct enc_hantrovcmd_dev *hantrovcmd_enc_data,
	int total_vcmd_core_num)
{
	u32 i;

	vcmd_release_AXIFE_IO(total_vcmd_core_num);
	for (i = 0; i < total_vcmd_core_num; i++) {
		if (hantrovcmd_enc_data[i].hwregs) {
			iounmap((void *)hantrovcmd_enc_data[i].hwregs);
			release_mem_region(hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_base_addr,
				hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_iosize);
			hantrovcmd_enc_data[i].hwregs = NULL;
		}
	}
}

static u32 enc_vcmd_pool_release(struct noncache_mem *enc_vcmd_mem_pool)
{
	iounmap((void *)enc_vcmd_mem_pool[0].virtual_address);
	release_mem_region(enc_vcmd_mem_pool[0].bus_address,
		enc_vcmd_mem_pool[0].size);
	iounmap((void *)enc_vcmd_mem_pool[1].virtual_address);
	release_mem_region(enc_vcmd_mem_pool[1].bus_address,
		enc_vcmd_mem_pool[1].size);
	iounmap((void *)enc_vcmd_mem_pool[2].virtual_address);
	release_mem_region(enc_vcmd_mem_pool[2].bus_address,
		enc_vcmd_mem_pool[2].size);
	return 0;
}

void enc_vcmd_delink_cmdbuf(struct enc_hantrovcmd_dev *dev,
	bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	while (1) {
		if (new_cmdbuf_node == NULL)
			break;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_data_linked)
			cmdbuf_obj->cmdbuf_data_linked = 0;
		else
			break;
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	dev->sw_cmdbuf_rdy_num = 0;
}

int enc_vcmd_open(struct drm_file *filp, void **enc_priv, void *data)
{
	int result = 0;
	unsigned long flags;
	struct drm_gem_object *vcmd_obj = NULL;
	struct enc_hantrovcmd_dev *dev = hantrovcmd_enc_data;
	bi_list_node *process_manager_node;
	struct process_manager_obj *process_manager_obj = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;
	struct drm_gb_vpu_work *args = (struct drm_gb_vpu_work *)data;

	enc_data = get_enc_private_handle();
	vcmd_obj = enc_data->vcmd_pool->obj;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
	drm_vma_node_allow(&vcmd_obj->vma_node, filp);
#else
	drm_vma_node_allow(&vcmd_obj->vma_node, filp->filp);
#endif
	args->offset_va = enc_data->vcmd_pool->flp_offset;
	args->offset_pa = enc_data->vcmd_pool->offset;
	gb_printf(KERN_INFO, "%s:%d vcmdbuf offset_va=0x%llx offset_pa=0x%llx\n",
		__func__, __LINE__, args->offset_va, args->offset_pa);

	*enc_priv = (void *)dev;

	process_manager_node = create_process_manager_node();
	if (process_manager_node == NULL) {
		gb_printf(KERN_ERR, "%s:%d-----enc vcmd open fail---\n",
			__func__, __LINE__);
		return -1;
	}
	process_manager_obj =
		(struct process_manager_obj *)process_manager_node->data;
	process_manager_obj->filp = filp;
	spin_lock_irqsave(&enc_vcmd_process_manager_lock, flags);
	bi_list_insert_node_tail(&(enc_data->enc_global_process_manager),
		process_manager_node);
	spin_unlock_irqrestore(&enc_vcmd_process_manager_lock, flags);

	gb_printf(KERN_INFO, "%s: dev opened finish!\n", __func__);
	return result;
}

static long enc_release_cmdbuf_node(bi_list *list, bi_list_node *cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	/*get cmdbuf object according to cmdbuf_id*/
	new_cmdbuf_node = cmdbuf_node;
	if (new_cmdbuf_node == NULL)
		return -1;
	//remove node from list
	new_cmdbuf_node = remove_cmdbuf_node_from_list(list, new_cmdbuf_node);
	if (new_cmdbuf_node) {
		//free node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		enc_data->enc_global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
		enc_free_cmdbuf_node(new_cmdbuf_node);
		return 0;
	}
	return 1;
}

int enc_vcmd_release(struct drm_file *filp, void *enc_priv)
{
	struct enc_hantrovcmd_dev *dev =
		(struct enc_hantrovcmd_dev *)enc_priv;
		//(struct enc_hantrovcmd_dev *)filp->driver_priv;
	u32 core_id = 0;
	unsigned long flags;
	u32 release_cmdbuf_num = 0;
	struct drm_gem_object *vcmd_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	bi_list_node *process_manager_node;
	struct process_manager_obj *process_manager_obj = NULL;
	long retVal = 0;
	u32 total_vcmd_core_num;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	vcmd_obj = enc_data->vcmd_pool->obj;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
	drm_vma_node_revoke(&vcmd_obj->vma_node, filp);
#else
	drm_vma_node_revoke(&vcmd_obj->vma_node, filp->filp);
#endif

	total_vcmd_core_num =
		sizeof(enc_vcmd_core_array) / sizeof(struct enc_vcmd_config);
	if (down_interruptible(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[dev->enc_vcmd_core_cfg.sub_module_type])))
		return -ERESTARTSYS;

	for (core_id = 0; core_id < total_vcmd_core_num; core_id++) {
		if ((&dev[core_id]) == NULL)
			continue;
		spin_lock_irqsave(dev[core_id].spinlock, flags);
		new_cmdbuf_node = dev[core_id].list_manager.head;
		while (1) {
			if (new_cmdbuf_node == NULL)
				break;
			cmdbuf_obj_temp =
			 (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (dev[core_id].hwregs && (cmdbuf_obj_temp->filp == filp)) {
				if (cmdbuf_obj_temp->cmdbuf_run_done) {
					cmdbuf_obj_temp->cmdbuf_need_remove = 1;
					retVal =
					 enc_release_cmdbuf_node(&dev[core_id].list_manager,
						new_cmdbuf_node);
					if (retVal == 1)
						cmdbuf_obj_temp->process_manager_obj
						 = NULL;
				} else if (cmdbuf_obj_temp->cmdbuf_data_linked
				 == 0) {
					cmdbuf_obj_temp->cmdbuf_data_linked = 1;
					cmdbuf_obj_temp->cmdbuf_run_done = 1;
					cmdbuf_obj_temp->cmdbuf_need_remove = 1;
					retVal =
					 enc_release_cmdbuf_node(&dev[core_id].list_manager,
							new_cmdbuf_node);
					if (retVal == 1)
						cmdbuf_obj_temp->process_manager_obj
						 = NULL;
				} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1
					&& dev[core_id].working_state
						== WORKING_STATE_IDLE) {
					cmdbuf_obj_temp->cmdbuf_run_done = 1;
					cmdbuf_obj_temp->cmdbuf_need_remove
						= 1;
					retVal =
						enc_release_cmdbuf_node(&dev[core_id].list_manager,
						new_cmdbuf_node);
					if (retVal == 1)
						cmdbuf_obj_temp->process_manager_obj = NULL;
				} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1
					&& dev[core_id].working_state
						== WORKING_STATE_WORKING) {
					bi_list_node *last_cmdbuf_node;
					u32 record_last_cmdbuf_rdy_num;
					//abort the vcmd and wait
					enc_data->software_triger_abort = 1;
					if (wait_event_interruptible(*dev[core_id].wait_abort_queue,
						enc_wait_abort_rdy(&dev[core_id]))) {
						spin_unlock_irqrestore(dev[core_id].spinlock, flags);
						up(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[dev->enc_vcmd_core_cfg.sub_module_type]));
						enc_data->software_triger_abort
							= 0;
						return -ERESTARTSYS;
					}
					enc_data->software_triger_abort = 0;
					cmdbuf_obj_temp->cmdbuf_run_done = 1;
					cmdbuf_obj_temp->cmdbuf_need_remove
						= 1;
					retVal =
						enc_release_cmdbuf_node(&dev[core_id].list_manager,
						new_cmdbuf_node);
					if (retVal == 1)
						cmdbuf_obj_temp->process_manager_obj
							= NULL;
					//link
					last_cmdbuf_node =
						find_last_linked_cmdbuf(dev[core_id].list_manager.tail);
					record_last_cmdbuf_rdy_num =
						dev->sw_cmdbuf_rdy_num;
					enc_vcmd_link_cmdbuf(dev,
						last_cmdbuf_node);
					//re-run
					if (dev[core_id].sw_cmdbuf_rdy_num)
						enc_vcmd_start(dev,
						 last_cmdbuf_node);
				}
				release_cmdbuf_num++;
				gb_printf(KERN_INFO, "%s: release reserved cmdbuf!\n", __func__);
			}
			new_cmdbuf_node = new_cmdbuf_node->next;
		}
		spin_unlock_irqrestore(dev[core_id].spinlock, flags);
	}
	if (release_cmdbuf_num)
		wake_up_interruptible_all(&enc_vcmd_cmdbuf_memory_wait);
	spin_lock_irqsave(&enc_vcmd_process_manager_lock, flags);
	process_manager_node = enc_data->enc_global_process_manager.head;
	while (1) {
		if (process_manager_node == NULL)
			break;
		process_manager_obj =
			(struct process_manager_obj *)process_manager_node->data;
		if (process_manager_obj->filp == filp)
			break;
		process_manager_node = process_manager_node->next;
	}
	//remove node from list
	bi_list_remove_node(&enc_data->enc_global_process_manager,
		process_manager_node);
	spin_unlock_irqrestore(&enc_vcmd_process_manager_lock, flags);
	free_process_manager_node(process_manager_node);
	up(&enc_data->enc_vcmd_reserve_cmdbuf_sem[dev->enc_vcmd_core_cfg.sub_module_type]);
	gb_printf(KERN_INFO, "%s: dev close finish!!\n", __func__);
	return 0;
}

static void enc_free_cmdbuf_node(bi_list_node *cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	if (cmdbuf_node == NULL) {
		gb_printf(KERN_ERR, "%s: remove_cmdbuf_node NULL\n", __func__);
		return;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)cmdbuf_node->data;

	enc_free_cmdbuf_mem(cmdbuf_obj->cmdbuf_id);
	free_cmdbuf_obj(cmdbuf_obj);
	bi_list_free_node(cmdbuf_node);
}

long enc_release_cmdbuf(struct drm_file *filp, u16 cmdbuf_id)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *last_cmdbuf_node = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list *list = NULL;
	u32 module_type;
	unsigned long flags;
	struct enc_hantrovcmd_dev *dev = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	/*get cmdbuf object according to cmdbuf_id*/
	new_cmdbuf_node = enc_data->enc_global_cmdbuf_node[cmdbuf_id];
	if (new_cmdbuf_node == NULL) {
		//should not happen
		gb_printf(KERN_ERR, "%s: ERROR cmdbuf_id !!!!\n", __func__);
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) {
		//should not happen
		gb_printf(KERN_ERR, "%s: ERROR cmdbuf_id !!\n", __func__);
		return -1;
	}
	module_type = cmdbuf_obj->module_type;
	//TODO
	if (down_interruptible
		(&enc_data->enc_vcmd_reserve_cmdbuf_sem[module_type])) {
		gb_printf(KERN_ERR, "%s:%d--down interruptible--\n",
			__func__, __LINE__);
		return -ERESTARTSYS;
	}
	dev = &hantrovcmd_enc_data[cmdbuf_obj->core_id];

	list = &dev->list_manager;
	cmdbuf_obj->cmdbuf_need_remove = 1;
	last_cmdbuf_node = new_cmdbuf_node->previous;
	while (1) {
		//remove current node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_need_remove == 1) {
			new_cmdbuf_node =
				remove_cmdbuf_node_from_list(list,
				 new_cmdbuf_node);
			if (new_cmdbuf_node) {
				//free node
				enc_data->enc_global_cmdbuf_node[cmdbuf_obj->cmdbuf_id]
					= NULL;
				if (cmdbuf_obj->process_manager_obj) {
					spin_lock_irqsave(&cmdbuf_obj->process_manager_obj->spinlock, flags);
					cmdbuf_obj->process_manager_obj->total_exe_time -=
						cmdbuf_obj->executing_time;
					spin_unlock_irqrestore(&cmdbuf_obj->process_manager_obj->spinlock, flags);
					wake_up_interruptible_all(&cmdbuf_obj->process_manager_obj->wait_queue);
				}
				enc_free_cmdbuf_node(new_cmdbuf_node);
			}
		}
		if (last_cmdbuf_node == NULL)
			break;
		new_cmdbuf_node = last_cmdbuf_node;
		last_cmdbuf_node = new_cmdbuf_node->previous;
	}
	//spin_unlock_irqrestore(dev->spinlock, flags);
	up(&enc_data->enc_vcmd_reserve_cmdbuf_sem[module_type]);
	return 0;
}

int gbenc_cmdbuf_reserve(struct drm_file *filp, void *data)
{
	int ret = -1;
	struct vpu_vcmd_buff vcmd_buf = {0};
	struct drm_gb_cmdbuf_resv *input_para =
		(struct drm_gb_cmdbuf_resv *)data;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	vcmd_buf.offset = enc_data->vcmd_pool->offset;
	vcmd_buf.vir_buff = enc_data->vcmd_pool->vir_buff;

	gb_printf(KERN_INFO, "%s:%d:-vcmd buff offset = 0x%llx, vir_buf = 0x%llx--\n",
		__func__, __LINE__,
		vcmd_buf.offset, (long long)vcmd_buf.vir_buff);
	ret = enc_reserve_cmdbuf(filp, input_para,
		&enc_data->enc_global_process_manager,
		enc_data->enc_global_cmdbuf_node,
		enc_data->enc_vcmd_mem_pool, vcmd_buf);
	if (ret == -1)
		gb_printf(KERN_INFO, "%s: enc VCMD Reserve %d\n", __func__,
			input_para->cmdbuf_id);

	return ret;
}


long enc_ioctl(struct drm_file *filp, unsigned int cmd,
	void *arg)
{
	int err = 0;
	size_t base_ddr_addr = 0;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	gb_printf(KERN_INFO, "%s: enc ioctl start cmd: 0x%08x\n", __func__, cmd);

	if (_IOC_TYPE(cmd) != HANTRO_IOC_MAGIC
	){
		gb_printf(KERN_ERR, "%s:%d--------ioctl fail---\n", __func__, __LINE__);
		return -ENOTTY;
	}
	if ((_IOC_TYPE(cmd) == HANTRO_IOC_MAGIC &&
		_IOC_NR(cmd) > HANTRO_IOC_MAXNR)
		) {
		gb_printf(KERN_ERR, "%s:%d--------ioctl fail---\n", __func__, __LINE__);
		return -ENOTTY;
	}
	if (_IOC_DIR(cmd) & _IOC_READ)
		err = !gb_access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd));
	if (_IOC_DIR(cmd) & _IOC_WRITE)
		err = !gb_access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd));
	if (err) {
		gb_printf(KERN_ERR, "%s:%d--------ioctl fail---\n", __func__, __LINE__);
		return -EFAULT;
	}
	gb_printf(KERN_INFO, "%s:------cmd = %x---\n", __func__, cmd);
	switch (cmd) {
	case HANTRO_IOCH_GET_VCMD_ENABLE:
	{
		__put_user(1, (unsigned long *)arg);
		break;
	}
	case HANTRO_IOCH_GET_CMDBUF_PARAMETER:
	{
		struct cmdbuf_mem_parameter local_cmdbuf_mem_data;
		struct noncache_mem	*vcmd_buf_mem_pool;
		struct noncache_mem	*vcmd_status_buf_mem_pool;

		vcmd_buf_mem_pool = &enc_data->enc_vcmd_mem_pool[0];
		vcmd_status_buf_mem_pool = &enc_data->enc_vcmd_mem_pool[1];
		base_ddr_addr = enc_data->enc_pcie.vram_paddr_base;

		local_cmdbuf_mem_data.cmdbuf_unit_size = CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_unit_size =
		 CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.cmdbuf_total_size =
		 CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_total_size =
			CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.phy_status_cmdbuf_addr =
			vcmd_status_buf_mem_pool->bus_address;
		local_cmdbuf_mem_data.phy_cmdbuf_addr =
			vcmd_buf_mem_pool->bus_address;

		gb_printf(KERN_INFO, "%s:%d----phy status cmdbuf addr=%lx--- \
			phy cmdbuf addr=%lx-base_ddr_addr=%lx-\n",
			__func__, __LINE__,
			local_cmdbuf_mem_data.phy_status_cmdbuf_addr,
			local_cmdbuf_mem_data.phy_cmdbuf_addr, base_ddr_addr);
		local_cmdbuf_mem_data.base_ddr_addr = base_ddr_addr;
		return copy_to_user((struct cmdbuf_mem_parameter *)arg,
		 &local_cmdbuf_mem_data,
		  sizeof(struct cmdbuf_mem_parameter));
	}
	case HANTRO_IOCH_GET_VCMD_PARAMETER:
	{
		struct enc_config_parameter input_para;
		int ret;

		gb_printf(KERN_INFO, "%s-%d: VCMD get vcmd config parameter\n",
			__func__, __LINE__);
		ret = copy_from_user(&input_para,
		 (struct enc_config_parameter *)arg,
		  sizeof(struct enc_config_parameter));
		if (enc_data->vcmd_type_core_num[input_para.module_type]) {
			input_para.submodule_main_addr =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_main_addr;
			input_para.submodule_dec400_addr =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_dec400_addr;
			input_para.submodule_L2Cache_addr =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_L2Cache_addr;
			input_para.submodule_MMU_addr[0] =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_MMU_addr[0];
			input_para.submodule_MMU_addr[1] =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_MMU_addr[1];
			input_para.submodule_axife_addr[0] =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_axife_addr[0];
			input_para.submodule_axife_addr[1] =
				enc_vcmd_manager[input_para.module_type][0]->enc_vcmd_core_cfg.submodule_axife_addr[1];
			input_para.config_status_cmdbuf_id =
				enc_vcmd_manager[input_para.module_type][0]->status_cmdbuf_id;
			input_para.vcmd_hw_version_id =
			 enc_vcmd_manager[input_para.module_type][0]->hw_version_id;
			input_para.vcmd_core_num =
			enc_data->vcmd_type_core_num[input_para.module_type];
			gb_printf(KERN_INFO, "%s:%d------module type = %x---\n",
				__func__, __LINE__, input_para.module_type);
		} else {
			input_para.submodule_main_addr = 0xffff;
			input_para.submodule_dec400_addr = 0xffff;
			input_para.submodule_L2Cache_addr = 0xffff;
			input_para.submodule_MMU_addr[0] = 0xffff;
			input_para.submodule_MMU_addr[1] = 0xffff;
			input_para.submodule_axife_addr[0] = 0xffff;
			input_para.submodule_axife_addr[1] = 0xffff;
			input_para.config_status_cmdbuf_id = 0;
			input_para.vcmd_core_num = 0;
			input_para.vcmd_hw_version_id = HW_ID_1_0_C;
		}
		return copy_to_user((struct enc_config_parameter *)arg,
		 &input_para, sizeof(struct enc_config_parameter));
	}
	case HANTRO_IOCH_GET_HWINFO_FROM_VCMD: {
		struct enc_hw_info *hw_info = &enc_data->hw_info;
		struct enc_hw_info uhw_info;

		memcpy(&uhw_info, hw_info, sizeof(struct enc_hw_info));
		gb_printf(KERN_INFO, "%s:ioctl get hw info from vcmd---\n", __func__);
		return copy_to_user((struct enc_hw_info *)arg,
			&uhw_info, sizeof(struct enc_hw_info));
	}
	default:
		gb_printf(KERN_INFO, "%s: enc IOCTL default, quit!!!\n", __func__);
	}
	return 0;
}

struct ENC_PRIVATE_DATA *get_enc_private_handle(void)
{
	return enc_private_data;
}

struct ENC_PRIVATE_DATA *enc_vmalloc_handle(int total_vcmd_core_num)
{
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	hantrovcmd_enc_data =
	 (struct enc_hantrovcmd_dev *)vmalloc(sizeof(struct enc_hantrovcmd_dev)
		* total_vcmd_core_num);
	if (hantrovcmd_enc_data == NULL)
		goto err1;
	memset(hantrovcmd_enc_data, 0,
		sizeof(struct enc_hantrovcmd_dev) * total_vcmd_core_num);

	enc_data =
	 (struct ENC_PRIVATE_DATA *)vmalloc(sizeof(struct ENC_PRIVATE_DATA));
	if (enc_data == NULL)
		goto err1;
	memset(enc_data, 0, sizeof(struct ENC_PRIVATE_DATA));
	return enc_data;
err1:
	if (hantrovcmd_enc_data != NULL)
		vfree(hantrovcmd_enc_data);
	if (enc_data != NULL)
		vfree(enc_data);
	gb_printf(KERN_ERR, "%s-%d: err1 module not vmalloc error\n", __func__, __LINE__);
	return NULL;
}

void enc_private_data_init(struct ENC_PRIVATE_DATA *enc_data,
	struct gb_pci_bar *pci_bars, struct gb_asic_dev_info *dev_info,
	struct vpu_offset_base vpu_offset, struct vpu_vcmd_buff *vcmd_pool)
{
	int i = 0, k;

	i = get_genbu_gpu_reg_bar_id(dev_info);
	enc_data->enc_pcie.vpu_paddr_base =
	 pci_bars[i].base + vpu_offset.enc_offset_base;
	enc_data->enc_pcie.vpu_vaddr_base =
	 pci_bars[i].mmio + vpu_offset.enc_offset_base;
	i = get_genbu_ddr_bar_id(dev_info);
	enc_data->enc_pcie.vram_vaddr_base = pci_bars[i].mmio;
	enc_data->enc_pcie.vram_paddr_base = pci_bars[i].base;
	enc_data->vcmd_buff.offset = vcmd_pool->offset;
	enc_data->vcmd_buff.vir_buff = vcmd_pool->vir_buff;
	enc_data->enc_base_ddr_addr = 0;
	enc_data->enc_cmdbuf_used_pos = 0;
	for	(i = 0; i < TOTAL_DISCRETE_CMDBUF_NUM; i++) {
		enc_data->enc_cmdbuf_used[i] = 0;
		enc_data->enc_global_cmdbuf_node[i] = NULL;
	}

	enc_data->enc_cmdbuf_used_residual = TOTAL_DISCRETE_CMDBUF_NUM;
	enc_data->enc_cmdbuf_used_pos = 1;
	enc_data->enc_cmdbuf_used[0] = 1;
	enc_data->enc_cmdbuf_used_residual -= 1;

	for (k = 0; k < MAX_VCMD_TYPE; k++) {
		enc_data->vcmd_type_core_num[k] = 0;
		enc_data->vcmd_position[k] = 0;
		for (i = 0; i < MAX_VCMD_NUMBER; i++)
			enc_vcmd_manager[k][i] = NULL;
	}
}

void enc_set_vcmd_init_status_addr(struct ENC_PRIVATE_DATA *enc_data,
	struct vpu_vcmd_buff *vcmd_pool, struct vpu_vcmd_buff *vcmd_reg)
{
	gb_printf(KERN_INFO, "%s:%d--vcmd pool offset =%llx, \
		vaddr = %llx, reg offset=%llx, vaddr=%llx--\n",
		__func__, __LINE__, vcmd_pool->offset,
		(long long)vcmd_pool->vir_buff,
	vcmd_reg->offset, (long long)vcmd_reg->vir_buff);
	enc_data->enc_vcmd_mem_pool[0].bus_address = vcmd_pool->offset;
	enc_data->enc_vcmd_mem_pool[1].bus_address = vcmd_pool->offset
		+ CMDBUF_MAX_SIZE;
	enc_data->enc_vcmd_mem_pool[2].bus_address = vcmd_reg->offset;

	enc_data->enc_vcmd_mem_pool[0].virtual_address = vcmd_pool->vir_buff;
	enc_data->enc_vcmd_mem_pool[1].virtual_address =
	 vcmd_pool->vir_buff + CMDBUF_MAX_SIZE;
	enc_data->enc_vcmd_mem_pool[2].virtual_address = vcmd_reg->vir_buff;
}

void hantrovcmd_enc_data_init(struct ENC_PRIVATE_DATA *enc_data)
{
	int i;
	static struct noncache_mem *enc_vcmd_registers_mem_pool;

	enc_vcmd_registers_mem_pool = &(enc_data->enc_vcmd_mem_pool[2]);
	for (i = 0; i < enc_data->total_vcmd_core_num; i++) {
		if (enc_vcmd_core_array[i].vcmd_irq != 0) {
			gb_printf(KERN_INFO, "%s-%d: enc before irq = %d, should irq=0\n",
			 __func__, __LINE__, enc_vcmd_core_array[i].vcmd_irq);
			enc_vcmd_core_array[i].vcmd_irq = 0;
		}
		hantrovcmd_enc_data[i].enc_vcmd_core_cfg =
		 enc_vcmd_core_array[i];
		hantrovcmd_enc_data[i].hwregs = NULL;
		hantrovcmd_enc_data[i].core_id = i;
		hantrovcmd_enc_data[i].working_state = WORKING_STATE_IDLE;
		hantrovcmd_enc_data[i].sw_cmdbuf_rdy_num = 0;
		hantrovcmd_enc_data[i].spinlock =
			&(enc_data->enc_owner_lock_vcmd[i]);
		spin_lock_init(&(enc_data->enc_owner_lock_vcmd[i]));
		hantrovcmd_enc_data[i].wait_queue =
			&(enc_data->enc_wait_queue_vcmd[i]);
		init_waitqueue_head(&(enc_data->enc_wait_queue_vcmd[i]));
		hantrovcmd_enc_data[i].wait_abort_queue =
			&(enc_data->enc_abort_queue_vcmd[i]);
		init_waitqueue_head(&(enc_data->enc_abort_queue_vcmd[i]));
		init_bi_list(&hantrovcmd_enc_data[i].list_manager);
		hantrovcmd_enc_data[i].duration_without_int = 0;
		enc_vcmd_manager[enc_vcmd_core_array[i].sub_module_type][enc_data->vcmd_type_core_num[enc_vcmd_core_array[i].sub_module_type]]
		 = &hantrovcmd_enc_data[i];
		enc_data->vcmd_type_core_num[enc_vcmd_core_array[i].sub_module_type]++;
		hantrovcmd_enc_data[i].vcmd_reg_mem_bus_address =
			enc_vcmd_registers_mem_pool->bus_address +
			i * VCMD_REGISTER_SIZE;
		hantrovcmd_enc_data[i].vcmd_reg_mem_virtual_address =
			enc_vcmd_registers_mem_pool->virtual_address +
			i * VCMD_REGISTER_SIZE;
		hantrovcmd_enc_data[i].vcmd_reg_mem_size = VCMD_REGISTER_SIZE;
#ifndef CONFIG_SW64
		memset(hantrovcmd_enc_data[i].vcmd_reg_mem_virtual_address, 0,
                        VCMD_REGISTER_SIZE);
#else
		memset_io(hantrovcmd_enc_data[i].vcmd_reg_mem_virtual_address, 0,
                       VCMD_REGISTER_SIZE);
#endif
	}
}
irqreturn_t enc_isr(int irq)
{
	irqreturn_t ret = 0;
	ret = enc_hantrovcmd_isr(irq, &hantrovcmd_enc_data[0]);
	return ret;
}

int enc_init(struct enc_hantrovcmd_dev *gbenc_data, void *pci_bars,
	struct gb_asic_dev_info *dev_info, struct vpu_vcmd_buff *vcmd_pool,
	struct vpu_vcmd_buff *vcmd_reg, struct vpu_offset_base vpu_offset)
{
	int i, total_vcmd_core_num;
	int result = -1;
	bi_list *tmp_enc_manager = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;
	u64 enc_paddr;
	struct vpu_vcmd_buff *vcmd_pool_tmp = NULL;

	total_vcmd_core_num =
		sizeof(enc_vcmd_core_array) / sizeof(struct enc_vcmd_config);
	enc_data = enc_vmalloc_handle(total_vcmd_core_num);
	if (enc_data == NULL)
		goto err1;

	vcmd_pool_tmp =
		(struct vpu_vcmd_buff *)vmalloc(sizeof(struct vpu_vcmd_buff));
	if (vcmd_pool_tmp == NULL)
		goto err1;
	memset(vcmd_pool_tmp, 0, sizeof(struct vpu_vcmd_buff));

	memcpy(vcmd_pool_tmp, vcmd_pool, sizeof(struct vpu_vcmd_buff));
	enc_data->vcmd_pool = vcmd_pool_tmp;

	gbenc_data = hantrovcmd_enc_data;
	enc_private_data = enc_data;

	enc_data->total_vcmd_core_num = total_vcmd_core_num;
	enc_private_data_init(enc_data, (struct gb_pci_bar *)pci_bars,
	 dev_info, vpu_offset, vcmd_pool);

	enc_set_vcmd_init_status_addr(enc_data, vcmd_pool, vcmd_reg);

	enc_paddr = enc_data->enc_pcie.vpu_paddr_base;
	result = enc_vcmd_init(total_vcmd_core_num, enc_paddr);
	if (result)
		goto err1;
	for (i = 0; i < total_vcmd_core_num; i++)
		gb_printf(KERN_INFO, "%s: enc init - vcmdcore[%d] addr =0x%lx\n",
		__func__, i, enc_vcmd_core_array[i].vcmd_base_addr);

	tmp_enc_manager = &(enc_data->enc_global_process_manager);
	init_bi_list(tmp_enc_manager);
	result = ConfigAXIFE(1, total_vcmd_core_num);
	if (result < 0) {
		vcmd_release_AXIFE_IO(total_vcmd_core_num);
		goto err1;
	}
	hantrovcmd_enc_data_init(enc_data);

	result = enc_vcmd_reserve(hantrovcmd_enc_data, total_vcmd_core_num,
	 (u8 *)enc_data->enc_pcie.vpu_vaddr_base);
	if (result < 0)
		goto err1;
	enc_vcmd_reset_asic(hantrovcmd_enc_data, total_vcmd_core_num);

	create_kernel_process_manager(tmp_enc_manager);
	for (i = 0; i < MAX_VCMD_TYPE; i++) {
		if (enc_data->vcmd_type_core_num[i] == 0)
			continue;
		sema_init(&(enc_data->enc_vcmd_reserve_cmdbuf_sem[i]), 1);
	}

	for (i = 0; i < MAX_VCMD_TYPE; i++) {
		if (enc_data->vcmd_type_core_num[i] == 0)
			continue;
		enc_read_main_module_all_registers(i, hantrovcmd_enc_data);
	}
	gb_printf(KERN_INFO, "%s-%d: enc_data[%d].enc_vcmd_core_cfg.vcmd_irq = %d\n",
	 __func__, __LINE__, 0,
	  hantrovcmd_enc_data[0].enc_vcmd_core_cfg.vcmd_irq);
	gb_printf(KERN_INFO, "%s-%d: enc vcmd finish !!!\n", __func__, __LINE__);
	return 0;
err1:
	if (hantrovcmd_enc_data != NULL)
		vfree(hantrovcmd_enc_data);
	if (enc_data != NULL)
		vfree(enc_data);
	if (vcmd_pool_tmp != NULL)
		vfree(vcmd_pool_tmp);
	gb_printf(KERN_ERR, "%s: err1 module not inserted\n", __func__);
	return result;
}

int gb_venc_irq_handle(int irq_id)
{
	int i, ret = -1;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
		/* get the IRQ line */
	for (i = 0; i < enc_data->total_vcmd_core_num; i++) {
		if (hantrovcmd_enc_data[i].hwregs == NULL)
			continue;
		if (hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_irq != -1) {
			gb_printf(KERN_INFO, "%s-%d: vpu enc irq_id = %d\n",
			 __func__, __LINE__, irq_id);
			ret =
			 request_irq(irq_id,
			  enc_hantrovcmd_isr,
			#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18))
			SA_INTERRUPT | SA_SHIRQ,
			#else
			IRQF_SHARED,
			#endif
			"vc8000", (void *)&hantrovcmd_enc_data[i]);
			if (ret == -EINVAL) {
				gb_printf(KERN_ERR, "%s: enc vcmd_irq error core_id = %d\n",
					__func__, i);
				enc_vcmd_release_IO(hantrovcmd_enc_data,
				 enc_data->total_vcmd_core_num);
				goto err;
			} else if (ret == -EBUSY) {
				gb_printf(KERN_ERR, "%s: id = %d,IRQ %d busy\n", __func__, i,
				 hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_irq);
				enc_vcmd_release_IO(hantrovcmd_enc_data,
				 enc_data->total_vcmd_core_num);
				goto err;
			} else
				gb_printf(KERN_INFO, "%s-%d: enc irq success, irq = %d",
				 __func__, __LINE__, irq_id);
		} else
			gb_printf(KERN_ERR, "%s: enc IRQ not in use!\n", __func__);
	}
	return 0;
err:
#ifdef HANTROMMU_SUPPORT
	enc_vcmd_pool_release(enc_data->enc_vcmd_mem_pool);
#endif
	if (hantrovcmd_enc_data != NULL)
		vfree(hantrovcmd_enc_data);
	if (enc_data != NULL)
		vfree(enc_data);
	gb_printf(KERN_ERR, "%s: err1 module not inserted\n", __func__);
	return ret;
}

static long enc_release_cmdbuf_node_cleanup(bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();

	while (1) {
		new_cmdbuf_node = list->head;
		if (new_cmdbuf_node == NULL)
			return 0;
		//remove node from list
		bi_list_remove_node(list, new_cmdbuf_node);
		//free node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		enc_data->enc_global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
		enc_free_cmdbuf_node(new_cmdbuf_node);
	}
	return 0;
}

void hantroenc_vcmd_cleanup(void)
{
	int i = 0;
	u32 result;
	struct ENC_PRIVATE_DATA *enc_data = NULL;

	enc_data = get_enc_private_handle();
	for (i = 0; i < enc_data->total_vcmd_core_num; i++) {
		if (hantrovcmd_enc_data[i].hwregs == NULL)
			continue;
		/* disable interrupt at first */
		vcmd_write_reg((const void *)hantrovcmd_enc_data[i].hwregs,
			VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		/* disable HW */
		vcmd_write_reg((const void *)hantrovcmd_enc_data[i].hwregs,
			VCMD_REGISTER_CONTROL_OFFSET, 0x0000);
		/* read status register */
		result =
		 vcmd_read_reg((const void *)hantrovcmd_enc_data[i].hwregs,
		 VCMD_REGISTER_INT_STATUS_OFFSET);
		/* clean status register */
		vcmd_write_reg((const void *)hantrovcmd_enc_data[i].hwregs,
			VCMD_REGISTER_INT_STATUS_OFFSET, result);

		/* free the vcmd IRQ */
		if (hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_irq != -1)
			free_irq(hantrovcmd_enc_data[i].enc_vcmd_core_cfg.vcmd_irq,
				(void *)&hantrovcmd_enc_data[i]);
		enc_release_cmdbuf_node_cleanup(&hantrovcmd_enc_data[i].list_manager);
	}

	release_process_node_cleanup(&enc_data->enc_global_process_manager);
	enc_vcmd_release_IO(hantrovcmd_enc_data, enc_data->total_vcmd_core_num);

	enc_vcmd_pool_release(enc_data->enc_vcmd_mem_pool);
	if (hantrovcmd_enc_data != NULL)
		vfree(hantrovcmd_enc_data);
	if (enc_data != NULL)
		vfree(enc_data);

	gb_printf(KERN_INFO, "%s: module removed finish\n", __func__);
}


